@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -27,7 +28,7 @@
 #include <string.h>
 #include "liberty.h"
 
-SGFTree *aftermath_sgftree;
+static SGFTree *aftermath_sgftree;
 
 /* Generate a move to definitely settle the position after the game
  * has been finished. The purpose of this is to robustly determine
@@ -123,11 +124,13 @@ aftermath_genmove(int *aftermath_move, int color,
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (!ON_BOARD(pos))
       continue;
-    else if (board[pos] == color && worm[pos].unconditional_status == ALIVE)
+    else if (board[pos] == color && worm[pos].invincible)
       distance[pos] = 0;
     else if (!do_capture_dead_stones
-	     && board[pos] == other 
-	     && worm[pos].unconditional_status == DEAD)
+	     && ((board[pos] == other 
+		  && worm[pos].unconditional_status == DEAD)
+		 || (board[pos] == color
+		     && worm[pos].unconditional_status == ALIVE)))
       distance[pos] = 0;
     else
       distance[pos] = -1;
@@ -256,9 +259,9 @@ aftermath_genmove(int *aftermath_move, int color,
 		  && libs == countlib(pos + dir)))
 	  && (DRAGON2(pos + dir).safety == INVINCIBLE
 	      || DRAGON2(pos + dir).safety == STRONGLY_ALIVE)) {
-	int score = 20 * (owl_hotspot[pos] + reading_hotspot[pos]);
-	if (score > best_score) {
-	  best_score = score;
+	int this_score = 20 * (owl_hotspot[pos] + reading_hotspot[pos]);
+	if (this_score > best_score) {
+	  best_score = this_score;
 	  best_scoring_move = pos;
 	}
       }
@@ -416,7 +419,7 @@ aftermath_genmove(int *aftermath_move, int color,
 	int origin = dragon[pos2].origin;
 	
 	if (k < 4) {
-	  if (dragon[pos2].matcher_status == ALIVE) {
+	  if (dragon[pos2].status == ALIVE) {
 	    safety = DEAD;
 	    break;
 	  }
@@ -426,7 +429,7 @@ aftermath_genmove(int *aftermath_move, int color,
 	  }
 	}
 
-	if (!mx[origin] && dragon[pos2].matcher_status == DEAD) {
+	if (!mx[origin] && dragon[pos2].status == DEAD) {
 	  bonus++;
 	  if (k < 4 
 	      && countlib(pos2) <= 2 
@@ -441,7 +444,7 @@ aftermath_genmove(int *aftermath_move, int color,
       else if (board[pos2] == color) {
 	dragons[pos] = pos2;
 	
-	if (safety == UNKNOWN && dragon[pos2].matcher_status == ALIVE)
+	if (safety == UNKNOWN && dragon[pos2].status == ALIVE)
 	  safety = ALIVE;
 	
 	if (DRAGON2(pos2).safety == INVINCIBLE)
@@ -465,11 +468,11 @@ aftermath_genmove(int *aftermath_move, int color,
 	    int important = 0;
 	    int safe_atari = 0;
 	    for (r = 0; r < 4; r++) {
-	      int d = delta[r];
+	      d = delta[r];
 	      if (!ON_BOARD(apos+d))
 		continue;
 	      if (board[apos+d] == other
-		  && dragon[apos+d].matcher_status == DEAD)
+		  && dragon[apos+d].status == DEAD)
 		important = 1;
 	      else if (board[apos+d] == EMPTY
 		       && !is_self_atari(apos+d, other))
@@ -557,8 +560,8 @@ aftermath_genmove(int *aftermath_move, int color,
 	|| !safe_move(move, color)
 	|| (DRAGON2(bb).safety != INVINCIBLE
 	    && DRAGON2(bb).safety != STRONGLY_ALIVE
-	    && owl_does_defend(move, bb) != WIN)
-	|| (!confirm_safety(move, color, 0, NULL, NULL, NULL))) {
+	    && owl_does_defend(move, bb, NULL) != WIN)
+	|| (!confirm_safety(move, color, NULL, NULL))) {
       score[move] = 0;
     }
     else {
@@ -584,7 +587,7 @@ aftermath_genmove(int *aftermath_move, int color,
 	    for (r = 0; r < neighbors; r++) {
 	      if (worm[adjs[r]].attack_codes[0] != 0
 		  && (find_defense(adjs[r], NULL)
-		      > worm[adjs[r]].defend_codes[0])) {
+		      > worm[adjs[r]].defense_codes[0])) {
 		DEBUG(DEBUG_AFTERMATH,
 		      "Blunder: %1m becomes tactically safer after %1m\n",
 		      adjs[r], move);
@@ -593,8 +596,8 @@ aftermath_genmove(int *aftermath_move, int color,
 	    }
 	    popgo();
 	    for (r = 0; r < neighbors && move_ok; r++) {
-	      if (dragon[adjs[r]].matcher_status == DEAD
-		  && !owl_does_attack(move, adjs[r])) {
+	      if (dragon[adjs[r]].status == DEAD
+		  && !owl_does_attack(move, adjs[r], NULL)) {
 		DEBUG(DEBUG_AFTERMATH,
 		      "Blunder: %1m becomes more alive after %1m\n",
 		      adjs[r], move);
@@ -620,7 +623,6 @@ aftermath_genmove(int *aftermath_move, int color,
    * dragons, carefully checking against mistakes.
    */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    int move;
     int target;
     int cc = NO_MOVE;
     int self_atari_ok = 0;
@@ -632,7 +634,7 @@ aftermath_genmove(int *aftermath_move, int color,
       if (!ON_BOARD(pos2))
 	continue;
       if (board[pos2] == other 
-	  && dragon[pos2].matcher_status != ALIVE
+	  && dragon[pos2].status != ALIVE
 	  && (do_capture_dead_stones 
 	      || worm[pos2].unconditional_status != DEAD)
 	  && DRAGON2(pos2).safety != INESSENTIAL) {
@@ -720,17 +722,17 @@ aftermath_genmove(int *aftermath_move, int color,
     /* Consult the owl code to determine whether the considered move
      * really is effective. Blunders should be detected here.
      */
-    if (owl_does_attack(move, target) == WIN) {
+    if (owl_does_attack(move, target, NULL) == WIN) {
       /* If we have an adjacent own dragon, which is not inessential,
        * verify that it remains safe.
        */
-      if (cc != NO_MOVE && !owl_does_defend(move, cc))
+      if (cc != NO_MOVE && !owl_does_defend(move, cc, NULL))
 	continue;
 
       /* If we don't allow self atari, also call confirm safety to
        * avoid setting up combination attacks.
        */
-      if (!self_atari_ok && !confirm_safety(move, color, 0, NULL, NULL, NULL))
+      if (!self_atari_ok && !confirm_safety(move, color, NULL, NULL))
 	continue;
 	  
       *aftermath_move = move;
@@ -804,7 +806,7 @@ reduced_genmove(int *move, int color)
    * we are behind, we have to play more daringly.
    */
   if (level >= 8) {
-    estimate_score(&lower_bound, &upper_bound);
+    estimate_score(&upper_bound, &lower_bound);
     if (verbose || showscore) {
       if (lower_bound == upper_bound)
 	gprintf("\nScore estimate: %s %f\n",
@@ -833,14 +835,11 @@ reduced_genmove(int *move, int color)
    * Ok, information gathering is complete. Now start to find some moves!
    */
 
-  /* Pick up tactical moves. */
-  worm_reasons(color);
-  
-  /* Pick up owl moves. */
+  /* Pick up moves that we know of already. */
   save_verbose = verbose;
   if (verbose > 0)
     verbose--;
-  owl_reasons(color);
+  collect_move_reasons(color);
   verbose = save_verbose;
   
   /* Look for combination attacks and defenses against them. */
@@ -848,7 +847,7 @@ reduced_genmove(int *move, int color)
   gg_assert(stackp == 0);
 
   /* Review the move reasons and estimate move values. */
-  if (review_move_reasons(move, &val, color, 0.0, lower_bound))
+  if (review_move_reasons(move, &val, color, 0.0, score, NULL))
     TRACE("Move generation likes %1m with value %f\n", *move, val);
   gg_assert(stackp == 0);
 
@@ -894,7 +893,7 @@ do_play_aftermath(int color, struct aftermath_data *a)
     }
     play_move(move, color_to_play);
     if (aftermath_sgftree)
-      sgftreeAddPlay(aftermath_sgftree, NULL, color_to_play, I(move), J(move));
+      sgftreeAddPlay(aftermath_sgftree, color_to_play, I(move), J(move));
     moves++;
     DEBUG(DEBUG_AFTERMATH, "%d %C move %1m (nodes %d, %d  total %d, %d)\n",
 	  movenum, color_to_play, move, get_owl_node_counter() - owl_nodes,
@@ -999,6 +998,17 @@ play_aftermath(int color)
       }
     }
   }
+
+  if (debug & DEBUG_AFTERMATH) {
+    gprintf("White captured: %d\n", a->white_captured);
+    gprintf("Black captured: %d\n", a->black_captured);
+    gprintf("White prisoners: %d\n", a->white_prisoners);
+    gprintf("Black prisoners: %d\n", a->black_prisoners);
+    gprintf("White territory: %d\n", a->white_territory);
+    gprintf("Black territory: %d\n", a->black_territory);
+    gprintf("White area: %d\n", a->white_area);
+    gprintf("Black area: %d\n", a->black_area);
+  }
 }
 
 float
