@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -20,6 +21,7 @@
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include <stdio.h>
+#include "hash.h"
 
 /*
  * This file, together with engine/hash.c implements hashing of go positions
@@ -48,14 +50,11 @@
  * The data1 field packs into 32 bits the following
  * fields:
  *
- * komaster:  3 bits (EMPTY, BLACK, WHITE, or GRAY)
- * kom_pos : 10 bits (allows MAX_BOARD up to 31)
- * routine :  4 bits (currently 10 different choices)
- * str1    : 10 bits
- * stackp  :  5 bits (actually remaining depth, depth - stackp)
- *
- * FIXME: Rename stackp to something like remaining_depth at some
- *        appropriate time.
+ * komaster	   :  3 bits (EMPTY, BLACK, WHITE, or GRAY)
+ * kom_pos	   : 10 bits (allows MAX_BOARD up to 31)
+ * routine	   :  4 bits (currently 10 different choices)
+ * str1 	   : 10 bits
+ * remaining_depth :  5 bits (depth - stackp)
  *
  * The data2 field packs into 32 bits the following
  * fields:
@@ -75,39 +74,27 @@ typedef struct read_result_t {
 } Read_result;
 
 /* Bit mask for the input bits in the data2 field. */
-#define RR_INPUT_DATA2 0x3ff
+#define RR_INPUT_DATA2		0x3ff
+
+/* Read_result entry status. */
+#define RR_STATUS_OPEN		(1 << 28)
+#define RR_STATUS_CLOSED	(2 << 28)
 
 /* Get parts of a Read_result identifying the input data. */
-#define rr_get_komaster(rr)   (((rr).data1  >> 29) & 0x07)
-#define rr_get_kom_pos(rr)    (((rr).data1  >> 19) & 0x3ff)
-#define rr_get_routine(rr)    (((rr).data1  >> 15) & 0x0f)
-#define rr_get_str1(rr)       (((rr).data1  >>  5) & 0x3ff)
-#define rr_get_stackp(rr)     (((rr).data1  >>  0) & 0x1f)
-#define rr_get_str2(rr)       (((rr).data2  >>  0) & 0x3ff)
-#define rr_get_str(rr)        rr_get_str1(rr)
+#define rr_get_komaster(rr)	    (((rr).data1  >> 29) & 0x07)
+#define rr_get_kom_pos(rr)	    (((rr).data1  >> 19) & 0x3ff)
+#define rr_get_routine(rr)	    (((rr).data1  >> 15) & 0x0f)
+#define rr_get_str1(rr) 	    (((rr).data1  >>  5) & 0x3ff)
+#define rr_get_remaining_depth(rr)  (((rr).data1  >>  0) & 0x1f)
+#define rr_get_str2(rr) 	    (((rr).data2  >>  0) & 0x3ff)
+#define rr_get_str(rr)		    rr_get_str1(rr)
 
 /* Set corresponding parts. */
-#define rr_input_data1(routine, komaster, kom_pos, str1, stackp) \
+#define rr_input_data1(routine, komaster, kom_pos, str1, remaining_depth) \
 	(((((((((komaster) << 10) | (kom_pos)) << 4) \
-	  | (routine)) << 10) | (str1)) << 5) | (stackp));
+	  | (routine)) << 10) | (str1)) << 5) | (remaining_depth))
 #define rr_input_data2(str2) (str2) \
 
-/* Set input data fields and at the same time set status to open. */
-#define rr_set_input_data(rr, routine, komaster, kom_pos, str, stackp) \
-       do { \
-         (rr).data1 = rr_input_data1(routine, komaster, kom_pos, str, stackp);\
-         (rr).data2 = (((rr).data2 & ~0x300003ff) | (1 << 28));\
-       } while (0)
-
-/* Variation for two distinct strings. */
-#define rr_set_input_data2(rr, routine, komaster, kom_pos, str1, str2, stackp)\
-       do { \
-         (rr).data1 = rr_input_data1(routine, komaster, kom_pos, \
-                                     str1, stackp); \
-         (rr).data2 = (((rr).data2 & ~0x3ff) | (1 << 28) \
-                       | rr_input_data2(str2)); \
-       } while (0)
-
 /* Get parts of a Read_result constituting the result of a search. */
 #define rr_get_status(rr)      (((rr).data2 >> 28) & 0x03)
 #define rr_get_result1(rr)     (((rr).data2 >> 24) & 0x0f)
@@ -115,14 +102,19 @@ typedef struct read_result_t {
 #define rr_get_move(rr)        (((rr).data2 >> 10) & 0x3ff)
 #define rr_get_result(rr)      rr_get_result1(rr)
 
-/* Set corresponding parts. */
+#define rr_is_free(rr)	       (((rr).data2 \
+				& (RR_STATUS_OPEN | RR_STATUS_CLOSED)) == 0)
+#define rr_is_open(rr)	       (((rr).data2 & RR_STATUS_OPEN) != 0)
+#define rr_is_closed(rr)       (((rr).data2 & RR_STATUS_CLOSED) != 0)
+
+/* Set corresponding parts. Closes the result entry. */
 #define rr_set_result_move(rr, result, move) \
-	(rr).data2 = (((rr).data2 & 0x3ff) \
-          | (2 << 28) | (((result) & 0x0f) << 24) | (((move) & 0x3ff) << 10))
+	(rr).data2 = (((rr).data2 & 0x3ff) | RR_STATUS_CLOSED \
+		      | (((result) & 0x0f) << 24) | (((move) & 0x3ff) << 10))
 
 /* Variation with two results. */
 #define rr_set_result_move2(rr, result1, result2, move) \
-	(rr).data2 = (((rr).data2 & 0x3ff) | (2 << 28) \
+	(rr).data2 = (((rr).data2 & 0x3ff) | RR_STATUS_CLOSED \
                       | (((result1) & 0x0f) << 24) \
                       | (((result2) & 0x0f) << 20) \
                       | (((move) & 0x3ff) << 10))
@@ -156,16 +148,22 @@ typedef struct hashnode_t {
  */
 
 typedef struct hashtable {
-  int            hashtablesize;	/* Number of hash buckets */
-  Hashnode     **hashtable;	/* Pointer to array of hashnode lists */
+  int		 hashtablesize;	/* Number of hash buckets. */
+  Hashnode     **hashtable;	/* Pointer to array of hashnode lists. */
 
-  int            num_nodes;	/* Total number of hash nodes */
-  Hashnode      *all_nodes;	/* Pointer to all allocated hash nodes. */
-  int            free_node;	/* Index to next free node. */
+  Hashnode	*all_nodes;	/* Pointer to all allocated hash nodes. */
+  Hashnode	*node_limit;	/* Pointer to the end of all_nodes[] array. */
+  Hashnode	*free_node;	/* Pointer to the first free node. */
 
-  int            num_results;	/* Total number of results */
-  Read_result   *all_results;	/* Pointer to all allocated results. */
-  int            free_result;	/* Index to next free result. */
+  Read_result	*all_results;	/* Pointer to all allocated results. */
+  Read_result	*result_limit;	/* Pointer to the enf of all_results[] array. */
+  Read_result	*free_result;	/* Pointer to the first free result. */
+
+  /* True if hashtable_partially_clear() hasn't been called yet. In this case
+   * allocated nodes and results (if any) go continuosly in memory which
+   * simplifies clearing.
+   */
+  int		 first_pass;
 } Hashtable;
 
 
@@ -202,8 +200,8 @@ void hashnode_dump(Hashnode *node, FILE *outfile);
 
 #define SETUP_TRACE_INFO2(name, str1, str2) \
   const char *read_function_name = name; \
-  int q1 = find_origin(str1); \
-  int q2 = find_origin(str2);
+  int q1 = board[str1] == EMPTY ? str1 : find_origin(str1); \
+  int q2 = board[str2] == EMPTY ? str2 : find_origin(str2);
 
 #else
 
@@ -224,9 +222,12 @@ void hashnode_dump(Hashnode *node, FILE *outfile);
 /* Trace messages in decidestring/decidedragon sgf file. */
 void sgf_trace(const char *func, int str, int move, int result,
 	       const char *message);
-/* Trace messages in decideconnection/decidesemeai sgf file. */
-void sgf_trace2(const char *func, int str1, int str2, int move, int result,
-		const char *message);
+/* Trace messages in decideconnection sgf file. */
+void sgf_trace2(const char *func, int str1, int str2, int move, 
+	        const char* result, const char *message);
+/* Trace messages in decidesemeai sgf file. */
+void sgf_trace_semeai(const char *func, int str1, int str2, int move, 
+		      int result1, int result2, const char *message);
 
 /* Macro to hide the call to sgf_trace(). Notice that a little black
  * magic is going on here. Before using this macro, SETUP_TRACE_INFO
@@ -243,16 +244,24 @@ void sgf_trace2(const char *func, int str1, int str2, int move, int result,
  */
 #define SGFTRACE2(move, result, message) \
   if (sgf_dumptree) \
-    sgf_trace2(read_function_name, q1, q2, move, result, message)
+    sgf_trace2(read_function_name, q1, q2, move, \
+	       result_to_string(result), message)
 
+#define SGFTRACE_SEMEAI(move, result1, result2, message) \
+  if (sgf_dumptree) \
+    sgf_trace_semeai(read_function_name, q1, q2, move, \
+	             result1, result2, message)
 
-extern Hashtable *movehash;
 
 int get_read_result(int routine, int komaster, int kom_pos,
 		    int *str, Read_result **read_result);
+int get_read_result_hash_modified(int routine, int komaster, int kom_pos,
+		    		  int *str, Hash_data *hash_modifier,
+				  Read_result **read_result);
 int get_read_result2(int routine, int komaster, int kom_pos,
 		     int *str1, int *str2, Read_result **read_result);
 
+
 /* ================================================================ */
 
 /*
@@ -297,14 +306,23 @@ int get_read_result2(int routine, int komaster, int kom_pos,
     return (value); \
   } while (0)
 
-#else
+#define READ_RETURN2(read_result, point, move, value_a, value_b) \
+  do { \
+    if ((value_a) != 0 && (point) != 0) *(point) = (move); \
+    if (read_result) { \
+      rr_set_result_move2(*(read_result), (value_a), (value_b), (move)); \
+    } \
+    return (value_a); \
+  } while (0)
+
+#else /* !TRACE_READ_RESULTS */
 
 #define READ_RETURN0(read_result) \
   do { \
     if (read_result) { \
       rr_set_result_move(*(read_result), 0, 0); \
     } \
-    gprintf("%o%s %1m %d 0 0 0 ", read_function_name, q, stackp); \
+    gprintf("%o%s %1m %d 0 0 ", read_function_name, q, stackp); \
     dump_stack(); \
     return 0; \
   } while (0)
@@ -315,7 +333,7 @@ int get_read_result2(int routine, int komaster, int kom_pos,
     if (read_result) { \
       rr_set_result_move(*(read_result), (value), (move)); \
     } \
-    gprintf("%o%s %1m %d %d %d ", read_function_name, q, stackp, \
+    gprintf("%o%s %1m %d %d %1m ", read_function_name, q, stackp, \
 	    (value), (move)); \
     dump_stack(); \
     return (value); \
@@ -327,7 +345,7 @@ int get_read_result2(int routine, int komaster, int kom_pos,
     if (read_result) { \
       rr_set_result_move2(*(read_result), (value_a), (value_b), (move)); \
     } \
-    gprintf("%o%s %1m %1m %d %d %d %d ", read_function_name, q1, q2, stackp, \
+    gprintf("%o%s %1m %1m %d %d %d %1m ", read_function_name, q1, q2, stackp, \
 	    (value_a), (value_b), (move)); \
     dump_stack(); \
     return; \
@@ -339,39 +357,59 @@ int get_read_result2(int routine, int komaster, int kom_pos,
     if (read_result) { \
       rr_set_result_move(*(read_result), (value), (move)); \
     } \
-    gprintf("%o%s %1m %1m %d %d %d ", read_function_name, q1, q2, stackp, \
+    gprintf("%o%s %1m %1m %d %d %1m ", read_function_name, q1, q2, stackp, \
 	    (value), (move)); \
     dump_stack(); \
     return (value); \
   } while (0)
 
+#define READ_RETURN2(read_result, point, move, value_a, value_b) \
+  do { \
+    if ((value_a) != 0 && (point) != 0) *(point) = (move); \
+    if (read_result) { \
+      rr_set_result_move2(*(read_result), (value_a), (value_b), (move)); \
+    } \
+    gprintf("%o%s %1m %d %d %d ", read_function_name, q, stackp, \
+	    (value_a), (move)); \
+    dump_stack(); \
+    return (value_a); \
+  } while (0)
+
+
 #endif
 
 /* ================================================================ */
 /* Routine numbers. */
 
-#define FIND_DEFENSE    0
-#define DEFEND1         1
-#define DEFEND2         2
-#define DEFEND3         3
-#define DEFEND4         4
+#define OWL_ATTACK      0
+#define OWL_DEFEND      1
+#define SEMEAI          2
 
-#define ATTACK          5
-#define ATTACK2         6
-#define ATTACK3         7
+#define FIND_DEFENSE    3
+#define ATTACK          4
 
-#define OWL_ATTACK      8
-#define OWL_DEFEND      9
-#define SEMEAI         10
+#define CONNECT         5
+#define DISCONNECT      6
 
-#define CONNECT        11
-#define DISCONNECT     12
+#define BREAK_IN	7
+#define BLOCK_OFF	8
 
-#define MAX_ROUTINE     DISCONNECT
+#define MAX_ROUTINE     BLOCK_OFF
 #define NUM_ROUTINES    (MAX_ROUTINE + 1)
   
-#endif
 
+/* Routine numbers for the persistent owl cache, in addition to
+ * OWL_ATTACK and OWL_DEFEND defined above.
+ */
+#define OWL_THREATEN_ATTACK    2
+#define OWL_THREATEN_DEFENSE   3
+#define OWL_DOES_DEFEND        4
+#define OWL_DOES_ATTACK        5
+#define OWL_CONNECTION_DEFENDS 6
+#define OWL_SUBSTANTIAL        7
+#define OWL_CONFIRM_SAFETY     8
+
+ 
 
 /* ================================================================ */
 /* This has actually nothing to do with caching, but is useful in
@@ -398,9 +436,9 @@ int get_read_result2(int routine, int komaster, int kom_pos,
 
 
 #define UPDATE_SAVED_KO_RESULT(savecode, save, code, move) \
-  if (code != 0 && (WIN - code) > savecode) { \
+  if (code != 0 && REVERSE_RESULT(code) > savecode) { \
     save = move; \
-    savecode = WIN - code; \
+    savecode = REVERSE_RESULT(code); \
   } \
 
 /* Same as above, except this should be used when there's no
@@ -412,6 +450,18 @@ int get_read_result2(int routine, int komaster, int kom_pos,
     savecode = code; \
   }
 
+
+/* This too isn't really related to caching but is convenient to have here.
+ * (Needs to be available in reading.c and persistent.c.)
+ *
+ * Minimum number of nodes for which DEBUG_READING_PERFORMANCE reports
+ * anything.
+ */
+#define MIN_READING_NODES_TO_REPORT 1000
+
+
+#endif /* _CACHE_H_ */
+
   
 /*
  * Local Variables:
