@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -46,7 +47,7 @@ combinations(int color)
 {
   int save_verbose;
   int attack_point;
-  int defense_point;
+  char defense_points[BOARDMAX];
   int other = OTHER_COLOR(color);
   int aa_val;
 
@@ -68,25 +69,19 @@ combinations(int color)
     add_my_atari_atari_move(attack_point, aa_val);
   }
   
-  aa_val = atari_atari(other, &attack_point, &defense_point, save_verbose);
+  aa_val = atari_atari(other, &attack_point, defense_points, save_verbose);
   if (aa_val > 0) {
-    int libs[2];
+    int pos;
     if (save_verbose)
-      gprintf("Combination attack for %C with size %d found at %1m, defense at %1m\n",
-	      other, aa_val, attack_point, defense_point);
-    if (safe_move(defense_point, color))
-      add_your_atari_atari_move(defense_point, aa_val);
-    /* Playing at the attack point is probably also a defense. */
-    if (safe_move(attack_point, color))
-      add_your_atari_atari_move(attack_point, aa_val);
-    /* If the attacking move gets two liberties, playing first on one
-     * of them is probably a defense.
-     */
-    if (approxlib(attack_point, other, 2, libs) == 2) {
-      if (safe_move(libs[0], color))
-	add_your_atari_atari_move(libs[0], aa_val);
-      if (safe_move(libs[1], color))
-	add_your_atari_atari_move(libs[1], aa_val);
+      gprintf("Combination attack for %C with size %d found at %1m\n",
+	      other, aa_val, attack_point);
+    
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+      if (ON_BOARD(pos) && defense_points[pos]) {
+	add_your_atari_atari_move(pos, aa_val);
+	if (save_verbose)
+	  gprintf("- defense at %1m\n", pos);
+      }
     }
   }
   verbose = save_verbose;
@@ -98,70 +93,73 @@ combinations(int color)
 static void
 find_double_threats(int color)
 {
-  int i, j;
   int ii;
   int k;
   int l;
 
-  for (i = 0; i < board_size; ++i)
-    for (j = 0; j < board_size; ++j) {
-      int num_a_threatened_groups;
-      int a_threatened_groups[MAX_THREATENED_STRINGS];
+  for (ii = BOARDMIN; ii < BOARDMAX; ii++) {
+    int num_a_threatened_groups;
+    int a_threatened_groups[MAX_THREATENED_STRINGS];
 #if 0
-      int num_d_threatened_groups;
-      int d_threatened_groups[MAX_THREATENED_STRINGS];
+    int num_d_threatened_groups;
+    int d_threatened_groups[MAX_THREATENED_STRINGS];
 #endif
 
-      ii = POS(i, j);
-      
-      /* Generate ATTACK_EITHER_MOVE move reasons for each pair of the 
-       * threatened strings.  We must also remove the threats, because
-       * otherwise we would get followup points for them also.     
-       *
-       * FIXME: 
-       *   - This is perhaps not the best way to do it, but realistically
-       *     it will be seldom that more than two strings are threatened
-       *     at the same point.  Still, we should find a better way.
-       *   - ATTACK_EITHER_MOVE should be generalized to more than two strings.
-       */
-      num_a_threatened_groups = get_attack_threats(ii, MAX_THREATENED_STRINGS,
-						   a_threatened_groups);
-      if (num_a_threatened_groups > 1) {
-	if (trymove(ii, color, "find_double_threats-A", ii, EMPTY, NO_MOVE)) {
-	  for (k = 0; k < num_a_threatened_groups - 1; ++k)
-	    for (l = k + 1; l < num_a_threatened_groups; ++l) {
-	      /* Note: If we used attack_either() here instead of trymove()
-	       *       and !defend_both(), we would not make use of the fact
-	       *       that we already know of a common threat point for
-	       *       the two strings.
-	       * Besides, attack_either is currently (3.1.11) not very good.
-	       *
-	       * The call to attack() is intended to detect the case
-	       * where the move at ii is a snapback capture.
-	       */
-	      if (board[a_threatened_groups[k]] == EMPTY
-		  || board[a_threatened_groups[l]] == EMPTY) {
-		if (!attack(ii, NULL)) {
-		  add_either_move(ii, ATTACK_STRING, a_threatened_groups[k], 
-				  ATTACK_STRING, a_threatened_groups[l]);
-		  remove_attack_threat_move(ii, a_threatened_groups[k]);
-		  remove_attack_threat_move(ii, a_threatened_groups[l]);
-		}
-	      }
-	      else if (!defend_both(a_threatened_groups[k],
-				    a_threatened_groups[l])) {
-		add_either_move(ii, ATTACK_STRING, a_threatened_groups[k], 
+    if (!ON_BOARD(ii))
+      continue;
+
+    /* Generate an EITHER_MOVE move reasons for each pair of the 
+     * threatened strings.  We must also remove the threats, because
+     * otherwise we would get followup points for them as well.
+     *
+     * FIXME: 
+     *   - This is perhaps not the best way to do it, but realistically
+     *     it will be seldom that more than two strings are threatened
+     *     at the same point.  Still, we should find a better way.
+     *   - EITHER_MOVE should be generalized to more than two strings.
+     */
+    num_a_threatened_groups = get_attack_threats(ii, MAX_THREATENED_STRINGS,
+						 a_threatened_groups);
+    if (num_a_threatened_groups > 1) {
+      if (trymove(ii, color, "find_double_threats-A", ii, EMPTY, NO_MOVE)) {
+	for (k = 0; k < num_a_threatened_groups - 1; ++k)
+	  for (l = k + 1; l < num_a_threatened_groups; ++l) {
+	    /* Note: If we used attack_either() here instead of trymove()
+	     *       and !defend_both(), we would not make use of the fact
+	     *       that we already know of a common threat point for
+	     *       the two strings.
+	     * Besides, attack_either is currently (3.1.11) not very good.
+	     *
+	     * The call to attack() is intended to detect the case
+	     * where the move at ii is a snapback capture.
+	     */
+	    if (board[a_threatened_groups[k]] == EMPTY
+		|| board[a_threatened_groups[l]] == EMPTY) {
+	      if (!attack(ii, NULL)) {
+		TRACE("Double threat at %1m, either %1m or %1m attacked.\n",
+		      ii, a_threatened_groups[k], a_threatened_groups[l]);
+		add_either_move(ii, ATTACK_STRING, a_threatened_groups[k],
 				ATTACK_STRING, a_threatened_groups[l]);
 		remove_attack_threat_move(ii, a_threatened_groups[k]);
 		remove_attack_threat_move(ii, a_threatened_groups[l]);
 	      }
 	    }
-	  popgo();
-	}
+	    else if (!defend_both(a_threatened_groups[k],
+				  a_threatened_groups[l])) {
+	      TRACE("Double threat at %1m, either %1m or %1m attacked.\n",
+		    ii, a_threatened_groups[k], a_threatened_groups[l]);
+	      add_either_move(ii, ATTACK_STRING, a_threatened_groups[k],
+			      ATTACK_STRING, a_threatened_groups[l]);
+	      remove_attack_threat_move(ii, a_threatened_groups[k]);
+	      remove_attack_threat_move(ii, a_threatened_groups[l]);
+	    }
+	  }
+	popgo();
       }
     }
-
-
+  }
+  
+  
   /* FIXME:
    *   TODO:
    *     - defense threats
@@ -212,16 +210,13 @@ struct aa_move {
 #define AA_MAX_MOVES MAX_BOARD * MAX_BOARD  
 static int aa_status[BOARDMAX]; /* ALIVE, DEAD or CRITICAL */
 static int forbidden[BOARDMAX];
-static int vital_string[BOARDMAX]; /* May not sacrifice these stones. */
 static int aa_values[BOARDMAX];
-static void compute_aa_status(int color, int saved_dragons[BOARDMAX],
-			      int saved_worms[BOARDMAX]);
+static void compute_aa_status(int color, const char safe_stones[BOARDMAX]);
 static void compute_aa_values(int color);
 static int get_aa_status(int pos);
-static int is_vital_string(int str);
-static int do_atari_atari(int color, int *attack_point,
-			  int *defense_point, int last_friendly,
-			  int save_verbose, int minsize,
+static int do_atari_atari(int color, int *attack_point, int *defense_point,
+			  char all_potential_defenses[BOARDMAX],
+			  int last_friendly, int save_verbose, int minsize,
 			  char goal[BOARDMAX]);
 static int atari_atari_succeeded(int color, int *attack_point,
                                 int *defense_point, int last_friendly,
@@ -232,7 +227,7 @@ static void atari_atari_find_attack_moves(int color, int minsize,
 static void atari_atari_attack_patterns(int color, int minsize,
 					struct aa_move attacks[AA_MAX_MOVES],
 					char goal[BOARDMAX]);
-static void atari_atari_attack_callback(int m, int n, int color,
+static void atari_atari_attack_callback(int anchor, int color,
 					struct pattern *pattern,
 					int ll, void *data);
 static int atari_atari_find_defense_moves(int targets[AA_MAX_TARGETS_PER_MOVE],
@@ -250,11 +245,14 @@ static void aa_sort_moves(struct aa_move attacks[AA_MAX_MOVES]);
 /* Set to 1 if you want verbose traces from this function. */
 
 int
-atari_atari(int color, int *attack_move, int *defense_move, int save_verbose)
+atari_atari(int color, int *attack_move, char defense_moves[BOARDMAX],
+	    int save_verbose)
 {
+  int other = OTHER_COLOR(color);
   int apos;
   int dpos;
   int aa_val;
+  char saved_defense_moves[BOARDMAX];
 
   /* Collect worm statuses of opponent's worms. We need to
    * know this because we only want to report unexpected
@@ -267,10 +265,12 @@ atari_atari(int color, int *attack_move, int *defense_move, int save_verbose)
     return 0;
   memset(forbidden, 0, sizeof(forbidden));
 
-  compute_aa_status(color, NULL, NULL);
+  compute_aa_status(color, NULL);
   compute_aa_values(color);
   
-  aa_val = do_atari_atari(color, &apos, &dpos, NO_MOVE,
+  if (defense_moves)
+    memset(defense_moves, 0, BOARDMAX);
+  aa_val = do_atari_atari(color, &apos, &dpos, defense_moves, NO_MOVE,
 			  save_verbose, 0, NULL);
 
   if (aa_val == 0)
@@ -284,11 +284,13 @@ atari_atari(int color, int *attack_move, int *defense_move, int save_verbose)
     
     if (attack_move)
       *attack_move = apos;
-    if (defense_move)
-      *defense_move = dpos;
     
     forbidden[apos] = 1;
-    new_aa_val = do_atari_atari(color, &apos, &dpos, NO_MOVE,
+    if (defense_moves) {
+      memcpy(saved_defense_moves, defense_moves, BOARDMAX);
+      memset(defense_moves, 0, BOARDMAX);
+    }
+    new_aa_val = do_atari_atari(color, &apos, &dpos, defense_moves, NO_MOVE,
 				save_verbose, aa_val, NULL);
 
     /* The last do_atari_atari call fails. When do_atari_atari fails,
@@ -296,32 +298,79 @@ atari_atari(int color, int *attack_move, int *defense_move, int save_verbose)
      * to a move that works and is necessary.
      */
     if (new_aa_val == 0)
-      return aa_val;
+      break;
     else
       aa_val = new_aa_val;
   }
 
-  /* We'll never get here, but the compiler may be more happy if it
-   * looks like we're returning something.
-   */
-  return 0;
+  if (defense_moves) {
+    int pos;
+    memcpy(defense_moves, saved_defense_moves, BOARDMAX);
+    /* defense_moves[] contains potential defense moves. Now we
+     * examine which of them really work.
+     */
+    forbidden[apos] = 0;
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+      if (!ON_BOARD(pos) || !defense_moves[pos])
+	continue;
+
+      if (!trymove(pos, other, "atari_atari", NO_MOVE, EMPTY, NO_MOVE)) {
+	defense_moves[pos] = 0;
+	if (save_verbose)
+	  gprintf("%1m deleted defense point, illegal\n", pos);
+	continue;
+      }
+
+      if (attack(pos, NULL)) {
+	defense_moves[pos] = 0;
+	popgo();
+	if (save_verbose)
+	  gprintf("%1m deleted defense point, unsafe\n", pos);
+	continue;
+      }
+      
+      if (do_atari_atari(color, &apos, &dpos, NULL, NO_MOVE,
+			 save_verbose, aa_val, NULL) > 0) {
+	if (save_verbose)
+	  gprintf("%1m deleted defense point, didn't work\n", pos);
+	defense_moves[pos] = 0;
+      }
+      
+      popgo();
+    }
+  }
+  return aa_val;
 }
 
 
-/* Ask the atari_atari code whether there appears any combination
- * attack which would capture at least minsize stones after playing at
- * (move). If this happens, (*defense) points to a move which prevents
- * this blunder.
- *
+/* Wrapper around atari_atari_blunder_size. Check whether a
+ * combination attack of size at least minsize appears after move
+ * at (move) has been made.
  * The arrays saved_dragons[] and saved_worms[] should be one for
  * stones belonging to dragons or worms respectively, which are
- * supposedly saved by (move). These may be NULL if no stones are
- * supposed to gaving been saved.
+ * supposedly saved by (move).
  */
 int
 atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
-			   int saved_dragons[BOARDMAX],
-			   int saved_worms[BOARDMAX])
+			   const char saved_dragons[BOARDMAX],
+			   const char saved_worms[BOARDMAX])
+{
+  char safe_stones[BOARDMAX];
+
+  mark_safe_stones(color, move, saved_dragons, saved_worms, safe_stones);
+
+  return (atari_atari_blunder_size(color, move, defense, safe_stones)
+	  >= minsize);
+}
+ 
+
+/* This function checks whether any new combination attack appears after
+ * move at (move) has been made, and returns its size (in points).
+ * safe_stones marks which of our stones are supposedly safe after this move.
+ */
+int
+atari_atari_blunder_size(int color, int move, int *defense,
+			 const char safe_stones[BOARDMAX])
 {
   int apos;
   int defense_point = NO_MOVE, after_defense_point = NO_MOVE;
@@ -332,47 +381,48 @@ atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
    * so in this respect the move is safe enough.
    */
   if (aa_depth < 2)
-    return 1;
+    return 0;
 
   memset(forbidden, 0, sizeof(forbidden));
 
-  compute_aa_status(other, saved_dragons, saved_worms);
+  /* FIXME: Maybe these should be moved after the tryko() below? */
+  compute_aa_status(other, safe_stones);
   compute_aa_values(other);
 
   /* Accept illegal ko capture here. */
   if (!tryko(move, color, NULL, EMPTY, NO_MOVE))
     /* Really shouldn't happen. */
-    abortgo(__FILE__, __LINE__, "trymove", I(move), J(move));
+    abortgo(__FILE__, __LINE__, "trymove", move);
   increase_depth_values();
 
-  aa_val = do_atari_atari(other, &apos, &defense_point, NO_MOVE, 0, minsize,
-			  NULL);
+  aa_val = do_atari_atari(other, &apos, &defense_point, NULL,
+			  NO_MOVE, 0, 0, NULL);
   after_aa_val = aa_val;
 
   if (aa_val == 0 || defense_point == NO_MOVE) {
 
-  /* No sufficiently large combination attack, so the move is safe from
-   * this danger.
-   *
-   * On rare occasions do_atari_atari might find a combination
-   * but no defense. In this case we assume that the combination
-   * is illusory.
-   */
+    /* No sufficiently large combination attack, so the move is safe from
+     * this danger.
+     *
+     * On rare occasions do_atari_atari might find a combination
+     * but no defense. In this case we assume that the combination
+     * is illusory.
+     */
 
     popgo();
     decrease_depth_values();
-    return 1;
+    return 0;
   }
 
-  while (aa_val >= after_aa_val) {
+  while (aa_val >= after_aa_val && defense_point != NO_MOVE) {
     /* Try dropping moves from the combination and see if it still
      * works. What we really want is to get the proper defense move
      * into (*defense).
      */
     after_defense_point = defense_point;
     forbidden[apos] = 1;
-    aa_val = do_atari_atari(other, &apos, &defense_point, NO_MOVE, 0, aa_val,
-			    NULL);
+    aa_val = do_atari_atari(other, &apos, &defense_point, NULL,
+			    NO_MOVE, 0, aa_val, NULL);
   }
 
   popgo();
@@ -381,16 +431,16 @@ atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
    * the original move at (aa) was really relevant. So we
    * try omitting it and see if a combination is still found.
    */
-  compute_aa_status(other, NULL, NULL);
+  compute_aa_status(other, NULL);
   compute_aa_values(other);
-  if (do_atari_atari(other, NULL, NULL, NO_MOVE,
-		     0, minsize, NULL) >= after_aa_val)
-    return 1;
-  else {
+  aa_val = do_atari_atari(other, NULL, NULL, NULL, NO_MOVE, 0, 0, NULL);
+  if (after_aa_val - aa_val > 0) {
     if (defense)
       *defense = after_defense_point;
-    return 0;
+    return after_aa_val - aa_val;
   }
+  else
+    return 0;
 }
 
 
@@ -399,13 +449,12 @@ atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
 /* ---------------------------------------------------------------- */
 
 
-/* Helper function for computing the aa_status for a string. It also
- * sets up the vital_string[] array.
+/* Helper function for computing the aa_status for all opponent's strings.
+ * If safe_stones is given, we just copy the information from there.
  */
 
 static void
-compute_aa_status(int color, int saved_dragons[BOARDMAX],
-		  int saved_worms[BOARDMAX])
+compute_aa_status(int color, const char safe_stones[BOARDMAX])
 {
   int other = OTHER_COLOR(color);
   int pos;
@@ -426,23 +475,26 @@ compute_aa_status(int color, int saved_dragons[BOARDMAX],
    */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (board[pos] == other) {
-      if (dragon[pos].matcher_status == DEAD)
-	aa_status[pos] = DEAD;
-      else if (dragon[pos].matcher_status == CRITICAL
-	       && (!saved_dragons || !saved_dragons[pos]))
-	aa_status[pos] = CRITICAL;
-      else if (worm[pos].attack_codes[0] != 0) {
-	if (worm[pos].defend_codes[0] != 0) {
-	  if (saved_worms && saved_worms[pos])
-	    aa_status[pos] = ALIVE;
-	  else
-	    aa_status[pos] = CRITICAL;
-	}
+      if (safe_stones) {
+	if (safe_stones[pos])
+	  aa_status[pos] = ALIVE;
 	else
 	  aa_status[pos] = DEAD;
       }
-      else
-	aa_status[pos] = ALIVE;
+      else {
+	if (dragon[pos].status == DEAD)
+	  aa_status[pos] = DEAD;
+	else if (dragon[pos].status == CRITICAL)
+	  aa_status[pos] = CRITICAL;
+	else if (worm[pos].attack_codes[0] != 0) {
+	  if (worm[pos].defense_codes[0] != 0)
+	    aa_status[pos] = CRITICAL;
+	  else
+	    aa_status[pos] = DEAD;
+	}
+	else
+	  aa_status[pos] = ALIVE;
+      }
     }
     else if (ON_BOARD(pos))
       aa_status[pos] = UNKNOWN;
@@ -483,22 +535,6 @@ compute_aa_status(int color, int saved_dragons[BOARDMAX],
     }
   }
 
-  /* Set up the vital strings array. This contains stones we can't be
-   * allowed to sacrifice. This is used by
-   * atari_atari_confirm_safety() to make sure that stones which are
-   * supposedly saved by the move can't be captured in a combination
-   * attack.
-   */
-
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (ON_BOARD(pos)
-	&& ((saved_dragons && saved_dragons[pos])
-	    || (saved_worms && saved_worms[pos])))
-      vital_string[pos] = 1;
-    else
-      vital_string[pos] = 0;
-  }
-  
   sgf_dumptree = save_sgf_dumptree;
   count_variations = save_count_variations;
   verbose = save_verbose;
@@ -532,23 +568,6 @@ get_aa_status(int pos)
 }
 
 
-/* Helper function to examine whether a stone is part of a vital
- * string. Since the vital_string[] array was generated at stackp == 0
- * new stones may have appeared on the board.
- */
-static int
-is_vital_string(int str)
-{
-  int stones[MAX_BOARD * MAX_BOARD];
-  int n = findstones(str, MAX_BOARD * MAX_BOARD, stones);
-  int k;
-  for (k = 0; k < n; k++)
-    if (vital_string[stones[k]])
-      return 1;
-
-  return 0;
-}
-
 
 /* Helper function for atari_atari. Here worms is the number of
  * opponent worms involved in the combination, and (last_friendly) is
@@ -575,8 +594,8 @@ is_vital_string(int str)
 
 static int
 do_atari_atari(int color, int *attack_point, int *defense_point,
-	       int last_friendly, int save_verbose, int minsize,
-	       char goal[BOARDMAX])
+	       char all_potential_defenses[BOARDMAX], int last_friendly,
+	       int save_verbose, int minsize, char goal[BOARDMAX])
 {
   int other = OTHER_COLOR(color);
   int k;
@@ -595,6 +614,15 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
       if (ON_BOARD(pos) && forbidden[pos])
 	gprintf("%o%1m ", pos);
     gprintf("\n");
+    gprintf("%ogoal: ");
+    if (!goal)
+      gprintf("none");
+    else {
+      for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+	if (ON_BOARD(pos) && goal[pos])
+	  gprintf("%o%1m ", pos);
+    }
+    gprintf("\n");
   }
 
   /* First look for strings adjacent to the last friendly move played
@@ -615,7 +643,7 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
     if (retval != 0) {
       if (sgf_dumptree)
 	/* FIXME: Better message. */
-	sgftreeAddComment(sgf_dumptree, NULL, "attack found");
+	sgftreeAddComment(sgf_dumptree, "attack found");
       return retval;
     }
   }
@@ -646,7 +674,18 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
     
     if (!trymove(apos, color, "do_atari_atari-A", str, EMPTY, NO_MOVE))
       continue;
-	
+    
+    if (all_potential_defenses) {
+      all_potential_defenses[apos] = 1;
+      if (countlib(apos) <= 2) {
+	int libs[2];
+	int num_libs = findlib(apos, 2, libs);
+	all_potential_defenses[libs[0]] = 1;
+	if (num_libs == 2)
+	  all_potential_defenses[libs[1]] = 1;
+      }
+    }
+
     if (!IS_STONE(board[str])) {
       /* Error situation. This could be caused by a wrong matcher status. */
       if (save_verbose || (debug & DEBUG_ATARI_ATARI))
@@ -671,6 +710,9 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
     for (r = 0; r < num_defense_moves; r++) {
       bpos = defense_moves[r];
 
+      if (all_potential_defenses)
+	all_potential_defenses[bpos] = 1;
+
       if (trymove(bpos, other, "do_atari_atari-B", str, EMPTY, NO_MOVE)) {
 	int new_aa_val;
 	char new_goal[BOARDMAX];
@@ -681,8 +723,8 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
 	modify_depth_values(2);
 	update_aa_goal(goal, new_goal, apos, color);
 	new_aa_val = do_atari_atari(color, NULL, defense_point,
-				    apos, save_verbose, minsize,
-				    new_goal);
+				    all_potential_defenses,
+				    apos, save_verbose, minsize, new_goal);
 	modify_depth_values(-2);
 	if (new_aa_val < aa_val)
 	  aa_val = new_aa_val;
@@ -708,7 +750,7 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
       }
       if (sgf_dumptree)
 	/* FIXME: Better message. */
-	sgftreeAddComment(sgf_dumptree, NULL, "attack found");
+	sgftreeAddComment(sgf_dumptree, "attack found");
     }
 
     if (attack_point)
@@ -747,66 +789,62 @@ static int
 atari_atari_succeeded(int color, int *attack_point, int *defense_point,
 		      int last_friendly, int save_verbose, int minsize)
 {
-  int m, n;
+  int pos;
+  int apos;
   int other = OTHER_COLOR(color);
 
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++) {
-      int ii = POS(m, n);
-      int aa;
-
-      if (board[ii] != other)
-       continue;
-
-      if (ii != find_origin(ii))
-       continue;
-
-      if (minsize > 0
-	  && get_aa_value(ii) < minsize
-	  && !is_vital_string(ii))
-       continue;
-
-      if (get_aa_status(ii) != ALIVE)
-       continue;
-
-      if (board[last_friendly] != EMPTY
-	  && !adjacent_strings(last_friendly, ii))
-       continue;
-
-      if (board[last_friendly] == EMPTY
-	  && !liberty_of_string(last_friendly, ii))
-       continue;
-
-      if (debug & DEBUG_ATARI_ATARI)
-	gprintf("Considering attack of %1m. depth = %d.\n", ii, depth);
-
-      if (attack(ii, &aa) && !forbidden[aa]) {
-	if (save_verbose || (debug & DEBUG_ATARI_ATARI)) {
-	  gprintf("%oThe worm %1m can be attacked at %1m after ", ii, aa);
-	  dump_stack();
-	}
-	if (attack_point)
-	  *attack_point = aa;
-
-	/* We look for a move defending the combination.
-	 * Normally this is found by find_defense but failing
-	 * that, if the attacking move is a safe move for color,
-	 * it probably defends.
-	 */
-	if (defense_point) {
-	  if (!find_defense(ii, defense_point)) {
-	    if (safe_move(aa, other))
-	      *defense_point = aa;
-	    else
-	      *defense_point = NO_MOVE;
-	  }
-	}
-	
-	DEBUG(DEBUG_ATARI_ATARI, "%oreturn value:%d (%1m)\n",
-	      get_aa_value(ii), ii);
-	return get_aa_value(ii);
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (board[pos] != other)
+      continue;
+    
+    if (pos != find_origin(pos))
+      continue;
+    
+    if (minsize > 0
+	&& get_aa_value(pos) < minsize)
+      continue;
+    
+    if (get_aa_status(pos) != ALIVE)
+      continue;
+    
+    if (board[last_friendly] != EMPTY
+	&& !adjacent_strings(last_friendly, pos))
+      continue;
+    
+    if (board[last_friendly] == EMPTY
+	&& !liberty_of_string(last_friendly, pos))
+      continue;
+    
+    if (debug & DEBUG_ATARI_ATARI)
+      gprintf("Considering attack of %1m. depth = %d.\n", pos, depth);
+    
+    if (attack(pos, &apos) && !forbidden[apos]) {
+      if (save_verbose || (debug & DEBUG_ATARI_ATARI)) {
+	gprintf("%oThe worm %1m can be attacked at %1m after ", pos, apos);
+	dump_stack();
       }
+      if (attack_point)
+	*attack_point = apos;
+      
+      /* We look for a move defending the combination.
+       * Normally this is found by find_defense but failing
+       * that, if the attacking move is a safe move for color,
+       * it probably defends.
+       */
+      if (defense_point) {
+	if (!find_defense(pos, defense_point)) {
+	  if (safe_move(apos, other))
+	    *defense_point = apos;
+	  else
+	    *defense_point = NO_MOVE;
+	}
+      }
+      
+      DEBUG(DEBUG_ATARI_ATARI, "%oreturn value:%d (%1m)\n",
+	    get_aa_value(pos), pos);
+      return get_aa_value(pos);
     }
+  }
   
   return 0;
 }
@@ -884,14 +922,14 @@ atari_atari_attack_patterns(int color, int minsize,
  * before or not. Only exclude already known attacking moves.
  */
 static void
-atari_atari_attack_callback(int m, int n, int color,
+atari_atari_attack_callback(int anchor, int color,
 			    struct pattern *pattern, int ll, void *data)
 {
   int move;
   int k;
   UNUSED(data);
 
-  move = AFFINE_TRANSFORM(pattern->movei, pattern->movej, ll, m, n);
+  move = AFFINE_TRANSFORM(pattern->move_offset, ll, anchor);
 
   if (forbidden[move])
     return;
@@ -900,7 +938,7 @@ atari_atari_attack_callback(int m, int n, int color,
    * if the pattern must be rejected.
    */
   if (pattern->autohelper_flag & HAVE_CONSTRAINT)
-    if (!pattern->autohelper(pattern, ll, move, color, 0))
+    if (!pattern->autohelper(ll, move, color, 0))
       return;
 
   /* If the pattern has a helper, call it to see if the pattern must
@@ -916,12 +954,11 @@ atari_atari_attack_callback(int m, int n, int color,
   for (k = 0; k < pattern->patlen; ++k) { /* match each point */
     if (pattern->patn[k].att == ATT_X) {
       /* transform pattern real coordinate */
-      int str = find_origin(AFFINE_TRANSFORM(pattern->patn[k].x,
-					     pattern->patn[k].y, ll, m, n));
+      int str = find_origin(AFFINE_TRANSFORM(pattern->patn[k].offset,
+					     ll, anchor));
 
       if (current_minsize > 0
-	  && get_aa_value(str) < current_minsize
-	  && !is_vital_string(str))
+	  && get_aa_value(str) < current_minsize)
 	continue;
 
       if (aa_move_known(current_attacks, move, str))
@@ -1076,7 +1113,7 @@ atari_atari_find_defense_moves(int targets[AA_MAX_TARGETS_PER_MOVE],
       gprintf("%o\n");
     }
   }
-  
+
   return num_moves;
 }
 
@@ -1111,8 +1148,9 @@ compute_aa_values(int color)
     value = 2 * countstones(pos);
 
     for (r = 0; r < liberties; r++) {
-      if (!mx[libs[r]] && (influence_moyo_color(libs[r]) == other
-			   || influence_moyo_color_opposite(libs[r]))) {
+      if (!mx[libs[r]]
+	  && (whose_moyo(&initial_black_influence, libs[r]) == other
+	      || whose_moyo(&initial_white_influence, libs[r]) == other)) {
 	mx[libs[r]] = 1;
 	value++;
       }
@@ -1122,8 +1160,8 @@ compute_aa_values(int color)
 	  continue;
 	mx[librd] = 1;
 	if (board[librd] == EMPTY
-	    && (influence_moyo_color(librd) == other
-		|| (influence_moyo_color_opposite(librd) == other)))
+	    && (whose_moyo(&initial_black_influence, librd) == other
+		|| (whose_moyo(&initial_white_influence, librd) == other)))
 	  value++;
       }
     }
@@ -1206,12 +1244,20 @@ update_aa_goal(char goal[BOARDMAX], char new_goal[BOARDMAX], int apos,
     queue[queue_end++] = apos;
   }
 
+#if 0
+  /* Disabled for now, since it does nothing but break atari_atari:16
+   * and trevorc:1540. It could be reactivated when the rest of the
+   * function would be modified in order to garanty that a forbidden
+   * move is strictly equivalent to a played move in terms of goal
+   * mapping. I doubt it would be anything worth though...
+   */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (ON_BOARD(pos) && forbidden[pos]) {
       dists[pos] = 1;
       queue[queue_end++] = pos;
     }
   }
+#endif
 
   if (queue_end == 0)
     return 0;
@@ -1246,7 +1292,9 @@ update_aa_goal(char goal[BOARDMAX], char new_goal[BOARDMAX], int apos,
      */
     for (k = 0; k < 4; k++) {
       int pos2 = pos + delta[k];
-      if ((board[pos] == other || pos == apos) && board[pos2] == EMPTY) {
+      if (!ON_BOARD(pos2))
+       continue;
+      if ((board[pos] != color || pos == apos) && board[pos2] == EMPTY) {
         ENQUEUE(pos2, dists[pos] + 1);
       }
       else if (board[pos] != other && board[pos2] == other) {
