@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -39,7 +40,7 @@ living_neighbor(int pos, int color)
   int k;
   for (k = 0; k < 4; k++) {
     if (board[pos + delta[k]] == color
-	&& dragon[pos + delta[k]].matcher_status != DEAD)
+	&& dragon[pos + delta[k]].status != DEAD)
       return 1;
   }
 
@@ -58,20 +59,20 @@ analyze_neighbor(int pos, int *found_black, int *found_white)
     case EMPTY:
       if (!(*found_black)
 	  && living_neighbor(pos, BLACK)
-	  && safe_move(pos, WHITE) != 3)
+	  && safe_move(pos, WHITE) != WIN)
 	*found_black = 1;
       
       if (!(*found_white)
 	  && living_neighbor(pos, WHITE)
-	  && safe_move(pos, BLACK) != 3)
+	  && safe_move(pos, BLACK) != WIN)
 	*found_white = 1;
       
       break;
 
     case BLACK:
       if (!worm[pos].inessential && DRAGON2(pos).safety != INESSENTIAL) {
-	if (dragon[pos].matcher_status == ALIVE
-	    || dragon[pos].matcher_status == UNKNOWN)
+	if (dragon[pos].status == ALIVE
+	    || dragon[pos].status == UNKNOWN)
 	  *found_black = 1;
 	else
 	  *found_white = 1;
@@ -80,8 +81,8 @@ analyze_neighbor(int pos, int *found_black, int *found_white)
 
     case WHITE:
       if (!worm[pos].inessential && DRAGON2(pos).safety != INESSENTIAL) {
-	if (dragon[pos].matcher_status == ALIVE
-	    || dragon[pos].matcher_status == UNKNOWN)
+	if (dragon[pos].status == ALIVE
+	    || dragon[pos].status == UNKNOWN)
 	  *found_white = 1;
 	else
 	  *found_black = 1;
@@ -103,8 +104,8 @@ analyze_neighbor(int pos, int *found_black, int *found_white)
 int 
 fill_liberty(int *move, int color)
 {
-  int m, n;
   int k;
+  int pos;
   int other = OTHER_COLOR(color);
   int defense_point;
   int potential_color[BOARDMAX];
@@ -116,212 +117,212 @@ fill_liberty(int *move, int color)
    * the process.
    */
   memset(potential_color, 0, sizeof(potential_color));
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++) {
-      int pos = POS(m, n);
-      if (board[pos] == EMPTY)
-	continue;
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!IS_STONE(board[pos]))
+      continue;
 
-      if (worm[pos].inessential || DRAGON2(pos).safety == INESSENTIAL)
-	continue;
-
-      if (dragon[pos].matcher_status != ALIVE) {
-	for (k = 0; k < 4; k++) {
-	  int pos2 = pos + delta[k];
-	  if (board[pos2] == EMPTY)
-	    potential_color[pos2] |= OTHER_COLOR(board[pos]);
-	}
-      }
-      
-      if (dragon[pos].matcher_status != DEAD) {
-	for (k = 0; k < 12; k++) {
-	  int d = delta[k%8];
-
-	  if (k >= 8) {
- 	    if (board[pos + d] != EMPTY)
- 	      continue;
-	    d *= 2;
-	  }
-	  if (board[pos + d] == EMPTY)
-	    potential_color[pos + d] |= board[pos];
-	}
+    if (worm[pos].inessential || DRAGON2(pos).safety == INESSENTIAL)
+      continue;
+    
+    if (dragon[pos].status != ALIVE) {
+      for (k = 0; k < 4; k++) {
+	int pos2 = pos + delta[k];
+	if (board[pos2] == EMPTY)
+	  potential_color[pos2] |= OTHER_COLOR(board[pos]);
       }
     }
-
-  
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++) {
-      int pos = POS(m, n);
-      /* It seems we can't trust an empty liberty to be gray-colored
-       * either as a cave or as a cavity. Instead we look for empty
-       * intersections with at least one neighbor of each color, where
-       * dead stones count as enemy stones. We also count empty
-       * neighbors to either color if the opponent can't play there.
-       */
-      int found_white = 0;
-      int found_black = 0;
-
-      if (board[pos] != EMPTY)
-	continue;
-
-      /* Quick rejection based on preliminary test above. */
-      if (potential_color[pos] != GRAY)
-	continue;
-
-      /* Loop over the neighbors. */
-      for (k = 0; k < 4; k++) {
-	int d = delta[k];
-	if (ON_BOARD(pos + d))
-	  analyze_neighbor(pos + d, &found_black, &found_white);
+    
+    if (dragon[pos].status != DEAD) {
+      for (k = 0; k < 12; k++) {
+	int d = delta[k%8];
+	
+	if (k >= 8) {
+	  if (board[pos + d] != EMPTY)
+	    continue;
+	  d *= 2;
+	}
+	if (board[pos + d] == EMPTY)
+	  potential_color[pos + d] |= board[pos];
       }
-      
-      /* Do we have neighbors of both colors? */
-      if (!(found_white && found_black))
+    }
+  }
+  
+  
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    /* It seems we can't trust an empty liberty to be gray-colored
+     * either as a cave or as a cavity. Instead we look for empty
+     * intersections with at least one neighbor of each color, where
+     * dead stones count as enemy stones. We also count empty
+     * neighbors to either color if the opponent can't play there.
+     */
+    int found_white = 0;
+    int found_black = 0;
+    
+    if (board[pos] != EMPTY)
+      continue;
+
+    /* Quick rejection based on preliminary test above. */
+    if (potential_color[pos] != GRAY)
+      continue;
+    
+    /* Loop over the neighbors. */
+    for (k = 0; k < 4; k++) {
+      int d = delta[k];
+      if (ON_BOARD(pos + d))
+	analyze_neighbor(pos + d, &found_black, &found_white);
+    }
+    
+    /* Do we have neighbors of both colors? */
+    if (!(found_white && found_black))
+      continue;
+    
+    /* Ok, we wish to play here, but maybe we can't. The following
+     * cases may occur:
+     * 1. Move is legal and safe.
+     * 2. Move is legal but not safe because it's in the middle of a seki.
+     * 3. Move is legal but not safe, can be played after backfilling.
+     * 4. Move is an illegal ko recapture.
+     * 5. Move is illegal but can be played after back-captures.
+     * 6. Move would violate confirm_safety.
+     */
+    
+    DEBUG(DEBUG_FILLLIB, "Filllib: Considering move at %1m.\n", pos);
+    
+    /* Legal and tactically safe, play it if it passes
+     * confirm_safety test, i.e. that it isn't a blunder which
+     * causes problems for other strings.
+     */
+    if (safe_move(pos, color) == WIN) {
+      DEBUG(DEBUG_FILLLIB, "Filllib: Tactically safe.\n");
+      if (filllib_confirm_safety(pos, color, &defense_point)) {
+	/* Safety confirmed. */
+	DEBUG(DEBUG_FILLLIB, "Filllib: Safety confirmed.\n");
+	*move = pos;
+	return 1;
+      }
+      else if (defense_point != NO_MOVE) {
+	/* Safety not confirmed because the move at (pos) would set
+	 * up a double threat. (defense_point) is assumed to defend
+	 * against this threat.
+	 *
+	 * FIXME: We should verify that (defense_point) really is effective.
+	 */
+	DEBUG(DEBUG_FILLLIB,
+	      "Filllib: Safety not confirmed, but %1m defends.\n",
+	      defense_point);
+	*move = defense_point;
+	return 1;
+      }
+      else {
+	/* The move causes problems somewhere else on the board, so
+	 * we have to discard it. If everything works right this
+	 * should not happen at this time.
+	 */
+	DEBUG(DEBUG_FILLLIB, "Filllib: Safety not confirmed, discarded.\n");
+	TRACE("Warning: Blunder detected in fill_liberty().\n");
 	continue;
-
-      /* Ok, we wish to play here, but maybe we can't. The following
-       * cases may occur:
-       * 1. Move is legal and safe.
-       * 2. Move is legal but not safe because it's in the middle of a seki.
-       * 3. Move is legal but not safe, can be played after backfilling.
-       * 4. Move is an illegal ko recapture.
-       * 5. Move is illegal but can be played after back-captures.
-       * 6. Move would violate confirm_safety.
-       */
-
-      DEBUG(DEBUG_FILLLIB, "Filllib: Considering move at %1m.\n", pos);
+      }
+    }
+    
+    /* Try to play the move. */
+    if (trymove(pos, color, "fill_liberty", NO_MOVE, EMPTY, NO_MOVE)) {
+      popgo();
+      /* Legal, but not safe. Look for backfilling move. */
+      DEBUG(DEBUG_FILLLIB,
+	    "Filllib: Legal but not safe, looking for backfilling move.\n");
       
-      /* Legal and tactically safe, play it if it passes
-       * confirm_safety test, i.e. that it isn't a blunder which
-       * causes problems for other strings.
-       */
-      if (safe_move(pos, color) == WIN) {
-	DEBUG(DEBUG_FILLLIB, "Filllib: Tactically safe.\n");
-	if (filllib_confirm_safety(pos, color, &defense_point)) {
-	  /* Safety confirmed. */
-	  DEBUG(DEBUG_FILLLIB, "Filllib: Safety confirmed.\n");
-	  *move = pos;
-	  return 1;
-	}
-	else if (defense_point != NO_MOVE) {
-	  /* Safety not confirmed because the move at (pos) would set
-           * up a double threat. (defense_point) is assumed to defend
-           * against this threat.
-	   *
-	   * FIXME: We should verify that (defense_point) really is effective.
-	   */
-	  DEBUG(DEBUG_FILLLIB,
-		"Filllib: Safety not confirmed, but %1m defends.\n",
-		defense_point);
-	  *move = defense_point;
-	  return 1;
-	}
-	else {
-	  /* The move causes problems somewhere else on the board, so
-           * we have to discard it. If everything works right this
-           * should not happen at this time.
-	   */
-	  DEBUG(DEBUG_FILLLIB, "Filllib: Safety not confirmed, discarded.\n");
-	  TRACE("Warning: Blunder detected in fill_liberty().\n");
+      if (find_backfilling_move(pos, color, move)) {
+	DEBUG(DEBUG_FILLLIB, "Filllib: Backfilling move at %1m.\n", *move);
+	/* In certain positions it may happen that an illegal move
+	 * is found. This probably only can happen if we try to play
+	 * a move inside a lost semeai. Anyway we should discard the
+	 * move.
+	 */
+	if (!is_legal(*move, color)) {
+	  DEBUG(DEBUG_FILLLIB, "Filllib: Was illegal, discarded.\n");
+	  *move = NO_MOVE;
 	  continue;
 	}
+	
+	/* If the move turns out to be strategically unsafe, or
+	 * setting up a double threat elsewhere, also discard it.
+	 */
+	if (!filllib_confirm_safety(*move, color, &defense_point)) {
+	  DEBUG(DEBUG_FILLLIB,
+		"Filllib: Safety not confirmed, discarded.\n");
+	  continue;
+	}
+	
+	/* Seems to be ok. */
+	return 1;
       }
-      
-      /* Try to play the move. */
-      if (trymove(pos, color, "fill_liberty", NO_MOVE, EMPTY, NO_MOVE)) {
-	popgo();
-	/* Legal, but not safe. Look for backfilling move. */
-	DEBUG(DEBUG_FILLLIB,
-	      "Filllib: Legal but not safe, looking for backfilling move.\n");
-
-	if (find_backfilling_move(pos, color, move)) {
-	  DEBUG(DEBUG_FILLLIB, "Filllib: Backfilling move at %1m.\n", *move);
-	  /* In certain positions it may happen that an illegal move
-	   * is found. This probably only can happen if we try to play
-	   * a move inside a lost semeai. Anyway we should discard the
-	   * move.
-	   */
-	  if (!is_legal(*move, color)) {
-	    DEBUG(DEBUG_FILLLIB, "Filllib: Was illegal, discarded.\n");
-	    *move = NO_MOVE;
-	    continue;
-	  }
-
-	  /* If the move turns out to be strategically unsafe, or
-           * setting up a double threat elsewhere, also discard it.
-	   */
-	  if (!filllib_confirm_safety(*move, color, &defense_point)) {
+      else {
+	/* If we captured some stones, this move should be ok anyway. */
+	if (does_capture_something(pos, color)) {
+	  DEBUG(DEBUG_FILLLIB,
+		"Filllib: Not tactically safe, but captures stones.\n");
+	  if (!filllib_confirm_safety(pos, color, &defense_point)) {
 	    DEBUG(DEBUG_FILLLIB,
 		  "Filllib: Safety not confirmed, discarded.\n");
 	    continue;
 	  }
-
-	  /* Seems to be ok. */
+	  *move = pos;
 	  return 1;
 	}
-	else {
-	  /* If we captured some stones, this move should be ok anyway. */
-	  if (does_capture_something(pos, color)) {
-	    DEBUG(DEBUG_FILLLIB,
-		  "Filllib: Not tactically safe, but captures stones.\n");
-	    if (!filllib_confirm_safety(pos, color, &defense_point)) {
-	      DEBUG(DEBUG_FILLLIB,
-		    "Filllib: Safety not confirmed, discarded.\n");
-	      continue;
-	    }
-	    *move = pos;
-	    return 1;
-	  }
+      }
+    }
+    else {
+      /* Move is illegal. Look for an attack on one of the neighbor
+       * worms. If found, return that move for back-capture.
+       */
+      DEBUG(DEBUG_FILLLIB, "Filllib: Illegal, looking for back-capture.\n");
+      for (k = 0; k < 4; k++) {
+	int d = delta[k];
+	if (board[pos + d] == other
+	    && worm[pos + d].attack_codes[0] == WIN) {
+	  *move = worm[pos + d].attack_points[0];
+	  DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
+	  return 1;
 	}
       }
-      else {
-	/* Move is illegal. Look for an attack on one of the neighbor
-	 * worms. If found, return that move for back-capture.
-	 */
-	DEBUG(DEBUG_FILLLIB, "Filllib: Illegal, looking for back-capture.\n");
-	for (k = 0; k < 4; k++) {
-	  int d = delta[k];
-	  if (board[pos + d] == other
-	      && worm[pos + d].attack_codes[0] == WIN) {
-	    *move = worm[pos + d].attack_points[0];
+      
+      DEBUG(DEBUG_FILLLIB,
+	    "Filllib: Nothing found, looking for ko back-capture.\n");
+      for (k = 0; k < 4; k++) {
+	int d = delta[k];
+	if (board[pos + d] == other
+	    && worm[pos + d].attack_codes[0] != 0
+	    && is_legal(worm[pos + d].attack_points[0], color)) {
+	  *move = worm[pos + d].attack_points[0];
+	  DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
+	  return 1;
+	}
+      }
+
+      DEBUG(DEBUG_FILLLIB,
+	    "Filllib: Nothing found, looking for threat to back-capture.\n");
+      for (k = 0; k < 4; k++) {
+	int d = delta[k];
+	if (board[pos + d] == other
+	    && worm[pos + d].attack_codes[0] != 0) {
+	  /* Just pick some other liberty. */
+	  /* FIXME: Something is odd about this code. */
+	  int libs[2];
+	  if (findlib(pos + d, 2, libs) > 1) {
+	    if (is_legal(libs[0], color))
+	      *move = libs[0];
+	    else if (is_legal(libs[1], color))
+	      *move = libs[1];
+	    else
+	      continue;
+	    
 	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
 	    return 1;
 	  }
 	}
-
-	DEBUG(DEBUG_FILLLIB,
-	      "Filllib: Nothing found, looking for ko back-capture.\n");
-	for (k = 0; k < 4; k++) {
-	  int d = delta[k];
-	  if (board[pos + d] == other
-	      && worm[pos + d].attack_codes[0] != 0
-	      && is_legal(worm[pos + d].attack_points[0], color)) {
-	    *move = worm[pos + d].attack_points[0];
-	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
-	    return 1;
-	  }
-	}
-
-	DEBUG(DEBUG_FILLLIB,
-	      "Filllib: Nothing found, looking for threat to back-capture.\n");
-	for (k = 0; k < 4; k++) {
-	  int d = delta[k];
-	  if (board[pos + d] == other
-	      && worm[pos + d].attack_codes[0] != 0) {
-	    /* Just pick some other liberty. */
-	    int libs[2];
-	    if (findlib(pos + d, 2, libs) > 1) {
-	      if (is_legal(libs[0], color))
-		*move = libs[0];
-	      else if (is_legal(libs[1], color))
-		*move = libs[1];
-	      DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
-	      return 1;
-	    }
-	  }
-	}
       }
     }
+  }
   
   /* Nothing found. */
   DEBUG(DEBUG_FILLLIB, "Filllib: No move found.\n");
@@ -364,6 +365,7 @@ find_backfilling_move(int move, int color, int *backfill_move)
   int success = 0;
   int acode;
   int saved_move = NO_MOVE;
+  int opponent_libs;
   
   /* Play (move) and identify all liberties and adjacent strings. */
   if (!trymove(move, color, "find_backfilling_move", move, EMPTY, NO_MOVE))
@@ -373,7 +375,7 @@ find_backfilling_move(int move, int color, int *backfill_move)
    * opponent. Save it for later use.
    */
   acode = attack(move, &apos);
-  gg_assert(acode != 0 && apos > NO_MOVE);
+  gg_assert(acode != 0 && apos != NO_MOVE);
   
   /* Find liberties. */
   liberties = findlib(move, MAXLIBS, libs);
@@ -390,24 +392,36 @@ find_backfilling_move(int move, int color, int *backfill_move)
    * trust it but keep it around if we don't find anything else. (See
    * filllib:17 for a position where this matters.)
    *
-   * FIXME: Maybe we should take care to find the neighbor with the
-   * fewest liberties, since that string probably can be removed
-   * fastest. For the moment we assume this to be nonimportant.
+   * It is also necessary to take care to first attack the string with
+   * the fewest liberties, which can probably be removed the fastest.
+   * See filllib:37 for an example (J5 tactically attacks K7 but the
+   * correct move is H5).
    *
    * FIXME: It seems we have to return immediately when we find an
    * attacking move, because recursing for further backfilling might
    * lead to moves which complete the capture but cannot be played
    * before the attacking move itself. This is not ideal but probably
    * good enough.
+   *
+   * In order to avoid losing unnecessary points while capturing dead
+   * stones, we try first to capture stones in atari, second defending
+   * at a liberty, and third capture stones with two or more
+   * liberties. See filllib:43 for a position where capturing dead
+   * stones (B10 or C8) loses a point compared to defending at a
+   * liberty (C6).
    */
-  for (k = 0; k < neighbors; k++) {
-    if (attack(adjs[k], &bpos) == WIN) {
-      if (liberty_of_string(bpos, adjs[k])) {
-	*backfill_move = bpos;
-	return 1;
+  for (opponent_libs = 1; opponent_libs <= 1; opponent_libs++) {
+    for (k = 0; k < neighbors; k++) {
+      if (opponent_libs < 5 && countlib(adjs[k]) != opponent_libs)
+	continue;
+      if (attack(adjs[k], &bpos) == WIN) {
+	if (liberty_of_string(bpos, adjs[k])) {
+	  *backfill_move = bpos;
+	  return 1;
+	}
+	else
+	  saved_move = bpos;
       }
-      else
-	saved_move = bpos;
     }
   }
   
@@ -422,6 +436,23 @@ find_backfilling_move(int move, int color, int *backfill_move)
     }
   }
 
+  if (!found_one) {
+    for (opponent_libs = 2; opponent_libs <= 5; opponent_libs++) {
+      for (k = 0; k < neighbors; k++) {
+	if (opponent_libs < 5 && countlib(adjs[k]) != opponent_libs)
+	  continue;
+	if (attack(adjs[k], &bpos) == WIN) {
+	  if (liberty_of_string(bpos, adjs[k])) {
+	    *backfill_move = bpos;
+	    return 1;
+	  }
+	  else
+	    saved_move = bpos;
+	}
+      }
+    }
+  }
+  
   /* If no luck so far, try with superstring liberties. */
   if (!found_one) {
     trymove(move, color, "find_backfilling_move", move, EMPTY, NO_MOVE);
@@ -515,7 +546,7 @@ filllib_confirm_safety(int move, int color, int *defense_point)
 	  && !play_attack_defend_n(color, 0, 1, move, pos2)) {
 	int adj;
 	adj = chainlinks(pos2, adjs);
-	/* It seems unlikely that we would ever get adjacent strings
+	/* It seems unlikely that we would ever get no adjacent strings
          * here, but if it should happen we simply give up and say the
          * move is unsafe.
 	 */
@@ -527,6 +558,24 @@ filllib_confirm_safety(int move, int color, int *defense_point)
       }
     }
 
+  /* Next attempt are diagonal neighbors. */
+  if (apos == NO_MOVE) {
+    for (k = 4; k < 8; k++)
+      if (board[move + delta[k]] == color) {
+	apos = move + delta[k];
+	break;
+      }
+  }
+
+  /* And two steps away. */
+  if (apos == NO_MOVE) {
+    for (k = 0; k < 4; k++)
+      if (board[move + 2 * delta[k]] == color) {
+	apos = move + 2 * delta[k];
+	break;
+      }
+  }
+  
   /* We should have found something by now. If not something's
    * probably broken elsewhere. Declare the move unsafe if it happens.
    */
@@ -538,11 +587,11 @@ filllib_confirm_safety(int move, int color, int *defense_point)
   save_verbose = verbose;
   if (verbose > 0)
     verbose--;
-  if (!owl_does_defend(move, apos))
+  if (!owl_does_defend(move, apos, NULL))
     return 0;
   verbose = save_verbose;
   
-  return confirm_safety(move, color, 0, defense_point, NULL, NULL);
+  return confirm_safety(move, color, defense_point, NULL);
 }
 
 
