@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -47,9 +48,9 @@
 #define LOWER_RIGHT 3
 
 /* Global variables remembering which symmetries the position has. */
-int horizontally_symmetric; /* symmetry with respect to K column */
-int vertically_symmetric;   /* symmetry with respect to 10 row */
-int diagonally_symmetric;   /* ... with respect to diagonal from UR to LL */
+static int horizontally_symmetric; /* symmetry with respect to K column */
+static int vertically_symmetric;   /* symmetry with respect to 10 row */
+static int diagonally_symmetric;   /* with respect to diagonal from UR to LL */
 
 /* This value must be lower than the value for an ongoing joseki. 
  * (Gets multiplied with board_size / 19.) 
@@ -250,13 +251,13 @@ static int fuseki_total_value;
 
 /* Callback for fuseki database pattern matching. */
 static void
-fuseki_callback(int ti, int tj, struct fullboard_pattern *pattern, int ll)
+fuseki_callback(int move, struct fullboard_pattern *pattern, int ll)
 {
   TRACE("Fuseki database move at %1m with relative weight %d, pattern %s+%d\n",
-	POS(ti, tj), (int) pattern->value, pattern->name, ll);
+	move, (int) pattern->value, pattern->name, ll);
 
   /* Store coordinates and relative weight for the found move. */
-  fuseki_moves[num_fuseki_moves] = POS(ti, tj);
+  fuseki_moves[num_fuseki_moves] = move;
   fuseki_value[num_fuseki_moves] = pattern->value;
   fuseki_total_value += pattern->value;
   num_fuseki_moves++;
