@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -27,7 +28,7 @@
 
 #include "liberty.h"
 #include "sgftree.h"
-#include "random.h"
+#include "gg_utils.h"
 
 /* Return one if x doesn't equal position_number and 0 otherwise.
  * After using this macro x will always have the value
@@ -36,12 +37,8 @@
 #define NEEDS_UPDATE(x) (x != position_number ? (x = position_number, 1) : 0)
 
 static int get_level(int *level);
-static int do_genmove(int *move, int color, float pure_threat_value);
-
-static double slowest_time = 0.0;
-static int    slowest_move = NO_MOVE;
-static int    slowest_movenum = 0;
-static double total_time = 0.0;
+static int do_genmove(int *move, int color, float pure_threat_value,
+		      int allowed_moves[BOARDMAX]);
 
 /* Position numbers for which various examinations were last made. */
 static int worms_examined = -1;
@@ -49,10 +46,15 @@ static int initial_influence_examined = -1;
 static int dragons_examined_without_owl = -1;
 static int dragons_examined = -1;
 static int initial_influence2_examined = -1;
+static int dragons_refinedly_examined = -1;
 
 static int revise_semeai(int color);
 static int revise_thrashing_dragon(int color, float advantage);
 
+static int find_mirror_move(int *move, int color);
+static int test_symmetry_after_move(int move, int color);
+static int should_resign(int color, float score);
+
 void sgfShowConsideredMoves(void);
 
 
@@ -71,25 +73,25 @@ reset_engine()
    * actually play through them, in order to get the right random
    * numbers.
    */
-  gg_srand(random_seed);
+  reuse_random_seed();
 
   /* Initialize things for hashing of positions. */
   reading_cache_clear();
   hashdata_recalc(&hashdata, board, board_ko_pos);
+
   worms_examined = -1;
   initial_influence_examined = -1;
   dragons_examined_without_owl = -1;
   dragons_examined = -1;
   initial_influence2_examined = -1;
+  dragons_refinedly_examined = -1;
 
   /* Prepare our table of move reasons. */
   clear_move_reasons();
+  clear_break_in_list();
 
-  /* Set up depth values. Level 10 should be more or less the same as level 9
-   * except that the superstring technology is used in the reading code. This
-   * makes us about 10% slower.
-   */
-  set_depth_values(level);
+  /* Set up depth values (see comments there for details). */
+  set_depth_values(level, 0);
 }
 
 /*
@@ -108,6 +110,8 @@ examine_position(int color, int how_much)
   int save_verbose = verbose;
 
   purge_persistent_reading_cache();
+  purge_persistent_connection_cache();
+  purge_persistent_breakin_cache();
   
   /* Don't print reading traces during make_worms and make_dragons unless 
    * the user really wants it (verbose == 3). 
@@ -117,9 +121,10 @@ examine_position(int color, int how_much)
 
   if (NEEDS_UPDATE(worms_examined)) {
     start_timer(0);
-    make_worms(save_verbose);
+    make_worms();
     time_report(0, "  make worms", NO_MOVE, 1.0);
   }
+
   if (how_much == EXAMINE_WORMS) {
     verbose = save_verbose;
     gg_assert(test_gray_border() < 0);
@@ -128,7 +133,7 @@ examine_position(int color, int how_much)
 
   if (stones_on_board(BLACK | WHITE) != 0) {
     if (NEEDS_UPDATE(initial_influence_examined))
-      compute_initial_influence(color, 0);
+      compute_worm_influence();
     if (how_much == EXAMINE_INITIAL_INFLUENCE) {
       verbose = save_verbose;
       gg_assert(test_gray_border() < 0);
@@ -137,14 +142,14 @@ examine_position(int color, int how_much)
 
     if (how_much == EXAMINE_DRAGONS_WITHOUT_OWL) {
       if (NEEDS_UPDATE(dragons_examined_without_owl))
-	make_dragons(color, 1, save_verbose);
+	make_dragons(color, 1);
       verbose = save_verbose;
       gg_assert(test_gray_border() < 0);
       return;
     }
     
     if (NEEDS_UPDATE(dragons_examined)) {
-      make_dragons(color, 0, save_verbose);
+      make_dragons(color, 0);
       /* We have automatically done a partial dragon analysis as well. */
       dragons_examined_without_owl = position_number;
     }
@@ -165,15 +170,24 @@ examine_position(int color, int how_much)
   
   verbose = save_verbose;
 
-  if (printworms)
-    show_dragons();
-
-  if (NEEDS_UPDATE(initial_influence2_examined))
-    compute_initial_influence(color, 1);
+  if (NEEDS_UPDATE(initial_influence2_examined)) {
+    compute_dragon_influence();
+  }
   if (how_much == EXAMINE_INITIAL_INFLUENCE2) {
     gg_assert(test_gray_border() < 0);
     return;
   }
+
+  if (NEEDS_UPDATE(dragons_refinedly_examined)) {
+    compute_refined_dragon_weaknesses();
+  }
+  if (how_much == FULL_EXAMINE_DRAGONS) {
+    gg_assert(test_gray_border() < 0);
+    return;
+  }
+
+  if (printworms)
+    show_dragons();
 }
 
 
@@ -205,60 +219,6 @@ silent_examine_position(int color, int how_much)
 }
 
 
-/*
- * Estimate the current score using the influence function. The value
- * is positive if white is thought to be ahead and negative otherwise.
- *
- * The color is the color to move. We only need this for ugly
- * technical reasons and it shouldn't have any effect on the score
- * estimate.
- */
-void
-old_estimate_score(int color, float *lower_bound, float *upper_bound)
-{
-  static int last_scored_position = -1;
-  static float lower = 0.0;
-  static float upper = 0.0;
-  float score1;
-  float score2;
-
-  if (position_number != last_scored_position) {
-    /* Find out information about the worms and dragons. */
-    examine_position(color, EXAMINE_ALL);
-    compute_initial_influence(OTHER_COLOR(color), 1);
-
-    /* The coefficients should match those close to the end of
-     * estimate_influence_value() in move_reasons.c.
-     */
-    score1 = influence_estimate_score(0.35, 0.13);
-    compute_initial_influence(color, 1);
-    score2 = influence_estimate_score(0.35, 0.13);
-    
-    DEBUG(DEBUG_SCORING, "%f %f %d %d\n", score1, score2,
-	  white_captured, black_captured);
-
-    if (score1 > score2) {
-      lower = score2;
-      upper = score1;
-    }
-    else {
-      lower = score1;
-      upper = score2;
-    }
-
-    lower += black_captured - white_captured;
-    upper += black_captured - white_captured;
- 
-    last_scored_position = position_number;
-  }
-
-  *lower_bound = lower + komi;
-  *upper_bound = upper + komi;
-  gg_assert(test_gray_border() < 0);
-}
-
-
-
 /* 
  * Generate computer move for COLOR.
  *
@@ -271,7 +231,17 @@ genmove(int *i, int *j, int color)
   int move;
   int retval;
 
-  retval = do_genmove(&move, color, 0.4);
+#if ORACLE
+  if (metamachine) {
+    retval = metamachine_genmove(i, j, color);
+    gg_assert(stackp == 0);
+    if (*i != -1)
+      return 1;
+  }
+#endif
+
+  retval = do_genmove(&move, color, 0.4, NULL);
+  gg_assert(retval < 0 || ON_BOARD1(move));
 
   if (i) *i = I(move);
   if (j) *j = J(move);
@@ -291,7 +261,7 @@ genmove_conservative(int *i, int *j, int color)
   int move;
   int retval;
 
-  retval = do_genmove(&move, color, 0.0);
+  retval = do_genmove(&move, color, 0.0, NULL);
 
   if (i) *i = I(move);
   if (j) *j = J(move);
@@ -300,15 +270,51 @@ genmove_conservative(int *i, int *j, int color)
 }
 
 
+int
+genmove_restricted(int *i, int *j, int color, int allowed_moves[BOARDMAX])
+{
+  int move;
+  int retval;
+
+  retval = do_genmove(&move, color, 0.0, allowed_moves);
+
+  if (i) *i = I(move);
+  if (j) *j = J(move);
+
+  return retval;
+}
+
+/* This function collects move reasons can be generated immediately from
+ * the data gathered in the examine_position() phase.
+ */
+void
+collect_move_reasons(int color)
+{
+  worm_reasons(color);
+  owl_reasons(color);
+  semeai_move_reasons(color);
+  break_in_move_reasons(color);
+}
+
 /* 
- * Perform the actual move generation. 
+ * Perform the actual move generation.
+ *
+ * The array allowed_moves restricts which moves may be considered. If
+ * NULL any move is allowed. Pass is always allowed and will be chosen
+ * if the move generation doesn't like any of the allowed moves (or
+ * overlooks them).
+ *
+ * Resignation is indicated by a negatively valued move (which is not
+ * a pass)
  */
   
 static int
-do_genmove(int *move, int color, float pure_threat_value)
+do_genmove(int *move, int color, float pure_threat_value,
+	   int allowed_moves[BOARDMAX])
 {
   float val;
   int save_verbose;
+  int save_depth;
 
   start_timer(0);
   
@@ -335,6 +341,20 @@ do_genmove(int *move, int color, float pure_threat_value)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
+  /* Store the depth value so we can check that it hasn't changed when
+   * we leave this function.
+   */
+  save_depth = depth;
+
+  /* If in mirror mode, try to find a mirror move. */
+  if (play_mirror_go
+      && (mirror_stones_limit < 0
+	  || stones_on_board(WHITE | BLACK) <= mirror_stones_limit)
+      && find_mirror_move(move, color)) {
+    TRACE("genmove() recommends mirror move at %1m\n", *move);
+    return 1.0;
+  }
+
   /* Find out information about the worms and dragons. */
   start_timer(1);
   examine_position(color, EXAMINE_ALL);
@@ -345,7 +365,7 @@ do_genmove(int *move, int color, float pure_threat_value)
    * we are behind, we have to play more daringly.
    */
   if (level >= 8) {
-    estimate_score(&lower_bound, &upper_bound);
+    estimate_score(&upper_bound, &lower_bound);
     if (verbose || showscore) {
       if (lower_bound == upper_bound)
 	gprintf("\nScore estimate: %s %f\n",
@@ -384,6 +404,8 @@ do_genmove(int *move, int color, float pure_threat_value)
     if (printboard == 1) {
       fprintf(stderr, "\n           owl_status display:\n\n");      
       showboard(3);
+      fprintf(stderr, "\n           matcher_status display:\n\n");      
+      showboard(4);
     }
   }
   
@@ -393,18 +415,17 @@ do_genmove(int *move, int color, float pure_threat_value)
    * Ok, information gathering is complete. Now start to find some moves!
    */
 
-  /* Pick up tactical moves. */
-  worm_reasons(color);
   
-  /* Pick up owl moves. */
+  /* Pick up moves that we know of already. */
   save_verbose = verbose;
   if (verbose > 0)
     verbose--;
-  owl_reasons(color);
+  collect_move_reasons(color);
   verbose = save_verbose;
   
   /* Try to find empty corner moves. */
-  fuseki(color);
+  if (!limit_search)
+    fuseki(color);
   gg_assert(stackp == 0);
 
   /* The general pattern database. */
@@ -420,7 +441,7 @@ do_genmove(int *move, int color, float pure_threat_value)
 
   /* Review the move reasons and estimate move values. */
   if (review_move_reasons(move, &val, color, 
-			  pure_threat_value, lower_bound))
+			  pure_threat_value, score, allowed_moves))
     TRACE("Move generation likes %1m with value %f\n", *move, val);
   gg_assert(stackp == 0);
   time_report(1, "review move reasons", NO_MOVE, 1.0);
@@ -429,12 +450,13 @@ do_genmove(int *move, int color, float pure_threat_value)
    * dragon dangerous and change its status from DEAD to
    * UNKNOWN. This may generate a move.
    */
-  if (val < 10.0 && !doing_scoring) {
+  if (val < 10.0 && !doing_scoring && !limit_search) {
     if (revise_thrashing_dragon(color, 15.0)) {
       shapes(color);
       if (!disable_endgame_patterns)
 	endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, score)) {
+      if (review_move_reasons(move, &val, color, pure_threat_value, score,
+			      allowed_moves)) {
 	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
 	      *move, val); 
       }
@@ -443,10 +465,12 @@ do_genmove(int *move, int color, float pure_threat_value)
   }
 
   /* If the move value is 6 or lower, we look for endgame patterns too. */
-  if (val <= 6.0 && !disable_endgame_patterns) {
+  if (val <= 6.0 && !disable_endgame_patterns && !limit_search) {
     endgame_shapes(color);
+    endgame(color);
     gg_assert(stackp == 0);
-    if (review_move_reasons(move, &val, color, pure_threat_value, score))
+    if (review_move_reasons(move, &val, color, pure_threat_value, score,
+			    allowed_moves))
       TRACE("Move generation likes %1m with value %f\n", *move, val);
     gg_assert(stackp == 0);
     time_report(1, "endgame", NO_MOVE, 1.0);
@@ -461,7 +485,8 @@ do_genmove(int *move, int color, float pure_threat_value)
 	|| revise_semeai(color)) {
       shapes(color);
       endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, score)) {
+      if (review_move_reasons(move, &val, color, pure_threat_value, score,
+			      allowed_moves)) {
 	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
 	      *move, val); 
       }
@@ -473,9 +498,11 @@ do_genmove(int *move, int color, float pure_threat_value)
    * all missing dame points.
    */
   if (val < 0.0 
-      && fill_liberty(move, color)) {
+      && fill_liberty(move, color)
+      && (!allowed_moves || allowed_moves[*move])) {
     val = 1.0;
     TRACE("Filling a liberty at %1m\n", *move);
+    record_top_move(*move, val);
     move_considered(*move, val);
     time_report(1, "fill liberty", NO_MOVE, 1.0);
   }
@@ -492,10 +519,12 @@ do_genmove(int *move, int color, float pure_threat_value)
 	  || (thrashing_dragon
 	      && ((color == BLACK && score < -15.0)
 		  || (color == WHITE && score > 15.0))))
-      && aftermath_genmove(move, color, NULL, 0) > 0) {
+      && aftermath_genmove(move, color, NULL, 0) > 0
+      && (!allowed_moves || allowed_moves[*move])) {
     ASSERT1(is_legal(*move, color), *move);
     val = 1.0;
     TRACE("Aftermath move at %1m\n", *move);
+    record_top_move(*move, val);
     move_considered(*move, val);
     time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
   }
@@ -506,7 +535,8 @@ do_genmove(int *move, int color, float pure_threat_value)
   if (val < 0.0
       && !doing_scoring
       && capture_all_dead 
-      && aftermath_genmove(move, color, NULL, 1) > 0) {
+      && aftermath_genmove(move, color, NULL, 1) > 0
+      && (!allowed_moves || allowed_moves[*move])) {
     ASSERT1(is_legal(*move, color), *move);
     val = 1.0;
     TRACE("Aftermath move at %1m\n", *move);
@@ -519,8 +549,16 @@ do_genmove(int *move, int color, float pure_threat_value)
     TRACE("I pass.\n");
     *move = NO_MOVE;
   }
-  else
+  else {
     TRACE("genmove() recommends %1m with value %f\n", *move, val);
+    /* Maybe time to resign...
+     */
+    if (resign_allowed && val < 10.0 && should_resign(color, score)) {
+      TRACE("... though, genmove() thinks the position is hopeless\n" );
+      /* Signal resignation by negating the move value */
+      val = -val;
+    }
+  }
   
   /* If statistics is turned on, this is the place to show it. */
   if (showstatistics) {
@@ -532,7 +570,7 @@ do_genmove(int *move, int color, float pure_threat_value)
     gprintf("Hash collisions:      %d\n", stats.hash_collisions);
   }
  
- if (showtime) {
+  if (showtime) {
     double spent = time_report(0, "TIME to generate move at ", *move, 1.0);
     total_time += spent;
     if (spent > slowest_time) {
@@ -540,18 +578,17 @@ do_genmove(int *move, int color, float pure_threat_value)
       slowest_move = *move;
       slowest_movenum = movenum + 1;
     }
-    if (*move == NO_MOVE) {
-      gprintf("\nSLOWEST MOVE: %d at %1m ", slowest_movenum, slowest_move);
-      fprintf(stderr, "(%.2f seconds)\n", slowest_time);
-      fprintf(stderr, "\nAVERAGE TIME: %.2f seconds per move\n",
-	      total_time / movenum);
-      fprintf(stderr, "\nTOTAL TIME: %.2f seconds\n",
-	      total_time);
-
-    }
   }
-  
+  if (0 && (debug & DEBUG_BREAKIN))
+    print_persistent_breakin_cache();
+
+  /* Some consistency checks to verify that things are properly
+   * restored and/or have not been corrupted.
+   */
+  gg_assert(stackp == 0);
   gg_assert(test_gray_border() < 0);
+  gg_assert(depth == save_depth);
+
   return val;
 }
 
@@ -599,176 +636,6 @@ get_level(int *level)
 }
   
 
-/* ================================================================ */
-/*       Set up fixed placement handicap stones for black side      */
-/* ================================================================ */
-
-
-/* Handicap stones are set up according to the following diagrams:
- *  
- * 2 stones:                    3 stones:           
- *
- *   A B C D E F G H J	  	  A B C D E F G H J  
- * 9 . . . . . . . . . 9  	9 . . . . . . . . . 9
- * 8 . . . . . . . . . 8  	8 . . . . . . . . . 8
- * 7 . . + . . . X . . 7  	7 . . X . . . X . . 7
- * 6 . . . . . . . . . 6  	6 . . . . . . . . . 6
- * 5 . . . . + . . . . 5  	5 . . . . + . . . . 5
- * 4 . . . . . . . . . 4  	4 . . . . . . . . . 4
- * 3 . . X . . . + . . 3  	3 . . X . . . + . . 3
- * 2 . . . . . . . . . 2  	2 . . . . . . . . . 2
- * 1 . . . . . . . . . 1  	1 . . . . . . . . . 1
- *   A B C D E F G H J	  	  A B C D E F G H J  
- *   
- * 4 stones:                    5 stones:           
- *						     
- *   A B C D E F G H J	          A B C D E F G H J  
- * 9 . . . . . . . . . 9 	9 . . . . . . . . . 9
- * 8 . . . . . . . . . 8 	8 . . . . . . . . . 8
- * 7 . . X . . . X . . 7 	7 . . X . . . X . . 7
- * 6 . . . . . . . . . 6 	6 . . . . . . . . . 6
- * 5 . . . . + . . . . 5 	5 . . . . X . . . . 5
- * 4 . . . . . . . . . 4 	4 . . . . . . . . . 4
- * 3 . . X . . . X . . 3 	3 . . X . . . X . . 3
- * 2 . . . . . . . . . 2 	2 . . . . . . . . . 2
- * 1 . . . . . . . . . 1 	1 . . . . . . . . . 1
- *   A B C D E F G H J	          A B C D E F G H J  
- *  
- * 6 stones:                    7 stones:           
- *						     
- *   A B C D E F G H J	          A B C D E F G H J  
- * 9 . . . . . . . . . 9 	9 . . . . . . . . . 9
- * 8 . . . . . . . . . 8 	8 . . . . . . . . . 8
- * 7 . . X . . . X . . 7 	7 . . X . . . X . . 7
- * 6 . . . . . . . . . 6 	6 . . . . . . . . . 6
- * 5 . . X . + . X . . 5 	5 . . X . X . X . . 5
- * 4 . . . . . . . . . 4 	4 . . . . . . . . . 4
- * 3 . . X . . . X . . 3 	3 . . X . . . X . . 3
- * 2 . . . . . . . . . 2 	2 . . . . . . . . . 2
- * 1 . . . . . . . . . 1 	1 . . . . . . . . . 1
- *   A B C D E F G H J	          A B C D E F G H J  
- *  
- * 8 stones:                    9 stones:           
- *						     
- *   A B C D E F G H J	          A B C D E F G H J  
- * 9 . . . . . . . . . 9   	9 . . . . . . . . . 9
- * 8 . . . . . . . . . 8   	8 . . . . . . . . . 8
- * 7 . . X . X . X . . 7   	7 . . X . X . X . . 7
- * 6 . . . . . . . . . 6   	6 . . . . . . . . . 6
- * 5 . . X . + . X . . 5   	5 . . X . X . X . . 5
- * 4 . . . . . . . . . 4   	4 . . . . . . . . . 4
- * 3 . . X . X . X . . 3   	3 . . X . X . X . . 3
- * 2 . . . . . . . . . 2   	2 . . . . . . . . . 2
- * 1 . . . . . . . . . 1   	1 . . . . . . . . . 1
- *   A B C D E F G H J	          A B C D E F G H J  
- *  
- * For odd-sized boards larger than 9x9, the same pattern is followed,
- * except that the edge stones are moved to the fourth line for 13x13
- * boards and larger.
- *
- * For even-sized boards at least 8x8, only the four first diagrams
- * are used, because there is no way to place the center stones
- * symmetrically. As for odd-sized boards, the edge stones are moved
- * to the fourth line for boards larger than 11x11.
- *
- * At most four stones are placed on 7x7 boards too (this size may or
- * may not be supported by the rest of the engine). No handicap stones
- * are ever placed on smaller boards.
- *
- * Notice that this function only deals with fixed handicap placement.
- * Larger handicaps can be added by free placement if the used
- * interface supports it.
- */
-
-
-/* This table contains the (coded) positions of the stones.
- *  2 maps to 2 or 3, depending on board size
- *  0 maps to center
- * -ve numbers map to  board_size - number
- *
- * The stones are placed in this order, *except* if there are
- * 5 or 7 stones, in which case center ({0, 0}) is placed, and
- * then as for 4 or 6.
- */
-
-static const int places[][2] = {
-
-  {2, -2}, {-2, 2}, {2, 2}, {-2, -2}, /* first 4 are easy */
-                                      /* for 5, {0,0} is explicitly placed */
-  
-  {0, 2}, {0, -2},                    /* for 6 these two are placed */
-                                      /* for 7, {0,0} is explicitly placed */
-  
-  {2, 0}, {-2, 0},                    /* for 8, these two are placed */
-
-  {0, 0},                             /* finally tengen for 9 */
-};
-
-
-/*
- * Sets up handicap stones, returning the number of placed handicap stones.
- */
-
-int
-placehand(int handicap)
-{
-  int x;
-  int maxhand;
-  int three = board_size > 11 ? 3 : 2;
-  int mid = board_size/2;
-  int retval = handicap;
-
-  /* A handicap of 1 just means that B plays first, no komi.
-   * Black is not told where to play the first stone so no handicap
-   * is set. 
-   */
-  if (handicap < 2)
-    return 0;
-  if ((board_size % 2 == 1) && (board_size >= 9))
-    maxhand = 9;
-  else if (board_size >= 7)
-    maxhand = 4;
-  else
-    maxhand = 0;
-
-  /* It's up to the caller of this function to notice if the handicap
-   * was too large for fixed placement and act upon that.
-   */
-  if (handicap > maxhand) {
-    handicap = maxhand;
-    retval = maxhand;
-  }
-
-  /* special cases: 5 and 7 */
-  if (handicap == 5 || handicap == 7) {
-    add_stone(POS(mid, mid), BLACK);
-    handicap--;
-  }
-
-  for (x = 0; x < handicap; ++x) {
-    int i = places[x][0];
-    int j = places[x][1];
-
-    /* translate the encoded values to board co-ordinates */
-    if (i == 2)  i = three;	/* 2 or 3 */
-    if (i == -2) i = -three;
-
-    if (j == 2)  j = three;
-    if (j == -2) j = -three;
-
-    if (i == 0) i = mid;
-    if (j == 0) j = mid;
-
-    if (i < 0) i += board_size-1;
-    if (j < 0) j += board_size-1;
-
-    add_stone(POS(i, j), BLACK);
-  }
-
-  return retval;
-}
-
-
 /* revise_semeai(color) changes the status of any DEAD dragon of
  * OPPOSITE_COLOR(color) which occurs in a semeai to UNKNOWN.
  * It returns true if such a dragon is found.
@@ -781,15 +648,21 @@ revise_semeai(int color)
   int found_one = 0;
   int other = OTHER_COLOR(color);
 
+  if (stones_on_board(BLACK | WHITE) == 0)
+    return 0;
+
+  if (doing_scoring)
+    return 0;
+  
   gg_assert(dragon2 != NULL);
 
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (ON_BOARD(pos)
 	&& dragon[pos].color == other
 	&& DRAGON2(pos).semeai
-	&& dragon[pos].matcher_status == DEAD) {
+	&& dragon[pos].status == DEAD) {
       found_one = 1;
-      dragon[pos].matcher_status = UNKNOWN;
+      dragon[pos].status = UNKNOWN;
       if (dragon[pos].origin == pos)
 	TRACE("revise_semeai: changed status of dragon %1m from DEAD to UNKNOWN\n",
 	      pos);
@@ -810,31 +683,227 @@ static int
 revise_thrashing_dragon(int color, float advantage)
 {
   int pos;
+  char safe_stones[BOARDMAX];
+  float strength[BOARDMAX];
 
   /* Trust the owl code's opinion if we are behind. */
   if ((color == BLACK && score > -advantage)
       || (color == WHITE && score < advantage))
     return 0;
 
-  /* If an owl threat has been found, a move reason has
-   * probably been generated, so we skip this step. */
-  if (dragon[thrashing_dragon].owl_threat_status == CAN_THREATEN_DEFENSE)
-    return 0;
-
   if (disable_threat_computation
       || !thrashing_dragon 
-      || dragon[thrashing_dragon].matcher_status != DEAD)
+      || dragon[thrashing_dragon].status != DEAD)
     return 0;
   
   for (pos = BOARDMIN; pos < BOARDMAX; pos++)
-    if (ON_BOARD(pos)
-	&& is_same_dragon(pos, thrashing_dragon))
-      dragon[pos].matcher_status = UNKNOWN;
+    if (ON_BOARD(pos) && thrashing_stone[pos]
+	&& worm[pos].unconditional_status != DEAD) {
+      dragon[pos].status = UNKNOWN;
+      DRAGON2(pos).safety = ALIVE;
+    }
+
+  set_strength_data(OTHER_COLOR(color), safe_stones, strength);
+  compute_influence(OTHER_COLOR(color), safe_stones, strength,
+      		    OPPOSITE_INFLUENCE(color),
+		    NO_MOVE, "revised thrashing dragon");
+  compute_refined_dragon_weaknesses();
+  
   return 1;
 }
 
 
+/* Look for a mirror move. First try the mirror of the last move. If
+ * none is available, test all moves to see if one makes the board
+ * symmetric.
+ *
+ * To be able to deal with handicap stones we use a somewhat weak
+ * definition of symmetry.
+ */
+#define MIRROR_MOVE(pos) POS(board_size - 1 - I(pos), board_size - 1 - J(pos))
 
+static int
+find_mirror_move(int *move, int color)
+{
+  int last_move = get_last_move();
+  int mirror_move;
+  if (last_move != NO_MOVE) {
+    mirror_move = MIRROR_MOVE(last_move);
+    if (test_symmetry_after_move(mirror_move, color)) {
+      *move = mirror_move;
+      return 1;
+    }
+  }
+  else {
+    for (mirror_move = BOARDMIN; mirror_move < BOARDMAX; mirror_move++) {
+      if (ON_BOARD(mirror_move)
+	  && test_symmetry_after_move(mirror_move, color)) {
+	*move = mirror_move;
+	return 1;
+      }
+    }
+  }
+  
+  return 0;
+}
+
+static int
+test_symmetry_after_move(int move, int color)
+{
+  int pos;
+  int result = 1;
+
+  if (board[move] != EMPTY)
+    return 0;
+  if (!trymove(move, color, "find_mirror_move", NO_MOVE, EMPTY, NO_MOVE))
+    return 0;
+  
+  for (pos = BOARDMIN; pos <= MIRROR_MOVE(pos); pos++) {
+    if ((board[pos] == EMPTY) ^ (board[MIRROR_MOVE(pos)] == EMPTY)) {
+      result = 0;
+      break;
+    }
+  }
+  
+  popgo();
+
+  return result;
+}
+
+/* Helper to decide whether GG should resign a game
+ */
+static int
+should_resign(int color, float score)
+{
+  float status;
+  int d;
+  /* We resign 19x19 games only, smaller board games are fast enough.
+   * We resign only if the margin is bigger than 45 pts and if we are
+   * behind (of course).
+   */
+  if (board_size < 19
+      || gg_abs(score) < 45
+      || (color == WHITE && score >= 0.0)
+      || (color == BLACK && score <= 0.0))
+    return 0;
+  /* Check dragon statuses. If a friendly dragon is critical, we are
+   * possibly not that much behind after we save it. If some hostile
+   * dragon is at least weak, we possibly have a chance to come back
+   * if we can kill it.
+   */
+  for (d = 0; d < number_of_dragons; d++) {
+    /* any friendly critical dragon ? */
+    if (board[dragon2[d].origin] == color
+	&& DRAGON(d).status == CRITICAL)
+      return 0;
+    /* any weak opponent dragon ? */
+    if (board[dragon2[d].origin] == OTHER_COLOR(color)
+	&& DRAGON(d).status != DEAD
+	&& DRAGON(d).effective_size >= 10
+	&& dragon_weak(dragon2[d].origin))
+      return 0;
+  }
+  /* Is it already too late to try something ? */
+  status = game_status(color);
+  if (status < 0.8)
+    /* Still "too early".
+     * Note: the 0.8 constant is very conservative, we actually could give
+     * up a bit earlier.
+     */
+    return 0;
+
+  /* well, it is probably reasonable and polite to give up this game */
+  return 1;
+}
+
+
+/*********************************************************************\
+ *                Mark a limited search area                         *
+\*********************************************************************/
+
+/* Mark a limited search area. Only affects the engine if the
+ * global variable limit_search is nonzero. In this case, genmove
+ * will only return moves within the area marked by the array
+ * search_mask.
+ */
+
+static int search_mask[BOARDMAX];
+
+/* The following function marks a diamond of radius 6 with center pos.
+ */
+
+void
+set_search_diamond(int pos)
+{
+  int i = I(pos);
+  int j = J(pos);
+  int m, n;
+  
+  for (m = 0; m < board_size; m++)
+    for (n = 0; n < board_size; n++) {
+      if (gg_abs(m - i) + gg_abs(n - j) <= 6)
+	search_mask[POS(m, n)] = 1;
+      else
+	search_mask[POS(m, n)] = 0;      
+    }
+  limit_search = pos;
+  if (0) draw_search_area();
+}
+
+/* unmarks the entire board */
+
+void
+reset_search_mask()
+{
+  memset(search_mask, 0, sizeof(search_mask));
+}
+
+/* marks a single vertex */
+
+void
+set_search_limit(int pos, int value)
+{
+  search_mask[pos] = value;
+}
+
+/* displays the search area */
+
+void
+draw_search_area(void)
+{
+  int m, n;
+
+  start_draw_board();
+  for (m = 0; m < board_size; m++)
+    for (n = 0; n < board_size; n++) {
+      int col, c;
+	
+      if (search_mask[POS(m,n)])
+	col = GG_COLOR_RED;
+      else
+	col = GG_COLOR_BLACK;
+      
+      if (board[POS(m, n)] == BLACK)
+	c = 'X';
+      else if (board[POS(m, n)] == WHITE)
+	c = 'O';
+      else if (search_mask[POS(m, n)])
+	c = '*';
+      else
+	c = '.';
+      draw_color_char(m, n, c, col);
+    }
+  end_draw_board();
+}
+
+/* returns true if the position is within the search area */
+int
+within_search_area(int pos)
+{
+  if (!limit_search)
+    return 1;
+  return search_mask[pos];
+}
 
 /*
  * Local Variables:
