@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -35,12 +36,16 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_VISUAL_C
+# include <crtdbg.h>
+#endif
+
 #include "sgftree.h"
 #include "clock.h"
 
 /* interface.c */
 /* Initialize the whole thing. Should be called once. */
-void init_gnugo(float memory);
+void init_gnugo(float memory, unsigned int random_seed);
 
 
 /* ================================================================ */
@@ -48,6 +53,10 @@ void init_gnugo(float memory);
 /* ================================================================ */
 
 
+/* Board size */
+
+#define DEFAULT_BOARD_SIZE 19
+
 /* Colors */
 #define EMPTY        0
 #define WHITE        1
@@ -60,15 +69,20 @@ void init_gnugo(float memory);
 #define ANCHOR_OTHER 7
 
 
-#define OTHER_COLOR(color)  (WHITE+BLACK-(color))
-#define IS_STONE(arg)  ((arg) == WHITE || (arg) == BLACK)
+#define OTHER_COLOR(color)  	(WHITE+BLACK-(color))
+#define IS_STONE(arg)   	((arg) == WHITE || (arg) == BLACK)
+#define BORDER_COLOR(color)	(color + WHITE_BORDER - WHITE)
 
 /* Return codes for reading functions */
 
-#define WIN  3
-#define KO_A 2
+#define WIN  5
+#define KO_A 4
+#define GAIN 3
+#define LOSS 2
 #define KO_B 1
 #define LOSE 0
+
+/* Used by break_through(). Must be different from 0 and WIN. */
 #define CUT  2
 
 /* Group statuses */
@@ -80,10 +94,8 @@ void init_gnugo(float memory);
 #define MAX_DRAGON_STATUS 4	/* used to size an array in matchpat.c */
 
 /* Dragon safety values. DEAD, ALIVE, and CRITICAL are reused. */
-#define INESSENTIAL     3
-#define TACTICALLY_DEAD 4
-#define WEAK            5
-#define WEAKLY_ALIVE    6
+#define INESSENTIAL     5
+#define TACTICALLY_DEAD 6
 #define ALIVE_IN_SEKI   7
 #define STRONGLY_ALIVE  8
 #define INVINCIBLE      9
@@ -91,14 +103,16 @@ void init_gnugo(float memory);
 #define CAN_THREATEN_ATTACK  11
 #define CAN_THREATEN_DEFENSE 12
 
+/* Surrounded */
+
+#define SURROUNDED 1
+#define WEAKLY_SURROUNDED 2
+
 /* Final statuses for empty vertices. */
 #define BLACK_TERRITORY 13
 #define WHITE_TERRITORY 14
 #define DAME            15
 
-/* Eye reading */
-#define LIFE_EYESIZE         10
-
 /* ================================================================ */
 /*                        Board manipulation                        */
 /* ================================================================ */
@@ -146,10 +160,10 @@ void  gnugo_who_wins(int color, FILE *outfile);
 float gnugo_estimate_score(float *upper, float *lower);
 void  gnugo_examine_position(int color, int how_much);
 
-int  gnugo_get_komi(void);
-void gnugo_get_board(int b[MAX_BOARD][MAX_BOARD]);
-int  gnugo_get_boardsize(void);
-int  gnugo_get_move_number(void);
+float gnugo_get_komi(void);
+void  gnugo_get_board(int b[MAX_BOARD][MAX_BOARD]);
+int   gnugo_get_boardsize(void);
+int   gnugo_get_move_number(void);
 
 /* ================================================================ */
 /*                           Game handling                          */
@@ -163,18 +177,15 @@ typedef struct {
   SGFTree   game_record;	/* Game record in sgf format. */
 
   int       computer_player;	/* BLACK, WHITE, or EMPTY (used as BOTH) */
-
-  char      outfilename[128];	/* Trickle file */
-  FILE      *outfile;
 } Gameinfo;
 
 void gameinfo_clear(Gameinfo *ginfo, int boardsize, float komi);
 void gameinfo_print(Gameinfo *ginfo);
 void gameinfo_load_sgfheader(Gameinfo *gameinfo, SGFNode *head);
 void gameinfo_play_move(Gameinfo *ginfo, int i, int j, int color);
-int  gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
+int  gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 			       const char *untilstr, int orientation);
-int  gameinfo_play_sgftree(Gameinfo *gameinfo, SGFNode *head,
+int  gameinfo_play_sgftree(Gameinfo *gameinfo, SGFTree *tree,
 			   const char *untilstr);
 
 
@@ -182,8 +193,6 @@ int  gameinfo_play_sgftree(Gameinfo *gameinfo, SGFNode *head,
 /*                           global variables                       */
 /* ================================================================ */
 
-/* Random seed */
-extern int random_seed;
 
 /* Miscellaneous debug options. */
 extern int quiet;		/* Minimal output. */
@@ -195,6 +204,14 @@ extern int printdragons;	/* print full data on each dragon */
 extern int printboard;		/* print board each move */
 extern int showstatistics;	/* print statistics */
 extern int profile_patterns;	/* print statistics of pattern usage */
+extern char outfilename[128];  /* output file (-o option) */
+extern int output_flags;       /* amount of output to outfile */
+
+/* output flag bits */
+#define OUTPUT_MARKDRAGONS         0x0001  /* mark dead and critical dragons */
+#define OUTPUT_MOVEVALUES          0x0002  /* output values of all moves in list */
+
+#define OUTPUT_DEFAULT             0 /* no debug output  by default */
 
 /* debug flag bits */
 /* NOTE : can specify -d0x... */
@@ -211,7 +228,7 @@ extern int profile_patterns;	/* print statistics of pattern usage */
 #define DEBUG_WORMS                 0x0400
 #define DEBUG_MOVE_REASONS          0x0800
 #define DEBUG_OWL_PERFORMANCE       0x1000
-#define DEBUG_LIFE                  0x2000
+#define DEBUG_BREAKIN		    0x2000
 #define DEBUG_FILLLIB               0x4000
 #define DEBUG_READING_PERFORMANCE   0x8000
 #define DEBUG_SCORING               0x010000
@@ -220,6 +237,10 @@ extern int profile_patterns;	/* print statistics of pattern usage */
 #define DEBUG_READING_CACHE         0x080000
 #define DEBUG_TERRITORY             0x100000
 #define DEBUG_OWL_PERSISTENT_CACHE  0x200000
+#define DEBUG_TOP_MOVES             0x400000
+#define DEBUG_MISCELLANEOUS         0x800000
+#define DEBUG_ORACLE_STREAM         0x1000000
+
 /* hash flag bits 
  *
  * Regarding HASH_DEFAULT:
@@ -228,48 +249,46 @@ extern int profile_patterns;	/* print statistics of pattern usage */
  */
 
 #define HASH_FIND_DEFENSE 0x0001  /* NOTE : can specify -d0x... */
-#define HASH_DEFEND1      0x0002
-#define HASH_DEFEND2      0x0004
-#define HASH_DEFEND3      0x0008
-#define HASH_DEFEND4      0x0010
 #define HASH_ATTACK       0x0020
-#define HASH_ATTACK2      0x0040
-#define HASH_ATTACK3      0x0080
 #define HASH_OWL_ATTACK   0x0100
 #define HASH_OWL_DEFEND   0x0200
 #define HASH_SEMEAI       0x0400
 #define HASH_CONNECT      0x0800
 #define HASH_DISCONNECT   0x1000
+#define HASH_BREAK_IN	  0x2000
+#define HASH_BLOCK_OFF	  0x4000
 #define HASH_NOTHING      0
 #define HASH_ALL          0xffff
 #define HASH_DEFAULT      (HASH_ATTACK | HASH_FIND_DEFENSE\
 			   | HASH_OWL_ATTACK | HASH_OWL_DEFEND | HASH_SEMEAI\
-                           | HASH_CONNECT | HASH_DISCONNECT)
+                           | HASH_CONNECT | HASH_DISCONNECT\
+			   | HASH_BREAK_IN | HASH_BLOCK_OFF)
 
 extern int debug;		/* debug flags */
 extern int hashflags;		/* hash flags */
-extern int life;                /* use "life" reading code */
-extern int life_eyesize;        /* max eye size for life code */
 extern int fusekidb;            /* use fuseki database */
 extern int disable_fuseki;      /* do not generate fuseki moves */
 extern int josekidb;            /* use joseki database */
 extern int level;		/* controls depth of reading */
 extern int semeai_variations;   /* max variations considered reading semeai */
-extern int urgent;              /* urgent move on board */
 extern int showtime;		/* print genmove time */
 extern int showscore;		/* print score */
 extern float score;
 extern float lower_bound;
 extern float upper_bound;
 extern int chinese_rules;       /* use chinese (area) rules for counting */
+extern int experimental_owl_ext;     /* use experimental owl (GAIN/LOSS) */
 extern int experimental_semeai;      /* use experimental semeai module */
 extern int experimental_connections; /* use experimental connection module */
 extern int alternate_connections;    /* use alternate connection module */
 extern int owl_threats;              /* compute owl threats */
-extern int experimental_influence;   /* use experimental influence module */
 extern int allow_suicide;            /* allow opponent to make suicide moves */
 extern int capture_all_dead;         /* capture all dead opponent stones */
 extern int play_out_aftermath; /* make everything unconditionally settled */
+extern int resign_allowed;           /* allows GG to resign hopeless games */
+extern int play_mirror_go;           /* try to play mirror go if possible */
+extern int mirror_stones_limit;      /* but stop at this number of stones */
+extern int gtp_version;              /* version of Go Text Protocol */
 
 #if EXPERIMENTAL_READING
 extern int defend_by_pattern;  /* use patterns for tactical reading defense */
@@ -281,12 +300,12 @@ extern int attack_by_pattern;  /* use patterns for tactical reading attack */
 extern int mandated_depth;
 extern int mandated_backfill_depth;
 extern int mandated_backfill2_depth;
+extern int mandated_break_chain_depth;
 extern int mandated_superstring_depth;
 extern int mandated_fourlib_depth;
 extern int mandated_ko_depth;
 extern int mandated_branch_depth;
 extern int mandated_aa_depth;
-extern int mandated_aa_threat_depth;
 extern int mandated_owl_distrust_depth;
 extern int mandated_owl_branch_depth;
 extern int mandated_owl_reading_depth;
@@ -295,7 +314,9 @@ extern int mandated_owl_node_limit;
 /* Keep this as 2D until we change the entire API. */
 extern float potential_moves[MAX_BOARD][MAX_BOARD];
 
-extern volatile int time_to_die;   /* set by signal handlers */
+extern int limit_search;  /* limit move search to a portion of the board */
+extern int oracle_exists; /* oracle is available for consultation        */
+extern int metamachine;   /* use metamachine_genmove                     */
 
 /* ================================================================ */
 /*                 tracing and debugging functions                  */
@@ -316,7 +337,8 @@ void start_draw_board(void);
 void draw_color_char(int m, int n, int c, int color);
 void draw_char(int m, int n, int c);
 void end_draw_board(void);
-void showboard(int xo);  /* ascii rep. of board to stdout */
+void showboard(int xo);  /* ascii rep. of board to stderr */
+void simple_showboard(FILE *outfile);  /* ascii rep. of board to outfile */
 
 /* printutils.c */
 int gprintf(const char *fmt, ...);
@@ -336,34 +358,19 @@ double gg_gettimeofday(void);
 /* influence.c */
 void debug_influence_move(int i, int j);
 
+
+#define TRACE  (!(verbose)) ? (void)0 : (void)gprintf
+
 #ifdef HAVE_VARIADIC_DEFINE
 
 /* gnuc allows variadic macros, so the tests can be done inline */
-#define TRACE(fmt, args...) \
-    do { if (verbose) gprintf(fmt, ##args); } while (0)
-#define RTRACE(fmt, args...) \
-    do { if (verbose >= 3) gprintf(fmt, ##args); } while (0)
-#define VTRACE(fmt, args...) \
-    do { if (verbose >= 4) gprintf(fmt, ##args); } while (0)
 #define DEBUG(level, fmt, args...) \
     do { if ((debug & (level))) gprintf(fmt, ##args); } while (0)
 
 #else /*HAVE_VARIADIC_DEFINE*/
 
-/* Using these without assignment causes compiler warnings on some systems. */
-extern int trace_dummy;
-# ifdef HAVE_VISUAL_C
-#  define TRACE_ASSIGN 
-# else
-#  define TRACE_ASSIGN trace_dummy =
-# endif
-
-#define TRACE  TRACE_ASSIGN (verbose) && gprintf
-#define RTRACE TRACE_ASSIGN (verbose >= 3) && gprintf
-#define VTRACE TRACE_ASSIGN (verbose >= 4) && gprintf
 /* if debug == 0, then can skip the function call. */
-#define DEBUG  TRACE_ASSIGN (debug) && DEBUG_func
-
+#define DEBUG  (!(debug)) ? (void)0 : (void)DEBUG_func
 int DEBUG_func(int level, const char *fmt, ...);
 
 #endif  /*HAVE_VARIADIC_DEFINE*/
@@ -376,6 +383,7 @@ int DEBUG_func(int level, const char *fmt, ...);
 #define EXAMINE_DRAGONS             4
 #define EXAMINE_OWL_REASONS         5
 #define EXAMINE_INITIAL_INFLUENCE2  6
+#define FULL_EXAMINE_DRAGONS        7
 
 #define EXAMINE_ALL                 99
 
@@ -390,8 +398,6 @@ void silent_examine_position(int color, int how_much);
 
 
 /* These are mostly used for GTP examination. */
-void reset_life_node_counter(void);
-int get_life_node_counter(void);
 void reset_owl_node_counter(void);
 int get_owl_node_counter(void);
 void reset_reading_node_counter(void);
@@ -421,6 +427,7 @@ void remove_stone(int pos);
 void play_move(int pos, int color);
 int undo_move(int n);
 int get_last_move(void);
+int get_last_player(void);
 int get_last_opponent_move(int color);
 int is_pass(int pos);
 int is_legal(int pos, int color);
@@ -428,15 +435,13 @@ int is_suicide(int pos, int color);
 int is_illegal_ko_capture(int pos, int color);
 int trymove(int pos, int color, const char *message, int str, 
 	    int komaster, int kom_pos);
-int semeai_trymove(int pos, int color, const char *message, 
-		   int str1, int str2, int owl_phase, int value);
 int tryko(int pos, int color, const char *message, 
 	  int komaster, int kom_pos);
 void popgo(void);
 
 /* utils.c */
-void change_matcher_status(int dr, int status);
-void who_wins(int color, FILE* outfile);
+void change_dragon_status(int dr, int status);
+void who_wins(int color, FILE *outfile);
 
 /* high-level routine to generate the best move for the given color */
 int genmove(int *i, int *j, int color);
@@ -448,14 +453,15 @@ int aftermath_final_status(int color, int pos);
 
 /* Basic information gathering. */
 /* worm.c */
-void make_worms(int save_verbose);
+void make_worms(void);
+void compute_worm_influence(void);
 
 /* dragon.c */
-void make_dragons(int color, int stop_before_owl, int save_verbose);
+void make_dragons(int color, int stop_before_owl);
 void initialize_dragon_data(void);
 void show_dragons(void);
+int crude_status(int pos);
 int dragon_status(int pos);
-int matcher_status(int pos);
 int same_dragon(int dr1, int dr2);
 
 /* moyo functions */
@@ -466,27 +472,29 @@ void prepare_pattern_profiling(void);
 void report_pattern_profiling(void);
 
 /* sgffile.c */
-void sgffile_move_made(int i, int j, int color, int value);
-void sgffile_put_stone(int i, int j, int color);
+void sgffile_add_debuginfo(SGFNode *node, int value);
+void sgffile_output(SGFTree *tree);
 
-int sgffile_open_file(const char *);
-int sgffile_close_file(void);
-int sgffile_write_gameinfo(Gameinfo *gameinfo, const char *);
-void sgffile_write_comment(const char *comment);
-void sgffile_write_circle_mark(int i, int j);
-void sgffile_printboard(int next);
+void sgffile_printsgf(int color_to_play, const char *filename);
+void sgffile_printboard(SGFTree *tree);
 void sgffile_recordboard(SGFNode *node);
 
 /* sgfdecide.c */
-void decide_string(int pos, const char *sgf_output);
-void decide_connection(int apos, int bpos, const char *sgf_output);
-void decide_dragon(int pos, const char *sgf_output);
-void decide_semeai(int apos, int bpos, const char *sgf_output);
-void decide_tactical_semeai(int apos, int bpos, const char *sgf_output);
-void decide_position(int color, const char *sgf_output);
-void decide_eye(int pos, const char *sgf_output);
-void decide_combination(int color, const char *sgf_output);
+void decide_string(int pos);
+void decide_connection(int apos, int bpos);
+void decide_owl(int pos);
+void decide_dragon_data(int pos);
+void decide_semeai(int apos, int bpos);
+void decide_tactical_semeai(int apos, int bpos);
+void decide_position(int color);
+void decide_eye(int pos);
+void decide_combination(int color);
+void decide_surrounded(int pos);
+void decide_oracle(Gameinfo *gameinfo, char *infilename, char *untilstring);
 
+/*oracle.c*/
+void dismiss_oracle(void);
+void oracle_clear_board(int boardsize);
 
 #endif  /* _GNUGO_H_ */
 
