@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -26,21 +27,51 @@
 
 #include "liberty.h"
 
-/* default attenuation coefficient. */
+
+/* STEPHANE NICOLET's modification : playing for influence in the fuseki */
+
+#if COSMIC_GNUGO
+
+
+/* We use much more influence than the defaults attenuation coefficients !
+ * The "TERR_.."-values are used in the influence computations used
+ * for territory evaluation. (initial_influence with dragons_known,
+ * move_influence)
+ */
+#define DEFAULT_ATTENUATION \
+       	(cosmic_importance * 2.7  + (1.0 - cosmic_importance) * 3.0)
+#define TERR_DEFAULT_ATTENUATION \
+	(cosmic_importance * 2.15 + (1.0 - cosmic_importance) * 2.4)
+
+/* Extra damping coefficient for spreading influence diagonally. */
+#define DIAGONAL_DAMPING \
+	(cosmic_importance * 2.5 + (1.0 - cosmic_importance) * 2.0)
+#define TERR_DIAGONAL_DAMPING \
+	(cosmic_importance * 2.5 + (1.0 - cosmic_importance) * 1.7)
+
+
+#else
+
+
+/* Default attenuation coefficient.
+ * The "TERR_.."-values are used in the influence computations used
+ * for territory evaluation. (initial_influence with dragons_known,
+ * move_influence)
+ */
 #define DEFAULT_ATTENUATION 3.0
-#define EXP_DEFAULT_ATTENUATION 2.4
+#define TERR_DEFAULT_ATTENUATION 2.4
 
 /* Extra damping coefficient for spreading influence diagonally. */
 #define DIAGONAL_DAMPING 2.0
-#define EXP_DIAGONAL_DAMPING 1.7
+#define TERR_DIAGONAL_DAMPING 1.7
+
+#endif
+
+
+
 
-/* Default strength of the influence from a stone. May be lowered if
- * it is unsafe.
- */
-#define DEFAULT_STRENGTH 100.0
 
 /* Smallest amount of influence that we care about distributing. */
-/*#define INFLUENCE_CUTOFF 0.02*/
 #define INFLUENCE_CUTOFF 0.02
 
 /* Value in delta_territory_cache indicating that the value has not
@@ -48,13 +79,9 @@
  */
 #define NOT_COMPUTED (-2.0 * MAX_BOARD * MAX_BOARD)
 
-/* Values for the float working area w when used only for marking. */
-#define UNMARKED 0.0
-#define MARKED 1.0
-
 /* Territory, moyo, and area are segmented into connected components
  * and given a number from the same series. These values are used in
- * region_type[][].
+ * region_type[].
  */
 #define WHITE_REGION    0
 #define BLACK_REGION    1
@@ -86,40 +113,78 @@ struct intrusion_data
 
 struct influence_data
 {
-  float white_influence[MAX_BOARD][MAX_BOARD]; /* Accumulated influence. */
-  float black_influence[MAX_BOARD][MAX_BOARD]; /* Accumulated influence. */
-  int p[MAX_BOARD][MAX_BOARD];           /* Working copy of board array. */
-  float white_strength[MAX_BOARD][MAX_BOARD];  /* Strength of influence source. */
-  float black_strength[MAX_BOARD][MAX_BOARD];  /* Strength of influence source. */
-  float white_attenuation[MAX_BOARD][MAX_BOARD]; /* Attenuation. */
-  float black_attenuation[MAX_BOARD][MAX_BOARD]; /* Attenuation. */
-  float white_permeability[MAX_BOARD][MAX_BOARD];
-  float black_permeability[MAX_BOARD][MAX_BOARD];
+  char safe[BOARDMAX];
 
-  int territory_segmentation[MAX_BOARD][MAX_BOARD];
-  int moyo_segmentation[MAX_BOARD][MAX_BOARD];
-  int area_segmentation[MAX_BOARD][MAX_BOARD];
+  float white_influence[BOARDMAX]; 	/* Accumulated influence. */
+  float black_influence[BOARDMAX]; 	/* Accumulated influence. */
+  float white_strength[BOARDMAX];  	/* Strength of influence source. */
+  float black_strength[BOARDMAX];  	/* Strength of influence source. */
+  float white_attenuation[BOARDMAX]; 
+  float black_attenuation[BOARDMAX];
+  float white_permeability[BOARDMAX];
+  float black_permeability[BOARDMAX];
+
+  int territory_segmentation[BOARDMAX];
+  int moyo_segmentation[BOARDMAX];
+  int area_segmentation[BOARDMAX];
   int region_type[MAX_REGIONS];
   int region_size[MAX_REGIONS];
+  float region_territorial_value[MAX_REGIONS];
   int number_of_regions;
 
-  float territory_value[MAX_BOARD][MAX_BOARD];
-  int non_territory[MAX_BOARD][MAX_BOARD];
+  int is_territorial_influence; /* 0 only if computing escape_influence.*/
+
+  float territory_value[BOARDMAX];
+  int non_territory[BOARDMAX];
+  int captured;
 
   int color_to_move; /* Which color is in turn to move. */
   
-  float w[MAX_BOARD][MAX_BOARD];         /* Working area. */
-  int queuei[MAX_BOARD * MAX_BOARD];     /* Points receiving influence. */
-  int queuej[MAX_BOARD * MAX_BOARD];
+  int queue[MAX_BOARD * MAX_BOARD];     /* Points receiving influence. */
 
   int intrusion_counter;
   struct intrusion_data intrusions[MAX_INTRUSIONS];
+
+  int id;
 };
 
 /* Typedef for pointer to either of the functions whose_territory(),
  * whose_moyo(), and whose_area().
  */
-typedef int (*owner_function_ptr)(struct influence_data *q, int m, int n);
+typedef int (*owner_function_ptr)(const struct influence_data *q, int pos);
+
+/* Used for tuning game advancement algorythm */
+#define WEIGHT_TERRITORY 10
+#define WEIGHT_MOYO       3
+#define WEIGHT_AREA       1
+
+
+
+#if COSMIC_GNUGO
+
+/* cosmic_importance is a number between 0.0 and 1.0 ;
+ * when cosmic_importance is 0.0, the default influence
+ * values are used; when cosmic_importance is 1.0, GNU Go
+ * will try to play an influence-oriented fuseki by 
+ * over-estimatingthe potential territory values of moyos.
+ * In the current implementation, cosmic_importance decreases 
+ * slowly for 19*19 games from 1.0 at move 4 to 0.0 at move 120.
+ */
+float cosmic_importance;
+
+
+/* Used in the whose_moyo() function */
+struct moyo_determination_data
+{
+  float influence_balance;
+  float my_influence_minimum;
+  float opp_influence_maximum;
+};
+
+
+#endif
+
+
 
 /*
  * Local Variables:
@@ -127,3 +192,4 @@ typedef int (*owner_function_ptr)(struct influence_data *q, int m, int n);
  * c-basic-offset: 2
  * End:
  */
+
