@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -28,7 +29,7 @@
 #include "liberty.h"
 #include "clock.h"
 
-#include <gg_utils.h>
+#include "gg_utils.h"
 
 /*
  * Initialize the gnugo engine. This needs to be called 
@@ -36,15 +37,20 @@
  */
 
 void
-init_gnugo(float memory)
+init_gnugo(float memory, unsigned int seed)
 {
+  set_random_seed(seed);
   reading_cache_init(memory * 1024 * 1024);
   clear_board();
 
+  transformation_init();
   dfa_match_init();
 #if EXPERIMENTAL_READING
   tree_match_init();
 #endif
+
+  clear_approxlib_cache();
+  clear_accuratelib_cache();
 }
 
 
@@ -59,6 +65,10 @@ gnugo_clear_board(int boardsize)
   gg_assert(MIN_BOARD <= boardsize && boardsize <= MAX_BOARD);
   board_size = boardsize;
   clear_board();
+#if 0
+  if (metamachine && oracle_exists)
+    oracle_clear_board(boardsize);
+#endif
 }
 
 /* Set the komi */
@@ -98,7 +108,14 @@ gnugo_is_pass(int i, int j)
 void
 gnugo_play_move(int i, int j, int color)
 {
+#if ORACLE
+  if (oracle_exists)
+    oracle_play_move(POS(i, j), color);
+  else
+    play_move(POS(i, j), color);
+#else
   play_move(POS(i, j), color);
+#endif
   clock_push_button(color);
 }
 
@@ -234,12 +251,12 @@ gnugo_is_suicide(int i, int j, int color)
 }
 
 
-/* Interface to placehand. Sets up handicap pieces and
+/* Interface to placehand. Sets up handicap stones and
  * returns the number of placed handicap stones. */
 int
 gnugo_placehand(int handicap)
 {
-  return placehand(handicap);
+  return place_fixed_handicap(handicap);
 }
 
 
@@ -251,12 +268,12 @@ gnugo_recordboard(SGFNode *root)
 }
 
 /* Interface to placehand. Sets up handicap stones and
- * returns the number of placed handicap stones, updating the sgf file
+ * returns the number of placed handicap stones, updating the sgf file.
  */
 int
 gnugo_sethand(int handicap, SGFNode *node)
 {
-  int stones = placehand(handicap);
+  int stones = place_fixed_handicap(handicap);
   sgffile_recordboard(node);
   return stones;
 }
@@ -322,7 +339,8 @@ gnugo_who_wins(int color, FILE *outfile)
 float
 gnugo_estimate_score(float *upper, float *lower)
 {
-  return estimate_score(lower, upper);
+  silent_examine_position(WHITE, EXAMINE_DRAGONS);
+  return estimate_score(upper, lower);
 }
 
 
@@ -338,7 +356,7 @@ gnugo_examine_position(int color, int how_much)
 
 /* Report the komi. */
 
-int 
+float
 gnugo_get_komi()
 {
   return komi;
@@ -388,9 +406,6 @@ gameinfo_clear(Gameinfo *ginfo, int boardsize, float komi)
 
   /* Info relevant to the computer player. */
   ginfo->computer_player = WHITE; /* Make an assumption. */
-
-  ginfo->outfilename[0] = '\0';
-  ginfo->outfile = NULL;
 }
 
 
@@ -447,7 +462,7 @@ gameinfo_load_sgfheader(Gameinfo *gameinfo, SGFNode *head)
   if (handicap > bsize * bsize - 1 || handicap < 0) {
     fprintf(stderr, " Handicap HA[%d] is unreasonable.\n", handicap);
     fprintf(stderr, " Modify SGF file.\n");
-    exit (1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -461,9 +476,7 @@ void
 gameinfo_play_move(Gameinfo *ginfo, int i, int j, int color)
 {
   gnugo_play_move(i, j, color);
-  sgftreeAddPlay(&ginfo->game_record, 0, color, i, j);
-
-  sgffile_move_made(i, j, color, 0);
+  sgftreeAddPlay(&ginfo->game_record, color, i, j);
 
   ginfo->to_move = OTHER_COLOR(color);
 }
@@ -475,14 +488,14 @@ gameinfo_play_move(Gameinfo *ginfo, int i, int j, int color)
  *
  * Returns the color of the next move to be made.
  *
- * Head is a sgf tree. 
+ * Head is an sgf tree. 
  * Untilstr is an optional string of the form either 'L12' or '120'
  * which tells it to stop playing at that move or move-number.
  * When debugging, this is the location of the move being examined.
  */
 
 int
-gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
+gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 			  const char *untilstr, int orientation)
 {
   int bs, handicap;
@@ -493,15 +506,13 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
   int until = 9999;
   int addstone = 0;          /* handicap stone detector */
   
-  SGFNode *node;
-  
-  if (!sgfGetFloatProperty(head, "KM", &komi)) {
+  if (!sgfGetFloatProperty(tree->root, "KM", &komi)) {
     if (gameinfo->handicap == 0)
       komi = 5.5;
     else
       komi = 0.5;
   }
-  if (!sgfGetIntProperty(head, "SZ", &bs))
+  if (!sgfGetIntProperty(tree->root, "SZ", &bs))
     bs = 19;
   gnugo_clear_board(bs);
   gnugo_set_komi(komi);
@@ -523,7 +534,7 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
     }
   }
   
-  if (sgfGetIntProperty(head, "HA", &handicap) && handicap > 1) {
+  if (sgfGetIntProperty(tree->root, "HA", &handicap) && handicap > 1) {
     gameinfo->handicap = handicap;
     next = WHITE;
   }
@@ -534,11 +545,11 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
    *
    * The sgf routines map AB[aa][bb][cc] into AB[aa]AB[bb]AB[cc]
    */
-  for (node = head; node; node = node->child) {
+  for (tree->lastnode = NULL; sgftreeForward(tree);) {
     SGFProperty *prop;
     int i, j;
       
-    for (prop = node->props; prop; prop = prop->next) {
+    for (prop = tree->lastnode->props; prop; prop = prop->next) {
       DEBUG(DEBUG_LOADSGF, "%c%c[%s]\n", 
 	    prop->name & 0xff, (prop->name >> 8), prop->value);
       switch (prop->name) {
@@ -551,16 +562,22 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
 	 * placed on the board.
 	 */
 	rotate(i, j, &i, &j, board_size, orientation);
-	gnugo_add_stone(i, j, BLACK);
-	sgffile_put_stone(i, j, BLACK);
+	if (BOARD(i, j) != EMPTY)
+	  gprintf("Illegal SGF! attempt to add a stone at occupied point %m\n",
+		  i, j);
+	else
+	  gnugo_add_stone(i, j, BLACK);
 	addstone = 1;
 	break;
 	      
       case SGFAW:
 	get_moveXY(prop, &i, &j, board_size);
 	rotate(i, j, &i, &j, board_size, orientation);
-	gnugo_add_stone(i, j, WHITE);
-	sgffile_put_stone(i, j, WHITE);
+	if (BOARD(i, j) != EMPTY)
+	  gprintf("Illegal SGF! attempt to add a stone at occupied point %m\n",
+		  i, j);
+	else
+	  gnugo_add_stone(i, j, WHITE);
 	addstone = 1;
 	break;
 	      
@@ -568,7 +585,7 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
 	/* following really should not be needed for proper sgf file */
 	if (movenum != 0 && !addstone) {
 	  gnugo_sethand(gameinfo->handicap, 0);
-	  sgfOverwritePropertyInt(head, "HA", handicap);
+	  sgfOverwritePropertyInt(tree->root, "HA", gameinfo->handicap);
 	}
 
 	/* Due to a bad comment in the SGF FF3 definition (in the
@@ -590,25 +607,36 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
 	/* following really should not be needed for proper sgf file */
 	if (movenum != 0 && !addstone) {
 	  gnugo_sethand(gameinfo->handicap, 0);
-	  sgfOverwritePropertyInt(head, "HA", handicap);
+	  sgfOverwritePropertyInt(tree->root, "HA", gameinfo->handicap);
 	}
 
 	if (movenum == until - 1) {
 	  gameinfo->to_move = next;
+	  /* go back so that variant will be added to the proper node */
+	  sgftreeBack(tree);
 	  return next;
 	}
 	      
 	if (get_moveXY(prop, &i, &j, board_size))
 	  if (i == untilm && j == untiln) {
 	    gameinfo->to_move = next;
+            /* go back so that variant will be added to the proper node */
+	    sgftreeBack(tree);
 	    return next;
 	  }
 
 	rotate(i, j, &i, &j, board_size, orientation);
-	gnugo_play_move(i, j, next);
-	sgffile_move_made(i, j, next, 0);
-	next = OTHER_COLOR(next);
-	      
+	if ((ON_BOARD2(i, j) && board[POS(i, j)] == EMPTY)
+	    || (i == -1 && j == -1)) {
+	  gnugo_play_move(i, j, next);
+	  next = OTHER_COLOR(next);
+	}
+	else {
+	  gprintf("WARNING: Move off board or on occupied position found in sgf-file.\n");
+	  gprintf("Move at %m ignored, trying to proceed.\n", i, j);
+	  return next;
+	}
+
 	break;
 
       case SGFIL:
@@ -645,9 +673,9 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
 /* Same as previous function, using standard orientation */
 
 int
-gameinfo_play_sgftree(Gameinfo *gameinfo, SGFNode *head, const char *untilstr)
+gameinfo_play_sgftree(Gameinfo *gameinfo, SGFTree *tree, const char *untilstr)
 {
-  return gameinfo_play_sgftree_rot(gameinfo, head, untilstr, 0);
+  return gameinfo_play_sgftree_rot(gameinfo, tree, untilstr, 0);
 }
 
 
