@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,7 +25,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
 #include "liberty.h"
 #include "gg_utils.h"
@@ -50,7 +50,6 @@ static int totals[6];
 static void
 clear_profile(struct pattern *pattern)
 {
-
   for (; pattern->patn; ++pattern) {
     pattern->hits = 0;
     pattern->reading_nodes = 0;
@@ -63,7 +62,7 @@ clear_profile(struct pattern *pattern)
 /* Print profiling information for one pattern struct array. */
 static void
 print_profile(struct pattern *pattern, int *total_hits,
-			   int *total_nodes, int *total_dfa_hits)
+	      int *total_nodes, int *total_dfa_hits)
 {
   for (; pattern->patn; ++pattern)
     if (pattern->hits > 0) {
@@ -98,6 +97,7 @@ prepare_pattern_profiling()
   clear_profile(owl_vital_apat_db.patterns);
   clear_profile(owl_defendpat_db.patterns);
   clear_profile(fusekipat_db.patterns);
+  clear_profile(oracle_db.patterns);
 #else
   fprintf(stderr,
 	  "Warning, no support for pattern profiling in this binary.\n");
@@ -128,6 +128,7 @@ report_pattern_profiling()
   print_profile(owl_vital_apat_db.patterns, &hits, &nodes, &dfa_hits);
   print_profile(owl_defendpat_db.patterns, &hits, &nodes, &dfa_hits);
   print_profile(fusekipat_db.patterns, &hits, &nodes, &dfa_hits);
+  print_profile(oracle_db.patterns, &hits, &nodes, &dfa_hits);
   fprintf(stderr, "------ ---------\n");
   fprintf(stderr, "%6d, %6d %9d\n", dfa_hits, hits, nodes);
 #endif
@@ -144,34 +145,26 @@ report_pattern_profiling()
 
 static void fixup_patterns_for_board_size(struct pattern *pattern);
 static void prepare_for_match(int color);
-static void do_matchpat(int m, int n, matchpat_callback_fn_ptr callback,
+static void do_matchpat(int anchor, matchpat_callback_fn_ptr callback,
 			int color, struct pattern *database,
 			void *callback_data, char goal[BOARDMAX]);
 static void matchpat_loop(matchpat_callback_fn_ptr callback, 
 			  int color, int anchor,
 			  struct pattern_db *pdb, void *callback_data,
 			  char goal[BOARDMAX], int anchor_in_goal);
-void transform(int i, int j, int *ti, int *tj, int trans);
-
-/* The pattern matcher still works in 2D and has a private copy of the
- * board here.
- */
-static Intersection p[MAX_BOARD][MAX_BOARD];
-static void board_to_p(void);
-
-static int matchpat_call_level = 0;
-static Intersection saved_p[MAX_BOARD][MAX_BOARD];
+void transform2(int i, int j, int *ti, int *tj, int trans);
 
 /* Precomputed tables to allow rapid checks on the piece at
  * the board. This table relies on the fact that color is
  * 1 or 2.
  *
- * For pattern element i,  require  (p[m][n] & andmask[i]) == valmask[i]
+ * For pattern element i,  require  (board[pos] & andmask[i]) == valmask[i]
  *
- * .XO) For i=0,1,2,  p[m][n] & 3 is a no-op, so we check p[][] == valmask
- * x)   For i=3, we are checking that p[][] is not color, so AND color and
- *      we get 0 for either empty or OTHER_COLOR, but color if it contains
- *      color
+ * .XO) For i=0,1,2, board[pos] & 3 is a no-op, so we check board[pos]
+ *	== valmask
+ * x)   For i=3, we are checking that board[pos] is not color, so AND
+ *	color and we get 0 for either empty or OTHER_COLOR, but color
+ *	if it contains color
  * o)   Works the other way round for checking it is not X.
  *
  *
@@ -192,7 +185,7 @@ static const int val_mask[2][8] = {
 
 /* and a table for checking classes quickly
  * class_mask[status][color] contains the mask to look for in class.
- * ie. if  pat[r].class & class_mask[dragon[x][y].status][p[x][y]]
+ * ie. if  pat[r].class & class_mask[dragon[pos].status][board[pos]]
  * is not zero then we reject it
  * Most elements if class_mask[] are zero - it is a sparse
  * matrix containing
@@ -232,7 +225,6 @@ static unsigned int class_mask[MAX_DRAGON_STATUS][3];
 static void
 fixup_patterns_for_board_size(struct pattern *pattern)
 {
-
   for (; pattern->patn; ++pattern)
     if (pattern->edge_constraints != 0) {
 
@@ -276,7 +268,6 @@ fixup_patterns_for_board_size(struct pattern *pattern)
 	if (pattern->minj > pattern->maxj - (board_size-1))
 	  pattern->minj = pattern->maxj - (board_size-1);
     }
-
 }
 
 
@@ -304,74 +295,46 @@ prepare_for_match(int color)
 }
 
 
-
-/* Compute the transform of (i, j) under transformation number trans.
- * *ti and *tj point to the transformed coordinates.
- * ORDER MATTERS : see texinfo documentation for details
- *
- * There is a copy of this table in mkpat.c
- */
-
-const int transformations[8][2][2] = {
-  {{ 1,  0}, { 0,  1}}, /* a - identity transformation matrix */
-  {{ 0,  1}, {-1,  0}}, /* g - rotate 270 counter-clockwise */
-  {{-1,  0}, { 0, -1}}, /* d - rotate 180 */
-  {{ 0, -1}, { 1,  0}}, /* f - rotate 90 counter-clockwise */
-  {{ 0, -1}, {-1,  0}}, /* h - rotate 90 and invert */
-  {{-1,  0}, { 0,  1}}, /* b - flip left */
-  {{ 0,  1}, { 1,  0}}, /* e - rotate 90 and flip left */
-  {{ 1,  0}, { 0, -1}}  /* c - invert */
-};
-
-
-/* Functional version for completeness. Prefer the TRANSFORM macro
- * in patterns.h.
+/* Functional version for completeness. Prefer the TRANSFORM2 macro
+ * in liberty.h.
  */
 
 void 
-transform(int i, int j, int *ti, int *tj, int trans)
+transform2(int i, int j, int *ti, int *tj, int trans)
 {
-  TRANSFORM(i, j, ti, tj, trans);
-}
-
-
-/* Compute the point offset by (di, dj), relative to a base point (basepos), 
- * taking into account a transformation.
- */
-
-int
-offset(int i, int j, int basepos, int trans)
-{
-  int ui, uj;
-  TRANSFORM(i, j, &ui, &uj, trans);
-  return basepos + DELTA(ui, uj);
+  TRANSFORM2(i, j, ti, tj, trans);
 }
 
 
 /*
- * Try all the patterns in the given array at (m, n). Invoke the
+ * Try all the patterns in the given array at (anchor). Invoke the
  * callback for any that matches. Classes X,O,x,o are checked here. It
  * is up to the callback to process the other classes, and any helper
  * or autohelper functions.
  *
- * If the support of goal[MAX_BOARD][MAX_BOARD] is a subset of the board,
- * patterns are rejected which do not involve this dragon. If goal is a 
- * null pointer, this parameter is ignored.
+ * If the support of goal[BOARDMAX] is a subset of the board, patterns
+ * are rejected which do not involve this dragon. If goal is a null
+ * pointer, this parameter is ignored.
  */
 
 static void
-do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
+do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
 	    struct pattern *pattern, void *callback_data,
 	    char goal[BOARDMAX]) 
 {
-  const int anchor_test = p[m][n] ^ color;  /* see below */
+  const int anchor_test = board[anchor] ^ color;  /* see below */
+  int m = I(anchor);
+  int n = J(anchor);
   int merged_val;
 
   /* Basic sanity checks. */
-  ASSERT_ON_BOARD2(m, n);
+  ASSERT_ON_BOARD1(anchor);
 
   /* calculate the merged value around [m][n] for the grid opt */
   {
+    /* FIXME: Convert this to 2D (using delta[]) but be aware that you'll
+     *	      also need to make corresponding changes in mkpat.c!
+     */
     int i, j;
     int shift = 30;
 
@@ -381,7 +344,7 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	unsigned int this;
 	if (!ON_BOARD2(i, j))
 	  this = 3;
-	else if ((this = p[i][j]) == 0)
+	else if ((this = BOARD(i, j)) == 0)
 	  continue;
 	else if (color == 2)
 	  this = OTHER_COLOR(this);
@@ -409,13 +372,13 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
        * pattern in one go.
        *
        * Patterns are always drawn from O perspective in .db,
-       * so p[m][n] is 'color' if the pattern is anchored
+       * so board[pos] is 'color' if the pattern is anchored
        * at O, or 'other' for X.
        * Since we require that this flag contains 3 for
        * anchored_at_X, we can check that
-       *   p[m][n] == (color ^ anchored_at_X)
+       *   board[pos] == (color ^ anchored_at_X)
        * which is equivalent to
-       *   (p[m][n] ^ color) == anchored_at_X)
+       *   (board[pos] ^ color) == anchored_at_X)
        * and the LHS is something we precomputed.
        */
 
@@ -479,8 +442,8 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	{
 	  int mi, mj, xi, xj;
 	  
-	  TRANSFORM(pattern->mini, pattern->minj, &mi, &mj, ll);
-	  TRANSFORM(pattern->maxi, pattern->maxj, &xi, &xj, ll);
+	  TRANSFORM2(pattern->mini, pattern->minj, &mi, &mj, ll);
+	  TRANSFORM2(pattern->maxi, pattern->maxj, &xi, &xj, ll);
 
 	  /* {min,max}{i,j} are the appropriate corners of the original
 	   * pattern, Once we transform, {m,x}{i,j} are still corners,
@@ -489,11 +452,12 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	   * to just test enough cases to be safe.
 	   */
 
-	  DEBUG(DEBUG_MATCHER, "---\nconsidering pattern '%s', rotation %d at %m. Range %d,%d -> %d,%d\n",
-		pattern->name, ll, m, n, mi, mj, xi, xj);
+	  DEBUG(DEBUG_MATCHER, 
+		"---\nconsidering pattern '%s', rotation %d at %1m. Range %d,%d -> %d,%d\n",
+		pattern->name, ll, anchor, mi, mj, xi, xj);
 
 	  /* now do the range-check */
-	  if (!ON_BOARD2(m+mi, n+mj) || !ON_BOARD2(m+xi, n+xj))
+	  if (!ON_BOARD2(m + mi, n + mj) || !ON_BOARD2(m + xi, n + xj))
 	    continue;  /* out of range */
 	}
 
@@ -505,28 +469,24 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	found_goal = 0;
 	found_nongoal = 0;
 	for (k = 0; k < pattern->patlen; ++k) { /* match each point */
-	  int x, y; /* absolute coords of (transformed) pattern element */
-
+	  int pos; /* absolute coords of (transformed) pattern element */
 	  int att = pattern->patn[k].att;  /* what we are looking for */
 
-
 	  /* Work out the position on the board of this pattern element. */
 
 	  /* transform pattern real coordinate... */
-	  TRANSFORM(pattern->patn[k].x, pattern->patn[k].y, &x, &y, ll);
-	  x += m;
-	  y += n;
+	  pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
 
-	  ASSERT_ON_BOARD2(x, y);
+	  ASSERT_ON_BOARD1(pos);
 
-	  /* ...and check that p[x][y] matches (see above). */
-	  if ((p[x][y] & and_mask[color-1][att]) != val_mask[color-1][att])
+	  /* ...and check that board[pos] matches (see above). */
+	  if ((board[pos] & and_mask[color-1][att]) != val_mask[color-1][att])
 	    goto match_failed;
 
-	  if (goal != NULL && p[x][y] != EMPTY) {
-	    if (goal[POS(x, y)])
+	  if (goal != NULL && board[pos] != EMPTY) {
+	    if (goal[pos])
 	      found_goal = 1;
-	    else if (p[x][y] == color)
+	    else if (board[pos] == color)
 	      found_nongoal = 1;
 	  }
 	  
@@ -534,7 +494,7 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	   * attributes - see patterns.db and above.
 	   */
 	  if ((pattern->class
-	       & class_mask[dragon[POS(x, y)].matcher_status][p[x][y]]) != 0)
+	       & class_mask[dragon[pos].status][board[pos]]) != 0)
 	    goto match_failed; 
 	  
 	} /* loop over elements */
@@ -545,7 +505,7 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	/* Make sure the grid optimisation wouldn't have 
            rejected this pattern */
 	ASSERT2((merged_val & pattern->and_mask[ll])
-	       == pattern->val_mask[ll], m, n);
+		== pattern->val_mask[ll], m, n);
 
 #endif /* we don't trust the grid optimisation */
 
@@ -573,7 +533,7 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 #endif
 	
 	/* A match!  - Call back to the invoker to let it know. */
-	callback(m, n, color, pattern, ll, callback_data);
+	callback(anchor, color, pattern, ll, callback_data);
 
 #if PROFILE_PATTERNS
 	pattern->reading_nodes += stats.nodes - nodes_before;
@@ -582,8 +542,8 @@ do_matchpat(int m, int n, matchpat_callback_fn_ptr callback, int color,
 	/* We jump to here as soon as we discover a pattern has failed. */
       match_failed:
 	DEBUG(DEBUG_MATCHER, 
-	      "end of pattern '%s', rotation %d at %m\n---\n", 
-	      pattern->name, ll, m, n);
+	      "end of pattern '%s', rotation %d at %1m\n---\n", 
+	      pattern->name, ll, anchor);
 	 
       } while (++ll < end_transformation); /* ll loop over symmetries */
     } /* if not rejected by maxwt */
@@ -609,14 +569,13 @@ matchpat_loop(matchpat_callback_fn_ptr callback, int color, int anchor,
 	      struct pattern_db *pdb, void *callback_data,
 	      char goal[BOARDMAX], int anchor_in_goal) 
 {
-  int i, j;
-
-  for (i = 0; i != board_size; i++)
-    for (j = 0; j != board_size; j++)
-      if (p[i][j] == anchor
-          && (!anchor_in_goal || goal[POS(i,j)] != 0))
-	do_matchpat(i, j, callback, color, 
-		    pdb->patterns, callback_data, goal);
+  int pos;
+  
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (board[pos] == anchor && (!anchor_in_goal || goal[pos] != 0))
+      do_matchpat(pos, callback, color, pdb->patterns,
+		  callback_data, goal);
+  }
 }
 
 #if EXPERIMENTAL_READING
@@ -663,14 +622,14 @@ tree_matchpat_loop(matchpat_callback_fn_ptr callback,
 
   for (i = 0; i != board_size; i++)
     for (j = 0; j != board_size; j++)
-      if (board[POS(i,j)] == anchor
+      if (BOARD(i,j) == anchor
           && (!anchor_in_goal || goal[POS(i,j)] != 0))
 	tree_do_matchpat(i, j, callback, color, 
 			 pdb, callback_data, goal, anchor_in_goal);
 
 }
 
-/* possibly cheeper to pass a little less recursively.
+/* Possibly cheaper to pass a little less recursively.
  * Note: This could even just be static data, unless the
  * tree pattern matcher is itself used recursively.*/
 struct rec_data {
@@ -689,14 +648,14 @@ do_tree_matchpat_rec(int color, int m, int n, int goal_found,
                      struct tree_node_list *tnl, 
                      struct rec_data *pdata)
 {
-  if (0 && !tnl) {
+  if (0 && !tnl)
     return;
-  }
+
   tnl = tnl->next;
   while (tnl) {
     struct tree_node *node = &(tnl->node);
     int x = m + node->x;
-    int y = n + node->y;;
+    int y = n + node->y;
     if (ON_BOARD2(x,y)) {
       int att = node->att;
       int point_color = BOARD(x,y);
@@ -711,8 +670,8 @@ do_tree_matchpat_rec(int color, int m, int n, int goal_found,
 	      &(pdata->database->patterns[match->patnum]);
             int ll = match->orientation;
 	    int mi, mj, xi, xj;
-	    TRANSFORM(pattern->mini, pattern->minj, &mi, &mj, ll);
-	    TRANSFORM(pattern->maxi, pattern->maxj, &xi, &xj, ll);
+	    TRANSFORM2(pattern->mini, pattern->minj, &mi, &mj, ll);
+	    TRANSFORM2(pattern->maxi, pattern->maxj, &xi, &xj, ll);
 	    /* now do the range-check */
             if (!goal_found
                 || !ON_BOARD2(m+mi, n+mj) 
@@ -722,11 +681,11 @@ do_tree_matchpat_rec(int color, int m, int n, int goal_found,
 	    else {
               if (0) {
                 gprintf("  P[%s, %d] matches at %m)\n",
-			pattern->name, 
-			match->orientation, x, y);
+			pattern->name, match->orientation, x, y);
               }
               /* A match! */
-              pdata->callback(m, n, color, pattern, ll, pdata->callback_data);
+              pdata->callback(POS(m, n), color, pattern, ll,
+			      pdata->callback_data);
             }
 
             match = match->next;
@@ -774,9 +733,12 @@ tree_do_matchpat(int m, int n, matchpat_callback_fn_ptr callback,
 
 /* The tree data structure is output with raw integer offsets
  * relative to a single array of tree_node_list and match_node
- * elements.  These offsets need to added to the actual base
+ * elements.  These offsets need to be added to the actual base
  * address of the list of elements for the pointers to be
  * meaningful.
+ *
+ * FIXME: This code is not portable. On certain platforms the size of
+ *        a pointer and the size of an int differ.
  */
 void 
 tree_initialize_pointers(struct tree_node_list *tnl,
@@ -805,16 +767,11 @@ tree_initialize_pointers(struct tree_node_list *tnl,
 
 #endif
 
-#
 
 /**************************************************************************/
 /* DFA matcher:                                                           */
 /**************************************************************************/
 
-/* If DFA_SORT, all matched patterns are sorted and checked 
- * in the same order as the standard scheme */
-#define DFA_SORT 0
-
 /* Set this to show the dfa board in action */
 /* #define DFA_TRACE 1 */
 
@@ -822,25 +779,26 @@ tree_initialize_pointers(struct tree_node_list *tnl,
 extern int board_size;
 static int dfa_board_size = -1;
 extern int dfa_p[DFA_MAX_BOARD * 4 * DFA_MAX_BOARD * 4];
-extern int spiral[8][MAX_ORDER];
-const int convert[3][4];
+extern int spiral[MAX_ORDER][8];
+
+/* This is used by the EXPECTED_COLOR macro. */
+extern const int convert[3][4];
 
 /* Forward declarations. */
 static void dfa_prepare_for_match(int color);
-static int scan_for_patterns(dfa_t *pdfa, int l, int m, int n, 
+static int scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos,
 			     int *pat_list);
-#if DFA_SORT
-static int compare_int(const void *a, const void *b);
-#endif
-static void do_dfa_matchpat(dfa_t *pdfa,
-			    int m, int n, matchpat_callback_fn_ptr callback,
+static void do_dfa_matchpat(dfa_rt_t *pdfa,
+			    int anchor, matchpat_callback_fn_ptr callback,
 			    int color, struct pattern *database,
-			    void *callback_data, char goal[BOARDMAX]);
-static void check_pattern_light(int m, int n, 
+			    void *callback_data, char goal[BOARDMAX],
+                            int anchor_in_goal);
+static void check_pattern_light(int anchor, 
 				matchpat_callback_fn_ptr callback,
 				int color, struct pattern *pattern, int ll,
 				void *callback_data,
-				char goal[BOARDMAX]);
+				char goal[BOARDMAX],
+                                int anchor_in_goal);
 static void dfa_matchpat_loop(matchpat_callback_fn_ptr callback,
 			      int color, int anchor,
 			      struct pattern_db *pdb, void *callback_data,
@@ -855,11 +813,6 @@ static void dfa_matchpat_loop(matchpat_callback_fn_ptr callback,
 void
 dfa_match_init(void)
 {
-  /* Copy the board to the p array.
-   * FIXME: Check whether this is needed here.
-   */
-  board_to_p();
-
   buildSpiralOrder(spiral);
 
   if (owl_vital_apat_db.pdfa != NULL)
@@ -909,8 +862,7 @@ dfa_prepare_for_match(int color)
   /* copy the board */
   for (i = 0; i < dfa_board_size; i++)
     for (j = 0; j < dfa_board_size; j++)
-      dfa_p[DFA_POS(i, j) + DFA_OFFSET] = 
-	EXPECTED_COLOR(color, p[i][j]);
+      dfa_p[DFA_POS(i, j) + DFA_OFFSET] = EXPECTED_COLOR(color, BOARD(i, j));
 
   prepare_for_match(color);
 }
@@ -925,7 +877,7 @@ dump_dfa_board(int m, int n)
   for (i = DFA_MAX_BOARD / 2; i < DFA_MAX_BOARD*1.5 ; i++) {
     for (j = DFA_MAX_BOARD / 2 ; j < DFA_MAX_BOARD*1.5 ; j++)
       if (i != (m+DFA_MAX_BOARD) || j != (n+DFA_MAX_BOARD))
-	fprintf(stderr, "%1d", dfa_p[DFA_PSO(i, j)]);
+	fprintf(stderr, "%1d", dfa_p[DFA_POS(i, j)]);
       else
 	fprintf(stderr, "*");
     fprintf(stderr, "\n");
@@ -941,95 +893,78 @@ dump_dfa_board(int m, int n)
  * Return the number of patterns found.
  */
 static int
-scan_for_patterns(dfa_t *pdfa, int l, int m, int n, int *pat_list)
+scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos, int *pat_list)
 {
-  int state, att, id, row;
-  int dfa_pos;
-
-  dfa_pos = DFA_POS(m, n) + DFA_OFFSET;
-  state = 1; /* initial state */
-  row = 0; /* initial row */
-  id = 0; /* position in id_list */ 
+  int delta;
+  int state = 1; /* initial state */
+  int row = 0; /* initial row */
+  int id = 0; /* position in id_list */ 
   
-  while (state != 0) {/* while not on error state */
+  do {
     /* collect patterns indexes */
-    att = pdfa->states[state].att;
+    int att = pdfa->states[state].att;
     while (att != 0) {
-      pat_list[id] = l + 8 * pdfa->indexes[att].val;
+      pat_list[id] = pdfa->indexes[att].val;
       id++;
       att = pdfa->indexes[att].next;
     }
       
     /* go to next state */
-    state = pdfa->states[state].next[dfa_p[dfa_pos + spiral[l][row]]];
+    delta = pdfa->states[state].next[dfa_pos[spiral[row][l]]];
+    state += delta;
     row++;
-  }
+  } while (delta != 0); /* while not on error state */
 
-  ASSERT2(row < MAX_ORDER, m, n);
+  gg_assert(row < MAX_ORDER);
   return id;
 }
 
 
-#if DFA_SORT
-/* used to sort patterns */
-static int
-compare_int(const void *a, const void *b)
-{
-  const int *da = (const int *) a;
-  const int *db = (const int *) b;
-     
-  return (*da > *db) - (*da < *db);
-}
-#endif
-
-
-/* perform pattern matching with dfa filtering */
+/* Perform pattern matching with dfa filtering. */
 static void 
-do_dfa_matchpat(dfa_t *pdfa,
-		int m, int n, matchpat_callback_fn_ptr callback,
+do_dfa_matchpat(dfa_rt_t *pdfa,
+		int anchor, matchpat_callback_fn_ptr callback,
 		int color, struct pattern *database,
-		void *callback_data, char goal[BOARDMAX])
+		void *callback_data, char goal[BOARDMAX],
+                int anchor_in_goal)
 {
+  int k = 0;
   int ll;      /* Iterate over transformations (rotations or reflections)  */
-  int matched; /* index in database[] of the matched pattern */
-
-  int reorder[DFA_MAX_MATCHED];
-  int *preorder = reorder;
-  int maxr = 0, k;
+  int patterns[DFA_MAX_MATCHED];
+  int *ll_patterns = patterns;
+  int num_matched = 0;
+  int num_transformations = (pdfa->pre_rotated ? 1 : 8);
+  int transformation_end[8];
+  int *dfa_pos = dfa_p + DFA_POS(I(anchor), J(anchor)) + DFA_OFFSET;
 
   /* Basic sanity checks. */
-  ASSERT_ON_BOARD2(m, n);
+  ASSERT_ON_BOARD1(anchor);
 
-  /* Geometrical pattern matching */
-  maxr = 0;
-
-  /* one scan by transformation */
-  for (ll = 0; ll != 8; ll++) {
-    maxr += scan_for_patterns(pdfa, ll, m, n, preorder);
-    preorder = reorder + maxr;
+  /* One scan by transformation */
+  for (ll = 0; ll < num_transformations; ll++) {
+    int ll_matched = scan_for_patterns(pdfa, ll, dfa_pos,
+				       patterns + num_matched);
+    
+    ll_patterns += ll_matched;
+    num_matched += ll_matched;
+    transformation_end[ll] = num_matched;
   }
 
-  ASSERT2(maxr < DFA_MAX_MATCHED, m, n);
+  ASSERT1(num_matched <= DFA_MAX_MATCHED, anchor);
 
-  /* Sorting patterns keep the same order as 
-   * standard pattern matching algorithm */
-#if DFA_SORT
-  gg_sort(reorder, maxr, sizeof(int), compare_int);
-#endif /* DFA_SORT */
-
-
-  /* Constraints and other tests */
-
-  for (k = 0; k != maxr ; k++) {
-    matched = reorder[k] / 8;
-    ll = reorder[k] % 8;
+  /* Constraints and other tests. */
+  for (ll = 0; ll < num_transformations; ll++) {
+    while (k < transformation_end[ll]) {
+      int matched = patterns[k];
 
 #if PROFILE_PATTERNS
-    database[matched].dfa_hits++;
+      database[matched].dfa_hits++;
 #endif
     
-    check_pattern_light(m, n, callback, color, database+matched, 
-			ll, callback_data, goal);
+      check_pattern_light(anchor, callback, color, database + matched,
+			  ll, callback_data, goal, anchor_in_goal);
+      k++;
+    }
   }
 }
 
@@ -1041,12 +976,13 @@ do_dfa_matchpat(dfa_t *pdfa,
  */
 
 static void
-check_pattern_light(int m, int n, matchpat_callback_fn_ptr callback, int color,
-	      struct pattern *pattern, int ll, void *callback_data,
-	      char goal[BOARDMAX])
+check_pattern_light(int anchor, matchpat_callback_fn_ptr callback, int color,
+		    struct pattern *pattern, int ll, void *callback_data,
+		    char goal[BOARDMAX], int anchor_in_goal)
 {
   int k;			/* Iterate over elements of pattern */
-  int found_goal, found_nongoal;
+  int found_goal = 0;
+  int found_nongoal = 0;
   
 #if PROFILE_PATTERNS
   int nodes_before;
@@ -1055,42 +991,56 @@ check_pattern_light(int m, int n, matchpat_callback_fn_ptr callback, int color,
 #ifdef PROFILE_MATCHER
   ++totals[1];
 #endif
+
+  if (0)
+    gprintf("check_pattern_light @ %1m rot:%d pattern: %s\n", 
+	    anchor, ll, pattern->name);
+
+  /* Throw out duplicating orientations of symmetric patterns. */
+  if (pattern->trfno == 5) {
+    if (ll < 2 || ll >= 6)
+      return;
+  }
+  else {
+    if (ll >= pattern->trfno)
+      return;
+  }
+
  
   /* Now iterate over the elements of the pattern. */
-  found_goal = 0;
-  found_nongoal = 0;
   for (k = 0; k < pattern->patlen; k++) {
   				/* match each point */
-    int x, y;			/* absolute (board) co-ords of 
+    int pos;			/* absolute (board) co-ords of 
   				   (transformed) pattern element */
 
     /* transform pattern real coordinate... */
-    TRANSFORM(pattern->patn[k].x, pattern->patn[k].y, &x, &y, ll);
-    x += m;
-    y += n;
-    ASSERT_ON_BOARD2(x, y);
+    pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
+    ASSERT_ON_BOARD1(pos);
 
-    /* goal check */
-    if (goal != NULL) {
-      if (goal[POS(x, y)])
-	found_goal = 1;
-      else if (p[x][y] == color)
-	found_nongoal = 1;
+    if (!anchor_in_goal) { 
+      /* goal check */
+      if (goal != NULL && board[pos] != EMPTY) {
+        if (goal[pos])
+	  found_goal = 1;
+        else if (board[pos] == color)
+	  found_nongoal = 1;
+      }
     }
 
-   /* class check */
-    ASSERT2(dragon[POS(x,y)].matcher_status < 4, x, y);
-    if ((pattern->class
-	 & class_mask[dragon[POS(x, y)].matcher_status][p[x][y]]) != 0)
+    /* class check */
+    ASSERT1(dragon[pos].status < 4, anchor);
+    if ((pattern->class & class_mask[dragon[pos].status][board[pos]]) != 0)
       goto match_failed;
     
-  }/* loop over elements */
+  } /* loop over elements */
   
   /* Make it here ==> We have matched all the elements to the board. */
-  if (goal != NULL && !found_goal)
-    goto match_failed;
-  if (goal != NULL && (pattern->class & CLASS_C) && !found_nongoal)
-    goto match_failed;
+  if (!anchor_in_goal) { 
+    if (goal != NULL && !found_goal)
+      goto match_failed;
+    if (goal != NULL && (pattern->class & CLASS_C) && !found_nongoal)
+      goto match_failed;
+  }
 
 #ifdef PROFILE_MATCHER
   ++totals[4];
@@ -1102,7 +1052,7 @@ check_pattern_light(int m, int n, matchpat_callback_fn_ptr callback, int color,
 #endif
   
   /* A match!  - Call back to the invoker to let it know. */
-  callback(m, n, color, pattern, ll, callback_data);
+  callback(anchor, color, pattern, ll, callback_data);
   
 #if PROFILE_PATTERNS
   pattern->reading_nodes += stats.nodes - nodes_before;
@@ -1110,8 +1060,8 @@ check_pattern_light(int m, int n, matchpat_callback_fn_ptr callback, int color,
   
   /* We jump to here as soon as we discover a pattern has failed. */
  match_failed:
-  DEBUG(DEBUG_MATCHER, "end of pattern '%s', rotation %d at %m\n---\n",
-	pattern->name, ll, m, n);
+  DEBUG(DEBUG_MATCHER, "end of pattern '%s', rotation %d at %1m\n---\n",
+	pattern->name, ll, anchor);
   
 } /* check_pattern_light */
 
@@ -1126,14 +1076,13 @@ dfa_matchpat_loop(matchpat_callback_fn_ptr callback, int color, int anchor,
 		  struct pattern_db *pdb, void *callback_data,
 		  char goal[BOARDMAX], int anchor_in_goal) 
 {
-  int i, j;
+  int pos;
 
-  for (i = 0; i != board_size; i++)
-    for (j = 0; j != board_size; j++)
-      if (p[i][j] == anchor
-          && (!anchor_in_goal || goal[POS(i,j)] != 0))
-	do_dfa_matchpat(pdb->pdfa, i, j, callback, color, pdb->patterns, 
-			callback_data, goal);
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (board[pos] == anchor && (!anchor_in_goal || goal[pos] != 0))
+      do_dfa_matchpat(pdb->pdfa, pos, callback, color, pdb->patterns,
+		      callback_data, goal, anchor_in_goal);
+  }
 }
 
 
@@ -1165,44 +1114,18 @@ matchpat(matchpat_callback_fn_ptr callback, int color,
 	 struct pattern_db *pdb, void *callback_data,
 	 char goal[BOARDMAX]) 
 {
-  matchpat_goal_anchor(callback, color, pdb, callback_data, goal, 0);
+  matchpat_goal_anchor(callback, color, pdb, callback_data, goal, 
+                       pdb->fixed_anchor);
 }
 
 void 
 matchpat_goal_anchor(matchpat_callback_fn_ptr callback, int color,
-	 struct pattern_db *pdb, void *callback_data,
-	 char goal[BOARDMAX], int anchor_in_goal) 
+		     struct pattern_db *pdb, void *callback_data,
+		     char goal[BOARDMAX], int anchor_in_goal) 
 {
   loop_fn_ptr_t loop = matchpat_loop;
   prepare_fn_ptr_t prepare = prepare_for_match;
 
-  /*
-   * Caution, dangerous workaround ahead.
-   *
-   * It's almost certainly not safe to make recursive calls to this
-   * function, but if we just make sure that we don't corrupt the
-   * contents of the p[][] array because of this, it seems to work
-   * anyway.
-   *
-   * Therefore we backup p[][] to saved_p[][] when we are called
-   * recursively and restore it when we are ready.
-   *
-   * FIXME: This is not a proper solution. In any case this
-   * implementation can only handle one level of recursion, which on
-   * the other hand is all we need.
-   */
-
-  /* Don't accept a second recursive call. */
-  gg_assert(matchpat_call_level <= 1);
-
-  if (matchpat_call_level == 1)
-    memcpy(saved_p, p, sizeof(p));
-
-  matchpat_call_level++;
-
-  /* Copy the board to the p array. */
-  board_to_p();
-
   /* check board size */
   if (pdb->fixed_for_size != board_size) {
     fixup_patterns_for_board_size(pdb->patterns);
@@ -1247,10 +1170,6 @@ matchpat_goal_anchor(matchpat_callback_fn_ptr callback, int color,
 	loop(callback, color, BLACK, pdb, callback_data, goal, anchor_in_goal);
       }
   }
-
-  matchpat_call_level--;
-  if (matchpat_call_level == 1)
-    memcpy(p, saved_p, sizeof(p));
 }
 
 
@@ -1264,23 +1183,14 @@ fullboard_matchpat(fullboard_matchpat_callback_fn_ptr callback, int color,
   int other = OTHER_COLOR(color);
   int ll;   /* Iterate over transformations (rotations or reflections)  */
   int k;    /* Iterate over elements of pattern */
-  int mid = (board_size-1)/2; /* We transform around the center point. */
-  int m, n;
-  int number_of_stones_on_board = 0;
+  /* We transform around the center point. */
+  int mid = POS((board_size-1)/2, (board_size-1)/2);
+  int number_of_stones_on_board = stones_on_board(BLACK | WHITE);
   
   /* Basic sanity check. */
   gg_assert(color != EMPTY);
-  gg_assert(board_size%2 == 1);
-  
-  /* Copy the board to the p array. */
-  board_to_p();
+  gg_assert(board_size % 2 == 1);
 
-  /* Count the number of stones on the board. */
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++)
-      if (p[m][n] != EMPTY)
-	number_of_stones_on_board++;
-  
   /* Try each pattern - NULL pattern marks end of list. */
   for (; pattern->patn; pattern++) { 
     /* The number of stones on the board must be right. This is not
@@ -1294,44 +1204,191 @@ fullboard_matchpat(fullboard_matchpat_callback_fn_ptr callback, int color,
     for (ll = 0; ll < 8; ll++) {
       /* Now iterate over the elements of the pattern. */
       for (k = 0; k < pattern->patlen; k++) { /* match each point */
-	int x, y; /* board co-ords of transformed pattern element */
+	int pos; /* board co-ords of transformed pattern element */
 	int att = pattern->patn[k].att;  /* what we are looking for */
 
 	/* Work out the position on the board of this pattern element. */
-	TRANSFORM(pattern->patn[k].x, pattern->patn[k].y, &x, &y, ll);
-	x += mid;
-	y += mid;
+	pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, mid);
 
-        ASSERT_ON_BOARD2(x, y);
+        ASSERT_ON_BOARD1(pos);
 
-	if ((att == ATT_O && p[x][y] != color)
-	    || (att == ATT_X && p[x][y] != other))
+	if ((att == ATT_O && board[pos] != color)
+	    || (att == ATT_X && board[pos] != other))
 	  break;
 	
       } /* loop over elements */
 	
       if (k == pattern->patlen) {
 	/* A match!  - Call back to the invoker to let it know. */
-	int x, y;
-	TRANSFORM(pattern->movei, pattern->movej, &x, &y, ll);
-	x += mid;
-	y += mid;
-	callback(x, y, pattern, ll);
+	int pos = AFFINE_TRANSFORM(pattern->move_offset, ll, mid);
+	callback(pos, pattern, ll);
       }
     }
   }
 }
 
-/* Copy the 1D board array to the 2D p array. */
+
+/**************************************************************************/
+/* Corner matcher                                                         */
+/**************************************************************************/
+
+/* These arrays specify anchor corner for each transformation. They _must_
+ * be in line with transformation2[][] array in patterns/transform.c.
+ */
+static const int corner_x[8] = {0, 0, 1, 1, 1, 1, 0, 0};
+static const int corner_y[8] = {0, 1, 1, 0, 1, 0, 0, 1};
+
+/* The number of stones in "corner area" for each board position. For example,
+ * corner area for position E3 when anchoring at A1 corner, looks like this:
+ *
+ *   |........		In general, NUM_STONES(pos) is the number of stones
+ *   |........		which are closer to the corner (stone at pos, if any,
+ * 3 |#####...		counts too) than pos. Note, that say G2 is not closer
+ *   |#####...		to the corner than E3, the reverse isn't true either.
+ * 1 |#####...		Their distances are "incomparable" since E < G but
+ *   +--------		3 > 2.
+ *    A   E
+ *
+ * Note that we need these values in at most MAX_BOARD x MAX_BOARD array.
+ * However, it may be anchored at any corner of the board, so if the board is
+ * small, we may calculate NUM_STONES() at negative coordinates.
+ */
+static int num_stones[2*BOARDMAX];
+#define NUM_STONES(pos) num_stones[(pos) + BOARDMAX]
+
+/* Stone locations are stored in this array. They might be needed by callback
+ * function.
+ */
+static int pattern_stones[BOARDMAX];
+
+
+/* Recursively performs corner matching. This function checks whether
+ * `num_variation' variations pointed by `variation' parameter match.
+ * If any of them does, the function calls itself recursively. If any
+ * pattern corresponding to those variations matches, it notifies
+ * callback function.
+ */
 static void
-board_to_p(void)
+do_corner_matchpat(int num_variations, struct corner_variation *variation,
+		   int match_color, corner_matchpat_callback_fn_ptr callback,
+		   int callback_color, int trans, int anchor, int stones)
 {
-  int m, n;
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++)
-      p[m][n] = BOARD(m, n);
+  for (; --num_variations >= 0; variation++) {
+    int move = AFFINE_TRANSFORM(variation->move_offset, trans, anchor);
+    char color_check = match_color ^ variation->xor_att;
+    struct corner_pattern *pattern = variation->pattern;
+
+    if (pattern && color_check == callback_color) {
+      int second_corner
+	  = AFFINE_TRANSFORM(pattern->second_corner_offset, trans, anchor);
+
+      if (NUM_STONES(second_corner) == stones
+	  && (!pattern->symmetric || trans < 4)) {
+	/* We have found a matching pattern. */
+	ASSERT1(board[move] == EMPTY, move);
+
+	callback(move, callback_color, pattern, trans, pattern_stones, stones);
+	continue;
+      }
+    }
+
+    if (variation->num_variations
+	&& NUM_STONES(move) == variation->num_stones
+	&& board[move] == color_check) {
+      /* A matching variation. */
+      pattern_stones[stones] = move;
+      do_corner_matchpat(variation->num_variations, variation->variations,
+			 match_color, callback, callback_color,
+			 trans, anchor, stones + 1);
+    }
+  }
 }
 
+
+/* Perform corner matching at all four corners and both possible
+ * transformations at each corner. `callback' is notified if any
+ * matching pattern is found.
+ */
+void
+corner_matchpat(corner_matchpat_callback_fn_ptr callback, int color,
+		struct corner_db *database)
+{
+  int k;
+
+  for (k = 0; k < 8; k++) {
+    int anchor = POS(corner_x[k] * (board_size - 1),
+		     corner_y[k] * (board_size - 1));
+    int i;
+    int j;
+    int dx = TRANSFORM(OFFSET(1, 0), k);
+    int dy = TRANSFORM(OFFSET(0, 1), k);
+    int pos;
+    struct corner_variation *variation = database->top_variations;
+
+    /* Fill in the NUM_STONES() array. We use `max_width' and `max_height'
+     * fields of database structure to stop working as early as possible.
+     */
+    NUM_STONES(anchor) = IS_STONE(board[anchor]);
+
+    pos = anchor;
+    for (i = 1; i < database->max_height; i++) {
+      pos += dx;
+      if (!ON_BOARD(pos)) {
+	do {
+	  NUM_STONES(pos) = BOARDMAX;
+	  pos += dx;
+	} while (++i < database->max_height);
+
+	break;
+      }
+
+      NUM_STONES(pos) = NUM_STONES(pos - dx) + IS_STONE(board[pos]);
+    }
+
+    pos = anchor;
+    for (j = 1; j < database->max_width; j++) {
+      pos += dy;
+      if (!ON_BOARD(pos)) {
+	do {
+	  NUM_STONES(pos) = BOARDMAX;
+	  pos += dy;
+	} while (++j < database->max_width);
+
+	break;
+      }
+      
+      NUM_STONES(pos) = NUM_STONES(pos - dy) + IS_STONE(board[pos]);
+    }
+    
+    for (i = 1; i < database->max_height; i++) {
+      pos = anchor + i * dy;
+      for (j = 1; j < database->max_width; j++) {
+	pos += dx;
+	NUM_STONES(pos) = NUM_STONES(pos - dx) + NUM_STONES(pos - dy)
+			- NUM_STONES(pos - dx - dy);
+	if (ON_BOARD1(pos) && IS_STONE(board[pos]))
+	  NUM_STONES(pos)++;
+      }
+    }
+
+    /* Try to match top variations. If any of them matches, we call
+     * do_corner_matchpat() to recurse that variation's tree.
+     */
+    for (i = 0; i < database->num_top_variations; i++) {
+      int move = AFFINE_TRANSFORM(variation->move_offset, k, anchor);
+
+      if (NUM_STONES(move) == 1 && IS_STONE(board[move])) {
+	pattern_stones[0] = move;
+	do_corner_matchpat(variation->num_variations, variation->variations,
+			   board[move], callback, color, k, anchor, 1);
+      }
+
+      variation++;
+    }
+  }
+}
+
+
 /*
  * Local Variables:
  * tab-width: 8
