@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -22,6 +23,7 @@
 /* values for move_reason.type */
 #define THREAT_BIT    1
 
+/* Only use even values for non-threat move reasons! */
 #define ATTACK_MOVE              2
 #define ATTACK_MOVE_GOOD_KO      4
 #define ATTACK_MOVE_BAD_KO       6
@@ -37,9 +39,10 @@
 #define SEMEAI_MOVE             18
 #define SEMEAI_THREAT           (SEMEAI_MOVE | THREAT_BIT)
 
-#define BLOCK_TERRITORY_MOVE    20
-#define EXPAND_TERRITORY_MOVE   22
-#define EXPAND_MOYO_MOVE        24
+/*      BLOCK_TERRITORY_MOVE    20   OBSOLETE */
+#define EXPAND_TERRITORY_MOVE   20
+#define EXPAND_MOYO_MOVE        22
+#define INVASION_MOVE           24
 
 #define OWL_ATTACK_MOVE         26
 #define OWL_ATTACK_MOVE_GOOD_KO 28
@@ -59,10 +62,13 @@
 #define YOUR_ATARI_ATARI_MOVE   52
 #define VITAL_EYE_MOVE          54
 
+#define OWL_ATTACK_MOVE_GAIN    60
+#define OWL_DEFEND_MOVE_LOSS    62
+
 #define ANTISUJI_MOVE           70
 
 #define EITHER_MOVE             100
-#define ALL_MOVE                101
+#define ALL_MOVE                102
 
 
 /* Bitmap values for move_reason.status */
@@ -72,7 +78,7 @@
 #define REDUNDANT               (TERRITORY_REDUNDANT | STRATEGICALLY_REDUNDANT)
 #define SECONDARY               4
 
-#define MAX_REASONS 40
+#define MAX_REASONS 120
 
 #define MAX_TRACE_LENGTH  160
 
@@ -105,12 +111,12 @@ struct move_data {
                                       experimental influence. */
   float reverse_followup_value;	/* Value of opponents followup move
 				   (reverse sente). */
-  float secondary_value;   /* Secondary move value. */
-
-  float min_value;         /* Minimum allowed value for the move. */
-  float max_value;         /* Maximum allowed value for the move. */
-  float min_territory;     /* Minimum territorial value. */
-  float max_territory;     /* Maximum territorial value. */
+  float secondary_value;      /* Secondary move value. */
+  float min_value;            /* Minimum allowed value for the move. */
+  float max_value;            /* Maximum allowed value for the move. */
+  float min_territory;        /* Minimum territorial value. */
+  float max_territory;        /* Maximum territorial value. */
+  float randomness_scaling;   /* Increase to randomize this move. */
 
   int reason[MAX_REASONS]; /* List of reasons for a move. */
   int move_safety;         /* Whether the move seems safe. */
@@ -126,41 +132,26 @@ struct move_data {
  * FIXME: Many of these could be optimized more for size (e.g. MAX_EYES)
  */
 
-#define MAX_MOVE_REASONS  1000
-#define MAX_WORMS         2*MAX_BOARD*MAX_BOARD/3
-#define MAX_DRAGONS       MAX_WORMS
-#define MAX_CONNECTIONS   4*MAX_WORMS
-#define MAX_WORM_PAIRS    MAX_WORMS
-#define MAX_EYES          MAX_BOARD*MAX_BOARD/2
-#define MAX_LUNCHES       MAX_WORMS
-#define MAX_EITHER        100
-#define MAX_ALL           100
+#define MAX_MOVE_REASONS	1000
+#define MAX_WORMS		2*MAX_BOARD*MAX_BOARD/3
+#define MAX_DRAGONS		MAX_WORMS
+#define MAX_CONNECTIONS 	4*MAX_WORMS
+#define MAX_EYES		MAX_BOARD*MAX_BOARD/2
+#define MAX_LUNCHES		MAX_WORMS
+#define MAX_EITHER		100
+#define MAX_ALL 		100
+#define MAX_ATTACK_THREATS	6
 
 
-float compute_shape_factor(int pos);
-
 extern struct move_data move[BOARDMAX];
 extern struct move_reason move_reasons[MAX_MOVE_REASONS];
 extern int next_reason;
 
-/* Worms */
-extern int worms[MAX_WORMS];
-extern int next_worm;
-
-/* Dragons */
-extern int dragons[MAX_DRAGONS];
-extern int next_dragon;
-
 /* Connections */
-extern int conn_dragon1[MAX_CONNECTIONS];
-extern int conn_dragon2[MAX_CONNECTIONS];
+extern int conn_worm1[MAX_CONNECTIONS];
+extern int conn_worm2[MAX_CONNECTIONS];
 extern int next_connection;
 
-/* Unordered worm pairs */
-extern int worm_pair1[MAX_WORM_PAIRS];
-extern int worm_pair2[MAX_WORM_PAIRS];
-extern int next_worm_pair;
-
 /* Unordered sets (currently pairs) of move reasons / targets */
 typedef struct {
   int reason1;
@@ -188,23 +179,26 @@ extern int replacement_map[BOARDMAX];
 
 
 
-int  find_worm(int str);
-int  find_dragon(int str);
-
-int  move_reason_known(int pos, int type, int what);
-int  attack_move_reason_known(int pos, int what);
-int  defense_move_reason_known(int pos, int what);
-int  owl_attack_move_reason_known(int pos, int what);
-int  owl_defense_move_reason_known(int pos, int what);
-int  get_biggest_owl_target(int pos);
-int  is_antisuji_move(int pos);
-
-int  move_connects_strings(int pos, int color);
-int  move_reasons_confirm_safety(int move, int color, int minsize);
+int find_worm(int str);
+int find_dragon(int str);
+   
+int move_reason_known(int pos, int type, int what);
+int attack_move_reason_known(int pos, int what);
+int defense_move_reason_known(int pos, int what);
+int owl_attack_move_reason_known(int pos, int what);
+int owl_defense_move_reason_known(int pos, int what);
+int owl_move_reason_known(int pos, int what);
+int get_biggest_owl_target(int pos);
+int is_antisuji_move(int pos);
 
 void discard_redundant_move_reasons(int pos);
 void list_move_reasons(int color);
 
+void mark_changed_dragon(int pos, int color, int affected, int affected2,
+			 int move_reason_type, char safe_stones[BOARDMAX],
+			 float strength[BOARDMAX], float *effective_size);
+void mark_changed_string(int affected, char changed_stones[BOARDMAX],
+			 float strength[BOARDMAX], char new_status);
 
 /*
  * Local Variables:
