@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -59,15 +60,15 @@ semeai(int color)
 
   for (d1 = 0; d1 < number_of_dragons; d1++) {
     if (DRAGON(d1).color != color
-	|| (DRAGON(d1).matcher_status != DEAD
-	    && DRAGON(d1).matcher_status != CRITICAL))
+	|| (DRAGON(d1).status != DEAD
+	    && DRAGON(d1).status != CRITICAL))
       continue;
 
     for (k = 0; k < dragon2[d1].neighbors; k++) {
       d2 = dragon2[d1].adjacent[k];
       if (DRAGON(d2).color != other
-	  || (DRAGON(d2).matcher_status != DEAD
-	      && DRAGON(d2).matcher_status != CRITICAL))
+	  || (DRAGON(d2).status != DEAD
+	      && DRAGON(d2).status != CRITICAL))
 	continue;
 
       /* Dragons d1 (our) and d2 (opponent) are adjacent and both DEAD
@@ -89,170 +90,183 @@ semeai(int color)
 }
 
 
-/* Experimental alternative to semeai() using owl reading */
+/* revision of semeai(). Differs from the old program in calling
+ * owl_analyze_semeai() instead of relying on static analysis.
+ */
+
+#define MAX_DRAGONS 50
 
 void
-new_semeai(int color)
+new_semeai()
 {
+  int semeai_results_first[MAX_DRAGONS][MAX_DRAGONS];
+  int semeai_results_second[MAX_DRAGONS][MAX_DRAGONS];
+  int semeai_move[MAX_DRAGONS][MAX_DRAGONS];
+  char semeai_certain[MAX_DRAGONS][MAX_DRAGONS];
   int d1, d2;
   int k;
-  int apos = NO_MOVE;
-  int bpos = NO_MOVE;
-  int other = OTHER_COLOR(color);
-  int best_result_a[MAX_NEIGHBOR_DRAGONS];
-  int best_result_b[MAX_NEIGHBOR_DRAGONS];
-  int worst_result_a[MAX_NEIGHBOR_DRAGONS];
-  int worst_result_b[MAX_NEIGHBOR_DRAGONS];
-  int a_best_status = UNKNOWN;  /* status if a plays first */
-  int b_worst_status = UNKNOWN; /* status if a plays first */
-  int a_worst_status = UNKNOWN; /* status if b plays first */
-  int b_best_status = UNKNOWN;  /* status if b plays first */
+  int num_dragons = number_of_dragons;
 
-  int move[MAX_NEIGHBOR_DRAGONS];
-  int a_status = UNKNOWN;
-  int b_status = UNKNOWN;
-  int pass;
+  if (num_dragons > MAX_DRAGONS) {
+    TRACE("Too many dragons!!! Might disregard some semeais.");
+    num_dragons = MAX_DRAGONS;
+  }
 
-  TRACE("Experimental Semeai Module is THINKING for %s!\n", 
-	color_to_string(color));
-  
-  for (d1 = 0; d1 < number_of_dragons; d1++) {
-    int semeai_found = 0;
-    if (DRAGON(d1).color != color
-	|| (DRAGON(d1).matcher_status != DEAD
-	    && DRAGON(d1).matcher_status != CRITICAL))
-      continue;
-    
-    /* First pass : collect data
-     * Second pass: update statuses
-     */
-    
-    for (k = 0; k < dragon2[d1].neighbors; k++) {
-      best_result_a[k] = DEAD;
-      worst_result_b[k] = ALIVE;
-      move[k] = NO_MOVE;
+  for (d1 = 0; d1 < num_dragons; d1++)
+    for (d2 = 0; d2 < num_dragons; d2++) {
+      semeai_results_first[d1][d2] = -1;
+      semeai_results_second[d1][d2] = -1;
     }
 
-    for (pass = 0; pass < 2; pass ++) {
-      for (k = 0; k < dragon2[d1].neighbors; k++) {
-	d2 = dragon2[d1].adjacent[k];
-	if (DRAGON(d2).color != other
-	    || (DRAGON(d2).matcher_status != DEAD
-		&& DRAGON(d2).matcher_status != CRITICAL))
-	  continue;
-	
-	/* Dragons d1 (our) and d2 (opponent) are adjacent and both DEAD
-	 * or CRITICAL.
-	 */
-	apos = DRAGON(d1).origin;
-	bpos = DRAGON(d2).origin;
-	
-	/* The following is something we want to do but
-	 * better wait until we have the semeai stuff working
-	 * with ko. See nicklas2 test 1401.
-	 */
-#if 0
-	if (DRAGON(d1).matcher_status == CRITICAL
-	    && DRAGON(d2).matcher_status == DEAD) {
-	  update_status(bpos, CRITICAL, CRITICAL);
-	  add_owl_attack_move(dragon[apos].owl_attack_point,
-			      bpos, WIN);
-	  continue;
-	}
-#endif
+  for (d1 = 0; d1 < num_dragons; d1++)
+    for (k = 0; k < dragon2[d1].neighbors; k++) {
+      int apos = DRAGON(d1).origin;
+      int bpos = DRAGON(dragon2[d1].adjacent[k]).origin;
+      int result_certain;
+      
+      d2 = dragon[bpos].id;
 
-	/* Ignore inessential worms or dragons */
-	if (worm[apos].inessential 
-	    || DRAGON2(apos).safety == INESSENTIAL
-	    || worm[bpos].inessential 
-	    || DRAGON2(bpos).safety == INESSENTIAL)
-	  continue;
-	
-	/* A dragon consisting of a single worm which is tactically dead or
-	 * critical and having just one neighbor should be ignored.
-	 */
-	if (dragon[apos].size == worm[apos].size
-	    && worm[apos].attack_codes[0] == WIN
-	    && DRAGON2(apos).hostile_neighbors == 1)
-	  continue;
-	if (dragon[bpos].size == worm[bpos].size
-	    && worm[bpos].attack_codes[0] == WIN
-	    && DRAGON2(bpos).hostile_neighbors == 1)
-	  continue;
+      /* Look for semeais */
+      
+      if (dragon[apos].color == dragon[bpos].color
+	  || (dragon[apos].status != DEAD
+	      && dragon[apos].status != CRITICAL)
+	  ||(dragon[bpos].status != DEAD
+	     && dragon[bpos].status != CRITICAL))
+	continue;
+      
+      /* A dragon consisting of a single worm which is tactically dead or
+       * critical and having just one neighbor should be ignored, since
+       * the owl code is more reliable than the semeai code in such cases.
+       * We do allow these cases if the worm has 4 liberties and can be
+       * defended.
+       */
+      
+      if (dragon[apos].size == worm[apos].size
+	  && worm[apos].attack_codes[0]
+	  && (worm[apos].liberties < 4
+	      || worm[apos].defense_codes[0] == 0))
+	continue;
+      
+      if (dragon[bpos].size == worm[bpos].size
+	  && worm[bpos].attack_codes[0]
+	  && (worm[bpos].liberties < 4
+	      || worm[bpos].defense_codes[0] == 0))
+	continue;
+      /* Ignore inessential worms or dragons */
+      
+      if (worm[apos].inessential 
+	  || DRAGON2(apos).safety == INESSENTIAL
+	  || worm[bpos].inessential 
+	  || DRAGON2(bpos).safety == INESSENTIAL)
+	continue;
 
-	/* If one dragon consist of a single stone, don't treat it
-	 * as a semeai. (But see nicklas1:1405 for an example where
-	 * this rule is bad.
-	 */
+      /* If either dragon is a single stone, this is best left
+       * to the owl code */
+      if (dragon[apos].size == 1 || dragon[bpos].size == 1)
+	continue;
 
-	if (dragon[apos].size == 1 || dragon[bpos].size == 1)
-	  continue;
+      /* The array semeai_results_first[d1][d2] will contain the status
+       * of d1 after the d1 d2 semeai, giving d1 the first move.
+       * The array semeai_results_second[d1][d2] will contain the status
+       * of d1 after the d1 d2 semeai, giving d2 the first move.
+       */
+      
+      DEBUG(DEBUG_SEMEAI, "Considering semeai between %1m and %1m\n",
+	    apos, bpos);
+      owl_analyze_semeai(apos, bpos,
+			 &(semeai_results_first[d1][d2]), 
+			 &(semeai_results_second[d1][d2]),
+			 &(semeai_move[d1][d2]), 1, &result_certain);
+      DEBUG(DEBUG_SEMEAI, "results if %s moves first: %s %s, %1m%s\n",
+	    board[apos] == BLACK ? "black" : "white",
+	    result_to_string(semeai_results_first[d1][d2]),
+	    result_to_string(semeai_results_second[d1][d2]),
+	    semeai_move[d1][d2], result_certain ? "" : " (uncertain)");
+      semeai_certain[d1][d2] = result_certain;
+    }
+  
+  for (d1 = 0; d1 < num_dragons; d1++) {
+    int semeai_found = 0;
+    int best_defense = 0;
+    int best_attack = 0;
+    int defense_move = PASS_MOVE;
+    int attack_move = PASS_MOVE;
+    int defense_certain = 0;
+    int attack_certain = 0;
+    
+    for (d2 = 0; d2 < num_dragons; d2++) {
+      if (semeai_results_first[d1][d2] == -1)
+	continue;
+      gg_assert(semeai_results_second[d1][d2] != -1);
+      semeai_found = 1;
 
-	semeai_found = 1;
-	a_best_status = UNKNOWN;
-	b_best_status = UNKNOWN;
-	a_worst_status = UNKNOWN;
-	b_worst_status = UNKNOWN;
-
-	if (pass == 0) {
-	  DEBUG(DEBUG_SEMEAI, "Considering semeai between %1m and %1m\n",
-		apos, bpos);
-	  owl_analyze_semeai(apos, bpos,
-			     best_result_a+k, worst_result_b+k, move+k, 1);
-	  if (a_best_status == DEAD
-	      || a_best_status == UNKNOWN
-	      || (a_best_status == ALIVE_IN_SEKI 
-		  && best_result_a[k] == ALIVE)) {
-	    a_best_status = best_result_a[k];
-	    b_worst_status = worst_result_b[k];
-	  }
-	  owl_analyze_semeai(bpos, apos, 
-			     best_result_b+k, worst_result_a+k, NULL, 1);
-	  if (b_best_status == DEAD
-	      || b_best_status == UNKNOWN
-	      || (b_best_status == ALIVE_IN_SEKI
-		  && best_result_b[k] == ALIVE)) {
-	    b_best_status = best_result_b[k];
-	    a_worst_status = worst_result_a[k];
-	  }
-	}
-	else { /* pass == 1 */
-	  if (a_status == CRITICAL
-	      && best_result_a[k] != DEAD)
-	    add_owl_defense_move(move[k], apos, WIN);
-	  if (b_status == CRITICAL
-	      && worst_result_b[k] == DEAD)
-	    add_owl_attack_move(move[k], bpos, WIN);
-	}
-      } /* loop over neighbor dragons */
-      if (pass == 0 && semeai_found) {
-	int a_matcher_status;
-	int b_matcher_status;
-
-	if (a_best_status != DEAD && a_worst_status == DEAD)
-	  a_status = CRITICAL;
-	else
-	  a_status = a_worst_status;
-	
-	if (b_best_status != DEAD && b_worst_status == DEAD)
-	  b_status = CRITICAL;
-	else
-	  b_status = b_worst_status;
-	
-	a_matcher_status = a_status;
-	b_matcher_status = b_status;
-	if (a_matcher_status == ALIVE_IN_SEKI)
-	  a_matcher_status = ALIVE;
-	if (b_matcher_status == ALIVE_IN_SEKI)
-	  b_matcher_status = ALIVE;
-	
-	update_status(apos, a_matcher_status, a_status);
-	update_status(bpos, b_matcher_status, b_status);
+      if (best_defense < semeai_results_first[d1][d2]
+	  || (best_defense == semeai_results_first[d1][d2]
+	      && defense_certain < semeai_certain[d1][d2])) {
+	best_defense = semeai_results_first[d1][d2];
+	defense_move = semeai_move[d1][d2];
+	defense_certain = semeai_certain[d1][d2];
+      }
+      if (best_attack < semeai_results_second[d2][d1]
+	  || (best_attack == semeai_results_second[d2][d1]
+	      && attack_certain < semeai_certain[d2][d1])) {
+	best_attack = semeai_results_second[d2][d1];
+	attack_move = semeai_move[d2][d1];
+	attack_certain = semeai_certain[d2][d1];
       }
     }
+    
+    if (semeai_found) {
+      dragon2[d1].semeai = 1;
+      if (best_defense != 0 && best_attack != 0) {
+	update_status(DRAGON(d1).origin, CRITICAL, CRITICAL);
+	dragon2[d1].semeai_defense_point = defense_move;
+	dragon2[d1].semeai_defense_certain = defense_certain;
+	dragon2[d1].semeai_attack_point = attack_move;
+	dragon2[d1].semeai_attack_certain = attack_certain;
+      }
+      else if (best_attack == 0 && attack_certain)
+	update_status(DRAGON(d1).origin, ALIVE, ALIVE);
+    }
   }
 }
 
+/* This function adds the semeai related move reasons, using the information
+ * stored in the dragon2 array.
+ *
+ * If the semeai had an uncertain result, and there is a owl move with
+ * certain result doing the same, we don't trust the semeai move.
+ */
+void
+semeai_move_reasons(int color)
+{
+  int other = OTHER_COLOR(color);
+  int d;
+
+  for (d = 0; d < number_of_dragons; d++)
+    if (dragon2[d].semeai && DRAGON(d).status == CRITICAL) {
+      if (DRAGON(d).color == color
+          && dragon2[d].semeai_defense_point
+	  && (dragon2[d].owl_defense_point == NO_MOVE
+	      || dragon2[d].semeai_defense_certain >= 
+	         dragon2[d].owl_defense_certain)) {
+	add_semeai_move(dragon2[d].semeai_defense_point, dragon2[d].origin);
+	DEBUG(DEBUG_SEMEAI, "Adding semeai defense move for %1m at %1m\n",
+	      DRAGON(d).origin, dragon2[d].semeai_defense_point);
+      }
+      else if (DRAGON(d).color == other
+	       && dragon2[d].semeai_attack_point
+	       && (dragon2[d].owl_attack_point == NO_MOVE
+		   || dragon2[d].semeai_attack_certain >= 
+		      dragon2[d].owl_attack_certain)) {
+	add_semeai_move(dragon2[d].semeai_attack_point, dragon2[d].origin);
+	DEBUG(DEBUG_SEMEAI, "Adding semeai attack move for %1m at %1m\n",
+	      DRAGON(d).origin, dragon2[d].semeai_attack_point);
+      }
+    }
+}
+
 /* liberty_of_dragon(pos, origin) returns true if the vertex at (pos) is a
  * liberty of the dragon with origin at (origin).
  */
@@ -281,19 +295,23 @@ static void
 update_status(int dr, int new_status, int new_safety)
 {
   int pos;
-  if (dragon[dr].matcher_status != new_status)
-    DEBUG(DEBUG_SEMEAI, "Changing matcher_status of %1m from %s to %s.\n", dr,
-	  status_to_string(dragon[dr].matcher_status),
+
+  if (dragon[dr].status != new_status
+      && (dragon[dr].status != CRITICAL || new_status != DEAD))  {
+    DEBUG(DEBUG_SEMEAI, "Changing status of %1m from %s to %s.\n", dr,
+	  status_to_string(dragon[dr].status),
 	  status_to_string(new_status));
-  if (DRAGON2(dr).safety != new_safety)
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+      if (IS_STONE(board[pos]) && is_same_dragon(dr, pos))
+	dragon[pos].status = new_status;
+  }
+
+  if (DRAGON2(dr).safety != new_safety
+      && (DRAGON2(dr).safety != CRITICAL || new_safety != DEAD)) {
     DEBUG(DEBUG_SEMEAI, "Changing safety of %1m from %s to %s.\n", dr,
 	  safety_to_string(DRAGON2(dr).safety), safety_to_string(new_safety));
-  
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++)
-    if (IS_STONE(board[pos]) && is_same_dragon(dr, pos))
-      dragon[pos].matcher_status = new_status;
-
-  DRAGON2(dr).safety = new_safety;
+    DRAGON2(dr).safety = new_safety;
+  }
 }
 
 
@@ -307,15 +325,14 @@ analyze_semeai(int my_dragon, int your_dragon)
   /* We start liberty counts at 1 since we will be subtracting
    * the number of worms. */
   int mylibs = 1, yourlibs = 1, commonlibs = 0; 
-  int yourlibi = -1, yourlibj = -1;
-  int commonlibi = -1, commonlibj = -1;
+  int yourlib = NO_MOVE;
+  int commonlib = NO_MOVE;
   int color = board[my_dragon];
-  int i, j;
-  int m, n;
   int my_status = UNKNOWN;
   int your_status = UNKNOWN;
   int margin_of_safety = 0;
   int owl_code_sufficient = 0;
+  int pos;
   
   DEBUG(DEBUG_SEMEAI, "semeai_analyzer: %1m (me) vs %1m (them)\n",
 	my_dragon, your_dragon);
@@ -324,19 +341,19 @@ analyze_semeai(int my_dragon, int your_dragon)
    * dragon owl_does_attack your dragon, add another owl attack move
    * reason.
    */
-  if (dragon[my_dragon].owl_status == CRITICAL
-      && dragon[your_dragon].owl_status == CRITICAL) {
-    if (dragon[your_dragon].owl_attack_point
-	== dragon[my_dragon].owl_defense_point)
+  if (DRAGON2(my_dragon).owl_status == CRITICAL
+      && DRAGON2(your_dragon).owl_status == CRITICAL) {
+    if (DRAGON2(your_dragon).owl_attack_point
+	== DRAGON2(my_dragon).owl_defense_point)
       return;
-    if (dragon[my_dragon].owl_defense_point != NO_MOVE) {
-      int acode = owl_does_attack(dragon[my_dragon].owl_defense_point,
-				  your_dragon);
+    if (DRAGON2(my_dragon).owl_defense_point != NO_MOVE) {
+      int acode = owl_does_attack(DRAGON2(my_dragon).owl_defense_point,
+				  your_dragon, NULL);
       if (acode != 0) {
-	add_owl_attack_move(dragon[my_dragon].owl_defense_point, your_dragon,
+	add_owl_attack_move(DRAGON2(my_dragon).owl_defense_point, your_dragon,
 			    acode);
 	DEBUG(DEBUG_SEMEAI, "added owl attack of %1m at %1m with code %d\n",
-	      your_dragon, dragon[my_dragon].owl_defense_point, acode);
+	      your_dragon, DRAGON2(my_dragon).owl_defense_point, acode);
 	owl_code_sufficient = 1;
       }
     }
@@ -347,21 +364,21 @@ analyze_semeai(int my_dragon, int your_dragon)
    * owl_does_defend my dragon, add another owl defense move reason
    * and possibly change the owl status of my dragon to critical.
    */
-  if ((dragon[my_dragon].owl_status == CRITICAL
-       || dragon[my_dragon].owl_status == DEAD)
-      && dragon[your_dragon].owl_status == CRITICAL) {
-    if (dragon[your_dragon].owl_attack_point
-	== dragon[my_dragon].owl_defense_point)
+  if ((DRAGON2(my_dragon).owl_status == CRITICAL
+       || DRAGON2(my_dragon).owl_status == DEAD)
+      && DRAGON2(your_dragon).owl_status == CRITICAL) {
+    if (DRAGON2(your_dragon).owl_attack_point
+	== DRAGON2(my_dragon).owl_defense_point)
       return;
-    if (dragon[your_dragon].owl_attack_point != NO_MOVE) {
-      int dcode = owl_does_defend(dragon[your_dragon].owl_attack_point,
-				  my_dragon);
+    if (DRAGON2(your_dragon).owl_attack_point != NO_MOVE) {
+      int dcode = owl_does_defend(DRAGON2(your_dragon).owl_attack_point,
+				  my_dragon, NULL);
       if (dcode != 0) {
-	add_owl_defense_move(dragon[your_dragon].owl_attack_point, my_dragon,
+	add_owl_defense_move(DRAGON2(your_dragon).owl_attack_point, my_dragon,
 			     dcode);
 	DEBUG(DEBUG_SEMEAI, "added owl defense of %1m at %1m with code %d\n",
-	      my_dragon, dragon[your_dragon].owl_attack_point, dcode);
-	if (dragon[my_dragon].owl_status == DEAD) {
+	      my_dragon, DRAGON2(your_dragon).owl_attack_point, dcode);
+	if (DRAGON2(my_dragon).owl_status == DEAD) {
 	  int pos;
 
 	  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
@@ -369,12 +386,12 @@ analyze_semeai(int my_dragon, int your_dragon)
 	      continue;
 	    if (board[pos] == board[my_dragon]
 		&& is_same_dragon(pos, my_dragon)) {
-	      dragon[pos].owl_status = CRITICAL;
-	      dragon[pos].matcher_status = CRITICAL;
+	      DRAGON2(pos).owl_status = CRITICAL;
+	      dragon[pos].status = CRITICAL;
 	    }
 	  }
 	  DEBUG(DEBUG_SEMEAI,
-		"changed owl_status and matcher_status of %1m to CRITICAL\n",
+		"changed owl_status and status of %1m to CRITICAL\n",
 		my_dragon);
 	}
 	owl_code_sufficient = 1;
@@ -397,40 +414,42 @@ analyze_semeai(int my_dragon, int your_dragon)
    * string which is adjacent to the other dragon which is owl
    * substantial.
    */
-  for (m = 0; m < board_size; m++)
-    for (n = 0; n < board_size; n++) {
-      int pos = POS(m, n);
-      if (worm[pos].origin == pos
-	  && worm[pos].attack_codes[0] == WIN)
-	if (dragon[pos].origin == my_dragon
-	    || dragon[pos].origin == your_dragon) {
-	  int adj;
-	  int adjs[MAXCHAIN];
-	  int r;
-	  
-	  adj = chainlinks(pos, adjs);
-	  for (r = 0; r < adj; r++) {
-	    int cpos = adjs[r];
-	    if (dragon[cpos].origin == my_dragon
-		|| dragon[cpos].origin == your_dragon)
-	      if (owl_substantial(pos)) {
-		DEBUG(DEBUG_SEMEAI, 
-		      "...tactical situation detected, exiting\n");
-		return;
-	      }
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!ON_BOARD(pos))
+      continue;
+
+    if (worm[pos].origin == pos
+	&& worm[pos].attack_codes[0] == WIN) {
+      if (dragon[pos].origin == my_dragon
+	  || dragon[pos].origin == your_dragon) {
+	int adj;
+	int adjs[MAXCHAIN];
+	int r;
+	
+	adj = chainlinks(pos, adjs);
+	for (r = 0; r < adj; r++) {
+	  int cpos = adjs[r];
+	  if (dragon[cpos].origin == my_dragon
+	      || dragon[cpos].origin == your_dragon) {
+	    if (owl_substantial(pos)) {
+	      DEBUG(DEBUG_SEMEAI, 
+		    "...tactical situation detected, exiting\n");
+	      return;
+	    }
 	  }
 	}
+      }
     }
+  }
   
   
   /* Mark the dragons as involved in semeai */
-  for (i = 0; i < board_size; i++)
-    for (j = 0; j < board_size; j++) {
-      int pos = POS(i, j);
-      if (is_same_dragon(pos, my_dragon)
-	  || is_same_dragon(pos, your_dragon))
-	DRAGON2(pos).semeai = 1;
-    }
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (ON_BOARD(pos)
+	&& (is_same_dragon(pos, my_dragon)
+	    || is_same_dragon(pos, your_dragon)))
+      DRAGON2(pos).semeai = 1;
+  }
   
   /* First we try to determine the number of liberties of each
    * dragon, and the number of common liberties. We subtract
@@ -440,48 +459,46 @@ analyze_semeai(int my_dragon, int your_dragon)
    * we try to find a liberty of the opponent's dragon, and a
    * common liberty, for future reference.
    */
-  for (i = 0; i < board_size; i++)
-    for (j = 0; j < board_size; j++) {
-      int pos = POS(i, j);
-      if (board[pos]
-	  && worm[pos].origin == pos) {
-	if (is_same_dragon(pos, my_dragon))
-	  mylibs--;
-	if (is_same_dragon(pos, your_dragon))
-	  yourlibs--;
-      }
-      else if (board[pos] == EMPTY) {
-	if (liberty_of_dragon(pos, your_dragon)) {
-	  yourlibs ++;
-	  if (liberty_of_dragon(pos, my_dragon)) {
-	    commonlibs++;
-	    mylibs++;
-	    commonlibi = i;
-	    commonlibj = j;
-	  }
-	  else {
-	    yourlibi = i;
-	    yourlibj = j;
-	  }
-	}
-	else if (liberty_of_dragon(pos, my_dragon))
-	  mylibs++;
-      }
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!ON_BOARD(pos))
+      continue;
+    if (IS_STONE(board[pos])
+	&& worm[pos].origin == pos) {
+      if (is_same_dragon(pos, my_dragon))
+	mylibs--;
+      if (is_same_dragon(pos, your_dragon))
+	yourlibs--;
     }
+    else if (board[pos] == EMPTY) {
+      if (liberty_of_dragon(pos, your_dragon)) {
+	yourlibs ++;
+	if (liberty_of_dragon(pos, my_dragon)) {
+	  commonlibs++;
+	  mylibs++;
+	  commonlib = pos;
+	}
+	else
+	  yourlib = pos;
+      }
+      else if (liberty_of_dragon(pos, my_dragon))
+	mylibs++;
+    }
+  }
+
   /* We add 1 to the
    * number of liberties of an owl critical dragon if the point
    * of attack is not a liberty of the dragon, since a move
    * may have to be invested in attacking it.
    */
 
-  if (dragon[my_dragon].owl_status == CRITICAL
-      && dragon[my_dragon].owl_attack_point != NO_MOVE
-      && !liberty_of_string(dragon[my_dragon].owl_attack_point, my_dragon))
+  if (DRAGON2(my_dragon).owl_status == CRITICAL
+      && DRAGON2(my_dragon).owl_attack_point != NO_MOVE
+      && !liberty_of_string(DRAGON2(my_dragon).owl_attack_point, my_dragon))
     mylibs++;
   
-  if (dragon[your_dragon].owl_status == CRITICAL
-      && dragon[your_dragon].owl_attack_point != NO_MOVE
-      && !liberty_of_string(dragon[your_dragon].owl_attack_point, your_dragon))
+  if (DRAGON2(your_dragon).owl_status == CRITICAL
+      && DRAGON2(your_dragon).owl_attack_point != NO_MOVE
+      && !liberty_of_string(DRAGON2(your_dragon).owl_attack_point, your_dragon))
     yourlibs++;
   
   /* Now we compute the statuses which result from a
@@ -538,8 +555,8 @@ analyze_semeai(int my_dragon, int your_dragon)
    * (2) If Y+C=M whoever moves first wins. CRITICAL.
    * (3) If Y+C<M I win.  */
 
-  if (DRAGON2(my_dragon).genus == 0
-      && DRAGON2(your_dragon).genus == 0) {
+  if (min_eyes(&DRAGON2(my_dragon).genus) == 0
+      && min_eyes(&DRAGON2(your_dragon).genus) == 0) {
     if (commonlibs == 0) {
       if (mylibs > yourlibs) {
 	my_status = ALIVE;
@@ -598,8 +615,8 @@ analyze_semeai(int my_dragon, int your_dragon)
       margin_of_safety = yourlibs - mylibs - commonlibs;
     }
   }
-  if (DRAGON2(my_dragon).genus > 0
-      && DRAGON2(your_dragon).genus > 0) {
+  if (min_eyes(&DRAGON2(my_dragon).genus) > 0
+      && min_eyes(&DRAGON2(your_dragon).genus) > 0) {
     if (mylibs > yourlibs + commonlibs) {
       my_status = ALIVE;
       your_status = DEAD;
@@ -637,8 +654,8 @@ analyze_semeai(int my_dragon, int your_dragon)
       margin_of_safety = 0;
     }
   }
-  if (DRAGON2(my_dragon).genus > 0
-      && DRAGON2(your_dragon).genus == 0) {
+  if (min_eyes(&DRAGON2(my_dragon).genus) > 0
+      && min_eyes(&DRAGON2(your_dragon).genus) == 0) {
     if (mylibs > commonlibs + yourlibs) {
       my_status = ALIVE;
       your_status = DEAD;
@@ -654,8 +671,8 @@ analyze_semeai(int my_dragon, int your_dragon)
       margin_of_safety = mylibs + commonlibs - yourlibs;
     }
   }
-  if (DRAGON2(my_dragon).genus == 0
-      && DRAGON2(your_dragon).genus > 0) {
+  if (min_eyes(&DRAGON2(my_dragon).genus) == 0
+      && min_eyes(&DRAGON2(your_dragon).genus) > 0) {
     if (yourlibs + commonlibs > mylibs) {
       my_status = DEAD;
       your_status = ALIVE;
@@ -684,7 +701,7 @@ analyze_semeai(int my_dragon, int your_dragon)
    * want to take it. So the matcher status is not changed.
    */
   
-  if (dragon[my_dragon].owl_status != CRITICAL) {
+  if (DRAGON2(my_dragon).owl_status != CRITICAL) {
     if (my_status == ALIVE)
       update_status(my_dragon, ALIVE, ALIVE_IN_SEKI);
     else if (my_status == CRITICAL)
@@ -710,45 +727,46 @@ analyze_semeai(int my_dragon, int your_dragon)
 
   if (my_status == CRITICAL || your_status == CRITICAL) {
     int found_one = 0;
-    if (dragon[my_dragon].owl_status == CRITICAL
-	&& dragon[my_dragon].owl_defense_point != NO_MOVE)
-      add_appropriate_semeai_moves(dragon[my_dragon].owl_defense_point,
+    if (DRAGON2(my_dragon).owl_status == CRITICAL
+	&& DRAGON2(my_dragon).owl_defense_point != NO_MOVE)
+      add_appropriate_semeai_moves(DRAGON2(my_dragon).owl_defense_point,
 				   my_dragon, your_dragon,
 				   my_status, your_status, margin_of_safety);
-    else if (dragon[your_dragon].owl_status == CRITICAL
-	     && dragon[your_dragon].owl_attack_point != NO_MOVE)
-      add_appropriate_semeai_moves(dragon[your_dragon].owl_attack_point,
+    else if (DRAGON2(your_dragon).owl_status == CRITICAL
+	     && DRAGON2(your_dragon).owl_attack_point != NO_MOVE)
+      add_appropriate_semeai_moves(DRAGON2(your_dragon).owl_attack_point,
 				   my_dragon, your_dragon,
 				   my_status, your_status, margin_of_safety);
     else if (commonlibs > 1) {
-      if (DRAGON2(my_dragon).heyes > 0)
+      if (eye_move_urgency(&DRAGON2(my_dragon).genus) > 0)
 	add_appropriate_semeai_moves(DRAGON2(my_dragon).heye,
 				     my_dragon, your_dragon,
 				     my_status, your_status, margin_of_safety);
-      if (DRAGON2(your_dragon).heyes > 0)
+      if (eye_move_urgency(&DRAGON2(your_dragon).genus))
 	add_appropriate_semeai_moves(DRAGON2(your_dragon).heye,
 				     my_dragon, your_dragon,
 				     my_status, your_status, margin_of_safety);
     }
     else {
-      for (i = 0; i < board_size-1; i++)
-	for (j = 0; j < board_size-1; j++) {
-	  int pos = POS(i, j);
-	  if (liberty_of_dragon(pos, your_dragon) 
-	      && !liberty_of_dragon(pos, my_dragon)
-	      && safe_move(pos, color)) {
-	    /* add move reasons for EVERY outside liberty where we can
-             * play safely. A move to win a semeai might not be a
-             * safe move if it is inside the opponent's eyespace. 
-             * However we hope that the reading code can analyze the
-             * semeai in cases where every safe liberty has been filled.
-	     */
-	    add_appropriate_semeai_moves(pos, my_dragon, your_dragon,
-					 my_status, your_status,
-					 margin_of_safety);
-	    found_one = 1;
-	  }
+      for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+	if (!ON_BOARD(pos))
+	  continue;
+
+	if (liberty_of_dragon(pos, your_dragon) 
+	    && !liberty_of_dragon(pos, my_dragon)
+	    && safe_move(pos, color)) {
+	  /* add move reasons for EVERY outside liberty where we can
+	   * play safely. A move to win a semeai might not be a
+	   * safe move if it is inside the opponent's eyespace. 
+	   * However we hope that the reading code can analyze the
+	   * semeai in cases where every safe liberty has been filled.
+	   */
+	  add_appropriate_semeai_moves(pos, my_dragon, your_dragon,
+				       my_status, your_status,
+				       margin_of_safety);
+	  found_one = 1;
 	}
+      }
       if (!found_one) {
 	/* No outside liberties --- look for common liberties.
 	 * Filling a common liberty is usually bad but if our 
@@ -757,15 +775,14 @@ analyze_semeai(int my_dragon, int your_dragon)
 	 * sanity check, we require filling a common liberty to
 	 * be a safe move.
 	 */
-	for (i = 0; i < board_size-1; i++)
-	  for (j = 0; j < board_size-1; j++) {
-	    int pos = POS(i, j);
-	    if (liberty_of_dragon(pos, your_dragon)
-		&& safe_move(pos, color))
-	      add_appropriate_semeai_moves(pos, my_dragon, your_dragon,
-					   my_status, your_status,
-					   margin_of_safety);
-	  }
+	for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+	  if (ON_BOARD(pos)
+	      && liberty_of_dragon(pos, your_dragon)
+	      && safe_move(pos, color))
+	    add_appropriate_semeai_moves(pos, my_dragon, your_dragon,
+					 my_status, your_status,
+					 margin_of_safety);
+	}
       }
     }
   }
@@ -783,7 +800,7 @@ add_appropriate_semeai_moves(int move, int my_dragon, int your_dragon,
   else if (margin_of_safety == 1)
     add_semeai_threat(move, my_dragon);
   if (your_status == CRITICAL)
-      add_semeai_move(move, your_dragon);
+    add_semeai_move(move, your_dragon);
   else if (margin_of_safety == 1)
     add_semeai_threat(move, your_dragon);
 }
@@ -847,14 +864,15 @@ small_semeai_analyzer(int apos, int bpos, int save_verbose)
 {
   int move;
   int resulta, resultb;
+  int dummy;
 
   /* FIXME: Not ko aware yet (since owl_analyze_semeai isn't).
    * Should be more careful if there is already a defense point.
    */
 
-  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 0);
-  if (resulta != DEAD
-      && worm[apos].defend_codes[0] == 0 
+  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 0, &dummy);
+  if (resulta != 0
+      && worm[apos].defense_codes[0] == 0
       && move != NO_MOVE) {
     if (save_verbose)
       gprintf("small semeai: changing defense of %1m to %1m\n",
