@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -39,7 +40,7 @@
  */
 
 void
-decide_string(int pos, const char *sgf_output)
+decide_string(int pos)
 {
   int aa, dd;
   int acode, dcode;
@@ -50,8 +51,8 @@ decide_string(int pos, const char *sgf_output)
     return ;
   }
 
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
 
   /* Prepare pattern matcher and reading code. */
   reset_engine();
@@ -105,10 +106,8 @@ decide_string(int pos, const char *sgf_output)
     }
   }
 
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
@@ -119,7 +118,7 @@ decide_string(int pos, const char *sgf_output)
  */
 
 void
-decide_connection(int apos, int bpos, const char *sgf_output)
+decide_connection(int apos, int bpos)
 {
   int move;
   int result;
@@ -138,8 +137,8 @@ decide_connection(int apos, int bpos, const char *sgf_output)
     return;
   }
 
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
 
   /* Prepare pattern matcher and reading code. */
   reset_engine();
@@ -154,6 +153,12 @@ decide_connection(int apos, int bpos, const char *sgf_output)
 	gprintf("%1m and %1m can be connected at %1m (%d variations)\n", 
 		apos, bpos, move, count_variations);
   }
+  else if (result == KO_A)
+    gprintf("%1m and %1m can be connected with ko (good) at %1m (%d variations)\n", 
+	    apos, bpos, move, count_variations);
+  else if (result == KO_B)
+    gprintf("%1m and %1m can be connected with ko (bad) at %1m (%d variations)\n", 
+	    apos, bpos, move, count_variations);
   else
     gprintf("%1m and %1m cannot be connected (%d variations)\n", 
 	    apos, bpos, count_variations);
@@ -168,30 +173,35 @@ decide_connection(int apos, int bpos, const char *sgf_output)
 	gprintf("%1m and %1m can be disconnected at %1m (%d variations)\n", 
 		apos, bpos, move, count_variations);
   }
+  else if (result == KO_A)
+    gprintf("%1m and %1m can be disconnected with ko (good) at %1m (%d variations)\n", 
+	    apos, bpos, move, count_variations);
+  else if (result == KO_B)
+    gprintf("%1m and %1m can be disconnected with ko (bad) at %1m (%d variations)\n", 
+	    apos, bpos, move, count_variations);
   else
     gprintf("%1m and %1m cannot be disconnected (%d variations)\n", 
 	    apos, bpos, count_variations);
   
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
 /* 
- * decide_dragon tries to attack and defend the dragon at (pos),
- * and then writes the number of variations considered in the attack
- * and defence to the sgf file.
+ * decide_owl (formerly called decide_dragon) tries to attack and defend 
+ * the dragon at (pos), and then writes the number of variations considered 
+ * in the attack and defence to the sgf file.
  */
 
 void
-decide_dragon(int pos, const char *sgf_output)
+decide_owl(int pos)
 {
   int move = NO_MOVE;
   int acode, dcode;
   SGFTree tree;
   int result_certain;
+  int kworm;
 
   if (board[pos] == EMPTY) {
     fprintf(stderr, "gnugo: --decide-dragon called on an empty vertex\n");
@@ -209,11 +219,11 @@ decide_dragon(int pos, const char *sgf_output)
    */
   reading_cache_clear();
   
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
 
   count_variations = 1;
-  acode = owl_attack(pos, &move, &result_certain);
+  acode = owl_attack(pos, &move, &result_certain, &kworm);
   if (acode) {
     if (acode == WIN) {
       if (move == NO_MOVE)
@@ -228,6 +238,9 @@ decide_dragon(int pos, const char *sgf_output)
     else if (acode == KO_B)
       gprintf("%1m can be attacked with ko (bad) at %1m (%d variations)", 
 	      pos, move, count_variations);
+    else if (acode == GAIN)
+      gprintf("%1m can be attacked with gain (captures %1m) at %1m (%d variations)", 
+	      pos, kworm, move, count_variations);
   }
   else 
     gprintf("%1m cannot be attacked (%d variations)", pos, count_variations);
@@ -237,8 +250,9 @@ decide_dragon(int pos, const char *sgf_output)
   else
     gprintf(" result uncertain\n");
 
+  reading_cache_clear();
   count_variations = 1;
-  dcode = owl_defend(pos, &move, &result_certain);
+  dcode = owl_defend(pos, &move, &result_certain, &kworm);
 
   if (dcode) {
     if (dcode == WIN) {
@@ -254,6 +268,9 @@ decide_dragon(int pos, const char *sgf_output)
     else if (dcode == KO_B)
       gprintf("%1m can be defended with ko (bad) at %1m (%d variations)", 
 	      pos, move, count_variations);
+    else if (dcode == LOSS)
+      gprintf("%1m can be defended with loss (loses %1m) at %1m (%d variations)", 
+	      pos, kworm, move, count_variations);
   }
   else
     gprintf("%1m cannot be defended (%d variations)",
@@ -264,18 +281,39 @@ decide_dragon(int pos, const char *sgf_output)
   else
     gprintf(" result uncertain\n");
   
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
+/* 
+ * decide_dragon_data prints the dragon data at (pos).
+ */
+
 void
-decide_semeai(int apos, int bpos, const char *sgf_output)
+decide_dragon_data(int pos)
+{
+  if (board[pos] == EMPTY) {
+    fprintf(stderr, "gnugo: --decide-dragon-data called on an empty vertex\n");
+    return ;
+  }
+  reset_engine();
+  silent_examine_position(board[pos], FULL_EXAMINE_DRAGONS);
+
+  gprintf("Dragon at %1m:\n", pos);
+  report_dragon(stderr, pos);
+}
+
+
+/* Print the result of the semeai code on the semeai at apos/bpos,
+ * optionally writing an sgf file.
+ */
+
+void
+decide_semeai(int apos, int bpos)
 {
   SGFTree tree;
-  int resulta, resultb, move;
+  int resulta, resultb, move, result_certain;
   int color = board[apos];
 
   if (color == EMPTY || board[bpos] != OTHER_COLOR(color)) {
@@ -294,35 +332,41 @@ decide_semeai(int apos, int bpos, const char *sgf_output)
    * from the cache. Thus we clear the cache here. */
   reading_cache_clear();
   
-  if (sgf_output)
-    begin_sgftreedump(&tree);
-  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 1);
-  gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
-	  color == BLACK ? "black" : "white",
-	  move,
-	  apos, safety_to_string(resulta),
-  	  bpos, safety_to_string(resultb),
-	  count_variations);
-  owl_analyze_semeai(bpos, apos, &resultb, &resulta, &move, 1);
-  gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
-	  color == BLACK ? "white" : "black",
-	  move,
-	  apos, safety_to_string(resulta),
-  	  bpos, safety_to_string(resultb),
-	  count_variations);
+  if (*outfilename)
+    sgffile_begindump(&tree);
 
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  gprintf("Analyzing semeai between %1m and %1m, %C moves first\n",
+	  apos, bpos, board[apos]);
+  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 1,
+		     &result_certain);
+  gprintf("Semeai defense of %1m: result %s %1m\n",
+	  apos, result_to_string(resulta), move);
+  gprintf("Semeai attack of %1m: result %s %1m\n",
+	  bpos, result_to_string(resultb), move);
+  gprintf("%d nodes%s\n\n", count_variations,
+	  result_certain ? "" : ", uncertain result");
+  
+  gprintf("Analyzing semeai between %1m and %1m, %C moves first\n",
+	  bpos, apos, board[bpos]);
+  owl_analyze_semeai(bpos, apos, &resultb, &resulta, &move, 1,
+		     &result_certain);
+  gprintf("Semeai defense of %1m: result %s %1m\n",
+	  bpos, result_to_string(resultb), move);
+  gprintf("Semeai attack of %1m: result %s %1m\n",
+	  apos, result_to_string(resulta), move);
+  gprintf("%d nodes%s\n", count_variations,
+	  result_certain ? "" : ", uncertain result");
+
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
 void
-decide_tactical_semeai(int apos, int bpos, const char *sgf_output)
+decide_tactical_semeai(int apos, int bpos)
 {
   SGFTree tree;
-  int resulta, resultb, move;
+  int resulta, resultb, move, dummy;
   int color = board[apos];
 
   if (color == EMPTY || board[bpos] != OTHER_COLOR(color)) {
@@ -341,16 +385,17 @@ decide_tactical_semeai(int apos, int bpos, const char *sgf_output)
    * from the cache. Thus we clear the cache here. */
   reading_cache_clear();
   
-  if (sgf_output)
-    begin_sgftreedump(&tree);
-  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 0);
+  if (*outfilename)
+    sgffile_begindump(&tree);
+
+  owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 0, &dummy);
   gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
 	  color == BLACK ? "black" : "white",
 	  move,
 	  apos, safety_to_string(resulta),
   	  bpos, safety_to_string(resultb),
 	  count_variations);
-  owl_analyze_semeai(bpos, apos, &resultb, &resulta, &move, 0);
+  owl_analyze_semeai(bpos, apos, &resultb, &resulta, &move, 0, &dummy);
   gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
 	  color == BLACK ? "white" : "black",
 	  move,
@@ -358,10 +403,8 @@ decide_tactical_semeai(int apos, int bpos, const char *sgf_output)
   	  bpos, safety_to_string(resultb),
 	  count_variations);
 
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
@@ -371,11 +414,12 @@ decide_tactical_semeai(int apos, int bpos, const char *sgf_output)
  */
 
 void
-decide_position(int color, const char *sgf_output)
+decide_position(int color)
 {
   int pos;
   int move = NO_MOVE;
   int acode = 0, dcode = 0;
+  int kworm;
   static const char *snames[] = {"dead", "alive", "critical", "unknown"};
   SGFTree tree;
 
@@ -388,8 +432,8 @@ decide_position(int color, const char *sgf_output)
    * from the cache. Thus we clear the cache here. */
   reading_cache_clear();
 
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
 
   count_variations = 1;
 
@@ -402,8 +446,8 @@ decide_position(int color, const char *sgf_output)
 
     gprintf("\nanalyzing %1m\n", pos);
     gprintf("status=%s, escape=%d\n", 
-	    snames[dragon[pos].status], DRAGON2(pos).escape_route);
-    acode = owl_attack(pos, &move, NULL);
+	    snames[dragon[pos].crude_status], DRAGON2(pos).escape_route);
+    acode = owl_attack(pos, &move, NULL, &kworm);
     if (acode) {
       if (acode == WIN) {
 	if (move == NO_MOVE)
@@ -418,9 +462,12 @@ decide_position(int color, const char *sgf_output)
       else if (acode == KO_B)
 	gprintf("%1m can be attacked with ko (bad) at %1m (%d variations)\n", 
 		pos, move, count_variations);
+      else if (acode == GAIN)
+	gprintf("%1m can be attacked with gain (captures %1m) at %1m (%d variations)", 
+		pos, kworm, move, count_variations);
       
       count_variations = 1;
-      dcode = owl_defend(pos, &move, NULL);
+      dcode = owl_defend(pos, &move, NULL, &kworm);
       if (dcode) {
 	if (dcode == WIN) {
 	  if (move == NO_MOVE)
@@ -435,6 +482,9 @@ decide_position(int color, const char *sgf_output)
 	else if (dcode == KO_B)
 	  gprintf("%1m can be defended with ko (bad) at %1m (%d variations)\n",
 		  pos, move, count_variations);
+	else if (dcode == LOSS)
+	  gprintf("%1m can be defended with loss (loses %1m) at %1m (%d variations)", 
+		  pos, kworm, move, count_variations);
       }
       else
 	gprintf("%1m cannot be defended (%d variations)\n", 
@@ -454,22 +504,21 @@ decide_position(int color, const char *sgf_output)
       gprintf("status of %1m revised to ALIVE\n", pos);
   }
   
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
 /*
- * Evaluates the eyespace at (pos) and prints a report.
+ * Evaluates the eyespace at (pos) and prints a report. You can get
+ * more information by adding -d0x02 to the command line.
  */
 
 void
-decide_eye(int pos, const char *sgf_output)
+decide_eye(int pos)
 {
   int color;
-  int max, min;
+  struct eyevalue value;
   int attack_point;
   int defense_point;
   int eyepos;
@@ -490,44 +539,36 @@ decide_eye(int pos, const char *sgf_output)
   if (printboard)
     showboard(0);
 
-  if (life)
-    reset_life_node_counter();
-
   /* Enable sgf output. */
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
   count_variations = 1;
   
   if (black_eye[pos].color == BLACK_BORDER) {
     eyepos = black_eye[pos].origin;
-    compute_eyes(eyepos, &max, &min, &attack_point, &defense_point,
+    compute_eyes(eyepos, &value, &attack_point, &defense_point,
 		 black_eye, half_eye, 0, EMPTY);
-    gprintf("Black eyespace at %1m: min=%d, max=%d\n", eyepos, min, max);
-    if (max != min) {
-      gprintf("  vital points: %1m (attack) %1m (defense)\n", attack_point,
-	      defense_point);
-    }
-  }
-  if (white_eye[pos].color == WHITE_BORDER) {
-    eyepos = white_eye[pos].origin;
-    compute_eyes(eyepos, &max, &min, &attack_point, &defense_point,
-		 white_eye, half_eye, 0, EMPTY);
-    gprintf("White eyespace at %1m: min=%d, max=%d\n", eyepos, min, max);
-    if (max != min) {
+    gprintf("Black eyespace at %1m: %s\n", eyepos, eyevalue_to_string(&value));
+    if (eye_move_urgency(&value) > 0) {
       gprintf("  vital points: %1m (attack) %1m (defense)\n", attack_point,
 	      defense_point);
     }
   }
   
-  if (life)
-    printf("%d positions examined by the life module.\n",
-	   get_life_node_counter());
-
-  /* Finish sgf output. */
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
+  if (white_eye[pos].color == WHITE_BORDER) {
+    eyepos = white_eye[pos].origin;
+    compute_eyes(eyepos, &value, &attack_point, &defense_point,
+		 white_eye, half_eye, 0, EMPTY);
+    gprintf("White eyespace at %1m: %s\n", eyepos, eyevalue_to_string(&value));
+    if (eye_move_urgency(&value) > 0) {
+      gprintf("  vital points: %1m (attack) %1m (defense)\n", attack_point,
+	      defense_point);
+    }
   }
+  
+  /* Finish sgf output. */
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
@@ -537,34 +578,92 @@ decide_eye(int pos, const char *sgf_output)
  */
 
 void
-decide_combination(int color, const char *sgf_output)
+decide_combination(int color)
 {
   int attack_move;
-  int defense_move;
+  char defense_moves[BOARDMAX];
   SGFTree tree;
+  int first = 1;
+  int pos;
 
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
   silent_examine_position(color, EXAMINE_ALL);
 
-  if (sgf_output)
-    begin_sgftreedump(&tree);
+  if (*outfilename)
+    sgffile_begindump(&tree);
   count_variations = 1;
 
-  if (atari_atari(color, &attack_move, &defense_move, verbose))
-    gprintf("Combination attack for %C at %1m, defense at %1m\n", color,
-	    attack_move, defense_move);
+  if (atari_atari(color, &attack_move, defense_moves, verbose)) {
+    gprintf("Combination attack for %C at %1m, defense at ", color,
+	    attack_move);
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+      if (ON_BOARD(pos) && defense_moves[pos]) {
+	if (first)
+	  first = 0;
+	else
+	  gprintf(", ");
+	gprintf("%1m", pos);
+      }
+    }
+    gprintf("\n");
+  }
   else
     gprintf("No Combination attack for %C\n", color);
   
-  if (sgf_output) {
-    end_sgftreedump(sgf_output);
-    count_variations = 0;
-  }
+  sgffile_enddump(outfilename);
+  count_variations = 0;
 }
 
 
+void
+decide_surrounded(int pos)
+{
+  int surround_status;
+
+  if (board[pos] == EMPTY) {
+    fprintf(stderr, "location must not be empty!\n");
+    return;
+  }
+
+  /* Prepare pattern matcher and reading code. */
+  reset_engine();
+
+  silent_examine_position(board[pos], EXAMINE_ALL);
+  surround_status = compute_surroundings(pos, NO_MOVE, 1, NULL);
+  if (surround_status == 1)
+    gprintf("the dragon at %1m is SURROUNDED!\n", pos);
+  else if (surround_status == 2)
+    gprintf("the dragon at %1m is WEAKLY SURROUNDED!\n", pos);
+  else
+    gprintf("the dragon at %1m is not surrounded.\n", pos);
+}  
+
+
+#if ORACLE
+
+void
+decide_oracle(Gameinfo *gameinfo, char *infilename, char *untilstring)
+{
+  SGFTree tree;
+
+  reset_engine();
+  if (*outfilename)
+    sgffile_begindump(&tree);
+
+  count_variations = 1;
+  summon_oracle();
+  oracle_loadsgf(infilename, untilstring);
+  consult_oracle(gameinfo->to_move);
+  sgffile_enddump(outfilename);
+  dismiss_oracle();
+  count_variations = 0;
+}
+
+#endif
+
+
 /*
  * Local Variables:
  * tab-width: 8
