@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -29,22 +30,13 @@
 
 /* Maximum number of dragons considered by a, B, C, and d class patterns. */
 #define MAX_DRAGONS_PER_PATTERN 5
+#define MAX_STRINGS_PER_PATTERN 5
 
 /* Values of joseki patterns. */
-#define U_VALUE 40
-#define J_VALUE 27
-#define j_VALUE 20
-#define t_VALUE 15
-
-/* Values of joseki patterns if using experimental influence. */
-#define EXP_J_VALUE 35
-#define EXP_j_VALUE 22
-#define EXP_t_VALUE 17
-
-/* Global variables set to joseki values acc. to experimental_influence. */
-float J_value = J_VALUE;
-float j_value = j_VALUE;
-float t_value = t_VALUE;
+#define U_VALUE 40.0
+#define J_VALUE 35.0
+#define j_VALUE 24.0
+#define t_VALUE 16.0
 
 
 /* 
@@ -52,61 +44,65 @@ float t_value = t_VALUE;
  */
 
 static void
-shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
+shapes_callback(int anchor, int color, struct pattern *pattern, int ll,
 		void *data)
 {
+  int other = OTHER_COLOR(color);
+  
   int k, l;
   int move;
+  /* For restricted search, the pattern must intersect the search area */
   
   /* Dragons of our color. */
   int my_dragons[MAX_DRAGONS_PER_PATTERN];
   int my_ndragons = 0;
-
+  
   /* Dragons of other color. */
   int your_dragons[MAX_DRAGONS_PER_PATTERN];
   int your_ndragons = 0;
 
+  /* Strings of our color. */
+  int my_strings[MAX_STRINGS_PER_PATTERN];
+  int my_nstrings = 0;
+  
+  /* Strings of other color. */
+  int your_strings[MAX_STRINGS_PER_PATTERN];
+  int your_nstrings = 0;
+
   /* Make a local copy of the classification that we may modify. */
   unsigned int class = pattern->class;
-  
-  /* Y patterns get rejected if F flag does not match with
-   * experimental_influence mode.
-   */
-  if (class & CLASS_Y)
-    if (!((experimental_influence && !(class & CLASS_F))
-	  ||(!experimental_influence && (class & CLASS_F))))
-      return;
 
   /* Don't accept fuseki marked patterns while scoring. */
   if (doing_scoring && (class & CLASS_F))
     return;
-  
+
   /* Don't need auxiliary data in this callback. */
   UNUSED(data);
-  
+
   /* Pick up the location of the move */
-  move = AFFINE_TRANSFORM(pattern->movei, pattern->movej, ll, m, n);
-  
+  move = AFFINE_TRANSFORM(pattern->move_offset, ll, anchor);
+
+  /* For restricted search, the pattern must intersect the search area */
+  if (limit_search && !within_search_area(move))
+    return;
+
   /* For some classes of patterns we need to find all dragons present
    * in the pattern.
    */
   if ((class & (CLASS_B | CLASS_C | CLASS_c | CLASS_a | CLASS_d | CLASS_O |
-		CLASS_J | CLASS_j | CLASS_U | CLASS_T | CLASS_t)) != 0)
-  {
-    int other = OTHER_COLOR(color);
-
+		CLASS_J | CLASS_j | CLASS_U | CLASS_T | CLASS_t)) != 0) {
     /* Match each point. */
-    for (k = 0; k < pattern->patlen; ++k) { 
+    for (k = 0; k < pattern->patlen; ++k) {
       int pos; /* absolute (board) co-ord of (transformed) pattern element */
       int origin; /* dragon origin */
       
       /* all the following stuff (currently) applies only at occupied cells */
       if (pattern->patn[k].att == ATT_dot)
 	continue;
-
+      
       /* transform pattern real coordinate */
-      pos = AFFINE_TRANSFORM(pattern->patn[k].x, pattern->patn[k].y, ll, m, n);
-
+      pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
+      
       /* Already, matchpat rejects O patterns containing a friendly stone with
        * DEAD or CRITICAL matcher_status. If the stone is tactically
        * CRITICAL it still could have matcher_status ALIVE since it might
@@ -151,16 +147,54 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
 	  your_dragons[l] = origin;
 	  your_ndragons++;
 	}
-      }      
+      }
+      
+      if (pattern->patn[k].att == ATT_O || pattern->patn[k].att == ATT_X) {
+	origin = find_origin(pos);
+	if (board[pos] == color && my_nstrings < MAX_STRINGS_PER_PATTERN) {
+	  for (l = 0; l < my_nstrings; l++) {
+	    if (my_strings[l] == origin)
+	      break;
+	  }
+	  if (l == my_nstrings) {
+	    /* We found another string of our color. Check that it
+	     * cannot be tactically captured before adding it to the
+	     * list of my_strings.
+	     */
+	    if (worm[pos].attack_codes[0] == 0
+		|| does_defend(move, pos)) {
+	      /* Ok, add the string to the list. */
+	      my_strings[l] = origin;
+	      my_nstrings++;
+	    }
+	  }
+	}
+	
+	if (board[pos] == other && your_nstrings < MAX_STRINGS_PER_PATTERN) {
+	  for (l = 0; l < your_nstrings; l++) {
+	    if (your_strings[l] == origin)
+	      break;
+	  }
+	  if (l == your_nstrings) {
+	    /* We found another opponent string, add it to the list. */
+	    your_strings[l] = origin;
+	    your_nstrings++;
+	  }
+	}
+      }
     } /* loop over elements */
   } /* if we need to loop over the elements */
 
   /* Nothing to connect. Remove C class bit. */
-  if (my_ndragons < 2)
+  if (my_ndragons < 2 && !experimental_connections)
+    class &= ~CLASS_C;
+  if (my_nstrings < 2 && experimental_connections)
     class &= ~CLASS_C;
 
   /* Nothing to cut. Remove B class bit. */
-  if (your_ndragons < 2)
+  if (your_ndragons < 2 && !experimental_connections)
+    class &= ~CLASS_B;
+  if (your_nstrings < 2 && experimental_connections)
     class &= ~CLASS_B;
   
   /*
@@ -201,7 +235,7 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
    */
   if (class & CLASS_n) {
     /* Allow ko unsafety. */
-    if (safe_move(move, OTHER_COLOR(color)) == 0) {
+    if (safe_move(move, other) == 0) {
       if (0)
 	TRACE("  opponent can't play safely at %1m, move discarded\n", move);
       return;
@@ -212,7 +246,7 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
    * if the pattern must be rejected.
    */
   if (pattern->autohelper_flag & HAVE_CONSTRAINT) {
-    if (!pattern->autohelper(pattern, ll, move, color, 0))
+    if (!pattern->autohelper(ll, move, color, 0))
       return;
   }
 
@@ -241,21 +275,25 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
   }
   
   /* does the pattern have an action? */
-  if (pattern->autohelper_flag & HAVE_ACTION) {
-    pattern->autohelper(pattern, ll, move, color, 1);
-  }
+  if (pattern->autohelper_flag & HAVE_ACTION)
+    pattern->autohelper(ll, move, color, 1);
 
-  /* Pattern class B, cut all combinations of opponent dragons. */
+  /* Pattern class B, try to cut all combinations of opponent strings. */
   if (class & CLASS_B) {
-    for (k = 0; k < your_ndragons; k++)
-      for (l = k+1; l < your_ndragons; l++) {
-	add_cut_move(move, your_dragons[k], your_dragons[l]);
-	TRACE("...cuts dragons %1m, %1m\n", your_dragons[k], your_dragons[l]);
+    for (k = 0; k < your_nstrings; k++)
+      for (l = k+1; l < your_nstrings; l++) {
+	if (string_connect(your_strings[k], your_strings[l], NULL)
+	    && !play_connect_n(color, 1, 1, move,
+			       your_strings[k], your_strings[l])) {
+	  add_cut_move(move, your_strings[k], your_strings[l]);
+	  TRACE("...cuts strings %1m, %1m\n",
+		your_strings[k], your_strings[l]);
+	}
       }
   }
 
   /* Pattern class C, connect all combinations of our dragons. */
-  if (class & CLASS_C) {
+  if ((class & CLASS_C) && !experimental_connections) {
     for (k = 0; k < my_ndragons; k++)
       for (l = k+1; l < my_ndragons; l++) {
 	add_connection_move(move, my_dragons[k], my_dragons[l]);
@@ -263,6 +301,20 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
       }
   }
 
+  /* Pattern class C, try to connect all combinations of our strings. */
+  if ((class & CLASS_C) && experimental_connections) {
+    for (k = 0; k < my_nstrings; k++)
+      for (l = k+1; l < my_nstrings; l++) {
+	if (disconnect(my_strings[k], my_strings[l], NULL)
+	    && !play_connect_n(color, 0, 1, move,
+			       my_strings[k], my_strings[l])) {
+	  add_connection_move(move, my_strings[k], my_strings[l]);
+	  TRACE("...connects strings %1m, %1m\n",
+		my_strings[k], my_strings[l]);
+	}
+      }
+  }
+
   /* Pattern class c, add strategical defense move reason for all our
    * dragons and a small shape bonus.
    *
@@ -279,11 +331,8 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
     TRACE("...shape value 1\n");
   }
 
-  /* Pattern class b, block to secure territory. */
-  if (class & CLASS_b) {
-    add_block_territory_move(move);
-    TRACE("...blocks territory\n");
-  }
+  /* Pattern class b is obsolete in the pattern databases handled here. */
+  gg_assert(!(class & CLASS_b));
 
   /* Pattern class e, expand to make territory. */
   if (class & CLASS_e) {
@@ -297,6 +346,12 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
     TRACE("...expands moyo\n");
   }
 
+  /* Pattern class i, an invasion. */
+  if (class & CLASS_I) {
+    add_invasion_move(move);
+    TRACE("...is an invasion\n");
+  }
+
   /* Pattern class a, strategical level attack on all opponent dragons. */
   if (class & CLASS_a) {
     for (k = 0; k < your_ndragons; k++) {
@@ -322,16 +377,16 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
     TRACE("...expands territory\n");
     add_expand_moyo_move(move);
     TRACE("...expands moyo\n");
-    set_minimum_move_value(move, J_value);
-    TRACE("... minimum move value %f\n", J_value);
+    set_minimum_move_value(move, J_VALUE);
+    TRACE("... minimum move value %f\n", J_VALUE);
   }
 
   /* Pattern class j, less urgent joseki move. Add expand territory and
-   * moyo, set a minimum value of j_value. If it is a fuseki pattern, set also
-   * the maximum value to j_value.
+   * moyo, set a minimum value of j_VALUE. If it is a fuseki pattern, set also
+   * the maximum value to j_VALUE.
    */
   if (class & CLASS_j) {
-    float min_value = j_value;
+    float min_value = j_VALUE;
     TRACE("...less urgent joseki move\n");
     add_expand_territory_move(move);
     TRACE("...expands territory\n");
@@ -350,6 +405,7 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
       min_value *= 1.005; /* Otherwise, j patterns not of CLASS_F would */
                           /* get preferred in value_move_reasons */
       set_maximum_move_value(move, min_value);
+      scale_randomness(move, 5.);
       TRACE("...move value %f (shape %f)\n", min_value, pattern->shape);
     }
     else 
@@ -362,7 +418,7 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
    * Set the (min-)value at t_value
    */
   if (class & CLASS_t) {
-    float min_value = t_value;
+    float min_value = t_VALUE;
     TRACE("...minor joseki move\n");
     
     /* Board size modification. */
@@ -377,15 +433,16 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
       min_value *= 1.005; /* Otherwise, j patterns not of CLASS_F would */
                           /* get preferred in value_move_reasons */
       set_maximum_move_value(move, min_value);
+      scale_randomness(move, 5.);
       TRACE("...move value %f (shape %f)\n", min_value, pattern->shape);
     }
-    else 
+    else
       TRACE("...minimum move value %f (shape %f)\n",
             min_value, pattern->shape);
     set_minimum_move_value(move, min_value);
   }
 
-  /* Pattern class U, very urgent move joseki. Add strategical defense
+  /* Pattern class U, very urgent joseki move. Add strategical defense
    * and attack, plus a shape bonus of 15 and a minimum value of 40.
    */
   if (class & CLASS_U) {
@@ -394,7 +451,6 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
       add_strategical_defense_move(move, my_dragons[k]);
       TRACE("...strategical defense of %1m\n", my_dragons[k]);
     }
-    urgent = 1;
     for (k = 0; k < your_ndragons; k++) {
       add_strategical_attack_move(move, your_dragons[k]);
       TRACE("...strategical attack on %1m\n", your_dragons[k]);
@@ -464,8 +520,206 @@ shapes_callback(int m, int n, int color, struct pattern *pattern, int ll,
 }
 
 
+/* This callback is invoked for each matched pattern from joseki database.
+ * This function is just a copy of relevant parts of shapes_callback().
+ */
+static void
+joseki_callback(int move, int color, struct corner_pattern *pattern,
+		int trans, int *stones, int num_stones)
+{
+  int k, l;
+  int class = pattern->class;
+
+  /* Dragons of our color. */
+  int my_dragons[MAX_DRAGONS_PER_PATTERN];
+  int my_ndragons = 0;
+
+  /* Dragons of other color. */
+  int your_dragons[MAX_DRAGONS_PER_PATTERN];
+  int your_ndragons = 0;
+
+  /* For restricted search, the pattern must intersect the search area. */
+  if (limit_search && !within_search_area(move))
+    return;
+
+  /* For urgent joseki patterns we need to find all dragons present in the
+   * pattern since such patterns are assumed to have strategical effect on
+   * them.
+   */
+  if (class & CLASS_U) {
+    /* Loop over all stones in the pattern. */
+    for (k = 0; k < num_stones; k++) {
+      int pos = stones[k];
+      int origin = dragon[pos].origin;
+
+      if (board[pos] == color && my_ndragons < MAX_DRAGONS_PER_PATTERN) {
+	for (l = 0; l < my_ndragons; l++) {
+	  if (my_dragons[l] == origin)
+	    break;
+	}
+
+	if (l == my_ndragons) {
+	  /* We found another dragon of our color. Check that it (or
+           * rather the underlying worm) cannot be tactically
+           * captured before adding it to the list of my_dragons.
+	   */
+	  if (worm[pos].attack_codes[0] == 0 || does_defend(move, pos)) {
+	    /* Ok, add the dragon to the list. */
+	    my_dragons[l] = origin;
+	    my_ndragons++;
+	  }
+	}
+      }
+
+      if (board[pos] != color && your_ndragons < MAX_DRAGONS_PER_PATTERN) {
+	for (l = 0; l < your_ndragons; l++) {
+	  if (your_dragons[l] == origin)
+	    break;
+	}
+	if (l == your_ndragons) {
+	  /* We found another opponent dragon, add it to the list. */
+	  your_dragons[l] = origin;
+	  your_ndragons++;
+	}
+      }
+    }
+  }
+
+  /* For joseki patterns we don't check if the proposed move is safe or legal.
+   */
+
+  /* If the pattern has a constraint, call the autohelper to see
+   * if the pattern must be rejected.
+   */
+  if (pattern->autohelper_flag & HAVE_CONSTRAINT) {
+    if (!pattern->autohelper(trans, move, color, 0))
+      return;
+  }
+
+  /* If using -a, want to see all matches even if not -v. */
+  if (allpats || verbose)
+    TRACE("pattern '%s'+%d matched at %1m\n", pattern->name, trans, move);
+
+  /* Does the pattern have an action? */
+  if (pattern->autohelper_flag & HAVE_ACTION)
+    pattern->autohelper(trans, move, color, 1);
+
+  /* Pattern class J, joseki standard move. Add expand territory and
+   * moyo, and require the value at least J_value.
+   */
+  if (class & CLASS_J) {
+    TRACE("...joseki standard move\n");
+    add_expand_territory_move(move);
+    TRACE("...expands territory\n");
+    add_expand_moyo_move(move);
+    TRACE("...expands moyo\n");
+    set_minimum_move_value(move, J_VALUE);
+    TRACE("... minimum move value %f\n", J_VALUE);
+  }
+
+  /* Pattern class j, less urgent joseki move. Add expand territory and
+   * moyo, set a minimum value of j_VALUE. If it is a fuseki pattern, set also
+   * the maximum value to j_VALUE.
+   */
+  if (class & CLASS_j) {
+    float min_value = j_VALUE;
+    TRACE("...less urgent joseki move\n");
+    add_expand_territory_move(move);
+    TRACE("...expands territory\n");
+    add_expand_moyo_move(move);
+    TRACE("...expands moyo\n");
+
+    /* Board size modification. */
+    min_value *= board_size / 19.0;
+
+    if (class & VALUE_SHAPE) {
+      min_value *= (1 + 0.01 * pattern->shape);
+      class &= ~VALUE_SHAPE;
+    };
+
+    if (board_size >= 17) {
+      min_value *= 1.005; /* Otherwise, j patterns not of CLASS_F would */
+                          /* get preferred in value_move_reasons(). */
+      set_maximum_move_value(move, min_value);
+      scale_randomness(move, 5.);
+      TRACE("...move value %f (shape %f)\n", min_value, pattern->shape);
+    }
+    else
+      TRACE("...minimum move value %f (shape %f)\n",
+            min_value, pattern->shape);
+    set_minimum_move_value(move, min_value);
+  }
+
+  /* Pattern class t, minor joseki move (tenuki OK).
+   * Set the (min-)value at t_value
+   */
+  if (class & CLASS_t) {
+    float min_value = t_VALUE;
+    TRACE("...minor joseki move\n");
+
+    /* Board size modification. */
+    min_value *= board_size / 19.0;
+
+    if (class & VALUE_SHAPE) {
+      min_value *= (1 + 0.01 * pattern->shape);
+      class &= ~VALUE_SHAPE;
+    };
+
+    if ((board_size >= 17) && (class & CLASS_F)) {
+      min_value *= 1.005; /* Otherwise, j patterns not of CLASS_F would */
+                          /* get preferred in value_move_reasons */
+      set_maximum_move_value(move, min_value);
+      scale_randomness(move, 5.);
+      TRACE("...move value %f (shape %f)\n", min_value, pattern->shape);
+    }
+    else
+      TRACE("...minimum move value %f (shape %f)\n",
+            min_value, pattern->shape);
+    set_minimum_move_value(move, min_value);
+  }
+
+  /* Pattern class U, very urgent joseki move. Add strategical defense
+   * and attack, plus a shape bonus of 15 and a minimum value of 40.
+   */
+  if (class & CLASS_U) {
+    TRACE("...joseki urgent move\n");
+    for (k = 0; k < my_ndragons; k++) {
+      add_strategical_defense_move(move, my_dragons[k]);
+      TRACE("...strategical defense of %1m\n", my_dragons[k]);
+    }
+    for (k = 0; k < your_ndragons; k++) {
+      add_strategical_attack_move(move, your_dragons[k]);
+      TRACE("...strategical attack on %1m\n", your_dragons[k]);
+    }
+    add_shape_value(move, 15);
+    TRACE("...shape value 15\n");
+    set_minimum_move_value(move, U_VALUE);
+    TRACE("...(min) move value %f\n", U_VALUE);
+  }
+
+  /* Pattern class T, joseki trick move. For the moment we never play these. */
+  if (class & CLASS_T) {
+    TRACE("...joseki trick move\n");
+    add_antisuji_move(move);
+    TRACE("...antisuji\n");
+  }
+
+  /* Pattern class N, antisuji move. */
+  if (class & CLASS_N) {
+    TRACE("...antisuji move\n");
+    add_antisuji_move(move);
+  }
+
+  /* Shape value specified. */
+  if (class & VALUE_SHAPE) {
+    add_shape_value(move, pattern->shape);
+    TRACE("...shape value %f\n", pattern->shape);
+  }
+}
+
+
 /*
- * Match all patterns in patterns.db and patterns2.db on all positions.  
+ * Match all patterns in patterns.db and patterns2.db on all positions.
  *
  * This function is one of the basic generators of move reasons, called
  * by genmove().
@@ -476,17 +730,15 @@ shapes(int color)
   TRACE("\nPattern matcher is looking for move reasons for %s!\n",
 	color_to_string(color));
 
-  /* Modify joseki values if using experimental influence. */
-  if (experimental_influence) {
-    J_value = EXP_J_VALUE;
-    j_value = EXP_j_VALUE;
-    t_value = EXP_t_VALUE;
-  }
-
   matchpat(shapes_callback, color, &pat_db, NULL, NULL);
 
-  if (josekidb)
+  /* Don't match joseki patterns while scoring. */
+  if (josekidb && !doing_scoring)
+#if 1
+    corner_matchpat(joseki_callback, color, &joseki_db);
+#else
     matchpat(shapes_callback, color, &joseki_db, NULL, NULL);
+#endif
 
   if (!disable_fuseki && !doing_scoring)
     matchpat(shapes_callback, color, &fusekipat_db, NULL, NULL);
