@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -149,16 +150,16 @@ is_hoshi_point(int m, int n)
 
 /* Print a line with coordinate letters above the board. */
 static void
-draw_letter_coordinates(void)
+draw_letter_coordinates(FILE *outfile)
 {
   int i;
   int ch;
   
-  fprintf(stderr, "  ");
+  fprintf(outfile, "  ");
   for (i = 0, ch = 'A'; i < board_size; i++, ch++) {
     if (ch == 'I')
       ch++;
-    fprintf(stderr, " %c", ch);
+    fprintf(outfile, " %c", ch);
   }
 }
 
@@ -186,7 +187,7 @@ void
 start_draw_board()
 {
   gg_init_color();
-  draw_letter_coordinates();
+  draw_letter_coordinates(stderr);
 }
 
 /* Draw a colored character. If c has the value EMPTY, either a "." or
@@ -234,7 +235,7 @@ void
 end_draw_board()
 {
   fprintf(stderr, "\n");
-  draw_letter_coordinates();
+  draw_letter_coordinates(stderr);
   fprintf(stderr, "\n");
 }
 
@@ -254,10 +255,12 @@ static void
 showchar(int i, int j, int empty, int xo)
 {
   struct dragon_data *d;  /* dragon data at (i, j) */
+  struct dragon_data2 *d2;
   int x;
   ASSERT_ON_BOARD2(i, j);
   x = BOARD(i, j);
   d = &(dragon[POS(i, j)]);
+  d2 = &(dragon2[d->id]);
 
   if (x == EMPTY) {
     if (xo != 2)
@@ -317,7 +320,7 @@ showchar(int i, int j, int empty, int xo)
     
     /* Now draw it. */
     if (xo == 1)
-      write_color_char(colors[BOARD(i, j)][d->status], w);
+      write_color_char(colors[BOARD(i, j)][d->crude_status], w);
     else if (xo == 2) {
       if (BOARD(i, j) == BLACK)
 	write_color_char(domain_colors[1], 'X');
@@ -325,7 +328,9 @@ showchar(int i, int j, int empty, int xo)
 	write_color_char(domain_colors[2], 'O');
     }
     else if (xo == 3)
-      write_color_char(colors[BOARD(i, j)][d->owl_status], w);
+      write_color_char(colors[BOARD(i, j)][d2->owl_status], w);
+    else if (xo == 4)
+      write_color_char(colors[BOARD(i, j)][d->status], w);
   }
 }
 
@@ -339,6 +344,7 @@ showchar(int i, int j, int empty, int xo)
  * xo=1:      colored dragon display
  * xo=2:      colored eye display
  * xo=3:      colored owl display
+ * xo=4:      colored matcher status display
  *
  */
 
@@ -365,14 +371,14 @@ showboard(int xo)
     
     fprintf(stderr, " %d", ii);
     
-    if ((xo == 0) && ((board_size < 10 && i == board_size-2)
-	|| (board_size >= 10 && i == 8)))
-      fprintf(stderr, "     WHITE has captured %d stones", black_captured);
+    if (xo == 0 && ((board_size < 10 && i == board_size-2)
+		    || (board_size >= 10 && i == 8)))
+      fprintf(stderr, "     WHITE (O) has captured %d stones", black_captured);
+    
+    if (xo == 0 && ((board_size < 10 && i == board_size-1)
+		    || (board_size >= 10 && i == 9)))
+      fprintf(stderr, "     BLACK (X) has captured %d stones", white_captured);
     
-    if ((xo == 0) && ((board_size < 10 && i == board_size-1)
-	|| (board_size >= 10 && i == 9)))
-      fprintf(stderr, "     BLACK has captured %d stones", white_captured);
-  
     if (xo == 3) {
       if (i == board_size-5)
 	write_color_string(GG_COLOR_GREEN, "    green=alive");
@@ -391,6 +397,40 @@ showboard(int xo)
 }
 
 
+/* Bare bones version of showboard(0). No fancy options, no hint of
+ * color, and you can choose where to write it.
+ */
+void
+simple_showboard(FILE *outfile)
+{
+  int i, j;
+
+  draw_letter_coordinates(outfile);
+  
+  for (i = 0; i < board_size; i++) {
+    fprintf(outfile, "\n%2d", board_size - i);
+    
+    for (j = 0; j < board_size; j++) {
+      if (BOARD(i, j) == EMPTY)
+	fprintf(outfile, " %c", is_hoshi_point(i, j) ? '+' : '.');
+      else
+	fprintf(outfile, " %c", BOARD(i, j) == BLACK ? 'X' : 'O');
+    }
+
+    fprintf(outfile, " %d", board_size - i);
+    
+    if ((board_size < 10 && i == board_size-2)
+	|| (board_size >= 10 && i == 8))
+      fprintf(outfile, "     WHITE (O) has captured %d stones", black_captured);
+    
+    if ((board_size < 10 && i == board_size-1)
+	|| (board_size >= 10 && i == 9))
+      fprintf(outfile, "     BLACK (X) has captured %d stones", white_captured);
+  }
+  
+  fprintf(outfile, "\n");
+  draw_letter_coordinates(outfile);
+}
 
 /*
  * Local Variables:
