@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
  * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
@@ -118,11 +118,11 @@ display_worm(int w)
   
     wmove(info_window, 10, 19 + i * 6);
     if (worm[w].defense_points[i] == 0)
-      gg_wprintw(info_window, "---:0 ", worm[w].defend_codes[i]);
+      gg_wprintw(info_window, "---:0 ", worm[w].defense_codes[i]);
     else
       gg_wprintw(info_window, "%3s:%d ", 
 		 location_to_string(worm[w].defense_points[i]),
-		 worm[w].defend_codes[i]);
+		 worm[w].defense_codes[i]);
 
     wmove(info_window, 11, 19 + i * 6);
     if (worm[w].attack_threat_points[i] == 0)
@@ -162,7 +162,7 @@ display_worm_tactical_data(int m, int n)
     if (worm[ii].attack_codes[i] != 0)
       display_board_char(I(worm[ii].attack_points[i]),
 			 J(worm[ii].attack_points[i]), 'A');
-    if (worm[ii].defend_codes[i] != 0)
+    if (worm[ii].defense_codes[i] != 0)
       display_board_char(I(worm[ii].defense_points[i]),
 			 J(worm[ii].defense_points[i]), 'D');
     if (worm[ii].attack_threat_codes[i] != 0)
@@ -223,8 +223,18 @@ prepare_dragons_tab()
 void
 display_dragon(int pos)
 {
-  struct dragon_data *d = &dragon[pos];
-  struct dragon_data2 *d2 = &(dragon2[d->id]);
+  struct dragon_data *d;
+  struct dragon_data2 *d2;
+
+  /* FIXME: We should clear the old entries here rather than just
+   * returning when called on an empty vertex. We can't continue now,
+   * however, since we may crash on bad field values.
+   */
+  if (board[pos] == EMPTY)
+    return;
+    
+  d = &dragon[pos];
+  d2 = &(dragon2[d->id]);
   wmove(info_window, 1, 2);
   gg_wprintw(info_window, "%3s: %5s dragon ",
 	     location_to_string(pos), color_to_string(d->color));
@@ -234,6 +244,7 @@ display_dragon(int pos)
   wmove(info_window, 3, 16);
   gg_wprintw(info_window, "%3d  %5.3f ", d->size, d->effective_size);
 
+#if 0
   wmove(info_window, 8, 18);
   if (board[pos] == EMPTY)
     gg_wprintw(info_window, "    ", d2->heyes);
@@ -247,6 +258,7 @@ display_dragon(int pos)
 
   wmove(info_window, 9, 18);
   gg_wprintw(info_window, "%d  ", d2->genus);
+#endif
   wmove(info_window, 10, 18);
   gg_wprintw(info_window, "%d  ", 
 	     board[pos] == EMPTY ? 0 : d2->escape_route);
@@ -258,47 +270,47 @@ display_dragon(int pos)
 
   /* status */
   wmove(info_window, 3, 55);
-  gg_wprintw(info_window, "%s     ", status_to_string(d->status));
+  gg_wprintw(info_window, "%s     ", status_to_string(d->crude_status));
 
   /* owl status */
   wmove(info_window, 4, 55);
-  gg_wprintw(info_window, "%-12s", status_to_string(d->owl_status));
-  if (d->owl_attack_point == NO_MOVE)
+  gg_wprintw(info_window, "%-12s", status_to_string(d2->owl_status));
+  if (d2->owl_attack_point == NO_MOVE)
     gg_wprintw(info_window, "[---] ");
   else
     gg_wprintw(info_window, "[%3s] ",
-	       location_to_string(d->owl_attack_point));
-  if (d->owl_defense_point == NO_MOVE)
+	       location_to_string(d2->owl_attack_point));
+  if (d2->owl_defense_point == NO_MOVE)
     gg_wprintw(info_window, "[---] ");
   else
     gg_wprintw(info_window, "[%3s] ",
-	       location_to_string(d->owl_defense_point));
+	       location_to_string(d2->owl_defense_point));
 
   wmove(info_window, 5, 55);
-  switch (d->owl_threat_status) {
+  switch (d2->owl_threat_status) {
   case DEAD:
   case ALIVE:
     gg_wprintw(info_window, "%-12s[---] [---]", 
-	       status_to_string(d->owl_threat_status));
+	       status_to_string(d2->owl_threat_status));
     break;
   case UNCHECKED:
     gg_wprintw(info_window, "unchecked   [---] [---]");
     break;
   case CAN_THREATEN_ATTACK:
     gg_wprintw(info_window, "att. threat [%3s] [---]",
-	       location_to_string(d->owl_second_attack_point));
+	       location_to_string(d2->owl_second_attack_point));
     break;
   case CAN_THREATEN_DEFENSE:
     gg_wprintw(info_window, "def. threat [---] [%3s]",
-	       location_to_string(d->owl_second_defense_point));
+	       location_to_string(d2->owl_second_defense_point));
     break;
   default:
-    gg_wprintw(info_window, "Error: %3d  [---] [---]", d->owl_threat_status);
+    gg_wprintw(info_window, "Error: %3d  [---] [---]", d2->owl_threat_status);
     break;
   }
 
   wmove(info_window, 6, 55);
-  gg_wprintw(info_window, "%s     ", status_to_string(d->matcher_status));
+  gg_wprintw(info_window, "%s     ", status_to_string(d->status));
 
   wmove(info_window, 8, 55);
   wmove(info_window, 9, 55);
@@ -382,12 +394,14 @@ display_eye(int color, struct eye_data eyedata[BOARDMAX], int pos)
   gg_wprintw(info_window, "%d   ", eyedata[pos].esize);
   wmove(info_window, 6, 18);
   gg_wprintw(info_window, "%d   ", eyedata[pos].msize);
+#if 0
   wmove(info_window, 7, 18);
   if (eyedata[pos].dragon == NO_MOVE)
     gg_wprintw(info_window, "---");
   else
     gg_wprintw(info_window, "%s ",
 	       location_to_string(eyedata[pos].dragon));
+#endif
 
   wmove(info_window, 11, 18);
   gg_wprintw(info_window, "%d  ", eyedata[pos].type);
@@ -395,9 +409,9 @@ display_eye(int color, struct eye_data eyedata[BOARDMAX], int pos)
   gg_wprintw(info_window, "%d  ", eyedata[pos].marginal);
 
   wmove(info_window, 4, 55);
-  gg_wprintw(info_window, "%d ", eyedata[pos].maxeye);
+  gg_wprintw(info_window, "%d ", max_eyes(&eyedata[pos].value));
   wmove(info_window, 5, 55);
-  gg_wprintw(info_window, "%d ", eyedata[pos].mineye);
+  gg_wprintw(info_window, "%d ", min_eyes(&eyedata[pos].value));
 
   wmove(info_window, 6, 55);
   if (eyedata[pos].attack_point == NO_MOVE)
