@@ -1,12 +1,12 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
  * To facilitate development of the Go Text Protocol, the two    *
  * files gtp.c and gtp.h are licensed under less restrictive     *
  * terms than the rest of GNU Go.                                *
  *                                                               *
- * Copyright 2001, 2002 by the Free Software Foundation.         *
+ * Copyright 2001, 2002, 2003 by the Free Software Foundation.   *
  *                                                               *
  * Permission is hereby granted, free of charge, to any person   *
  * obtaining a copy of this file gtp.c, to deal in the Software  *
@@ -62,6 +62,12 @@ static int gtp_boardsize = -1;
 static gtp_transform_ptr vertex_transform_input_hook = NULL;
 static gtp_transform_ptr vertex_transform_output_hook = NULL;
 
+/* Current id number. We keep track of this internally rather than
+ * pass it to the functions processing the commands, since those can't
+ * do anything useful with it anyway.
+ */
+static int current_id;
+
 /* Read filehandle gtp_input linewise and interpret as GTP commands. */
 void
 gtp_main_loop(struct gtp_command commands[], FILE *gtp_input)
@@ -70,7 +76,6 @@ gtp_main_loop(struct gtp_command commands[], FILE *gtp_input)
   char command[GTP_BUFSIZE];
   char *p;
   int i;
-  int id;
   int n;
   int status = GTP_OK;
   
@@ -79,17 +84,34 @@ gtp_main_loop(struct gtp_command commands[], FILE *gtp_input)
     if (!fgets(line, GTP_BUFSIZE, gtp_input))
       break; /* EOF or some error */
 
-    /* Remove comments. */
-    if ((p = strchr(line, '#')) != NULL)
-      *p = 0;
-    
+    /* Preprocess the line. */
+    for (i = 0, p = line; line[i]; i++) {
+      char c = line[i];
+      /* Convert HT (9) to SPACE (32). */
+      if (c == 9)
+	*p++ = 32;
+      /* Remove CR (13) and all other control characters except LF (10). */
+      else if ((c > 0 && c <= 9)
+	       || (c >= 11 && c <= 31)
+	       || c == 127)
+	continue;
+      /* Remove comments. */
+      else if (c == '#')
+	break;
+      /* Keep ordinary text. */
+      else
+	*p++ = c;
+    }
+    /* Terminate string. */
+    *p = 0;
+	
     p = line;
 
     /* Look for an identification number. */
-    if (sscanf(p, "%d%n", &id, &n) == 1)
+    if (sscanf(p, "%d%n", &current_id, &n) == 1)
       p += n;
     else
-      id = -1; /* No identification number. */
+      current_id = -1; /* No identification number. */
 
     /* Look for command name. */
     if (sscanf(p, " %s %n", command, &n) < 1)
@@ -101,12 +123,12 @@ gtp_main_loop(struct gtp_command commands[], FILE *gtp_input)
      */
     for (i = 0; commands[i].name != NULL; i++) {
       if (strcmp(command, commands[i].name) == 0) {
-	status = (*commands[i].function)(p, id);
+	status = (*commands[i].function)(p);
 	break;
       }
     }
     if (commands[i].name == NULL)
-      gtp_failure(id, "unknown command: '%s'", command);
+      gtp_failure("unknown command");
 
     if (status == GTP_FATAL)
       gtp_panic();
@@ -121,7 +143,7 @@ gtp_internal_set_boardsize(int size)
 }
 
 /* If you need to transform the coordinates on input or output, use
- * this functions to set hook functions which are called any time
+ * these functions to set hook functions which are called any time
  * coordinates are read or about to be written. In GNU Go this is used
  * to simulate rotated boards in regression tests.
  */
@@ -135,7 +157,7 @@ gtp_set_vertex_transform_hooks(gtp_transform_ptr in, gtp_transform_ptr out)
 /*
  * This function works like printf, except that it only understands
  * very few of the standard formats, to be precise %c, %d, %f, %s.
- * But it also accepts %m, which takes two integers and writes a move,
+ * But it also accepts %m, which takes two integers and writes a vertex,
  * and %C, which takes a color value and writes a color string.
  */
 void 
@@ -217,17 +239,17 @@ gtp_printf(const char *format, ...)
  * given.
  */
 void
-gtp_printid(int id, int status)
+gtp_start_response(int status)
 {
   if (status == GTP_SUCCESS)
     gtp_printf("=");
   else
     gtp_printf("?");
   
-  if (id < 0)
+  if (current_id < 0)
     gtp_printf(" ");
   else
-    gtp_printf("%d ", id);
+    gtp_printf("%d ", current_id);
 }
 
 
@@ -244,10 +266,10 @@ gtp_finish_response()
  * is just like one to printf.
  */
 int
-gtp_success(int id, const char *format, ...)
+gtp_success(const char *format, ...)
 {
   va_list ap;
-  gtp_printid(id, GTP_SUCCESS);
+  gtp_start_response(GTP_SUCCESS);
   va_start(ap, format);
   vfprintf(stdout, format, ap);
   va_end(ap);
@@ -257,10 +279,10 @@ gtp_success(int id, const char *format, ...)
 
 /* Write a full failure response. The call is identical to gtp_success. */
 int
-gtp_failure(int id, const char *format, ...)
+gtp_failure(const char *format, ...)
 {
   va_list ap;
-  gtp_printid(id, GTP_FAILURE);
+  gtp_start_response(GTP_FAILURE);
   va_start(ap, format);
   vfprintf(stdout, format, ap);
   va_end(ap);
@@ -343,12 +365,13 @@ gtp_decode_coord(char *s, int *i, int *j)
 
 /* Convert a move, i.e. "b" or "w" followed by a vertex to a color and
  * coordinates. Return the number of characters read from the string
- * s.
+ * s. The vertex may be "pass" and then the coordinates are set to (-1, -1).
  */
 int
 gtp_decode_move(char *s, int *color, int *i, int *j)
 {
   int n1, n2;
+  int k;
 
   assert(gtp_boardsize > 0);
 
@@ -357,9 +380,18 @@ gtp_decode_move(char *s, int *color, int *i, int *j)
     return 0;
 
   n2 = gtp_decode_coord(s + n1, i, j);
-  if (n2 == 0)
-    return 0;
-
+  if (n2 == 0) {
+    char buf[6];
+    if (sscanf(s + n1, "%5s%n", buf, &n2) != 1)
+      return 0;
+    for (k = 0; k < (int) strlen(buf); k++)
+      buf[k] = tolower((int) buf[k]);
+    if (strcmp(buf, "pass") != 0)
+      return 0;
+    *i = -1;
+    *j = -1;
+  }
+  
   return n1 + n2;
 }
 
