@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -21,13 +22,13 @@
 
 #include "gnugo.h"
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
 #ifdef HAVE_UNISTD_H
+/* For isatty(). */
 #include <unistd.h>
 #else
 #include <io.h>
@@ -44,15 +45,12 @@
 # endif
 #endif
 
-#include <signal.h>
-
-#include <liberty.h>
+#include "liberty.h"
 
 #include "gg-getopt.h"
-#include <gg_utils.h>
+#include "gg_utils.h"
 
 #include "interface.h"
-#include "gmp.h"
 #include "sgftree.h"
 #include "random.h"
 
@@ -77,27 +75,36 @@ enum {OPT_BOARDSIZE=127,
       OPT_QUIET,
       OPT_GTP_INPUT,
       OPT_GTP_INITIAL_ORIENTATION,
+      OPT_GTP_VERSION,
       OPT_SHOWCOPYRIGHT,
       OPT_REPLAY_GAME,
       OPT_DECIDE_STRING,
       OPT_DECIDE_CONNECTION,
-      OPT_DECIDE_DRAGON,
+      OPT_DECIDE_OWL,
+      OPT_DECIDE_DRAGON_DATA,
       OPT_DECIDE_SEMEAI,
+      OPT_DECIDE_SURROUNDED,
       OPT_DECIDE_TACTICAL_SEMEAI,
+      OPT_DECIDE_ORACLE,
       OPT_EXPERIMENTAL_SEMEAI,
-      OPT_SEMEAI_VARIATIONS,
+      OPT_EXPERIMENTAL_OWL_EXT,
+      OPT_SEMEAI_NODE_LIMIT,
       OPT_EXPERIMENTAL_CONNECTIONS,
       OPT_EXPERIMENTAL_INFLUENCE,
       OPT_ALTERNATE_CONNECTIONS,
+      OPT_WITH_BREAK_IN,
+      OPT_WITHOUT_BREAK_IN,
       OPT_OPTIONS,
       OPT_STANDARD_SEMEAI,
       OPT_STANDARD_CONNECTIONS,
       OPT_STANDARD_INFLUENCE,
+      OPT_PRINT_LEVELS,
       OPT_DECIDE_POSITION,
       OPT_DECIDE_EYE,
       OPT_DECIDE_COMBINATION,
       OPT_BRANCH_DEPTH,
       OPT_BACKFILL2_DEPTH,
+      OPT_BREAK_CHAIN_DEPTH,
       OPT_SUPERSTRING_DEPTH,
       OPT_AA_DEPTH,
       OPT_DEBUG_FLAGS,
@@ -105,12 +112,13 @@ enum {OPT_BOARDSIZE=127,
       OPT_OWL_BRANCH,
       OPT_OWL_READING,
       OPT_OWL_NODE_LIMIT,
-      OPT_LIFE,
-      OPT_LIFE_EYESIZE,
       OPT_NOFUSEKIDB,
       OPT_NOFUSEKI,
       OPT_NOJOSEKIDB,
       OPT_LEVEL,
+      OPT_MIN_LEVEL,
+      OPT_MAX_LEVEL,
+      OPT_LIMIT_SEARCH,
       OPT_SHOWTIME,
       OPT_SHOWSCORE,
       OPT_DEBUG_INFLUENCE,
@@ -125,7 +133,12 @@ enum {OPT_BOARDSIZE=127,
       OPT_CAPTURE_ALL_DEAD,
       OPT_PLAY_OUT_AFTERMATH,
       OPT_ATTACK_BY_PATTERN,
-      OPT_DEFEND_BY_PATTERN
+      OPT_DEFEND_BY_PATTERN,
+      OPT_MIRROR,
+      OPT_MIRROR_LIMIT,
+      OPT_METAMACHINE,
+      OPT_RESIGN_ALLOWED,
+      OPT_NEVER_RESIGN
 };
 
 /* names of playing modes */
@@ -136,6 +149,7 @@ enum mode {
   MODE_ASCII_EMACS,
   MODE_GTP,
   MODE_GMP,
+  MODE_SGMP,
   MODE_SGF,
   MODE_LOAD_AND_ANALYZE,
   MODE_LOAD_AND_SCORE,
@@ -144,12 +158,15 @@ enum mode {
   MODE_REPLAY,
   MODE_DECIDE_STRING,
   MODE_DECIDE_CONNECTION,
-  MODE_DECIDE_DRAGON,
+  MODE_DECIDE_OWL,
+  MODE_DECIDE_DRAGON_DATA,
   MODE_DECIDE_SEMEAI,
   MODE_DECIDE_TACTICAL_SEMEAI,
   MODE_DECIDE_POSITION,
   MODE_DECIDE_EYE,
-  MODE_DECIDE_COMBINATION
+  MODE_DECIDE_COMBINATION,
+  MODE_DECIDE_SURROUNDED,
+  MODE_DECIDE_ORACLE
 };
 
 
@@ -170,9 +187,11 @@ static struct gg_option const long_options[] =
   {"orientation",    required_argument, 0, OPT_GTP_INITIAL_ORIENTATION},
   {"gtp-initial-orientation",
   		     required_argument, 0, OPT_GTP_INITIAL_ORIENTATION},
+  {"gtp-version",    required_argument, 0, OPT_GTP_VERSION},
   {"infile",         required_argument, 0, 'l'},
   {"until",          required_argument, 0, 'L'},
   {"outfile",        required_argument, 0, 'o'},
+  {"output-flags",   required_argument, 0, 'O'},
   {"boardsize",      required_argument, 0, OPT_BOARDSIZE},
   {"color",          required_argument, 0, OPT_COLOR},
   {"handicap",       required_argument, 0, OPT_HANDICAPSTONES},
@@ -182,12 +201,14 @@ static struct gg_option const long_options[] =
   {"version",        no_argument,       0, 'v'},
   {"allpats",        no_argument,       0, 'a'},
   {"printboard",     no_argument,       0, 'T'},
+  {"printeyes",      no_argument,       0, 'E'},
   {"debug",          required_argument, 0, 'd'},
   {"debug-flags",    no_argument,       0,  OPT_DEBUG_FLAGS},
   {"depth",          required_argument, 0, 'D'},
   {"backfill-depth", required_argument, 0, 'B'},
   {"branch-depth",   required_argument, 0, OPT_BRANCH_DEPTH},
   {"backfill2-depth",   required_argument, 0, OPT_BACKFILL2_DEPTH},
+  {"break_chain-depth", required_argument, 0, OPT_BREAK_CHAIN_DEPTH},
   {"superstring-depth", required_argument, 0, OPT_SUPERSTRING_DEPTH},
   {"fourlib-depth",  required_argument, 0, 'F'},
   {"ko-depth",       required_argument, 0, 'K'},
@@ -196,7 +217,11 @@ static struct gg_option const long_options[] =
   {"owl-branch",     required_argument, 0, OPT_OWL_BRANCH},
   {"owl-reading",    required_argument, 0, OPT_OWL_READING},
   {"owl-node-limit", required_argument, 0, OPT_OWL_NODE_LIMIT},
+  {"print-levels",   no_argument,       0, OPT_PRINT_LEVELS},
   {"level",          required_argument, 0, OPT_LEVEL},
+  {"min-level",      required_argument, 0, OPT_MIN_LEVEL},
+  {"max-level",      required_argument, 0, OPT_MAX_LEVEL},
+  {"limit-search",   required_argument, 0, OPT_LIMIT_SEARCH},
   {"clock",          required_argument, 0, OPT_CLOCK_TIME},
   {"byo-time",       required_argument, 0, OPT_CLOCK_BYO_TIME},
   {"byo-period",     required_argument, 0, OPT_CLOCK_BYO_PERIOD},
@@ -204,15 +229,16 @@ static struct gg_option const long_options[] =
   {"chinese-rules",  no_argument,       0, OPT_CHINESE_RULES},
   {"japanese-rules", no_argument,       0, OPT_JAPANESE_RULES},
   {"experimental-semeai",  no_argument, 0, OPT_EXPERIMENTAL_SEMEAI},
-  {"semeai-variations",   required_argument, 0, OPT_SEMEAI_VARIATIONS},
+  {"experimental-owl-ext",  no_argument, 0, OPT_EXPERIMENTAL_OWL_EXT},
+  {"semeai-node-limit",   required_argument, 0, OPT_SEMEAI_NODE_LIMIT},
   {"experimental-connections",  no_argument, 0, OPT_EXPERIMENTAL_CONNECTIONS},
   {"owl-threats",     no_argument,      0, OPT_OWL_THREATS},
   {"no-owl-threats",  no_argument,      0, OPT_NO_OWL_THREATS},
-  {"experimental-influence",  no_argument, 0, OPT_EXPERIMENTAL_INFLUENCE},
-  {"standard-influence",  no_argument, 0, OPT_STANDARD_INFLUENCE},
   {"standard-connections",  no_argument, 0, OPT_STANDARD_CONNECTIONS},
-  {"standard_semeai", no_argument,      0, OPT_STANDARD_SEMEAI},
+  {"standard-semeai", no_argument,      0, OPT_STANDARD_SEMEAI},
   {"alternate-connections",  no_argument, 0, OPT_ALTERNATE_CONNECTIONS},
+  {"with-break-in",  	no_argument, 0, OPT_WITH_BREAK_IN},
+  {"without-break-in",  no_argument, 0, OPT_WITHOUT_BREAK_IN},
   {"options",        no_argument, 0, OPT_OPTIONS},
   {"allow-suicide",  no_argument,       0, OPT_ALLOW_SUICIDE},
   {"capture-all-dead",   no_argument,   0, OPT_CAPTURE_ALL_DEAD},
@@ -227,16 +253,18 @@ static struct gg_option const long_options[] =
   {"seed",           required_argument, 0, 'r'},
   {"decide-string",  required_argument, 0, OPT_DECIDE_STRING},
   {"decide-connection", required_argument, 0, OPT_DECIDE_CONNECTION},
-  {"decide-dragon",  required_argument, 0, OPT_DECIDE_DRAGON},
+  {"decide-dragon",  required_argument, 0, OPT_DECIDE_OWL},
+  {"decide-owl",     required_argument, 0, OPT_DECIDE_OWL},
+  {"decide-dragon-data",  required_argument, 0, OPT_DECIDE_DRAGON_DATA},
   {"decide-semeai",  required_argument, 0, OPT_DECIDE_SEMEAI},
   {"decide-tactical-semeai", required_argument, 0, OPT_DECIDE_TACTICAL_SEMEAI},
-  {"decide-position", no_argument,       0, OPT_DECIDE_POSITION},
+  {"decide-position", no_argument,      0, OPT_DECIDE_POSITION},
+  {"decide-surrounded",  required_argument, 0, OPT_DECIDE_SURROUNDED},
   {"decide-eye",     required_argument, 0, OPT_DECIDE_EYE},
   {"decide-combination", no_argument,   0, OPT_DECIDE_COMBINATION},
-  {"life",           no_argument,       0, OPT_LIFE},
-  {"life-eyesize",   required_argument, 0, OPT_LIFE_EYESIZE},
+  {"decide-oracle",  no_argument,       0, OPT_DECIDE_ORACLE},
   {"nofusekidb",     no_argument,       0, OPT_NOFUSEKIDB},
-  {"nofuseki",       no_argument,         0, OPT_NOFUSEKI},
+  {"nofuseki",       no_argument,       0, OPT_NOFUSEKI},
   {"nojosekidb",     no_argument,       0, OPT_NOJOSEKIDB},
   {"debug-influence", required_argument, 0, OPT_DEBUG_INFLUENCE},
   {"showtime",       no_argument,       0, OPT_SHOWTIME},
@@ -246,13 +274,14 @@ static struct gg_option const long_options[] =
   {"profile-patterns", no_argument,     0, OPT_PROFILE_PATTERNS},
   {"attack-by-pattern", no_argument,    0, OPT_ATTACK_BY_PATTERN},
   {"defend-by-pattern", no_argument,    0, OPT_DEFEND_BY_PATTERN},
+  {"mirror",         no_argument,       0, OPT_MIRROR},
+  {"mirror-limit",   required_argument, 0, OPT_MIRROR_LIMIT},
+  {"metamachine",    no_argument,       0, OPT_METAMACHINE},
+  {"resign-allowed", no_argument,       0, OPT_RESIGN_ALLOWED},
+  {"never-resign",   no_argument,       0, OPT_NEVER_RESIGN},
   {NULL, 0, NULL, 0}
 };
 
-float memory = (float) DEFAULT_MEMORY;	  /* Megabytes used for hash table. */
-
-static void sigterm_handler(int);
-
 
 int
 main(int argc, char *argv[])
@@ -260,7 +289,7 @@ main(int argc, char *argv[])
   Gameinfo gameinfo;
   SGFTree sgftree;
 
-  int i, umove;
+  int i;
   int mandated_color = EMPTY;
   enum mode playmode = MODE_UNKNOWN;
   int replay_color = EMPTY;
@@ -269,6 +298,7 @@ main(int argc, char *argv[])
   char *untilstring = NULL;
   char *scoringmode = NULL;
   char *outfile = NULL;
+  char *outflags = NULL;
   char *gtpfile = NULL;
   
   char *printsgffile = NULL;
@@ -278,27 +308,28 @@ main(int argc, char *argv[])
   char debuginfluence_move[4] = "\0";
   
   int benchmark = 0;  /* benchmarking mode (-b) */
-  FILE *gtp_input_FILE;
+  FILE *gtp_input_FILE, *output_check;
   int orientation = 0;
 
+  float memory = (float) DEFAULT_MEMORY; /* Megabytes used for hash table. */
+
   /* If seed is zero, GNU Go will play a different game each time. If
    * it is set using -r, GNU Go will play the same game each time.
    * (Change seed to get a different game).
    */
   int seed = 0;
+  int seed_specified = 0;
 
-  board_size = 19;
   komi = 0.0;
   
-  /* Set SIGTERM handler. */
-  signal(SIGTERM, sigterm_handler);
-
   level = DEFAULT_LEVEL;
-  semeai_variations = DEFAULT_SEMEAI_VARIATIONS;
-
+  min_level = 0;
+  max_level = 10;
+  
   mandated_depth               = -1;
   mandated_backfill_depth      = -1;
   mandated_backfill2_depth     = -1;
+  mandated_break_chain_depth   = -1;
   mandated_superstring_depth   = -1;
   mandated_fourlib_depth       = -1;
   mandated_ko_depth            = -1;
@@ -310,8 +341,6 @@ main(int argc, char *argv[])
   mandated_aa_depth            = -1;
 
   debug = 0;
-  life  = 0;
-  life_eyesize = LIFE_EYESIZE;
   fusekidb = 1;
   disable_fuseki = 0;
   josekidb = 1;
@@ -323,27 +352,25 @@ main(int argc, char *argv[])
     owl_threats = 1;
   else
     owl_threats = 0;
+  experimental_owl_ext = EXPERIMENTAL_OWL_EXT;
   experimental_semeai = EXPERIMENTAL_SEMEAI;
   experimental_connections = EXPERIMENTAL_CONNECTIONS;
-  experimental_influence = EXPERIMENTAL_INFLUENCE;
 
   allow_suicide = 0;
   capture_all_dead = 0;
   play_out_aftermath = 0;
+  limit_search = 0;
+  resign_allowed = RESIGNATION_ALLOWED;
 
   /* Default parameters for clock and auto level systems. */
   clock_init(3600, 0, 0);      /* One hour sudden death. */
-  clock_init_autolevel(2, 10); /* 2 < level < 10 */
 
   sgftree_clear(&sgftree);
   gameinfo_clear(&gameinfo, board_size, komi);
   
-  /* Set some standard options. */
-  umove = BLACK;
-  
-  /* Now weed through all of the command line options. */
+  /* Weed through all of the command line options. */
   while ((i = gg_getopt_long(argc, argv, 
-			     "-ab:B:d:D:EF:gh::H:K:l:L:M:m:o:p:r:fsStTvw",
+                            "-ab:B:d:D:EF:gh::H:K:l:L:M:m:o:O:p:r:fsStTvw",
 			     long_options, NULL)) != EOF)
     {
       switch (i) {
@@ -356,15 +383,17 @@ main(int argc, char *argv[])
 	break;
 	
       case 'b': benchmark = atoi(gg_optarg); playmode = MODE_SOLO; break;
-      case 'r': seed = atoi(gg_optarg); break;
+      case 'r': seed = atoi(gg_optarg); seed_specified = 1; break;
       case 'S': showstatistics = 1; break;
       case 'w': printworms = 1; break;
-      case 'm': printmoyo = strtol(gg_optarg, NULL, 0);  /* allows 0x... */ break;
+      case 'm': printmoyo = strtol(gg_optarg, NULL, 0);  /* allows 0x... */ 
+	break;
       case 'd': debug ^= strtol(gg_optarg, NULL, 0);  /* allows 0x... */ break;
       case 'D': mandated_depth = atoi(gg_optarg); break;
 
       case 'M': memory = atof(gg_optarg); break; /* floating point number */
-      case 'H': hashflags = strtol(gg_optarg, NULL, 0);  /* allows 0x... */ break;
+      case 'H': hashflags = strtol(gg_optarg, NULL, 0);  /* allows 0x... */ 
+	break;
 
       case 'E': printboard = 2; break;
       case 'B': mandated_backfill_depth = atoi(gg_optarg); break;
@@ -377,7 +406,24 @@ main(int argc, char *argv[])
 	
       case 'o':
 	outfile = gg_optarg;
-	strcpy(gameinfo.outfilename, gg_optarg);
+	strcpy(outfilename, gg_optarg);
+	break;
+
+      case 'O':
+	outflags = gg_optarg;
+	output_flags = 0;
+	if (outflags)
+	  while (*outflags){
+	    switch (*outflags) {
+	    case 'd':
+	      output_flags |= OUTPUT_MARKDRAGONS;
+	      break;
+	    case 'v':
+	      output_flags |= OUTPUT_MOVEVALUES;
+	      break;
+	    }
+	    outflags++;
+	  }
 	break;
 	
       case OPT_QUIET:
@@ -397,10 +443,17 @@ main(int argc, char *argv[])
 	}
 	break;
 	
+      case OPT_GTP_VERSION:
+	gtp_version = atoi(gg_optarg);
+	break;
+	
       case OPT_OPTIONS:
-	if (EXPERIMENTAL_INFLUENCE)
+	if (USE_BREAK_IN)
 	  fprintf(stderr,
-		  "configure option enabled: experimental influence\n");
+		  "configure option enabled: experimental break-ins\n");
+	if (COSMIC_GNUGO)
+	  fprintf(stderr,
+		  "configure option enabled: cosmic GNU Go \n");
 	if (EXPERIMENTAL_CONNECTIONS)
 	  fprintf(stderr,
 		  "configure option enabled: experimental connections\n");
@@ -410,6 +463,9 @@ main(int argc, char *argv[])
 	if (EXPERIMENTAL_SEMEAI)
 	  fprintf(stderr,
 		  "configure option enabled: experimental semeai\n");
+	if (EXPERIMENTAL_OWL_EXT)
+	  fprintf(stderr,
+		  "configure option enabled: experimental GAIN/LOSS codes\n");
 	if (EXPERIMENTAL_READING)
 	  fprintf(stderr,
 		  "configure option enabled: experimental reading\n");
@@ -418,6 +474,11 @@ main(int argc, char *argv[])
 	if (OWL_THREATS)
 	  fprintf(stderr,
 		  "configure option enabled: owl threats\n");
+	fprintf(stderr,
+		"Owl node limit: %d\n", OWL_NODE_LIMIT);
+	fprintf(stderr,
+		"Semeai node limit: %d\n", SEMEAI_NODE_LIMIT);
+
 	return EXIT_SUCCESS;
 	break;
 
@@ -475,16 +536,24 @@ main(int argc, char *argv[])
 	owl_threats = 0;
 	break;
 
+      case OPT_METAMACHINE:
+        metamachine = 1;
+	break;
+
       case OPT_JAPANESE_RULES: 
 	chinese_rules = 0;
 	break;
 
+      case OPT_EXPERIMENTAL_OWL_EXT:
+	experimental_owl_ext = 1;
+	break;
+
       case OPT_EXPERIMENTAL_SEMEAI:
 	experimental_semeai = 1;
 	break;
 
-      case OPT_SEMEAI_VARIATIONS:
-	semeai_variations = atoi(gg_optarg);
+      case OPT_SEMEAI_NODE_LIMIT:
+	semeai_node_limit = atoi(gg_optarg);
 	break;
 
       case OPT_STANDARD_SEMEAI: 
@@ -503,12 +572,12 @@ main(int argc, char *argv[])
 	alternate_connections = !alternate_connections;
 	break;
 
-      case OPT_EXPERIMENTAL_INFLUENCE:
-	experimental_influence = 1;
+      case OPT_WITH_BREAK_IN:
+	experimental_break_in = 1;
 	break;
 
-      case OPT_STANDARD_INFLUENCE: 
-	experimental_influence = 0;
+      case OPT_WITHOUT_BREAK_IN:
+	experimental_break_in = 0;
 	break;
 
       case OPT_ALLOW_SUICIDE:
@@ -523,6 +592,14 @@ main(int argc, char *argv[])
 	play_out_aftermath = 1;
 	break;
 
+      case OPT_RESIGN_ALLOWED:
+	resign_allowed = 1;
+	break;
+
+      case OPT_NEVER_RESIGN:
+	resign_allowed = 0;
+	break;
+
       case OPT_MODE: 
 	if (strcmp(gg_optarg, "ascii") == 0) 
 	  playmode = MODE_ASCII;
@@ -532,6 +609,8 @@ main(int argc, char *argv[])
 	  playmode = MODE_GTP;
 	else if (strcmp(gg_optarg, "gmp") == 0)
 	  playmode = MODE_GMP;
+	else if (strcmp(gg_optarg, "sgmp") == 0)
+	  playmode = MODE_SGMP;
 	else {
 	  fprintf(stderr, "Invalid mode selection: %s\n", gg_optarg);
 	  fprintf(stderr, "Try `gnugo --help' for more information.\n");
@@ -567,13 +646,22 @@ main(int argc, char *argv[])
 	playmode = MODE_DECIDE_CONNECTION;
 	break;
 	
-      case OPT_DECIDE_DRAGON:
+      case OPT_DECIDE_OWL:
 	if (strlen(gg_optarg) > 3) {
 	  fprintf(stderr, "Invalid board coordinate: %s\n", gg_optarg);
 	  exit(EXIT_FAILURE);
 	}
 	strcpy(decide_this, gg_optarg);
-	playmode = MODE_DECIDE_DRAGON;
+	playmode = MODE_DECIDE_OWL;
+	break;
+	
+      case OPT_DECIDE_DRAGON_DATA:
+	if (strlen(gg_optarg) > 3) {
+	  fprintf(stderr, "Invalid board coordinate: %s\n", gg_optarg);
+	  exit(EXIT_FAILURE);
+	}
+	strcpy(decide_this, gg_optarg);
+	playmode = MODE_DECIDE_DRAGON_DATA;
 	break;
 	
       case OPT_DECIDE_SEMEAI:
@@ -608,7 +696,6 @@ main(int argc, char *argv[])
 		  "usage: --decide-tactical-semeai [first dragon]/[second dragon]\n");
 	  return EXIT_FAILURE;
 	}
-
 	playmode = MODE_DECIDE_TACTICAL_SEMEAI;
 	break;
 	
@@ -629,6 +716,19 @@ main(int argc, char *argv[])
 	playmode = MODE_DECIDE_COMBINATION;
 	break;
 	
+      case OPT_DECIDE_SURROUNDED:
+	if (strlen(gg_optarg) > 3) {
+	  fprintf(stderr, "Invalid board coordinate: %s\n", gg_optarg);
+	  exit(EXIT_FAILURE);
+	}
+	strcpy(decide_this, gg_optarg);
+	playmode = MODE_DECIDE_SURROUNDED;
+	break;
+	
+      case OPT_DECIDE_ORACLE:
+	playmode = MODE_DECIDE_ORACLE;
+	break;
+	
       case OPT_BRANCH_DEPTH:
 	mandated_branch_depth = atoi(gg_optarg);
 	break;
@@ -637,12 +737,17 @@ main(int argc, char *argv[])
 	mandated_backfill2_depth = atoi(gg_optarg);
 	break;
 	
+      case OPT_BREAK_CHAIN_DEPTH:
+	mandated_break_chain_depth = atoi(gg_optarg);
+	break;
+	
       case OPT_SUPERSTRING_DEPTH:
 	mandated_superstring_depth = atoi(gg_optarg);
 	break;
 	
       case OPT_AA_DEPTH:
 	mandated_aa_depth = atoi(gg_optarg);
+	break;
 
       case OPT_OWL_DISTRUST:
 	mandated_owl_distrust_depth = atoi(gg_optarg);
@@ -660,15 +765,6 @@ main(int argc, char *argv[])
 	mandated_owl_node_limit = atoi(gg_optarg);
 	break;
 	
-      case OPT_LIFE:
- 	life = 1;
-	break;
-
-      case OPT_LIFE_EYESIZE:
-	life = 1;
-	life_eyesize = atoi(gg_optarg);
-	break;
-	
       case OPT_NOFUSEKIDB:
 	fusekidb = 0;
 	break;
@@ -685,6 +781,27 @@ main(int argc, char *argv[])
 	level = atoi(gg_optarg);
 	break;
 
+      case OPT_MIN_LEVEL:
+	min_level = atoi(gg_optarg);
+	break;
+
+      case OPT_MAX_LEVEL:
+	max_level = atoi(gg_optarg);
+	break;
+
+      case OPT_LIMIT_SEARCH:
+	{
+	  int m, n;
+
+	  if (!string_to_location(board_size, gg_optarg, &m, &n)) {
+	    fprintf(stderr, "gnugo: use --limit-search <pos>\n");
+	    return EXIT_FAILURE;
+	  }
+	  rotate(m, n, &m, &n, board_size, orientation);
+	  set_search_diamond(POS(m, n));
+	}
+	break;
+
       case OPT_CLOCK_TIME:
 
 	clock_init(atoi(gg_optarg), -1, -1);
@@ -775,6 +892,14 @@ main(int argc, char *argv[])
 
 #endif
 
+      case OPT_MIRROR:
+        play_mirror_go = 1;
+        break;
+
+      case OPT_MIRROR_LIMIT:
+        mirror_stones_limit = atoi(gg_optarg);
+        break;
+
       case 'v':
 	show_version();
 	return EXIT_SUCCESS;
@@ -808,6 +933,16 @@ main(int argc, char *argv[])
 	break;
 
 	/* NOTE: getopt returns '?' if an illegal option is supplied. */
+
+      case OPT_PRINT_LEVELS:
+	{
+	  int lev;
+	  for(lev = 10; lev > 0; lev--)
+	    set_depth_values(lev, 1);
+	}
+	return EXIT_SUCCESS;
+	break;
+
       case '?':
       default:
 	fprintf(stderr, "Try `gnugo --help' for more information.\n");
@@ -824,16 +959,12 @@ main(int argc, char *argv[])
   }
   
   /* Start random number seed. */
-  if (!seed)
+  if (!seed_specified)
     seed = time(0);
-  gg_srand(seed);
-  random_seed = seed;
-
   
   /* Initialize the GNU Go engine. */
-  init_gnugo(memory);
+  init_gnugo(memory, seed);
 
-  
   /* Read the infile if there is one. Also play up the position. */
   if (infilename) {
     if (!sgftree_readfile(&sgftree, infilename)) {
@@ -841,12 +972,11 @@ main(int argc, char *argv[])
       exit(EXIT_FAILURE);
     }
     
-    gameinfo_play_sgftree_rot(&gameinfo, sgftree.root,
-			      untilstring, orientation);
+    gameinfo_play_sgftree_rot(&gameinfo, &sgftree, untilstring,
+                             orientation);
   }
-
+  else
   /* Initialize and empty sgf tree if there was no infile. */
-  if (!sgftree.root)
     sgftreeCreateHeaderNode(&sgftree, board_size, komi);
 
   /* Set the game_record to be identical to the loaded one or the
@@ -869,25 +999,37 @@ main(int argc, char *argv[])
     else
       playmode = (isatty(0)) ? MODE_ASCII : MODE_GMP;
   }
-  
-  if (outfile) {
-    if (playmode != MODE_DECIDE_STRING
-	&& playmode != MODE_DECIDE_CONNECTION
-	&& playmode != MODE_DECIDE_DRAGON
-	&& playmode != MODE_DECIDE_POSITION
-	&& playmode != MODE_DECIDE_SEMEAI
-	&& playmode != MODE_DECIDE_TACTICAL_SEMEAI
-	&& playmode != MODE_DECIDE_EYE
-	&& playmode != MODE_DECIDE_COMBINATION)
-      if (!sgffile_open_file(outfile)) {
-	fprintf(stderr, "Error: could not open '%s'\n", gg_optarg);
-	exit(EXIT_FAILURE);
-      }
+
+  if (outfile && playmode != MODE_LOAD_AND_PRINT) {
+    output_check = fopen(outfile, "w");
+    if (!output_check) {
+      fprintf(stderr, "Error: could not open '%s' for writing\n", outfile);
+      exit(EXIT_FAILURE);
+    }
+    fclose(output_check);
   }
-  
+
   switch (playmode) {
-  case MODE_GMP:     
-    play_gmp(&gameinfo);
+  case MODE_GMP:
+  case MODE_SGMP:
+
+    /* not supported by the protocol */
+    resign_allowed = 0;
+
+#if ORACLE
+    if (metamachine)
+      summon_oracle();
+#endif
+
+    /* EMPTY is valid for play_gmp.c. */
+    gameinfo.computer_player = mandated_color;
+    play_gmp(&gameinfo, playmode == MODE_SGMP);
+
+#if ORACLE
+    if (metamachine)
+      dismiss_oracle();
+#endif
+
     break;
     
   case MODE_SOLO:
@@ -895,26 +1037,42 @@ main(int argc, char *argv[])
     break;
     
   case MODE_REPLAY:    
-    if (!sgftree.root) {
+    if (!infilename) {
       fprintf(stderr, "You must use -l infile with replay mode.\n");
       exit(EXIT_FAILURE);
     }
-    play_replay(sgftree.root, replay_color);
+    play_replay(&gameinfo, replay_color);
     break;
     
   case MODE_LOAD_AND_ANALYZE:
     if (mandated_color != EMPTY)
       gameinfo.to_move = mandated_color;
     
-    if (!sgftree.root) {
+    if (!infilename) {
       fprintf(stderr, "You must use -l infile with load and analyze mode.\n");
       exit(EXIT_FAILURE);
     }
-    load_and_analyze_sgf_file(&gameinfo, benchmark);
+
+#if ORACLE
+    if (metamachine) {
+      summon_oracle();
+      oracle_loadsgf(infilename, untilstring);
+    }
+#endif
+
+    load_and_analyze_sgf_file(&gameinfo);
+
+#if ORACLE
+    dismiss_oracle();
+#endif
+
     break;
     
   case MODE_LOAD_AND_SCORE:
-    if (!sgftree.root) {
+    if (mandated_color != EMPTY)
+      gameinfo.to_move = mandated_color;
+
+    if (!infilename) {
       fprintf(stderr, "gnugo: --score must be used with -l\n");
       exit(EXIT_FAILURE);
     }
@@ -922,14 +1080,16 @@ main(int argc, char *argv[])
     break;
     
   case MODE_LOAD_AND_PRINT:
-    if (!sgftree.root) {
+    if (!infilename) {
       fprintf(stderr, "gnugo: --printsgf must be used with -l\n");
       exit(EXIT_FAILURE);
     }
+
     else {
-      sgffile_open_file(printsgffile);
-      sgffile_write_gameinfo(&gameinfo, "load and print"); 
-      sgffile_printboard(gameinfo.to_move);
+      if (mandated_color != EMPTY)
+        gameinfo.to_move = mandated_color;
+
+      sgffile_printsgf(gameinfo.to_move, printsgffile);
     }
     break;
     
@@ -948,7 +1108,7 @@ main(int argc, char *argv[])
       }
 
       rotate(m, n, &m, &n, board_size, orientation);
-      decide_string(POS(m, n), outfile);
+      decide_string(POS(m, n));
     }
     break;
   
@@ -962,22 +1122,24 @@ main(int argc, char *argv[])
       }
       
       if (!string_to_location(board_size, decide_this, &ai, &aj)) {
-	fprintf(stderr, "usage: --decide-connection [first string]/[second string]\n");
+	fprintf(stderr,
+		"usage: --decide-connection [first string]/[second string]\n");
 	return EXIT_FAILURE;
       }
       
       if (!string_to_location(board_size, decide_that, &bi, &bj)) {
-	fprintf(stderr, "usage: --decide-connection [first string]/[second string]\n");
+	fprintf(stderr,
+		"usage: --decide-connection [first string]/[second string]\n");
 	return EXIT_FAILURE;
       }
 
       rotate(ai, aj, &ai, &aj, board_size, orientation);
       rotate(bi, bj, &bi, &bj, board_size, orientation);
-      decide_connection(POS(ai, aj), POS(bi, bj), outfile);
+      decide_connection(POS(ai, aj), POS(bi, bj));
     }
-  break;
+    break;
   
-  case MODE_DECIDE_DRAGON:
+  case MODE_DECIDE_OWL:
     {
       int m, n;
       
@@ -992,7 +1154,26 @@ main(int argc, char *argv[])
       }
 
       rotate(m, n, &m, &n, board_size, orientation);
-      decide_dragon(POS(m, n), outfile);
+      decide_owl(POS(m, n));
+    }
+    break;
+  
+  case MODE_DECIDE_DRAGON_DATA:
+    {
+      int m, n;
+      
+      if (!infilename) {
+	fprintf(stderr, "gnugo: --decide-dragon-data must be used with -l\n");
+	return EXIT_FAILURE;
+      }
+      
+      if (!string_to_location(board_size, decide_this, &m, &n)) {
+	fprintf(stderr, "gnugo: --decide-dragon-data: strange coordinate \n");
+	return EXIT_FAILURE;
+      }
+
+      rotate(m, n, &m, &n, board_size, orientation);
+      decide_dragon_data(POS(m, n));
     }
     break;
   
@@ -1019,7 +1200,7 @@ main(int argc, char *argv[])
 
       rotate(ai, aj, &ai, &aj, board_size, orientation);
       rotate(bi, bj, &bi, &bj, board_size, orientation);
-      decide_semeai(POS(ai, aj), POS(bi, bj), outfile);
+      decide_semeai(POS(ai, aj), POS(bi, bj));
     }
     break;
     
@@ -1047,7 +1228,7 @@ main(int argc, char *argv[])
 
       rotate(ai, aj, &ai, &aj, board_size, orientation);
       rotate(bi, bj, &bi, &bj, board_size, orientation);
-      decide_tactical_semeai(POS(ai, aj), POS(bi, bj), outfile);
+      decide_tactical_semeai(POS(ai, aj), POS(bi, bj));
     }
     break;
     
@@ -1062,7 +1243,7 @@ main(int argc, char *argv[])
       color = gameinfo.to_move;
       if (mandated_color != EMPTY)
 	color = mandated_color;
-      decide_position(color, outfile);
+      decide_position(color);
     }
     break;
     
@@ -1081,7 +1262,7 @@ main(int argc, char *argv[])
       }
       
       rotate(m, n, &m, &n, board_size, orientation);
-      decide_eye(POS(m, n), outfile);
+      decide_eye(POS(m, n));
     }
     break;
   
@@ -1095,10 +1276,41 @@ main(int argc, char *argv[])
       color = gameinfo.to_move;
       if (mandated_color != EMPTY)
 	color = mandated_color;
-      decide_combination(color, outfile);
+      decide_combination(color);
     }
     break;
     
+  case MODE_DECIDE_SURROUNDED:
+    {
+      int m, n;
+
+      if (!string_to_location(board_size, decide_this, &m, &n)) {
+	fprintf(stderr, 
+		"usage: --decide-surrounded [pos]\n");
+	return EXIT_FAILURE;
+      }
+
+      rotate(m, n, &m, &n, board_size, orientation);
+      decide_surrounded(POS(m, n));
+      break;
+    }
+
+#if ORACLE
+  case MODE_DECIDE_ORACLE:
+    {
+      if (mandated_color != EMPTY)
+	gameinfo.to_move = mandated_color;
+      
+      if (!infilename) {
+	fprintf(stderr, "You must use -l infile with load and analyze mode.\n");
+	exit(EXIT_FAILURE);
+      }
+
+      decide_oracle(&gameinfo, infilename, untilstring);
+      break;
+    }
+#endif
+
   case MODE_GTP:  
     if (gtpfile != NULL) {
       gtp_input_FILE = fopen(gtpfile, "r");
@@ -1115,66 +1327,30 @@ main(int argc, char *argv[])
 
   case MODE_ASCII_EMACS:  
     if (mandated_color != EMPTY)
-      umove = mandated_color;
-    gameinfo.computer_player = OTHER_COLOR(umove);
+      gameinfo.computer_player = OTHER_COLOR(mandated_color);
+
     play_ascii_emacs(&sgftree, &gameinfo, infilename, untilstring);
     break;
 
   case MODE_ASCII:  
   default:     
     if (mandated_color != EMPTY)
-      umove = mandated_color;
-    gameinfo.computer_player = OTHER_COLOR(umove);
+      gameinfo.computer_player = OTHER_COLOR(mandated_color);
+
     play_ascii(&sgftree, &gameinfo, infilename, untilstring);
     break;
   }
   
-  sgffile_close_file();
-  
   if (profile_patterns)
     report_pattern_profiling();
 
-  clock_report_autolevel(NULL, OTHER_COLOR(umove));
+  clock_report_autolevel(NULL, gameinfo.computer_player);
  
   return 0;
 }  /* end main */
 
 
 
-/* 
- * Cgoban sends us a sigterm when it wants us to die. But it doesn't
- * close the pipe, so we cannot rely on gmp to pick up an error.
- * 
- * We want to keep control so we can close the output sgf file
- * properly, so we trap the signal.
- */
-
-volatile int time_to_die = 0;   /* set by signal handlers */
-
-static void 
-sigterm_handler(int sig)
-{
-  time_to_die = 1;
-  if (!quiet)
-    write(2, "*SIGTERM*\n", 10);  /* bad to use stdio in a signal handler */
-
-  close(0);                  /* this forces gmp.c to return an gmp_err */
-
-  /* I thought signal handlers were one-shot, yet on my linux box it is kept.
-   * Restore the default behaviour so that a second signal has the
-   * original effect - in case we really are stuck in a loop.
-   */
-  signal(sig, SIG_DFL);
-
-  /* schedule a SIGALRM in 5 seconds, in case we haven't cleaned up by then
-   * - cgoban sends the SIGTERM only once 
-   */
-#ifdef HAVE_ALARM
-  alarm(5);
-#endif
-}
-
-
 static void
 show_version(void)
 {
@@ -1199,11 +1375,12 @@ show_version(void)
 Usage: gnugo [-opts]\n\
 \n\
 Main Options:\n\
-       --mode <mode>     Force the playing mode ('ascii', 'gmp', or 'gtp').\n\
-                         Default is ASCII.\n\
+       --mode <mode>     Force the playing mode ('ascii', 'gmp', 'sgmp',\n\
+                         or 'gtp'). Default is ASCII.\n\
                          If no terminal is detected GMP (Go Modem Protocol)\n\
                          will be assumed.\n\
        --quiet           Don't print copyright and informational messages\n\
+       --level <amount>  strength (default %d, up to 10 supported)\n\
        --gtp-input <file>Read gtp commands from file instead of stdin\n\
    -l, --infile <file>   Load name sgf file\n\
    -L, --until <move>    Stop loading just before move is played. <move>\n\
@@ -1211,36 +1388,41 @@ Main Options:\n\
    -o, --outfile <file>  Write sgf output to file\n\
    --printsgf <file>     Write position as a diagram to file (use with -l)\n\
 \n\
-Options that affect strength (higher = stronger, slower):\n\
+Other options affecting strength (higher = stronger, slower):\n\
+(Unless you are working on the program --level <n> is all you need.)\n\
    -D, --depth <depth>          deep reading cutoff (default %d)\n\
    -B, --backfill-depth <depth> deep reading cutoff (default %d)\n\
    -F, --fourlib-depth <depth>  deep reading cutoff (default %d)\n\
    -K, --ko-depth <depth>       deep reading cutoff (default %d)\n\
    --branch-depth <depth>       deep reading cutoff (default %d)\n\
    --backfill2-depth <depth>    deep reading cutoff (default %d)\n\
+   --break_chain-depth <depth>  deep reading cutoff (default %d)\n\
    --superstring-depth <depth>  deep reading cutoff (default %d)\n\
    --aa-depth <depth>           deep reading cutoff (default %d)\n\
    --owl-distrust <depth>       owl distrust depth (default %d)\n\
    --owl-branch <depth>         owl branching depth (default %d)\n\
    --owl-reading <depth>        owl reading depth (default %d)\n\
    --owl-node-limit <limit>     max nodes for owl reading (default %d)\n\
-   --level <amount>             strength (default %d, up to 10 supported)\n\
-   --autolevel                  adapt gnugo level during game to respect\n\
-                                the time specified by --clock <sec>.\n\
 "
 
-#define USAGE1 "\n\
+#define USAGE1 "\
+   --min-level <amount>         minimum level for adjustment schemes\n\
+   --max-level <amount>         maximum level for adjustment schemes\n\
+   --autolevel                  adapt gnugo level during game to respect\n\
+                                the time specified by --clock <sec>.\n\
 Experimental options:\n\
-   --life                  use life code\n\
-   --life-eyesize <n>      use life code, life_eyesize=n (default %d)\n\
+   --with-break-in         use the break-in code (on at level 10 by default)\n\
+   --without-break-in      do not use the break-in code\n\
    --nofusekidb            turn off fuseki database\n\
    --nofuseki              turn off fuseki moves entirely\n\
    --nojosekidb            turn off joseki database\n\
+   --mirror                try to play mirror go\n\
+   --mirror-limit <n>      stop mirroring when n stones on board\n\n\
 Scoring:\n\
    --score estimate        estimate score at loaded position\n\
    --score finish          generate moves to finish game, then score\n\
    --score aftermath       generate moves to finish, use best algorithm\n\
-   --score aftermath --capture-all-dead --chinese rules   Tromp-Taylor score\n\
+   --score aftermath --capture-all-dead --chinese-rules   Tromp-Taylor score\n\
 \n\
 Cache size (higher=more memory usage, faster unless swapping occurs):\n\
    -M, --cache-size <megabytes>  RAM cache for hashing (default %4.1f Mb)\n\
@@ -1252,7 +1434,7 @@ Game Options: (--mode ascii)\n\
        --komi <num>      Set the komi\n\
        --clock <sec>     Initialize the timer.\n\
        --byo-time <sec>  Initialize the byo-yomi timer.\n\
-       --byo-period <stones>  Initialize the byo-yomi period.\n\n\
+       --byo-period <stones>  Initialize the byo-yomi period.\n\
 \n\
 Informative Output:\n\
    -v, --version         Display the version of GNU Go\n\
@@ -1265,9 +1447,10 @@ Informative Output:\n\
 
 
 #define COPYRIGHT "\n\n\
-This is GNU GO, a Go program. Contact gnugo@gnu.org, or see\n\
+This is GNU Go, a Go program. Contact gnugo@gnu.org, or see\n\
 http://www.gnu.org/software/gnugo/ for more information.\n\n\
-Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation.\n\n\
+Copyright 1999, 2000, 2001, 2002 and 2003 by\n\
+the Free Software Foundation.\n\n\
 This program is free software; you can redistribute it and/or\n\
 modify it under the terms of the GNU General Public License\n\
 as published by the Free Software Foundation - version 2.\n\n\
@@ -1294,7 +1477,7 @@ Debugging Options:\n\
                      both:  replay all moves\n\
    -a, --allpats                 test all patterns\n\
    -T, --printboard              colored display of dragons\n\
-   -E                            colored display of eye spaces\n\
+   -E, --printeyes               colored display of eye spaces\n\
    -d, --debug <flags>           debugging output (see next item for bits)\n\
    --debug-flags                 print the debug flags for previous item\n\
    -H, --hash <level>            hash (see gnugo.h for bits)\n\
@@ -1303,6 +1486,10 @@ Debugging Options:\n\
    -b, --benchmark num           benchmarking mode - can be used with -l\n\
    -S, --statistics              print statistics (for debugging purposes)\n\n\
    -t, --trace                   verbose tracing\n\
+   -O, --output-flags <flags>    optional output (use with -o)\n\
+                    d: mark dead and critical dragons\n\
+                    v: show values of considered moves\n\
+                    specify either 'd', 'v' or 'dv' (nothing by default)\n\
    --showtime                    print timing diagnostic\n\
    --replay <color>              replay game. Use with -o.\n\
    --showscore                   print estimated score\n\
@@ -1316,7 +1503,6 @@ Debugging Options:\n\
        --decide-eye <string>     evaluate the eye\n\
        --decide-combination      search for combination attack (try with -o)\n\
        --genmove <color>         generate a move for color\n\
-       --life <eyesize>          use eye reading code\n\
        --nofusekidb              turn off fuseki database\n\
        --nofuseki                turn off fuseki moves entirely\n\
        --nojosekidb              turn off joseki database\n\
@@ -1341,7 +1527,7 @@ DEBUG_READING               0x0200\n\
 DEBUG_WORMS                 0x0400\n\
 DEBUG_MOVE_REASONS          0x0800\n\
 DEBUG_OWL_PERFORMANCE       0x1000\n\
-DEBUG_LIFE                  0x2000\n\
+DEBUG_BREAKIN               0x2000\n\
 DEBUG_FILLLIB               0x4000\n\
 DEBUG_READING_PERFORMANCE   0x8000\n\
 DEBUG_SCORING               0x010000\n\
@@ -1349,7 +1535,10 @@ DEBUG_AFTERMATH             0x020000\n\
 DEBUG_ATARI_ATARI           0x040000\n\
 DEBUG_READING_CACHE         0x080000\n\
 DEBUG_TERRITORY             0x100000\n\
-DEBUG_OWL_PERSISTENT_CACHE  0x200000\n\
+DEBUG_OWL_PERSISTENT_CACHE  0X200000\n\
+DEBUG_TOP_MOVES             0x400000\n\
+DEBUG_MISCELLANEOUS         0x800000\n\
+DEBUG_ORACLE_STREAM         0x1000000\n\
 "
 
 /*
@@ -1359,14 +1548,13 @@ DEBUG_OWL_PERSISTENT_CACHE  0x200000\n\
 static void
 show_help(void)
 {
-  set_depth_values(DEFAULT_LEVEL);
-  fprintf(stderr, USAGE,
+  set_depth_values(DEFAULT_LEVEL, 0);
+  fprintf(stderr, USAGE, level,
 	  depth, backfill_depth, fourlib_depth, ko_depth, branch_depth,
-	  backfill2_depth, superstring_depth, aa_depth, 
+	  backfill2_depth, break_chain_depth, superstring_depth, aa_depth, 
 	  owl_distrust_depth, owl_branch_depth,
-	  owl_reading_depth, owl_node_limit, DEFAULT_LEVEL);
+	  owl_reading_depth, owl_node_limit);
   fprintf(stderr, USAGE1,
-	   LIFE_EYESIZE,
 	  (float) DEFAULT_MEMORY, MIN_BOARD, MAX_BOARD, MAX_HANDICAP);
 }
 
