@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,12 +25,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <ctype.h>
 
 #include "liberty.h"
 #include "interface.h"
 #include "sgftree.h"
+#include "gg_utils.h"
 
 #define DEBUG_COMMANDS "\
 capture <pos>    try to capture indicated group\n\
@@ -46,24 +47,25 @@ showterri        display territory\n\
 static int opt_showboard = 1;
 static int showdead = 0;
 static int emacs = 0;
-SGFTree sgftree;
-SGFNode *curnode = 0;
+static SGFTree sgftree;
 static int last_move_i;      /* The position of the last move */
 static int last_move_j;      /* -""-                          */
+static int resignation_allowed;
 
 /* Unreasonable score used to detect missing information. */
 #define NO_SCORE 4711
 /* Keep track of the score estimated before the last computer move. */
 static int current_score_estimate = NO_SCORE;
 
-static void endgame(Gameinfo *gameinfo);
+static int ascii_endgame(Gameinfo *gameinfo, int reason);
+static void ascii_count(Gameinfo *gameinfo);
 static void showcapture(char *line);
 static void showdefense(char *line);
 static void ascii_goto(Gameinfo *gameinfo, char *line);
-static int ascii2pos(char *line, int *i, int *j);
+static void ascii_free_handicap(Gameinfo *gameinfo, char *handicap);
 
 /* If sgf game info is written can't reset parameters like handicap, etc. */
-static int sgf_initialized = 0;
+static int sgf_initialized;
 
 /*
  * Create letterbar for the top and bottom of the ASCII board.
@@ -114,14 +116,6 @@ set_handicap_spots(int boardsize)
   
   memset(hspots, '.', sizeof(hspots));
 
-  /* small sizes are easier to hardwire... */
-  if (boardsize == 2 || boardsize == 4)
-    return;
-  if (boardsize == 3) {
-    /* just the middle one */
-    hspots[boardsize/2][boardsize/2] = '+';
-    return;
-  }
   if (boardsize == 5) {
     /* place the outer 4 */
     hspots[1][1] = '+';
@@ -201,8 +195,8 @@ ascii_showboard(void)
   set_handicap_spots(board_size);
 
   printf("\n");
-  printf("    White has captured %d pieces\n", black_captured);
-  printf("    Black has captured %d pieces\n", white_captured);
+  printf("    White (O) has captured %d pieces\n", black_captured);
+  printf("    Black (X) has captured %d pieces\n", white_captured);
   if (showscore) {
     if (current_score_estimate == NO_SCORE)
       printf("    No score estimate is available yet.\n");
@@ -230,7 +224,7 @@ ascii_showboard(void)
 	pos_is_move = 128;
       else
 	pos_is_move = 0;
-      dead = (matcher_status(POS(i, j))==DEAD) && showdead;
+      dead = (dragon_status(POS(i, j))==DEAD) && showdead;
       switch (BOARD(i, j) + pos_is_move + last_pos_was_move) {
 	case EMPTY+128:
 	case EMPTY:
@@ -266,7 +260,7 @@ ascii_showboard(void)
 	  last_pos_was_move = 0;
 	  break;
 	default: 
-	  fprintf(stderr, "Illegal board value %d\n", BOARD(i, j));
+	  fprintf(stderr, "Illegal board value %d\n", (int) BOARD(i, j));
 	  exit(EXIT_FAILURE);
 	  break;
       }
@@ -301,49 +295,51 @@ static void
 show_commands(void)
 {
   printf("\nCommands:\n");
-  printf(" back             Take back your last move\n");
-  printf(" boardsize        Set boardsize (on move 1 only!)\n");
-  printf(" comment          Write a comment to outputfile\n");
-  printf(" depth <num>      Set depth for reading\n");
-  printf(" display          Display game board\n");
-  printf(" exit             Exit GNU Go\n");
-  printf(" force <move>     Force a move for current color\n");
-  printf(" forward          Go to next node in game tree\n");
-  printf(" goto <movenum>   Go to movenum in game tree\n");
-  printf(" level <amount>   Playing level (default = 10)\n");
-  printf(" handicap         Set handicap (on move 1 only!)\n");
-  printf(" help             Display this help menu\n");
-  printf(" helpdebug        Display debug help menu\n");
-  printf(" info             Display program settings\n");
-  printf(" komi             Set komi (on move 1 only!)\n");
-  printf(" last             Goto last node in game tree\n");
-  printf(" pass             Pass on your move\n");
-  printf(" play <num>       Play <num> moves\n");
-  printf(" playblack        Play as Black (switch if White)\n");
-  printf(" playwhite        Play as White (switch if Black)\n");
-  printf(" quit             Exit GNU Go\n");
-  printf(" resign           Resign the current game\n");
-  printf(" save <file>      Save the current game\n");
-  printf(" load <file>      Load a game from file\n");
-  printf(" score            Toggle display of score On/Off\n");
-  printf(" showboard        Toggle display of board On/Off\n");
-  printf(" switch           Switch the color you are playing\n");
-  printf(" undo             Take the last move back (same as back)\n");
-  printf(" <move>           A move of the format <letter><number>");
+  printf(" back                Take back your last move\n");
+  printf(" boardsize           Set boardsize (on move 1 only)\n");
+  printf(" comment             Write a comment to outputfile\n");
+  printf(" depth <num>         Set depth for reading\n");
+  printf(" display             Display game board\n");
+  printf(" exit                Exit GNU Go\n");
+  printf(" force <move>        Force a move for current color\n");
+  printf(" forward             Go to next node in game tree\n");
+  printf(" goto <movenum>      Go to movenum in game tree\n");
+  printf(" level <amount>      Playing level (default = 10)\n");
+  printf(" handicap <num>      Set fixed handicap (on move 1 only)\n");
+  printf(" freehandicap <num>  Place free handicap (on move 1 only)\n");
+  printf("                     Omit <num> to place handicap yourself\n");
+  printf(" help                Display this help menu\n");
+  printf(" helpdebug           Display debug help menu\n");
+  printf(" info                Display program settings\n");
+  printf(" komi                Set komi (on move 1 only)\n");
+  printf(" last                Goto last node in game tree\n");
+  printf(" pass                Pass on your move\n");
+  printf(" play <num>          Play <num> moves\n");
+  printf(" playblack           Play as Black (switch if White)\n");
+  printf(" playwhite           Play as White (switch if Black)\n");
+  printf(" quit                Exit GNU Go\n");
+  printf(" resign              Resign the current game\n");
+  printf(" save <file>         Save the current game\n");
+  printf(" load <file>         Load a game from file\n");
+  printf(" score               Toggle display of score On/Off\n");
+  printf(" showboard           Toggle display of board On/Off\n");
+  printf(" switch              Switch the color you are playing\n");
+  printf(" undo                Take the last move back (same as back)\n");
+  printf(" <move>              A move of the format <letter><number>");
   printf("\n");
 }
 
 enum commands {INVALID=-1, END, EXIT, QUIT, RESIGN, 
 	       PASS, MOVE, FORCE, SWITCH,
 	       PLAY, PLAYBLACK, PLAYWHITE,
-	       SETHANDICAP, SETBOARDSIZE, SETKOMI,
+	       SETHANDICAP, FREEHANDICAP, SETBOARDSIZE, SETKOMI,
 	       SETDEPTH,
                INFO, DISPLAY, SHOWBOARD, HELP, UNDO, COMMENT, SCORE,
                CMD_DEAD, CMD_BACK, CMD_FORWARD, CMD_LAST,
                CMD_CAPTURE, CMD_DEFEND,
                CMD_HELPDEBUG, CMD_SHOWAREA, CMD_SHOWMOYO, CMD_SHOWTERRI,
                CMD_GOTO, CMD_SAVE, CMD_LOAD, CMD_SHOWDRAGONS, CMD_LISTDRAGONS,
-	       SETHURRY, SETLEVEL, NEW, COUNT
+	       SETHURRY, SETLEVEL, NEW, COUNT, CONTINUE
 };
 
 
@@ -378,7 +374,8 @@ get_command(char *command)
   if (!strncmp(command, "showdragons", 9)) return CMD_SHOWDRAGONS;
   if (!strncmp(command, "listdragons", 9)) return CMD_LISTDRAGONS;
   if (!strncmp(command, "boardsize", 9)) return SETBOARDSIZE;
-  if (!strncmp(command, "handicap", 8)) return SETHANDICAP;
+  if (!strncmp(command, "freehandicap", 9)) return FREEHANDICAP;
+  if (!strncmp(command, "handicap", 5)) return SETHANDICAP;
   if (!strncmp(command, "display", 7)) return DISPLAY;
   if (!strncmp(command, "helpdebug", 7)) return CMD_HELPDEBUG;
   if (!strncmp(command, "resign", 6)) return RESIGN;
@@ -412,7 +409,8 @@ get_command(char *command)
   if (!strncmp(command, "last", 2)) return CMD_LAST;
   if (!strncmp(command, "goto", 2)) return CMD_GOTO;
   if (!strncmp(command, "game", 2)) return NEW;
-  if (!strncmp(command, "count", 2)) return COUNT;
+  if (!strncmp(command, "count", 3)) return COUNT;
+  if (!strncmp(command, "continue", 4)) return CONTINUE;
 
   /* No valid command was found. */
   return INVALID;
@@ -424,17 +422,17 @@ get_command(char *command)
  */
 
 static void
-init_sgf(Gameinfo *ginfo, SGFNode *root)
+init_sgf(Gameinfo *ginfo)
 {
   if (sgf_initialized)
     return;
   sgf_initialized = 1;
 
-  sgffile_write_gameinfo(ginfo, "ascii");
+  sgf_write_header(sgftree.root, 1, get_random_seed(), komi,
+      		   level, chinese_rules);
+  sgfOverwritePropertyInt(sgftree.root, "HA", ginfo->handicap);
   if (ginfo->handicap > 0)
-    gnugo_recordboard(root);
-  else
-    ginfo->to_move = BLACK;
+    gnugo_recordboard(sgftree.root);
 }
 
 
@@ -442,36 +440,52 @@ init_sgf(Gameinfo *ginfo, SGFNode *root)
  * Generate the computer move. 
  */
 
-static void
+static int
 computer_move(Gameinfo *gameinfo, int *passes)
 {
   int i, j;
   int move_val;
+  int resignation_declined = 0;
+
+  init_sgf(gameinfo);
 
-  init_sgf(gameinfo, sgftree.root);
-  
   /* Generate computer move. */
   move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
+  if (resignation_allowed && move_val < 0.0 && ON_BOARD2(i, j)) {
+    int state = ascii_endgame(gameinfo, 2);
+    if (state != -1)
+      return state;
+
+    /* The opponent declined resignation. Remember not to resign again. */
+    resignation_allowed = 0;
+    resignation_declined = 1;
+    move_val = -move_val;
+  }
+
   if (showscore) {
-    gnugo_estimate_score(&lower_bound, &upper_bound);
+    gnugo_estimate_score(&upper_bound, &lower_bound);
     current_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
   }
     
   last_move_i = i;
   last_move_j = j;
   
-  mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move), movenum+1, i, j);
+  mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move),
+	  movenum+1, i, j);
   if (is_pass(POS(i, j)))
     (*passes)++;
   else
     *passes = 0;
 
   gnugo_play_move(i, j, gameinfo->to_move);
-  curnode = sgfAddPlay(curnode, gameinfo->to_move, i, j);
+  sgffile_add_debuginfo(sgftree.lastnode, move_val);
+  sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
+  if (resignation_declined)
+    sgftreeAddComment(&sgftree, "GNU Go resignation was declined");
+  sgffile_output(&sgftree);
 
-  sgffile_move_made(i, j, gameinfo->to_move, move_val);
-  
   gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
+  return 0;
 }
 
 
@@ -479,28 +493,28 @@ computer_move(Gameinfo *gameinfo, int *passes)
  * Make a move.
  */
 
-static void
+static int
 do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
 {
   int i, j;
 
-  if (!ascii2pos(command, &i, &j)) {
+  if (!string_to_location(board_size, command, &i, &j)) {
     printf("\nInvalid move: %s\n", command);
-    return;
+    return 0;
   }
   
   if (!gnugo_is_legal(i, j, gameinfo->to_move)) {
     printf("\nIllegal move: %s", command);
-    return;
+    return 0;
   }
 
   *passes = 0;
   TRACE("\nyour move: %m\n\n", i, j);
+  init_sgf(gameinfo);
   gnugo_play_move(i, j, gameinfo->to_move);
-  /* FIXME: This call to init_sgf should not be here. */
-  init_sgf(gameinfo, sgftree.root);
-  sgffile_move_made(i, j, gameinfo->to_move, 0);
-  curnode = sgfAddPlay(curnode, gameinfo->to_move, i, j);
+  sgffile_add_debuginfo(sgftree.lastnode, 0);
+  sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
+  sgffile_output(&sgftree);
 
   last_move_i = i;
   last_move_j = j;
@@ -509,13 +523,16 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
     ascii_showboard();
     printf("GNU Go is thinking...\n");
   }
+
   if (force) {
     gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
     gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
-    return;
+    sgftreeAddComment(&sgftree, "forced");
+    return 0;
   }
+
   gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
-  computer_move(gameinfo, passes);
+  return computer_move(gameinfo, passes);
 }
 
 
@@ -523,21 +540,24 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
  * Make a pass.
  */
 
-static void
+static int
 do_pass(Gameinfo *gameinfo, int *passes, int force)
 {
   (*passes)++;
+  init_sgf(gameinfo);
   gnugo_play_move(-1, -1, gameinfo->to_move);
-  init_sgf(gameinfo, sgftree.root);
-  sgffile_move_made(-1, -1, gameinfo->to_move, 0);
-  curnode = sgfAddPlay(curnode, gameinfo->to_move, -1, -1);
+  sgffile_add_debuginfo(sgftree.lastnode, 0);
+  sgftreeAddPlay(&sgftree, gameinfo->to_move, -1, -1);
+  sgffile_output(&sgftree);
 
   gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
   if (force) {
     gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
-    return;
+    sgftreeAddComment(&sgftree, "forced");
+    return 0;
   }
-  computer_move(gameinfo, passes);
+
+  return computer_move(gameinfo, passes);
 }
 
 
@@ -549,7 +569,7 @@ void
 play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 {
   int m, num;
-  int sz = 0;
+  int sz;
   float fnum;
   int passes = 0;  /* two passes and its over */
   int tmp;
@@ -565,46 +585,45 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
   setbuf(stdout, NULL); /* else set it to completely UNBUFFERED */
 #endif
   
+  sgftree = *tree;
+
+  if (filename) {
+    gameinfo_load_sgfheader(gameinfo, sgftree.root);
+    gameinfo->to_move = gameinfo_play_sgftree(gameinfo, &sgftree, until);
+    sgf_initialized = 1;
+  }
+  else {
+    if (sgfGetIntProperty(sgftree.root, "SZ", &sz)) 
+      gnugo_clear_board(sz);
+    if (gameinfo->handicap == 0)
+      gameinfo->to_move = BLACK;
+    else {
+      gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
+      gameinfo->to_move = WHITE;
+    }
+    sgf_initialized = 0;
+  }
+
   while (state == 1) {
+    state = 0;
 
-    sgftree = *tree;
-    
     /* No score is estimated yet. */
     current_score_estimate = NO_SCORE;
-    
-    if (filename) {
-      gameinfo_load_sgfheader(gameinfo, sgftree.root);
-      sgffile_write_gameinfo(gameinfo, "ascii");
-      gameinfo->to_move = gameinfo_play_sgftree(gameinfo, sgftree.root, until);
-      sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
-      sgf_initialized = 1;
-      curnode = sgftreeNodeCheck(&sgftree, 0);
-    }
-    else {
-      if (sz)
-	sgfOverwritePropertyInt(sgftree.root, "SZ", sz);
-      if (sgfGetIntProperty(sgftree.root, "SZ", &sz)) 
-	gnugo_clear_board(sz);
-      if (gameinfo->handicap == 0)
-	gameinfo->to_move = BLACK;
-      else {
-	gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
-	sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
-	gameinfo->to_move = WHITE;
-      }
-      
-      curnode = sgftree.root;
-    }
-    
+
+    /* Allow resignation at interface level (the engine may still be not
+     * allowed to resign.
+     */
+    resignation_allowed = 1;
+
     printf("\nBeginning ASCII mode game.\n\n");
     gameinfo_print(gameinfo);
-    
+
     /* Does the computer play first?  If so, make a move. */
     if (gameinfo->computer_player == gameinfo->to_move)
-      computer_move(gameinfo, &passes);
-    
+      state = computer_move(gameinfo, &passes);
+
     /* main ASCII Play loop */
-    while (passes < 2 && !time_to_die) {
+    while (state == 0) {
       /* Display game board. */
       if (opt_showboard)
 	ascii_showboard();
@@ -617,40 +636,40 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
       if (!fgets(line, 80, stdin)) {
 	printf("\nThanks! for playing GNU Go.\n\n");
 	return ;
-      }      
-      while (command = strtok(line_ptr, ";"), line_ptr = 0, command) {
-	
+      }
+
+      while (state == 0 &&
+	     (command = strtok(line_ptr, ";"), line_ptr = 0, command)) {
 	/* Get the command or move. */
 	switch (get_command(command)) {
 	case RESIGN:
-	  printf("\nGNU Go wins by resignation.");
-	  sgftreeWriteResult(&sgftree,
-			     gameinfo->to_move == WHITE ? -1000.0 : 1000.0,
-			     1);
+	  state = ascii_endgame(gameinfo, 1);
+	  break;
+
 	case END:
 	case EXIT:
 	case QUIT:
 	  printf("\nThanks! for playing GNU Go.\n\n");
-	  return ;
-	  break;
+	  return;
+
 	case HELP:
 	  show_commands();
 	  break;
+
 	case CMD_HELPDEBUG:
 	  printf(DEBUG_COMMANDS);
 	  break;
+
 	case SHOWBOARD:
 	  opt_showboard = !opt_showboard;
 	  break;
+
 	case INFO:
 	  printf("\n");
 	  gameinfo_print(gameinfo);
 	  break;
+
 	case SETBOARDSIZE:
-	  if (movenum > 0) {
-	    printf("Boardsize can be modified on move 1 only!\n");
-	    break;
-	  }
 	  if (sgf_initialized) {
 	    printf("Boardsize cannot be changed after record is started!\n");
 	    break;
@@ -660,7 +679,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
-	  if (num < 2 || num > 25) {
+	  if (num < MIN_BOARD || num > MAX_BOARD) {
 	    printf("\nInvalid board size: %d\n", num);
 	    break;
 	  }
@@ -672,11 +691,8 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  sgfOverwritePropertyInt(sgftree.root, "SZ", sz);
 	  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  break;
+
 	case SETHANDICAP:
-	  if (movenum > 0) {
-	    printf("Handicap can be modified on move 1 only!\n");
-	    break;
-	  }
 	  if (sgf_initialized) {
 	    printf("Handicap cannot be changed after game is started!\n");
 	    break;
@@ -695,15 +711,21 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  /* Place stones on board but don't record sgf 
 	   * in case we change more info. */
 	  gameinfo->handicap = gnugo_placehand(num);
-	  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  printf("\nSet handicap to %d\n", gameinfo->handicap);
-	  gameinfo->to_move = WHITE;
+          gameinfo->to_move = (gameinfo->handicap ? WHITE : BLACK);
 	  break;
-	case SETKOMI:
-	  if (movenum > 0) {
-	    printf("Komi can be modified on move 1 only!\n");
+
+	case FREEHANDICAP:
+	  if (sgf_initialized) {
+	    printf("Handicap cannot be changed after game is started!\n");
 	    break;
 	  }
+	  while (*command && *command != ' ')
+	    command++;
+	  ascii_free_handicap(gameinfo, command);
+	  break;
+
+	case SETKOMI:
 	  if (sgf_initialized) {
 	    printf("Komi cannot be modified after game record is started!\n");
 	    break;
@@ -716,54 +738,53 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  komi = fnum;
 	  printf("\nSet Komi to %.1f\n", komi);
 	  break;
+
 	case SETDEPTH:
-	  {
-	    command += 6;
-	    if (sscanf(command, "%d", &num) != 1) {
-	      printf("\nInvalid command syntax!\n");
-	      break;
-	    }
-	    mandated_depth = num;
-	    printf("\nSet depth to %d\n", mandated_depth);
+	  command += 6;
+	  if (sscanf(command, "%d", &num) != 1) {
+	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
+	  mandated_depth = num;
+	  printf("\nSet depth to %d\n", mandated_depth);
+	  break;
+
 	case SETLEVEL:
-	  {
-	    command += 6;
-	    if (sscanf(command, "%d", &num) != 1) {
-	      printf("\nInvalid command syntax!\n");
-	      break;
-	    }
-	    level = num;
-	    printf("\nSet level to %d\n", level);
+	  command += 6;
+	  if (sscanf(command, "%d", &num) != 1) {
+	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
+	  level = num;
+	  printf("\nSet level to %d\n", level);
+	  break;
+
 	  /* Level replaces hurry as of 2.7.204. This option is retained
 	   * for compatibility with gnugoclient. 
 	   */
 	case SETHURRY:
-	  {
-	    command += 6;
-	    if (sscanf(command, "%d", &num) != 1) {
-	      printf("\nInvalid command syntax!\n");
-	      break;
-	    }
-	    level = 10 - num;
-	    printf("\nSet hurry to %d\n", 10 - level);
+	  command += 6;
+	  if (sscanf(command, "%d", &num) != 1) {
+	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
+	  level = 10 - num;
+	  printf("\nSet hurry to %d\n", 10 - level);
+	  break;
+
 	case DISPLAY:
 	  if (!opt_showboard)
 	    ascii_showboard();
 	  break;
+
 	case FORCE:
 	  command += 6; /* skip the force part... */
 	  switch (get_command(command)) {
 	  case MOVE:
-	    do_move(gameinfo, command, &passes, 1);
+	    state = do_move(gameinfo, command, &passes, 1);
 	    break;
 	  case PASS:
-	    do_pass(gameinfo, &passes, 1);
+	    state = do_pass(gameinfo, &passes, 1);
 	    break;
 	  default:
 	    printf("Illegal forced move: %s %d\n", command,
@@ -771,12 +792,15 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    break;
 	  }
 	  break;
+
 	case MOVE:
-	  do_move(gameinfo, command, &passes, 0);
+	  state = do_move(gameinfo, command, &passes, 0);
 	  break;
+
 	case PASS:
-	  do_pass(gameinfo, &passes, 0);
+	  state = do_pass(gameinfo, &passes, 0);
 	  break;
+
 	case PLAY:
 	  command += 5;
 	  if (sscanf(command, "%d", &num) != 1) {
@@ -787,7 +811,9 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    for (m = 0; m < num; m++) {
 	      gameinfo->computer_player 
 		= OTHER_COLOR(gameinfo->computer_player);
-	      computer_move(gameinfo, &passes);
+	      state = computer_move(gameinfo, &passes);
+	      if (state)
+		break;
 	      if (passes >= 2)
 		break;
 	    }
@@ -796,69 +822,78 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    break;
 	  }
 	  break;
+
 	case PLAYBLACK:
 	  if (gameinfo->computer_player == WHITE)
 	    gameinfo->computer_player = BLACK;
 	  if (gameinfo->computer_player == gameinfo->to_move)
-	    computer_move(gameinfo, &passes);
+	    state = computer_move(gameinfo, &passes);
 	  break;
+
 	case PLAYWHITE:
 	  if (gameinfo->computer_player == BLACK)
 	    gameinfo->computer_player = WHITE;
 	  if (gameinfo->computer_player == gameinfo->to_move)
-	    computer_move(gameinfo, &passes);
+	    state = computer_move(gameinfo, &passes);
 	  break;
+
 	case SWITCH:
 	  gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
-	  computer_move(gameinfo, &passes);
+	  state = computer_move(gameinfo, &passes);
 	  break;
+
 	case UNDO:
 	case CMD_BACK:
 	  if (gnugo_undo_move(1)) {
-	    sgffile_write_comment("undo");
-	    curnode = curnode->parent;
+            sgftreeAddComment(&sgftree, "undone");
+	    sgftreeBack(&sgftree);
 	    gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
 	  }
 	  else
 	    printf("\nCan't undo.\n");
 	  break;
+
 	case CMD_FORWARD:
-	  if (curnode->child) {
-	    gameinfo->to_move = gnugo_play_sgfnode(curnode->child,
-						   gameinfo->to_move);
-	    curnode = curnode->child;
-	  }
+         if (sgftreeForward(&sgftree))
+           gameinfo->to_move = gnugo_play_sgfnode(sgftree.lastnode,
+						  gameinfo->to_move);
 	  else
 	    printf("\nEnd of game tree.\n");
 	  break;
+
 	case CMD_LAST:
-	  while (curnode->child) {
-	    gameinfo->to_move = gnugo_play_sgfnode(curnode->child,
-						   gameinfo->to_move);
-	    curnode = curnode->child;
-	  }
+         while (sgftreeForward(&sgftree))
+           gameinfo->to_move = gnugo_play_sgfnode(sgftree.lastnode,
+						  gameinfo->to_move);
 	  break;
+
 	case COMMENT:
 	  printf("\nEnter comment. Press ENTER when ready.\n");
 	  fgets(line, 80, stdin);
-	  sgfAddComment(curnode, line);
+	  sgftreeAddComment(&sgftree, line);
 	  break;
+
 	case SCORE:
 	  showscore = !showscore;
 	  if (!showscore)
 	    current_score_estimate = NO_SCORE;
 	  break;
+
 	case CMD_DEAD:
+	  examine_position(gameinfo->to_move, FULL_EXAMINE_DRAGONS);
 	  showdead = !showdead;
 	  break;
+
 	case CMD_CAPTURE:
 	  strtok(command, " ");
 	  showcapture(strtok(NULL, " "));
 	  break;
+
 	case CMD_DEFEND:
 	  strtok(command, " ");
 	  showdefense(strtok(NULL, " "));
 	  break;
+
 	case CMD_SHOWMOYO:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_MOYO;
@@ -866,6 +901,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
+
 	case CMD_SHOWTERRI:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_TERRITORY;
@@ -873,6 +909,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
+
 	case CMD_SHOWAREA:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_AREA;
@@ -880,23 +917,26 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
+
 	case CMD_SHOWDRAGONS:
 	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
 	  showboard(1);
 	  break;
+
 	case CMD_GOTO:
 	  strtok(command, " ");
 	  ascii_goto(gameinfo, strtok(NULL, " "));
 	  break;
+
 	case CMD_SAVE:
 	  strtok(command, " ");
 	  tmpstring = strtok(NULL, " ");
 	  if (tmpstring) {
 	    /* discard newline */
 	    tmpstring[strlen(tmpstring)-1] = 0;
-	    sgf_write_header(sgftree.root, 1, random_seed, komi, level);
+	    /* make sure we are saving proper handicap */
+	    init_sgf(gameinfo);
 	    writesgf(sgftree.root, tmpstring);
-	    sgf_initialized = 0;
 	    printf("You may resume the game");
 	    printf(" with -l %s --mode ascii\n", tmpstring);
 	    printf("or load %s\n", tmpstring);
@@ -904,19 +944,21 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  else
 	    printf("Please specify filename\n");
 	  break;
+
 	case CMD_LOAD:
 	  strtok(command, " ");
 	  tmpstring = strtok(NULL, " ");
 	  if (tmpstring) {
 	    /* discard newline */
 	    tmpstring[strlen(tmpstring)-1] = 0;
-	    if (!sgftree_readfile(&sgftree, tmpstring))
-	      {
-		fprintf(stderr, "Cannot open or parse '%s'\n", tmpstring);
-		break;
-	      }
-	    sgf_initialized = 0;
-	    gameinfo_play_sgftree(gameinfo, sgftree.root, NULL);
+	    if (!sgftree_readfile(&sgftree, tmpstring)) {
+	      fprintf(stderr, "Cannot open or parse '%s'\n", tmpstring);
+	      break;
+	    }
+            /* to avoid changing handicap etc. */
+            sgf_initialized = 1;
+            gameinfo_load_sgfheader(gameinfo, sgftree.root);
+            gameinfo_play_sgftree(gameinfo, &sgftree, NULL);
 	    sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  }
 	  else
@@ -927,71 +969,33 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
 	  show_dragons();
 	  break;
-	case COUNT:
-	case NEW:
-	case INVALID:
+
 	default:
 	  printf("\nInvalid command: %s", command);
 	  break;
 	}
-      }
-    }
-    
-    /* two passes : game over */
-    
-    if (passes >= 2)
-      gnugo_who_wins(gameinfo->computer_player, stdout);
-    printf("\nIf you disagree, we may count the game together.\n");
-    printf("You may optionally save the game as an SGF file.\n");
-    state = 0;
-    while (state == 0) {
-      printf("\n\
-Type \"save <filename>\" to save,\n\
-     \"count\" to recount,\n\
-     \"quit\" to quit\n\
- or  \"game\" to play again\n");
-      line_ptr = line;
-      if (!fgets(line, 80, stdin))
-	break;
-      command = strtok(line_ptr, "");
-      switch (get_command(command)) {
-      case CMD_SAVE:
-	strtok(command, " ");
-	tmpstring = strtok(NULL, " ");
-	if (tmpstring) {
-	  /* discard newline */
-	  tmpstring[strlen(tmpstring)-1] = 0;
-	  sgf_write_header(sgftree.root, 1, random_seed, komi, level);
-	  writesgf(sgftree.root, tmpstring);
-	    sgf_initialized = 0;
-	}
-	else
-	  printf("Please specify filename\n");
-	break;
-	
-      case NEW:
-	state = 1;
-	break;
-	
-      case COUNT:
-	endgame(gameinfo);
-	break;
 
-      case QUIT:
-	state = 2;
-	break;
-	
-      default:
-	state = 0;
+	if (passes >= 2)
+	  state = ascii_endgame(gameinfo, 0);
       }
     }
+
+    sgffile_output(&sgftree);
     passes = 0;
-    showdead = 0;
-    sgf_initialized = 0;
+    
     /* Play a different game next time. */
     update_random_seed();
+
+    /* Free the sgf tree and prepare for a new game. */
+    sgfFreeNode(sgftree.root);
+    sgftree_clear(&sgftree);
+    sgftreeCreateHeaderNode(&sgftree, board_size, komi);
+    sgf_initialized = 0;
+
+    gameinfo_clear(gameinfo, board_size, komi);
   }
-  printf("\nThanks for playing GNU Go.\n\n");
+
+  printf("\nThanks! for playing GNU Go.\n\n");
 }
 
 void
@@ -1003,12 +1007,93 @@ play_ascii_emacs(SGFTree *tree, Gameinfo *gameinfo,
 }
 
 
-/*
- * endgame() scores the game.
- */
+/* Communicates with user after a game has ended. */
+static int
+ascii_endgame(Gameinfo *gameinfo, int reason)
+{
+  char line[80];
+  char *line_ptr;
+  char *command;
+  char *tmpstring;
+  int state = 0;
 
+  if (reason == 0) {		/* Two passes, game is over. */
+    gnugo_who_wins(gameinfo->computer_player, stdout);
+    printf("\nIf you disagree, we may count the game together.\n");
+
+    sgftreeWriteResult(&sgftree, estimate_score(NULL, NULL), 1);
+  }
+  else {
+    int color = OTHER_COLOR(gameinfo->to_move);
+
+    if (reason == 1)		/* Our opponent has resigned. */
+      printf("GNU Go wins by resignation.\n");
+    else			/* We have resigned. */
+      printf("You win by resignation.\n");
+
+    printf("Result: %c+Resign\n\n", color == WHITE ? 'W' : 'B');
+    sgftreeWriteResult(&sgftree, color == WHITE ? 1000.0 : -1000.0, 1);
+  }
+
+  while (state == 0) {
+    printf("You may optionally save the game as an SGF file.\n\n");
+    printf("Type \"save <filename>\" to save,\n");
+    if (reason == 0)
+      printf("     \"count\" to recount,\n");
+    else if (reason == 2)
+      printf("     \"continue\" to decline resignation and continue the game,\n");
+    printf("     \"quit\" to quit\n");
+    printf(" or  \"game\" to play again\n");
+
+    line_ptr = line;
+    if (!fgets(line, 80, stdin))
+      break;
+
+    command = strtok(line_ptr, "");
+    switch (get_command(command)) {
+    case CMD_SAVE:
+      strtok(command, " ");
+      tmpstring = strtok(NULL, " ");
+      if (tmpstring) {
+	/* discard newline */
+	tmpstring[strlen(tmpstring)-1] = 0;
+	init_sgf(gameinfo);
+	writesgf(sgftree.root, tmpstring);
+      }
+      else
+	printf("Please specify filename\n");
+      break;
+
+    case COUNT:
+      if (reason == 0)
+	ascii_count(gameinfo);
+      break;
+
+    case CONTINUE:
+      state = -1;
+      break;
+
+    case NEW:
+      state = 1;
+      break;
+
+    case QUIT:
+      state = 2;
+      break;
+
+    default:
+      state = 0;
+    }
+  }
+
+  return state;
+}
+
+
+/* ascii_count() scores the game.
+ */
 static void
-endgame(Gameinfo *gameinfo)
+ascii_count(Gameinfo *gameinfo)
 {
   char line[12];
   int done = 0;
@@ -1020,14 +1105,14 @@ endgame(Gameinfo *gameinfo)
   ascii_showboard();
   while (!done) {
     printf("Dead stones are marked with small letters (x,o).\n");
-    printf("\nIf you don't agree, enter the location of a group\n\
-to toggle its state or \"done\".\n");
+    printf("\nIf you don't agree, enter the location of a group\n");
+    printf("to toggle its state or \"done\".\n");
 
     if (!fgets(line, 12, stdin))
       return; /* EOF or some error */
     
     for (i = 0; i < 12; i++)
-      line[i] = (isupper ((int) line[i]) ? tolower ((int) line[i]) : line[i]);
+      line[i] = tolower((int) line[i]);
     if (!strncmp(line, "done", 4))
       done = 1;
     else if (!strncmp(line, "quit", 4))
@@ -1052,16 +1137,18 @@ to toggle its state or \"done\".\n");
       printf("dead group, or type \"done\"\n");
     }      
     else if (!strncmp(line, "undo", 4)) {
-      printf("UNDO not allowed anymore. The status of the stones now toggles after entering the location of it.\n");
+      printf("UNDO not allowed anymore. The status of the stones now\n");
+      printf("toggles after entering the location of it.\n");
       ascii_showboard();
     }
     else {
-      if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY)
+      if (!string_to_location(board_size, line, &i, &j)
+	  || BOARD(i, j) == EMPTY)
 	printf("\ninvalid!\n");
       else {
-	int status = matcher_status(POS(i, j));
+	int status = dragon_status(POS(i, j));
 	status = (status == DEAD) ? ALIVE : DEAD;
-	change_matcher_status(POS(i, j), status);
+	change_dragon_status(POS(i, j), status);
 	ascii_showboard();
       }
     }
@@ -1075,10 +1162,12 @@ showcapture(char *line)
 {
   int i, j, x, y;
   if (line)
-    if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY) {
+    if (!string_to_location(board_size, line, &i, &j)
+	|| BOARD(i, j) == EMPTY) {
       printf("\ninvalid point!\n");
       return;
     }
+  
   if (gnugo_attack(i, j, &x, &y))
     mprintf("\nSuccessfull attack of %m at %m\n", i, j, x, y);
   else
@@ -1091,7 +1180,8 @@ showdefense(char *line)
 {
   int i, j, x, y;
   if (line)
-    if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY) {
+    if (!string_to_location(board_size, line, &i, &j)
+	|| BOARD(i, j) == EMPTY) {
       printf("\ninvalid point!\n");
       return;
     }
@@ -1110,50 +1200,93 @@ showdefense(char *line)
 static void
 ascii_goto(Gameinfo *gameinfo, char *line)
 {
-  int movenumber = 0;
+  const char *movenumber = line;
+
   if (!line)
     return;
+  
   if (!strncmp(line, "last", 4))
-    movenumber = 9999;
-  else {
-    if (!strncmp(line, "first", 4))
-      movenumber = 1;
-    else
-      sscanf(line, "%i", &movenumber);
-  }
-  printf("goto %i\n", movenumber);
-  curnode = sgftree.root;
-  gameinfo->to_move = gnugo_play_sgftree(curnode, &movenumber, &curnode);
-  return;
+    movenumber = "9999";
+  else if (!strncmp(line, "first", 4))
+    movenumber = "1";
+  
+  printf("goto %s\n", movenumber);
+  gameinfo_play_sgftree(gameinfo, &sgftree, movenumber);
 }
 
 
-/* Convert a coordinate pair from ascii text to two integers.
- * FIXME: Check that this function is indeed equivalent to
- * string_to_location() and then replace it.
- */
-static int
-ascii2pos(char *line, int *i, int *j)
+static void
+ascii_free_handicap(Gameinfo *gameinfo, char *handicap)
 {
-  int d;
-  char c;
-  if (sscanf(line, "%c%d", &c, &d) != 2)
-    return 0;
+  int handi;
+  int i;
+  char line[80];
+  int stones[MAX_BOARD*MAX_BOARD];
+  int x, y, pos;
 
-  /* No 'I' in the coordinate system. */
-  if (tolower((int) c) == 'i')
-    return 0;
-  
-  *j = tolower((int) c) - 'a';
-  if (tolower((int) c) > 'i')
-    --*j;
-  
-  *i = board_size - d;
+  if (sscanf(handicap, "%d", &handi) == 1) {
+    /* Gnu Go is to place handicap */
+    if (handi < 0 || handi == 1) {
+      printf("\nInvalid command syntax!\n");
+      return;
+    }
 
-  if (*i < 0 || *i >= board_size || *j < 0 || *j >= board_size)
-    return 0;
-  
-  return 1;
+    gnugo_clear_board(board_size);
+    handi = place_free_handicap(handi);
+    printf("\nPlaced %d stones of free handicap.\n", handi);
+  }
+  else { /* User is to place handicap */
+    gnugo_clear_board(board_size);
+    handi = 0;
+
+    while (1) {
+      ascii_showboard();
+      printf("\nType in coordinates of next handicap stone, or one of the following commands:\n");
+      printf("  undo        take back the last stone placed\n");
+      printf("  clear       remove all the stones placed so far\n");
+      printf("  done        finish placing handicap\n\n");
+      printf("You have placed %d handicap stone(s) so far.\n\n", handi);
+
+      if (!fgets(line, 80, stdin))
+        return; /* EOF or some error */
+      for (i = 0; i < 80; i++)
+        line[i] = tolower((int) line[i]);
+
+      if (!strncmp(line, "undo", 4)) {
+        if (!handi)
+	  printf("\nNothing to undo.\n");
+	else {
+	  remove_stone(stones[--handi]);
+	  gprintf("\nRemoved the stone at %m.\n", I(stones[handi]),
+		  J(stones[handi]));
+	}
+      }
+      else if (!strncmp(line, "clear", 5)) {
+        gnugo_clear_board(board_size);
+        handi = 0;
+      }
+      else if (!strncmp(line, "done", 4)) {
+	if (handi == 1) /* Don't bother with checks later */
+	  printf("\nHandicap cannot be one stone. Either add "
+		 "some more, or delete the only stone.\n");
+	else
+	  break;
+      }
+      else if (string_to_location(board_size, line, &x, &y)) {
+	pos = POS(x,y);
+	if (board[pos] != EMPTY)
+	  printf("\nThere's already a stone there.\n");
+	else {
+	  add_stone(pos, BLACK);
+	  stones[handi++] = pos;
+	}
+      }
+      else
+	printf("\nInvalid command: %s", line);
+    }
+  }
+  gameinfo->handicap = handi;
+  gameinfo->to_move = (handi ? WHITE : BLACK);
 }
 
 
