@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -20,6 +21,7 @@
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include "gnugo.h"
+#include "liberty.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,12 +30,13 @@
 #include "interface.h"
 #include "gmp.h"
 #include "sgftree.h"
+#include "gg_utils.h"
 
 /* --------------------------------------------------------------*/
 /* Play a game against a go-modem-protocol (GMP) client.         */
 /* --------------------------------------------------------------*/
 void
-play_gmp(Gameinfo *gameinfo)
+play_gmp(Gameinfo *gameinfo, int simplified)
 {
   SGFTree sgftree;
 
@@ -46,45 +49,20 @@ play_gmp(Gameinfo *gameinfo)
   int passes = 0; /* two passes and its over */
   int to_move;  /* who's turn is next ? */
 
-  int mycolor;  /* who has which color */
+  int mycolor = -1;  /* who has which color */
   int yourcolor;
-  SGFNode *curnode = NULL; /* current SGFNode */
-  
-  mycolor = gameinfo->computer_player;
+
+  if (gameinfo->computer_player == WHITE)
+    mycolor = 1;
+  else if (gameinfo->computer_player == BLACK)
+    mycolor = 0;
 
   sgftree_clear(&sgftree);
   sgftreeCreateHeaderNode(&sgftree, gnugo_get_boardsize(), gnugo_get_komi());
 
-  yourcolor = OTHER_COLOR(mycolor);
-
   ge = gmp_create(0, 1);
   TRACE("board size=%d\n", gnugo_get_boardsize());
 
-  curnode = sgftree.root;
-  
-  /* Leave all the -1's so the client can negotiate the game parameters. */
-  if (chinese_rules)
-    gmp_startGame(ge, -1, -1, 5.5, -1, -1);
-  else
-    gmp_startGame(ge, -1, -1, 5.5, 0, -1);	
-  do {
-    message = gmp_check(ge, 1, NULL, NULL, &error);
-  } while (!time_to_die
-	   && (message == gmp_nothing
-	       || message == gmp_reset));
-  
-  if (message == gmp_err)  {
-    fprintf(stderr, "gnugo-gmp: Error \"%s\" occurred.\n", error);
-    exit(1);
-  }
-  else if (message != gmp_newGame)  {
-    fprintf(stderr, "gnugo-gmp: Expecting a newGame, got %s\n",
-	    gmp_resultString(message));
-    exit(1);
-  }
-
-  gameinfo->handicap = gmp_handicap(ge);
-  gnugo_clear_board(gmp_size(ge));
   /* 
    * The specification of the go modem protocol doesn't even discuss
    * komi. So we have to guess the komi. If the komi is set on the
@@ -98,6 +76,43 @@ play_gmp(Gameinfo *gameinfo)
       gnugo_set_komi(0.5);
   }
 
+  if (!simplified) {
+    /* Leave all the -1's so the client can negotiate the game parameters. */
+    if (chinese_rules)
+      gmp_startGame(ge, -1, -1, 5.5, -1, mycolor, 0);
+    else
+      gmp_startGame(ge, -1, -1, 5.5, 0, mycolor, 0);
+  }
+  else {
+    gmp_startGame(ge, board_size, gameinfo->handicap,
+		  gnugo_get_komi(), chinese_rules, mycolor, 1);
+  }
+
+  do {
+    message = gmp_check(ge, 1, NULL, NULL, &error);
+  } while (message == gmp_nothing || message == gmp_reset);
+  
+  if (message == gmp_err)  {
+    fprintf(stderr, "gnugo-gmp: Error \"%s\" occurred.\n", error);
+    exit(EXIT_FAILURE);
+  }
+  else if (message != gmp_newGame)  {
+    fprintf(stderr, "gnugo-gmp: Expecting a newGame, got %s\n",
+	    gmp_resultString(message));
+    exit(EXIT_FAILURE);
+  }
+
+  gameinfo->handicap = gmp_handicap(ge);
+  gnugo_clear_board(gmp_size(ge));
+
+  /* Let's pretend GMP knows about komi in case something will ever change. */
+  gnugo_set_komi(gmp_komi(ge));
+
+#if ORACLE
+  if (metamachine && oracle_exists)
+    oracle_clear_board(gnugo_get_boardsize());
+#endif
+
   sgfOverwritePropertyInt(sgftree.root, "SZ", gnugo_get_boardsize());
 
   TRACE("size=%d, handicap=%d, komi=%f\n", gnugo_get_boardsize(),
@@ -118,12 +133,13 @@ play_gmp(Gameinfo *gameinfo)
   }
 
   gameinfo->computer_player = mycolor;
-  sgffile_write_gameinfo(gameinfo, "gmp");
+  sgf_write_header(sgftree.root, 1, get_random_seed(), gnugo_get_komi(),
+		   level, chinese_rules);
   gameinfo->handicap = gnugo_sethand(gameinfo->handicap, sgftree.root);
   sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 
   /* main GMP loop */
-  while (passes < 2 && !time_to_die) {
+  while (passes < 2) {
 
     if (to_move == yourcolor) {
       moveval = 0;
@@ -132,7 +148,8 @@ play_gmp(Gameinfo *gameinfo)
 
       if (message == gmp_err) {
 	fprintf(stderr, "GNU Go: Sorry, error from gmp client\n");
-	sgffile_close_file();
+        sgftreeAddComment(&sgftree, "got error from gmp client");
+        sgffile_output(&sgftree);
 	return;
       }
 
@@ -146,8 +163,8 @@ play_gmp(Gameinfo *gameinfo)
 		    j - k);
 	    break;
 	  }
-	  sgffile_write_comment("undo");
-	  curnode = curnode->parent;
+	  sgftreeAddComment(&sgftree, "undone");
+	  sgftreeBack(&sgftree);
 	  to_move = OTHER_COLOR(to_move);
 	}
 	continue;
@@ -155,17 +172,17 @@ play_gmp(Gameinfo *gameinfo)
 
       if (message == gmp_pass) {
 	++passes;
-        curnode = sgfAddPlay(curnode, to_move, -1, -1);
+        sgftreeAddPlay(&sgftree, to_move, -1, -1);
 	gnugo_play_move(-1, -1, yourcolor);
-	sgffile_move_made(-1, -1, to_move, moveval);
+	sgffile_output(&sgftree);
       }
       else {
 	/* not pass */
 	passes = 0;
-        curnode = sgfAddPlay(curnode, to_move, i, j);
+        sgftreeAddPlay(&sgftree, to_move, i, j);
 	TRACE("\nyour move: %m\n\n", i, j);
 	gnugo_play_move(i, j, yourcolor);
-	sgffile_move_made(i, j, to_move, moveval);
+	sgffile_output(&sgftree);
       }
 
     }
@@ -173,22 +190,23 @@ play_gmp(Gameinfo *gameinfo)
       /* Generate my next move. */
       moveval = gnugo_genmove(&i, &j, mycolor);
       gnugo_play_move(i, j, mycolor);
+      sgffile_add_debuginfo(sgftree.lastnode, moveval);
       
       if (moveval < 0) {
 	/* pass */
-        curnode = sgfAddPlay(curnode, to_move, -1, -1);
+        sgftreeAddPlay(&sgftree, to_move, -1, -1);
 	gmp_sendPass(ge);
-	sgffile_move_made(-1, -1, to_move, moveval);
 	++passes;
       }
       else {
 	/* not pass */
-        curnode = sgfAddPlay(curnode, to_move, i, j);
+        sgftreeAddPlay(&sgftree, to_move, i, j);
 	gmp_sendMove(ge, j, i);
-	sgffile_move_made(i, j, to_move, moveval);
 	passes = 0;
 	TRACE("\nmy move: %m\n\n", i, j);
       }
+      sgffile_add_debuginfo(sgftree.lastnode, 0);
+      sgffile_output(&sgftree);
     }
     
     to_move = OTHER_COLOR(to_move);
@@ -197,30 +215,49 @@ play_gmp(Gameinfo *gameinfo)
   /* two passes: game over */
   gmp_sendPass(ge);   
   
-  /* We hang around here until cgoban asks us to go, since
-   * sometimes cgoban crashes if we exit first.
-   */
-  
   if (!quiet)
     fprintf(stderr, "Game over - waiting for client to shut us down\n");
-  sgffile_close_file();
   who_wins(mycolor, stderr);
+
+  if (showtime) {
+    gprintf("\nSLOWEST MOVE: %d at %1m ", slowest_movenum, slowest_move);
+    fprintf(stderr, "(%.2f seconds)\n", slowest_time);
+    fprintf(stderr, "\nAVERAGE TIME: %.2f seconds per move\n",
+	    total_time / movenum);
+    fprintf(stderr, "\nTOTAL TIME: %.2f seconds\n",
+	    total_time);
+  }
   
   
   /* play_gmp() does not return to main(), therefore the score
    * writing code is here.
    */
-  score = gnugo_estimate_score(&lower_bound, &upper_bound);
+  score = gnugo_estimate_score(&upper_bound, &lower_bound);
 
   sgfWriteResult(sgftree.root, score, 1);
-  
-  while (!time_to_die) {
-    message = gmp_check(ge, 1, &j, &i, &error);
-    if (!quiet)
-      fprintf(stderr, "Message %d from gmp\n", message);
-    if (message == gmp_err)
-      break;
+  sgffile_output(&sgftree);
+
+  if (!simplified) {
+    /* We hang around here until cgoban asks us to go, since
+     * sometimes cgoban crashes if we exit first.
+     *
+     * FIXME: Check if this is still needed.  I made it dependand on
+     *	      `simplifed' just to avoid changes in GMP mode.
+     */
+    while (1) {
+      message = gmp_check(ge, 1, &j, &i, &error);
+      if (!quiet)
+	fprintf(stderr, "Message %d from gmp\n", message);
+      if (message == gmp_err)
+	break;
+    }
   }
+
+#if ORACLE
+  if (metamachine && oracle_exists)
+    dismiss_oracle();
+#endif
+
   if (!quiet)
     fprintf(stderr, "gnugo going down\n");
 }
