@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,7 +25,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <math.h>
 
 #include "interface.h"
@@ -35,10 +35,10 @@
 #include "random.h"
 #include "gg_utils.h"
 
-
 void
 play_solo(Gameinfo *gameinfo, int moves)
 {
+  SGFTree sgftree;
   int passes = 0; /* num. consecutive passes */
   int move_val;
   double t1, t2;
@@ -57,7 +57,12 @@ play_solo(Gameinfo *gameinfo, int moves)
   int n = 6 + 2*gg_rand()%5;
   int i, j;
 
-  sgffile_write_gameinfo(gameinfo, "solo");
+  gnugo_set_komi(5.5);
+
+  sgftree_clear(&sgftree);
+  sgftreeCreateHeaderNode(&sgftree, gnugo_get_boardsize(), gnugo_get_komi());
+  sgf_write_header(sgftree.root, 1, get_random_seed(), 5.5,
+                   level, chinese_rules);
  
   /* Generate some random moves. */
   if (boardsize > 6) {
@@ -66,17 +71,25 @@ play_solo(Gameinfo *gameinfo, int moves)
 	i = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
 	j = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
       } while (!gnugo_is_legal(i, j, gameinfo->to_move));
-      
-      gameinfo_play_move(gameinfo, i, j, gameinfo->to_move);
+
+      gnugo_play_move(i, j, gameinfo->to_move);
+      sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
+      sgftreeAddComment(&sgftree, "random move");
+      gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
     } while (--n > 0);
   }
   
   t1 = gg_cputime();
   memset(&totalstats, '\0', sizeof(totalstats));
-  while (passes < 2 && --moves >= 0 && !time_to_die) {
+  while (passes < 2 && --moves >= 0) {
     reset_owl_node_counter();
     move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
-    gameinfo_play_move(gameinfo, i, j, gameinfo->to_move);
+
+    gnugo_play_move(i, j, gameinfo->to_move);
+    sgffile_add_debuginfo(sgftree.lastnode, move_val);
+    sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
+    sgffile_output(&sgftree);
+    gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
 
     if (move_val < 0) {
       ++passes;
@@ -101,6 +114,10 @@ play_solo(Gameinfo *gameinfo, int moves)
   /* Two passes and it's over. (EMPTY == BOTH) */
   gnugo_who_wins(EMPTY, stdout);
 
+  score = gnugo_estimate_score(&upper_bound, &lower_bound);
+  sgfWriteResult(sgftree.root, score, 1);
+  sgffile_output(&sgftree);
+
 #if 0
   if (t2 == t1)
     printf("%.3f moves played\n", (double) (save_moves-moves));
@@ -123,44 +140,39 @@ play_solo(Gameinfo *gameinfo, int moves)
 /* ================================================================ */
 
 
-/* FIXME: This should be in a separate source file.
- */
-
-
 /*
  * Load SGF file and run genmove().
  */
 
 void 
-load_and_analyze_sgf_file(Gameinfo *gameinfo, int benchmark)
+load_and_analyze_sgf_file(Gameinfo *gameinfo)
 {
+  SGFTree sgftree;
   int i, j;
   int next;
-  int r;
+  int move_val;
   
   next = gameinfo->to_move;
-  gameinfo->computer_player = next;
-  sgffile_write_gameinfo(gameinfo, "load and analyze");
+  sgftree = gameinfo->game_record;
 
-  if (benchmark) {
-    for (r = 0; r < benchmark; ++r) {
-      genmove(&i, &j, next);
-      next = OTHER_COLOR(next);
-    }
-  }
+  if (metamachine)
+    sgffile_begindump(&sgftree);
+
+  move_val = gnugo_genmove(&i, &j, next);
+
+  if (is_pass(POS(i, j)))
+    gprintf("%s move: PASS!\n", next == WHITE ? "white (O)" : "black (X)");
+  else
+    gprintf("%s move %m\n", next == WHITE ? "white (O)" : "black (X)", i, j);
+
+  if (metamachine)
+    sgffile_enddump(outfilename);
   else {
-    genmove(&i, &j, next);
-    
-    if (is_pass(POS(i, j))) {
-      gprintf("%s move: PASS!\n", next == WHITE ? "white (O)" : "black (X)");
-      sgffile_move_made(i, j, next, 0);
-    }
-    else {
-      gprintf("%s move %m\n", next == WHITE ? "white (O)" : "black (X)",
-	      i, j);
-      gnugo_play_move(i, j, next);
-      sgffile_move_made(i, j, next, 0);
-    }
+    gnugo_play_move(i, j, next);
+    sgftreeAddPlay(&sgftree, next, i, j);
+    sgftreeAddComment(&sgftree, "load and analyze mode");
+    sgffile_add_debuginfo(sgftree.lastnode, move_val);
+    sgffile_output(&sgftree);
   }
 }
 
@@ -174,36 +186,59 @@ load_and_analyze_sgf_file(Gameinfo *gameinfo, int benchmark)
  *             get an accurate score
  */
 
+#define ESTIMATE  0
+#define FINISH    1
+#define AFTERMATH 2
+
 void 
 load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo, 
 			const char *scoringmode)
 {
   int i, j, move_val;
-  int until;
-  float result;
   char *tempc = NULL;
-  char dummy;
   char text[250];
   char winner;
   int next;
   int pass = 0;
-  SGFTree score_tree;
-  SGFNode *node;
+  int method;
+  SGFTree local_tree;
+  SGFTree *score_tree = tree;
   
-  sgftree_clear(&score_tree);
-  node = sgftreeCreateHeaderNode(&score_tree, board_size, komi);
-  sgftreeSetLastNode(&score_tree, node);
-  sgftree_printboard(&score_tree);
+  /* Default scoring method is ESTIMATE since it's fastest. */
+  method = ESTIMATE;
+  if (strcmp(scoringmode, "finish") == 0)
+    method = FINISH;
+  else if (strcmp(scoringmode, "aftermath") == 0)
+    method = AFTERMATH;
+
+  /* For aftermath scoring we compress the previous moves to a static
+   * board position in the output sgf. This helps a lot when debugging
+   * scoring mistakes. We don't do this for the finish method,
+   * however, since users may be better served by having GNU Go's
+   * selfplay added to the original game record.
+   */
+  if (method == AFTERMATH) {
+    sgftree_clear(&local_tree);
+    /* Modify komi to compensate for captured stones. We start at a
+     * setup position and since there is no standard sgf property to
+     * tell the number of captured stones, a modified komi is the best
+     * available solution.
+     */
+    sgftreeCreateHeaderNode(&local_tree, board_size,
+			    komi + black_captured - white_captured);
+    sgffile_printboard(&local_tree);
+    sgfAddProperty(local_tree.lastnode, "PL",
+		   gameinfo->to_move == WHITE ? "W" : "B");
+    score_tree = &local_tree;
+  }
   
-  sgffile_write_gameinfo(gameinfo, "load and score");
   next = gameinfo->to_move;
-  sgffile_printboard(next);
-  doing_scoring = 1;
   reset_engine();
   
-  if (!strcmp(scoringmode, "finish") || !strcmp(scoringmode, "aftermath")) {
-    until = 9999;
-    do {
+  /* Complete the game by selfplay for the finish and aftermath methods. */
+  if (method != ESTIMATE) {
+    doing_scoring = 1;
+    while (pass < 2) {
       move_val = genmove_conservative(&i, &j, next);
       if (move_val >= 0) {
 	pass = 0;
@@ -211,81 +246,83 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
 		next == WHITE ? "white (O)" : "black (X)", i, j);
       }
       else {
-	++pass;
+	pass++;
 	gprintf("%d %s move : PASS!\n", movenum, 
 		next == WHITE ? "white (O)" : "black (X)");
       }
       play_move(POS(i, j), next);
-      sgffile_move_made(i, j, next, move_val);
-      sgftreeAddPlay(&score_tree, NULL, next, i, j);
+      sgffile_add_debuginfo(score_tree->lastnode, move_val);
+      sgftreeAddPlay(score_tree, next, i, j);
+      sgffile_output(score_tree);
       next = OTHER_COLOR(next);
-    } while (movenum <= until && pass < 2);
+    }
+    doing_scoring = 0;
+  }
+  
+  /* Calculate the score. */
+  if (method == AFTERMATH)
+    score = aftermath_compute_score(next, komi, score_tree);
+  else {
+    /* Before we call estimate_score() we must make sure that the
+     * dragon status is computed. Therefore the call to
+     * examine_position().
+     */
+    examine_position(next, EXAMINE_ALL);
+    score = gnugo_estimate_score(NULL, NULL);
+  }
+  
+  if (score < 0.0) {
+    sprintf(text, "Black wins by %1.1f points\n", -score);
+    winner = 'B';
+  }
+  else if (score > 0.0) {
+    sprintf(text, "White wins by %1.1f points\n", score);
+    winner = 'W';
+  }
+  else {
+    sprintf(text, "Jigo\n");
+    winner = '0';
+  }
+  fputs(text, stdout);
+  sgftreeAddComment(score_tree, text);
 
-    if (pass >= 2) {
-      node = score_tree.lastnode;
-      /* Calculate the score */
-      if (!strcmp(scoringmode, "aftermath"))
-	score = aftermath_compute_score(next, komi, &score_tree);
+  /* For the finish and aftermath methods we compare the score with
+   * what's stored in the game record.
+   *
+   * FIXME: No comparison is made if the stored result was 0. Wins by
+   *        time or forfeit or not handled either.
+   *
+   * FIXME: Does anybody actually care about this information? Just
+   *        removing this piece of code is a tempting alternative.
+   */
+  if (method != ESTIMATE && sgfGetCharProperty(tree->root, "RE", &tempc)) {
+    char dummy;
+    float result;
+    if (sscanf(tempc, "%1c%f", &dummy, &result) == 2) {
+      fprintf(stdout, "Result from file: %c+%1.1f\n", dummy, result);
+      fputs("GNU Go result and result from file are ", stdout);
+      if (result == fabs(score) && winner == dummy)
+	fputs("identical\n", stdout);
       else
-	score = gnugo_estimate_score(&lower_bound, &upper_bound);
-
-      if (score < 0.0) {
-	sprintf(text, "Black wins by %1.1f points\n", -score);
-	winner = 'B';
+	fputs("different\n", stdout);
+      
+    }
+    else {
+      if (tempc[2] == 'R') {
+	fprintf(stdout, "Result from file: Resign\n");
+	fputs("GNU Go result and result from file are ", stdout);
+	if (tempc[0] == winner)
+	  fputs("identical\n", stdout);
+	else
+	  fputs("different\n", stdout);
       }
-      else if (score > 0.0) {
-	sprintf(text, "White wins by %1.1f points\n", score);
-	winner = 'W';
-      }
-      else {
-	sprintf(text, "Jigo\n");
-	winner = '0';
-      }
-      fputs(text, stdout);
-      sgfAddComment(node, text);
-      sgffile_write_comment(text);
-      if (sgfGetCharProperty(tree->root, "RE", &tempc)) {
-	if (sscanf(tempc, "%1c%f", &dummy, &result) == 2) {
-	  fprintf(stdout, "Result from file: %1.1f\n", result);
-	  fputs("GNU Go result and result from file are ", stdout);
-	  if (result == fabs(score) && winner == dummy)
-	    fputs("identical\n", stdout);
-	  else
-	    fputs("different\n", stdout);
-	      
-	}
-	else {
-	  if (tempc[2] == 'R') {
-	    fprintf(stdout, "Result from file: Resign\n");
-	    fputs("GNU Go result and result from file are ", stdout);
-	    if (tempc[0] == winner)
-	      fputs("identical\n", stdout);
-	    else
-	      fputs("different\n", stdout);
-	  }
-	}
-      }
-      sgfWriteResult(score_tree.root, score, 1);
     }
   }
-  doing_scoring = 0;
 
-
-  if (!strcmp(scoringmode, "aftermath")) {
-    if (gameinfo->outfile)
-      writesgf(score_tree.root, gameinfo->outfilename);
-    return;
-  }
-
-  /* Before we call estimate_score() we must make sure that the dragon
-   * status is computed. Therefore the call to examine_position().
-   */
-  examine_position(next, EXAMINE_ALL);
-  score = estimate_score(NULL, NULL);
-
-  fprintf(stdout, "\n%s seems to win by %1.1f points\n",
-	  score < 0 ? "B" : "W",
-	  score < 0 ? -score : score);
+  if (method != ESTIMATE)
+    sgfWriteResult(score_tree->root, score, 1);
+  
+  sgffile_output(score_tree);
 }
 
 
