@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,7 +25,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <ctype.h>
 
 #include "interface.h"
@@ -34,71 +34,81 @@
 
 static void replay_node(SGFNode *node, int color_to_test);
 
+static float replay_score;
+static float total_score;
+
 
 /* --------------------------------------------------------------*/
 /* replay a game */
 /* --------------------------------------------------------------*/
 
 void
-play_replay(SGFNode *sgf_head, int color_to_replay)
+play_replay(Gameinfo *gameinfo, int color_to_replay)
 {
-  Gameinfo  gameinfo;
   int tmpi;
   float tmpf;
   char *tmpc = NULL;
 
-  SGFNode *node;
+  SGFTree tree = gameinfo->game_record;
+  SGFNode *node = tree.root;
 
   /* Get the board size. */
-  if (!sgfGetIntProperty(sgf_head, "SZ", &tmpi)) {
+  if (!sgfGetIntProperty(node, "SZ", &tmpi)) {
     fprintf(stderr, "Couldn't find the size (SZ) attribute!\n");
     exit(EXIT_FAILURE);
   }
-  gameinfo_clear(&gameinfo, tmpi, 5.5);
-   
-  /* Get the number of handicap stones. */
-  if (sgfGetIntProperty(sgf_head, "HA", &tmpi)) {
-    /* Handicap stones should appear as AW,AB properties in the sgf file. */
-    gameinfo.handicap = tmpi;
-  }
+
+  gnugo_clear_board(tmpi);
 
   /* Get the komi. */
-  if (sgfGetFloatProperty(sgf_head, "KM", &tmpf))
+  if (sgfGetFloatProperty(node, "KM", &tmpf))
     komi = tmpf;
 
   if (!quiet) {
-    if (sgfGetCharProperty(sgf_head, "RU", &tmpc))
+    if (sgfGetCharProperty(node, "RU", &tmpc))
       printf("Ruleset:      %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "GN", &tmpc))
+    if (sgfGetCharProperty(node, "GN", &tmpc))
       printf("Game Name:    %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "DT", &tmpc))
+    if (sgfGetCharProperty(node, "DT", &tmpc))
       printf("Game Date:    %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "GC", &tmpc))
+    if (sgfGetCharProperty(node, "GC", &tmpc))
       printf("Game Comment: %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "US", &tmpc))
+    if (sgfGetCharProperty(node, "US", &tmpc))
       printf("Game User:    %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "PB", &tmpc))
+    if (sgfGetCharProperty(node, "PB", &tmpc))
       printf("Black Player: %s\n", tmpc);
-    if (sgfGetCharProperty(sgf_head, "PW", &tmpc))
+    if (sgfGetCharProperty(node, "PW", &tmpc))
       printf("White Player: %s\n", tmpc);
 
-    gameinfo_print(&gameinfo);
+    gameinfo_print(gameinfo);
+
+    replay_score = 0.0;
+    total_score = 0.0;
   }
 
-  sgffile_write_gameinfo(&gameinfo, "replay game");         
-   
   /*
    * Now actually run through the file.  This is the interesting part.
    * We need to traverse the SGF tree, and every time we encounter a node
    * we need to check what move GNU Go would make, and see if it is OK. 
    */
-  node = sgf_head;
   while (node) {
     replay_node(node, color_to_replay);
+    sgffile_output(&tree);
     node = node->child;
   }
 
-   sgffile_close_file();
+  if (!quiet) {
+    printf("\nGlobal score: %.2f / %.2f", replay_score, total_score);
+  }
+
+  if (showtime) {
+      gprintf("\nSLOWEST MOVE: %d at %1m ", slowest_movenum, slowest_move);
+      fprintf(stderr, "(%.2f seconds)\n", slowest_time);
+      fprintf(stderr, "\nAVERAGE TIME: %.2f seconds per move\n",
+	      total_time / movenum);
+      fprintf(stderr, "\nTOTAL TIME: %.2f seconds\n",
+	      total_time);
+  }
 }
 
 
@@ -118,6 +128,8 @@ replay_node(SGFNode *node, int color_to_replay)
   int m, n; /* Move from file. */
   int i, j; /* Move generated by GNU Go. */
 
+  char buf[127];
+
   /* Handle any AB / AW properties, and note presence
    * of move properties.
    */
@@ -128,15 +140,11 @@ replay_node(SGFNode *node, int color_to_replay)
       /* add black */
       gnugo_add_stone(get_moveX(sgf_prop, boardsize),
 		      get_moveY(sgf_prop, boardsize), BLACK);
-      sgffile_put_stone(get_moveX(sgf_prop, boardsize),
-			get_moveY(sgf_prop, boardsize), BLACK);
       break;
     case SGFAW:
       /* add white */
       gnugo_add_stone(get_moveX(sgf_prop, boardsize),
 		      get_moveY(sgf_prop, boardsize), WHITE);
-      sgffile_put_stone(get_moveX(sgf_prop, boardsize),
-			get_moveY(sgf_prop, boardsize), WHITE);
       break;
     case SGFB:
     case SGFW:
@@ -155,34 +163,56 @@ replay_node(SGFNode *node, int color_to_replay)
 
   if (color == color_to_replay || color_to_replay == GRAY) {
     /* Get a move from the engine for color. */
-    gnugo_genmove(&i, &j, color);
+    float move_val = gnugo_genmove(&i, &j, color);
     /* Now report on how well the computer generated the move. */
     if (i != m || j != n || !quiet) {
       mprintf("Move %d (%C): ", movenum + 1, color);
     
-      mprintf("GNU Go plays %m ", i, j);
-      if (!gnugo_is_pass(i, j))
-	printf("(%.2f) ", potential_moves[i][j]);
+      if (move_val < 0.0 && !gnugo_is_pass(i, j))
+	printf("GNU Go resigns ");
+      else {
+	mprintf("GNU Go plays %m ", i, j);
+	if (!gnugo_is_pass(i, j))
+	  printf("(%.2f) ", potential_moves[i][j]);
+      }
       mprintf("- Game move %m ", m, n);
       if (!gnugo_is_pass(m, n) && potential_moves[m][n] > 0.0)
 	printf("(%.2f) ", potential_moves[m][n]);
       printf("\n");
+
+      if (!quiet) {
+	replay_score += (potential_moves[i][j] - potential_moves[m][n]);
+	total_score += potential_moves[i][j];
+      }
     }
     if (i != m || j != n) {
-      char buf[127];
-      gg_snprintf(buf, 127, "GNU Go plays %s(%.2f) - Game move %s(%.2f)",
-	location_to_string(POS(i,j)),
-	gnugo_is_pass(i, j) ? 0 : potential_moves[i][j],
-	location_to_string(POS(m,n)),
-        gnugo_is_pass(m, n) && potential_moves[m][n] < 0.0 ? 0 : potential_moves[m][n]);
-      sgffile_write_comment(buf);
-      sgffile_write_circle_mark(i,j);
+      if (move_val < 0.0 && !gnugo_is_pass(i, j))
+	gg_snprintf(buf, 127, "GNU Go resigns - Game move %s (%.2f)",
+		    location_to_string(POS(m, n)),
+		    gnugo_is_pass(m, n)
+		    && potential_moves[m][n] < 0.0 ?
+			0 : potential_moves[m][n]);
+      else {      
+	gg_snprintf(buf, 127, "GNU Go plays %s (%.2f) - Game move %s (%.2f)",
+		    location_to_string(POS(i, j)),
+		    gnugo_is_pass(i, j) ? 0 : potential_moves[i][j],
+		    location_to_string(POS(m, n)),
+		    gnugo_is_pass(m, n)
+		    && potential_moves[m][n] < 0.0 ?
+			0 : potential_moves[m][n]);
+	sgfCircle(node, i, j);
+      }
     }
+    else
+      gg_snprintf(buf, 127, "GNU Go plays the same move %s (%.2f)",
+                 location_to_string(POS(i, j)),
+                 gnugo_is_pass(i, j) ? 0 : potential_moves[i][j]);
+    sgfAddComment(node, buf);
+    sgffile_add_debuginfo(node, 0);
   }
 
   /* Finally, do play the move from the file. */
   gnugo_play_move(m, n, color);
-  sgffile_move_made(m, n, color, 0);
 }
 
 
