@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,9 +25,6 @@
 #include "patterns.h"
 
 
-#include <stdlib.h>
-#include <assert.h>
-
 /* Try to match all (permutations of) connection patterns at (m,n).
  * For each match, if it is a B pattern, set cutting point in worm
  * data structure and make eye space marginal for the connection
@@ -35,10 +33,9 @@
  */
 
 static void
-cut_connect_callback(int m, int n, int color, struct pattern *pattern,
+cut_connect_callback(int anchor, int color, struct pattern *pattern,
 		     int ll, void *data)
 {
-  int anchor = POS(m, n);
   int move;
   int k;
   int first_dragon  = NO_MOVE;
@@ -46,21 +43,50 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
 
   int other = OTHER_COLOR(color);
   UNUSED(data);
+
+  /* Only match W patterns with standard connections. */
+  if ((pattern->class & CLASS_W) && experimental_connections)
+    return;
+
+  /* Only match Y patterns with experimental connections. */
+  if ((pattern->class & CLASS_Y) && !experimental_connections)
+    return;
   
-  move = AFFINE_TRANSFORM(pattern->movei, pattern->movej, ll, m, n);
+  move = AFFINE_TRANSFORM(pattern->move_offset, ll, anchor);
   
   if ((pattern->class & CLASS_B) && !safe_move(move, other))
     return;
 
-  /* If C pattern, test if there are more than one dragon in this
-   * pattern so that there is something to connect, before doing any
-   * expensive reading.
+  /* Reject C patterns in which an INHIBIT_CONNECTION was set
+   * previously during find_cuts.
    */
+
   if (pattern->class & CLASS_C) {
+    if (!experimental_connections) {
+      for (k = 0; k < pattern->patlen; ++k) { /* match each point */
+	/* transform pattern real coordinate */
+	int pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
+	if (board[pos]==EMPTY
+	    && ((color == WHITE
+		 && (white_eye[pos].type & INHIBIT_CONNECTION))
+		|| (color == BLACK
+		  && (black_eye[pos].type & INHIBIT_CONNECTION)))) {
+	  DEBUG(DEBUG_DRAGONS, 
+		"Connection pattern of type %s inhibited at %1m\n",
+		pattern->name, pos);
+	  return;
+	}
+      }
+    }
+
+    /* If C pattern, test if there are more than one dragon in this
+     * pattern so that there is something to connect, before doing any
+     * expensive reading.
+     */
+
     for (k = 0; k < pattern->patlen; ++k) { /* match each point */
       /* transform pattern real coordinate */
-      int pos = AFFINE_TRANSFORM(pattern->patn[k].x, pattern->patn[k].y,
-				 ll, m, n);
+      int pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
       
       /* Look for distinct dragons. */
       if (pattern->patn[k].att == ATT_O) {
@@ -82,7 +108,7 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
    * if the pattern must be rejected.
    */
   if (pattern->autohelper_flag & HAVE_CONSTRAINT) {
-    if (!pattern->autohelper(pattern, ll, move, color, 0))
+    if (!pattern->autohelper(ll, move, color, 0))
       return;
   }
 
@@ -94,14 +120,13 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
       return;
   }
 
-  if ((pattern->class & (CLASS_B | CLASS_E | CLASS_e))
+  if ((pattern->class & CLASS_B)
       && !(pattern->class & CLASS_s)) {
     /* Require that the X stones in the pattern are tactically safe. */
     for (k = 0; k < pattern->patlen; ++k) { /* match each point */
       if (pattern->patn[k].att == ATT_X) {
 	/* transform pattern real coordinate */
-	int pos = AFFINE_TRANSFORM(pattern->patn[k].x, pattern->patn[k].y,
-				   ll, m, n);
+	int pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
 
 	if (worm[pos].attack_codes[0] == WIN
 	  && (move == NO_MOVE
@@ -113,23 +138,17 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
 
   /* Get here => Pattern matches. */
   if (pattern->class & CLASS_B) {
-    TRACE("Cutting pattern %s+%d found at %1m\n",
+    DEBUG(DEBUG_DRAGONS, "Cutting pattern %s+%d found at %1m\n",
 	  pattern->name, ll, anchor);
-    TRACE("cutting point %1m\n", move);
+    DEBUG(DEBUG_DRAGONS, "cutting point %1m\n", move);
   }
   else if (pattern->class & CLASS_C)
-    TRACE("Connecting pattern %s+%d found at %1m\n",
-	  pattern->name, ll, anchor);
-  else if (pattern->class & CLASS_E)
-    TRACE("Eye space modifying pattern %s+%d found at %1m\n",
-	  pattern->name, ll, anchor);
-  else if (pattern->class & CLASS_e)
-    TRACE("Eye space modifying pattern %s+%d found at %1m\n",
+    DEBUG(DEBUG_DRAGONS, "Connecting pattern %s+%d found at %1m\n",
 	  pattern->name, ll, anchor);
 
   /* does the pattern have an action? */
   if (pattern->autohelper_flag & HAVE_ACTION) {
-    pattern->autohelper(pattern, ll, move, color, 1);
+    pattern->autohelper(ll, move, color, 1);
   }
 
   /* If it is a B pattern, set cutting point in eye data and make eye
@@ -162,14 +181,15 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
   second_dragon = NO_MOVE;
   for (k = 0; k < pattern->patlen; ++k) { /* match each point */
     /* transform pattern real coordinate */
-    int pos = AFFINE_TRANSFORM(pattern->patn[k].x, pattern->patn[k].y,
-			       ll, m, n);
+    int pos = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
 
     /* Look for dragons to amalgamate. Never amalgamate stones which
      * can be attacked.
      */
-    if ((pattern->class & CLASS_C) && (board[pos] == color)
-	&& (worm[pos].attack_codes[0] == 0)) {
+    if ((pattern->class & CLASS_C)
+	&& board[pos] == color
+	&& pattern->patn[k].att == ATT_O
+	&& ((pattern->class & CLASS_s) || worm[pos].attack_codes[0] == 0)) {
       if (first_dragon == NO_MOVE)
 	first_dragon = dragon[pos].origin;
       else if (second_dragon == NO_MOVE
@@ -180,22 +200,25 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
 	if (verbose || (debug & DEBUG_DRAGONS))
 	  gprintf("Pattern %s joins %C dragons %1m, %1m\n",
 		  pattern->name, color, first_dragon, second_dragon);
-	fflush(stdout);
 	join_dragons(second_dragon, first_dragon);
 	/* Now look for another second dragon. */
 	second_dragon = NO_MOVE;
 	first_dragon = dragon[pos].origin;
       }
     }
-
+    
     /* Inhibit connections */
     if (pattern->class & CLASS_B) {
       if (pattern->patn[k].att != ATT_not)
 	break; /* The inhibition points are guaranteed to come first. */
-      if (color == WHITE && white_eye[pos].color == WHITE_BORDER)
+      if (color == WHITE && white_eye[pos].color == WHITE_BORDER) {
 	white_eye[pos].type |= INHIBIT_CONNECTION;
-      else if (color == BLACK && black_eye[pos].color == BLACK_BORDER)
+	DEBUG(DEBUG_DRAGONS, "inhibiting connection at %1m\n", pos);
+      }
+      else if (color == BLACK && black_eye[pos].color == BLACK_BORDER) {
 	black_eye[pos].type |= INHIBIT_CONNECTION;
+	DEBUG(DEBUG_DRAGONS, "inhibiting connection at %1m\n", pos);
+      }
     }
   } /* loop over elements */
 }
@@ -203,30 +226,21 @@ cut_connect_callback(int m, int n, int color, struct pattern *pattern,
 
 /* Only consider B patterns. */
 static void
-cut_callback(int m, int n, int color, struct pattern *pattern, int ll,
+cut_callback(int anchor, int color, struct pattern *pattern, int ll,
 	     void *data)
 {
   if (pattern->class & CLASS_B)
-    cut_connect_callback(m, n, color, pattern, ll, data);
+    cut_connect_callback(anchor, color, pattern, ll, data);
 }
   
 
 /* Consider C patterns and those without classification. */
 static void
-conn_callback(int m, int n, int color, struct pattern *pattern, int ll,
+conn_callback(int anchor, int color, struct pattern *pattern, int ll,
 	      void *data)
 {
-  if (!(pattern->class & (CLASS_B | CLASS_e)))
-    cut_connect_callback(m, n, color, pattern, ll, data);
-}
-  
-/* Only consider e patterns. */
-static void
-modify_eye_callback(int m, int n, int color, struct pattern *pattern,
-		     int ll, void *data)
-{
-  if (pattern->class & CLASS_e)
-    cut_connect_callback(m, n, color, pattern, ll, data);
+  if (!(pattern->class & CLASS_B))
+    cut_connect_callback(anchor, color, pattern, ll, data);
 }
   
 /* Find cutting points which should inhibit amalgamations and sever
@@ -245,12 +259,6 @@ find_connections(void)
   matchpat(conn_callback, ANCHOR_COLOR, &conn_db, NULL, NULL);
 }
 
-void
-modify_eye_spaces(void)
-{
-  matchpat(modify_eye_callback, ANCHOR_COLOR, &conn_db, NULL, NULL);
-}
-
 
 /*
  * Local Variables:
