@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -129,12 +130,6 @@ struct invariant_hash {
 /* This is defined in engine/matchpat.c */
 extern const int transformations[8][2][2];
 
-#define TRANSFORM(i, j, ti, tj, trans) \
-  do { \
-    *ti = transformations[trans][0][0] * (i) + transformations[trans][0][1] * (j); \
-    *tj = transformations[trans][1][0] * (i) + transformations[trans][1][1] * (j); \
-  } while (0)
-
 
 /* A situation is the combination of a board position and the move to
  * be made. We use the invariant hashes excluding and including the move
@@ -263,7 +258,7 @@ init_zobrist_table(unsigned int hash[8][MAX_BOARD][MAX_BOARD])
   for (k = 1; k < 8; k++)
     for (m = 0; m < board_size; m++)
       for (n = 0; n < board_size; n++) {
-	TRANSFORM(m-mid, n-mid, &i, &j, k);
+	TRANSFORM2(m-mid, n-mid, &i, &j, k);
 	hash[k][m][n] = hash[0][i+mid][j+mid];
       }
   
@@ -1216,9 +1211,11 @@ main(int argc, char *argv[])
     
     /* Tell the user everything worked out fine */
     fprintf(stderr, "The pattern database was produced with no errors.\n");
-    for (i = 0; i < number_of_games; i++) {
-      if (unused_games[i]) number_of_unused_games++;
-    }
+
+    for (i = 0; i < number_of_games; i++)
+      if (unused_games[i])
+	number_of_unused_games++;
+
     fprintf(stderr, "Out of %d games, %d were not used.\n", 
 	    number_of_games, number_of_unused_games);
   }
