@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -23,17 +24,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <string.h>
 #include <ctype.h>
 
 #include "../sgf/sgftree.h"
 #include "liberty.h"
 
-#ifndef VARIADIC_MACROS
-int trace_dummy;
-#endif
-
 #define USAGE "\
 Usage : joseki prefix filename\n\
 "
@@ -46,6 +42,16 @@ Usage : joseki prefix filename\n\
 #define ANTISUJI  4
 #define TENUKI_OK 5
 
+
+/* We don't want to play moves on edges of board which might have been
+ * cropped, since there might appear an accidential capture.
+ */
+#define SAFE_ON_BOARD(i, j) ((i) >= 0 && (j) >= 0\
+			     && (i) < MAX_BOARD - 1 && (j) < MAX_BOARD - 1)
+
+static int boardsize;
+
+
 /* Identify the type of joseki move.
  * FIXME: We might want the relax the requirement that this info comes
  *        as the very first character.
@@ -136,7 +142,7 @@ write_diagram(int movei, int movej, int color, int marki, int markj,
   int i, j;
   
   for (i = -1; i <= marki; i++) {
-    for (j = markj; j < board_size; j++) {
+    for (j = markj; j >= 0; j--) {
       if (i == -1)
 	printf("-");
       else if (labels && labels[i][j])
@@ -146,7 +152,7 @@ write_diagram(int movei, int movej, int color, int marki, int markj,
       else if (BOARD(i, j) == color)
 	printf("O");
       else if (BOARD(i, j) == OTHER_COLOR(color))
-	printf("X");
+	printf("X");               
       else
 	printf(".");
     }
@@ -159,10 +165,10 @@ write_diagram(int movei, int movej, int color, int marki, int markj,
 
 /* Write the colon line of the pattern. */
 static void
-write_colon_line(int move_type, char *text)
+write_colon_line(int move_type, char symmetry, char *text)
 {
   char *p;
-  
+
   /* Locate a possible colon line in the sgf file comment. */
   if (!text)
     p = NULL;
@@ -174,7 +180,7 @@ write_colon_line(int move_type, char *text)
       p += 2;
   }
 
-  printf(":8,sF");
+  printf(":%c,sF", symmetry);
   switch (move_type) {
   case URGENT:
     printf("U");
@@ -191,7 +197,9 @@ write_colon_line(int move_type, char *text)
   case TENUKI_OK:
     printf("t");
     break;
-  /* Antisuji moves handled elsewhere. */
+  case ANTISUJI:
+    printf("N");
+    break;
   }
 
   if (p) {
@@ -208,6 +216,24 @@ write_colon_line(int move_type, char *text)
 }
 
 
+/* Check if the board and labels are symmetric. */
+static int
+board_is_symmetric(int n, char labels[MAX_BOARD][MAX_BOARD])
+{
+  int i;
+  int j;
+
+  for (i = 0; i <= n; i++) {
+    for (j = 0; j < i; j++) {
+      if (BOARD(i, j) != BOARD(j, i)
+	  || (labels && labels[i][j] != labels[j][i]))
+	return 0;
+    }
+  }
+
+  return 1;
+}
+
 /* Write a pattern to stdout. */
 static void
 make_pattern(int movei, int movej, int color,
@@ -217,24 +243,27 @@ make_pattern(int movei, int movej, int color,
 {
   static int pattern_number = 0;
   int move_type;
-  
+  char symmetry = '8';
+
   pattern_number++;
   move_type = identify_move_type(text);
-  
+
   printf("Pattern %s%d\n", prefix, pattern_number);
 
   /* Write comments. */
   write_selected_lines(text, '#');
   printf("\n");
-  
+
   /* Write the main diagram. */
   write_diagram(movei, movej, color, marki, markj, NULL);
   printf("\n");
 
   /* Write the colon line. */
-  write_colon_line(move_type, text);
+  if (movei == movej && marki == markj && board_is_symmetric(marki, labels))
+    symmetry = '/';
+  write_colon_line(move_type, symmetry, text);
   printf("\n");
-  
+
   /* Write the constraint diagram if there are any labels, a
    * constraint line, or an action line.
    */
@@ -248,15 +277,11 @@ make_pattern(int movei, int movej, int color,
     /* Write constraint and action lines. */
     write_selected_lines(text, ';');
     write_selected_lines(text, '>');
-    if (move_type == ANTISUJI)
-      printf(">antisuji(*);\n");
     printf("\n");
   }
-  else if (move_type == ANTISUJI)
-    printf(">antisuji(*);\n\n");
 
   printf("\n");
-  
+
   /* Basic sanity checking. We do this at the end to simplify debugging. */
   if (multiple_marks)
     fprintf(stderr, "Warning: Multiple square marks in pattern %s%d\n",
@@ -300,26 +325,32 @@ analyze_node(SGFNode *node, const char *prefix)
     case SGFMA: /* Mark */
       if (marki != -1)
 	multiple_marks = 1;
-      else
-	get_moveXY(prop, &marki, &markj, board_size);
+      else {
+	get_moveXY(prop, &marki, &markj, boardsize);
+	markj = boardsize - 1 - markj;
+      }
       break;
       
     case SGFW: /* White move */
       color = WHITE;
-      get_moveXY(prop, &movei, &movej, board_size);
+      get_moveXY(prop, &movei, &movej, boardsize);
+      movej = boardsize - 1 - movej;
       break;
       
     case SGFB: /* Black move */
       color = BLACK;
-      get_moveXY(prop, &movei, &movej, board_size);
+      get_moveXY(prop, &movei, &movej, boardsize);
+      movej = boardsize - 1 - movej;
       break;
 
     case SGFLB: /* Label, with value like "mh:A" */
-      get_moveXY(prop, &i, &j, board_size);
-      assert(i >= 0 && i < board_size && j >= 0 && j < board_size);
-      assert(prop->value[2] == ':');
-      labels[i][j] = prop->value[3];
-      label_found = 1;
+      get_moveXY(prop, &i, &j, boardsize);
+      j = boardsize - 1 - j;
+      gg_assert(prop->value[2] == ':');
+      if (ON_BOARD2(i, j)) {
+	labels[i][j] = prop->value[3];
+	label_found = 1;
+      }
       break;
 
     case SGFC: /* Comment */
@@ -329,16 +360,16 @@ analyze_node(SGFNode *node, const char *prefix)
   }
 
   /* If we have a move and a square mark, produce a pattern. */
-  if (movei != -1 && marki != -1)
+  if (SAFE_ON_BOARD(movei, movej) && ON_BOARD2(marki, markj))
     make_pattern(movei, movej, color, marki, markj, multiple_marks,
 		 (label_found ? labels : NULL), comment, prefix);
 
   /* Traverse child, if any. */
   if (node->child) {
-    if (movei != -1)
+    if (SAFE_ON_BOARD(movei, movej))
       tryko(POS(movei, movej), color, NULL, EMPTY, NO_MOVE);
     analyze_node(node->child, prefix);
-    if (movei != -1)
+    if (SAFE_ON_BOARD(movei, movej))
       popgo();
   }
 
@@ -347,6 +378,7 @@ analyze_node(SGFNode *node, const char *prefix)
     analyze_node(node->next, prefix);
 }
 
+
 int
 main(int argc, char *argv[])
 {
@@ -372,10 +404,11 @@ main(int argc, char *argv[])
 
 #define PREAMBLE "\
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
-# This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       #\n\
+# This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       #\n\
 # http://www.gnu.org/software/gnugo/ for more information.          #\n\
 #                                                                   #\n\
-# Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. #\n\
+# Copyright 1999, 2000, 2001, 2002 and 2003                         #\n\
+# by the Free Software Foundation.                                  #\n\
 #                                                                   #\n\
 # This program is free software; you can redistribute it and/or     #\n\
 # modify it under the terms of the GNU General Public License as    #\n\
@@ -390,7 +423,7 @@ main(int argc, char *argv[])
 # License along with this program; if not, write to the Free        #\n\
 # Software Foundation, Inc., 59 Temple Place - Suite 330,           #\n\
 # Boston, MA 02111, USA.                                            #\n\
-#                                                                   #\n\
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
 # This file is automatically generated by joseki. Do not edit       #\n\
 # it directly. Instead, edit the corresponding sgf file.            #\n\
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
@@ -400,8 +433,15 @@ main(int argc, char *argv[])
   printf(PREAMBLE);
 
   /* Call the engine to setup and clear the board. */
+  board_size = MAX_BOARD;
   clear_board();
   
+  /* Determine board size of the file. */
+  if (!sgfGetIntProperty(sgf, "SZ", &boardsize)) {
+    fprintf(stderr, "joseki: error: can't determine file board size\n");
+    return 1;
+  }
+  
   /* Walk through the tree and make patterns. */
   analyze_node(sgf, prefix);
     
