@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-# This program is distributed with GNU GO, a Go program.        #
+# This program is distributed with GNU Go, a Go program.        #
 #                                                               #
 # Write gnugo@gnu.org or see http://www.gnu.org/software/gnugo/ #
 # for more information.                                         #
@@ -201,7 +201,9 @@ $pidg = open3($goprog_in, $goprog_out, $goprog_err, $goprog)
   or die "Couldn't launch GNU Go: $!";
 print "goprog pid: $pidg\n" if $verbose > 1;
 my ($goprog_exe) = split (" ", $goprog);
--e $goprog_exe  or die "Couldn't locate go program: $goprog_exe";
+-e $goprog_exe
+  or ($goprog_exe = "$goprog_exe.exe") && -e $goprog_exe 
+  or die "Couldn't locate go program: $goprog_exe";
 $goprog_timestamp = (stat $goprog_exe)->mtime;
 
 go_command("name");
@@ -520,7 +522,8 @@ sub tally_result {
   $failures++ if $status eq "failed";
   $unexpected_fail++ if $status eq "FAILED";
   
-  if ($status ne "skipped") {
+  if (($verbose and $status ne "skipped") or
+      (!$verbose and ($status eq "PASSED" or $status eq "FAILED")) ) {
     print "$g_curtestfile:$number: $status: correct: $correct  answer: $incorrect\n";
   }
   
@@ -919,7 +922,7 @@ sub allTargets {
       chop($_ = <MAKEFILE>);
       while ($_) {
         push @files, $_ =~ /\s+(\w+\.tst)/;
-        chop($_ = <MAKEFILE>);
+        chop if defined($_ = <MAKEFILE>);
       }
     }
   }
