@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -56,8 +57,6 @@
 /*                     Some utility functions.                      */
 /* ================================================================ */
 
-/* General FIXME:  Move these somewhere? */
-
 /*
  * Utility: a checking, initializing malloc
  */
@@ -68,8 +67,8 @@ xalloc(unsigned int size)
   void *pt = malloc(size);
 
   if (!pt) {
-    fprintf(stderr, "%s: Out of memory!\n", "xalloc");
-    exit(2);
+    fprintf(stderr, "xalloc: Out of memory!\n");
+    exit(EXIT_FAILURE);
   }
 
   memset(pt, 0, (unsigned long) size);
@@ -82,8 +81,8 @@ xrealloc(void *pt, unsigned int size)
   void *ptnew = realloc(pt, size);
 
   if (!ptnew) {
-    fprintf(stderr, "%s: Out of memory!\n", "xrealloc");
-    exit(2);
+    fprintf(stderr, "xrealloc: Out of memory!\n");
+    exit(EXIT_FAILURE);
   }
   return ptnew;
 }
@@ -232,6 +231,24 @@ sgfGetCharProperty(SGFNode *node, const char *name, char **value)
 }
 
 
+/*
+ * Is there a property of this type in the node?
+ */
+
+static int
+sgfHasProperty(SGFNode *node, const char *name)
+{
+  SGFProperty *prop;
+  short nam = name[0] | name[1] << 8;
+
+  for (prop = node->props; prop; prop = prop->next)
+    if (prop->name == nam)
+      return 1;
+
+  return 0;
+}
+
+
 /*
  * Overwrite a property from an SGF node with text or create a new
  * one if it does not exist.
@@ -343,25 +360,18 @@ sgfRoot(SGFNode *node)
 
 /*
  * Make an SGF property.
- *
- * FIXME: To speed up, we should have MkMoveProperty.
- *      Comment: We *never* have any speed problems with these functions. /iw
  */
-
-SGFProperty *
-sgfMkProperty(const char *name, const  char *value,
-	      SGFNode *node, SGFProperty *last)
+static SGFProperty *
+do_sgf_make_property(short sgf_name,  const char *value,
+		     SGFNode *node, SGFProperty *last)
 {
   SGFProperty *prop;
 
   prop = (SGFProperty *) xalloc(sizeof(SGFProperty));
-  prop->value = xalloc(strlen(value)+1);
-  prop->next = NULL;
-  if (strlen(name) == 1) 
-    prop->name = name[0] | (short) (' ' << 8);
-  else
-    prop->name = name[0] | name[1] << 8;
+  prop->name = sgf_name;
+  prop->value = xalloc(strlen(value) + 1);
   strcpy(prop->value, value);
+  prop->next = NULL;
 
   if (last == NULL)
     node->props = prop;
@@ -371,6 +381,65 @@ sgfMkProperty(const char *name, const  char *value,
   return prop;
 }
 
+
+/* Make an SGF property.  In case of a property with a range it
+ * expands it and makes several properties instead.
+ */
+SGFProperty *
+sgfMkProperty(const char *name, const  char *value,
+	      SGFNode *node, SGFProperty *last)
+{
+  static const short properties_allowing_ranges[12] = {
+    /* Board setup properties. */
+    SGFAB, SGFAW, SGFAE,
+
+    /* Markup properties. */
+    SGFCR, SGFMA, SGFSQ, SGFTR, SGFDD, SGFSL,
+
+    /* Miscellaneous properties. */
+    SGFVW,
+
+    /* Go-specific properties. */
+    SGFTB, SGFTW
+  };
+
+  int k;
+  short sgf_name;
+
+  if (strlen(name) == 1)
+    sgf_name = name[0] | (short) (' ' << 8);
+  else
+    sgf_name = name[0] | name[1] << 8;
+
+  for (k = 0; k < 12; k++) {
+    if (properties_allowing_ranges[k] == sgf_name)
+      break;
+  }
+
+  if (k < 12
+      && strlen(value) == 5
+      && value[2] == ':') {
+    char x1 = value[0];
+    char y1 = value[1];
+    char x2 = value[3];
+    char y2 = value[4];
+    char new_value[] = "xy";
+
+    if (x1 <= x2 && y1 <= y2) {
+      for (new_value[0] = x1; new_value[0] <= x2; new_value[0]++) {
+	for (new_value[1] = y1; new_value[1] <= y2; new_value[1]++)
+	  last = do_sgf_make_property(sgf_name, new_value, node, last);
+      }
+
+      return last;
+    }
+  }
+
+  /* Not a range property. */
+  return do_sgf_make_property(sgf_name, value, node, last);
+}
+
+
 /*
  * Recursively free an SGF property.
  *
@@ -418,23 +487,21 @@ sgfAddPlay(SGFNode *node, int who, int movex, int movey)
 {
   char move[3];
   SGFNode *new;
-
+  
   /* a pass move? */
   if (movex == -1 && movey == -1)
     move[0] = 0;
   else
     sprintf(move, "%c%c", movey + 'a', movex + 'a');
 
-  if (node->child) {
-    node = sgfStartVariantFirst(node->child);
-    sgfAddProperty(node, (who == BLACK) ? "B" : "W", move);
-
-    return node;
+  if (node->child)
+    new = sgfStartVariantFirst(node->child);
+  else {
+    new = sgfNewNode();
+    node->child = new;
+    new->parent = node;
   }
-
-  new = sgfNewNode();
-  node->child = new;
-  new->parent = node;
+  
   sgfAddProperty(new, (who == BLACK) ? "B" : "W", move);
 
   return new;
@@ -555,6 +622,39 @@ sgfTriangle(SGFNode *node, int i, int j)
 }
 
 
+/*
+ * Place a label on the board at position (i, j).
+ */
+
+SGFNode *
+sgfLabel(SGFNode *node, const char *label, int i, int j)
+{
+  /* allows 12 chars labels - more than enough */
+  char text[16];
+
+  gg_snprintf(text, 16, "%c%c:%s", j+'a', i+'a', label);
+  sgfAddProperty(node, "LB", text);
+
+  return node;
+}
+
+
+/*
+ * Place a numeric label on the board at position (i, j).
+ */
+
+SGFNode *
+sgfLabelInt(SGFNode *node, int num, int i, int j)
+{
+  char text[16];
+
+  gg_snprintf(text, 16, "%c%c:%d", j+'a', i+'a', num);
+  sgfAddProperty(node, "LB", text);
+
+  return node;
+}
+
+
 /*
  * Place a circle mark on the board at position (i, j).
  */
@@ -663,6 +763,76 @@ sgfAddChild(SGFNode *node)
 }
 
 
+/*
+ * Write result of the game to the game tree.
+ */
+
+void
+sgfWriteResult(SGFNode *node, float score, int overwrite)
+{
+  char text[8];
+  char winner;
+  float s;
+  int dummy;
+
+  /* If not writing to the SGF file, skip everything and return now. */
+  if (!node)
+    return;
+
+  /* If not overwriting and there already is a result property, return. */
+  if (!overwrite)
+    if (sgfGetIntProperty(node, "RE", &dummy))
+      return;
+
+  if (score > 0.0) {
+    winner = 'W';
+    s = score;
+  }
+  else if (score < 0.0) {
+    winner = 'B';
+    s = -score;
+  }
+  else {
+    winner = '0';
+    s = 0;
+  }
+
+  if (winner == '0')
+    gg_snprintf(text, 8, "0");
+  else if (score < 1000.0 && score > -1000.0)
+    gg_snprintf(text, 8, "%c+%3.1f", winner, s);
+  else
+    gg_snprintf(text, 8, "%c+%c", winner, 'R');
+  sgfOverwriteProperty(node, "RE", text);
+}
+
+
+void
+sgf_write_header(SGFNode *root, int overwrite, int seed, float komi,
+		 int level, int rules)
+{
+  time_t curtime = time(NULL);
+  struct tm *loctime = localtime(&curtime);
+  char str[128];
+  int dummy;
+
+  gg_snprintf(str, 128, "GNU Go %s Random Seed %d level %d", 
+	      VERSION, seed, level);
+  if (overwrite || !sgfGetIntProperty(root, "GN", &dummy))
+    sgfOverwriteProperty(root, "GN", str);
+  gg_snprintf(str, 128, "%4.4i-%2.2i-%2.2i",
+	      loctime->tm_year+1900, loctime->tm_mon+1, loctime->tm_mday);
+  if (overwrite || !sgfGetIntProperty(root, "DT", &dummy))
+    sgfOverwriteProperty(root, "DT", str);
+  if (overwrite || !sgfGetIntProperty(root, "AP", &dummy))
+    sgfOverwriteProperty(root, "AP", "GNU Go:"VERSION);
+  if (overwrite || !sgfGetIntProperty(root, "RU", &dummy))
+    sgfOverwriteProperty(root, "RU", rules ? "Chinese" : "Japanese");
+  sgfOverwriteProperty(root, "FF", "4");
+  sgfOverwritePropertyFloat(root, "KM", komi);
+}
+
+
 /* ================================================================ */
 /*                          Read SGF tree                           */
 /* ================================================================ */
@@ -757,9 +927,10 @@ match(int expected)
 static void
 propident(char *buffer, int size)
 {
-  if (lookahead == -1 || !isupper(lookahead)) 
+  if (lookahead == EOF || !isupper(lookahead)) 
     parse_error("Expected an upper case letter.", 0);
-  while (lookahead != -1 && isalpha(lookahead)) {
+  
+  while (lookahead != EOF && isalpha(lookahead)) {
     if (isupper(lookahead) && size > 1) {
       *buffer++ = lookahead;
       size--;
@@ -776,7 +947,7 @@ propvalue(char *buffer, int size)
   char *p = buffer;
 
   match('[');
-  while (lookahead != ']' && lookahead != '\0') {
+  while (lookahead != ']' && lookahead != EOF) {
     if (lookahead == '\\') {
       lookahead = sgf_getch();
       /* Follow the FF4 definition of backslash */
@@ -798,6 +969,7 @@ propvalue(char *buffer, int size)
     lookahead = sgf_getch();
   }
   match(']');
+  
   /* Remove trailing whitespace */
   --p;
   while (p > buffer && isspace((int) *p))
@@ -826,7 +998,7 @@ node(SGFNode *n)
 {
   SGFProperty *last = NULL;
   match(';');
-  while (lookahead != -1 && isupper(lookahead))
+  while (lookahead != EOF && isupper(lookahead))
     last = property(n, last);
 }
 
@@ -853,7 +1025,7 @@ gametree(SGFNode **p, SGFNode *parent, int mode)
     match('(');
   else
     for (;;) {
-      if (lookahead == -1) {
+      if (lookahead == EOF) {
 	parse_error("Empty file?", 0);
 	break;
       }
@@ -899,7 +1071,7 @@ gametreefuseki(SGFNode **p, SGFNode *parent, int mode,
     match('(');
   else
     for (;;) {
-      if (lookahead == -1) {
+      if (lookahead == EOF) {
 	parse_error("Empty file?", 0);
 	break;
       }
@@ -1050,106 +1222,109 @@ readsgffile(const char *filename)
 
 #define OPTION_STRICT_FF4 0
 
-
-/*
- * p->child is the next move.
- * p->next  is the next variation
- */
-
-
-#if 0
-void
-sgf_putch(char c)
-{
-  fputc(c, ttsgfout);
-}
-
-
-void
-sgf_puts(const char *s)
-{
-  fputs(s, ttsgfout);
-}
-
-
-void
-sgf_putint(int i)
-{
-  char buf[10];
-  char *p = buf;
-  
-  if (i < 0) {
-    sgf_putch('-');
-    i = -i;
-  }
-  do {
-    *p++ = i % 10;
-    i /= 10;
-  } while (i > 9);
-  while (p > buf)
-    sgf_putch((char) (*--p + '0'));
-  /* MS-C complains even of this conversion, if not cast... */
-}
-
+static int sgf_column = 0;
 
 static void
-unparse_prop(SGFProperty *p)
+sgf_putc(int c, FILE *file)
 {
-  char name[3], *cp;
-
-  name[2] = 0;
-  while (p != NULL) {
-    name[0] = p->name & 255;
-    name[1] = p->name >> 8;
-    if (name[1] == ' ')
-      name[1] = 0;
-    sgf_puts(name);
-    sgf_putch('[');
-    for (cp = p->value; *cp; cp++) {
-      if (*cp == '[' || *cp == ']' || *cp == '\\')
-	sgf_putch('\\');
-      sgf_putch(*cp);
-    }
-    sgf_putch(']');
-    p = p->next;
-  }
-}
-#endif
-
-
-/*
- * Print all unprinted property values at node N to file.
- */
-
-static void
-sgfPrintAllProperties(FILE *file, SGFNode *node)
-{
-  SGFProperty *prop;
-  SGFProperty *last;
-  
-  prop = node->props;
-  if (!prop)
+  if (c == '\n' && sgf_column == 0)
     return;
 
-  do {
-    if (!(prop->name & 0x20)) {
-      last = prop;
-          
-      fprintf(file, "%c%c[%s]", prop->name&0xff, (prop->name>>8) & 0xff, 
-	      prop->value);
-      prop = prop->next;
-      while (prop) {
-	if (last->name == prop->name) {
-	  fprintf(file, "[%s]", prop->value);
-	  prop->name |= 0x20;  /* indicate already printed */
-	}
-	prop = prop->next;
-      }
-      prop = last;
-      prop->name |= 0x20;  /* indicate already printed */
+  fputc(c, file);
+
+  if (c == '\n')
+    sgf_column = 0;
+  else
+    sgf_column++;
+
+  if (c == ']' && sgf_column > 60) {
+    fputc('\n', file);
+    sgf_column = 0;
+  }
+}
+
+static void
+sgf_puts(const char *s, FILE *file)
+{
+  for (; *s; s++) {
+    if (*s == '[' || *s == ']' || *s == '\\') {
+      fputc('\\', file);
+      sgf_column++;
     }
-    prop = prop->next;
-  } while (prop);
+    fputc((int) *s, file);
+    sgf_column++;
+  }
+}
+
+/* Print all properties with the given name in a node to file and mark
+ * them as printed.
+ *
+ * If is_comment is 1, multiple properties are concatenated with a
+ * newline. I.e. we write
+ *
+ * C[comment1
+ * comment2]
+ *
+ * instead of
+ *
+ * C[comment1][comment2]
+ *
+ * Most other property types should be written in the latter style.
+ */
+
+static void
+sgf_print_name(FILE *file, short name)
+{
+  sgf_putc(name & 0xff, file);
+  if (name >> 8 != ' ')
+    sgf_putc(name >> 8, file);
+}
+
+static void
+sgf_print_property(FILE *file, SGFNode *node, short name, int is_comment)
+{
+  int n = 0;
+  SGFProperty *prop;
+
+  for (prop = node->props; prop; prop = prop->next) {
+    if (prop->name == name) {
+      prop->name |= 0x20;  /* Indicate already printed. */
+      if (n == 0) {
+	sgf_print_name(file, name);
+	sgf_putc('[', file);
+      }
+      else if (is_comment)
+	sgf_putc('\n', file);
+      else {
+	sgf_putc(']', file);
+	sgf_putc('[', file);
+      }
+      
+      sgf_puts(prop->value, file);
+      n++;
+    }
+  }
+
+  if (n > 0)
+    sgf_putc(']', file);
+
+  /* Add a newline after certain properties. */
+  if (name == SGFAB || name == SGFAW || name == SGFAE || (is_comment && n > 1))
+    sgf_putc('\n', file);
+}
+
+/*
+ * Print all remaining unprinted property values at node N to file.
+ */
+
+static void
+sgfPrintRemainingProperties(FILE *file, SGFNode *node)
+{
+  SGFProperty *prop;
+
+  for (prop = node->props; prop; prop = prop->next)
+    if (!(prop->name & 0x20))
+      sgf_print_property(file, node, prop->name, 0);
 }
 
 
@@ -1157,28 +1332,12 @@ sgfPrintAllProperties(FILE *file, SGFNode *node)
  * Print the property values of NAME at node N and mark it as printed. 
  */
 
-int
+static void
 sgfPrintCharProperty(FILE *file, SGFNode *node, const char *name)
 {
-  int first = 1;
-  SGFProperty *prop;
   short nam = name[0] | name[1] << 8;
-
-  for (prop = node->props; prop; prop = prop->next) {
-    if (prop->name == nam) {
-      prop->name |= 0x20;  /* indicate already printed */
-      if (first) {
-	if (name[1] == ' ')
-	  fprintf(file, "%c", name[0]);
-	else
-	  fprintf(file, "%s", name);
-	first = 0;
-      }
-      fprintf(file, "[%s]", prop->value);
-    }
-  }
-
-  return !first;
+  
+  sgf_print_property(file, node, nam, 0);
 }
 
 
@@ -1189,137 +1348,82 @@ sgfPrintCharProperty(FILE *file, SGFNode *node, const char *name)
  *       what the sgfspec says.
  */
 
-int
+static void
 sgfPrintCommentProperty(FILE *file, SGFNode *node, const char *name)
 {
-  int first = 1;
-  SGFProperty *prop;
   short nam = name[0] | name[1] << 8;
-
-  for (prop = node->props; prop; prop = prop->next) {
-    if (prop->name == nam) {
-      prop->name |= 0x20;  /*indicate already printed*/
-      if (first) {
-	if (name[1] == ' ')
-	  fprintf(file, "%c[%s", name[0], prop->value);
-	else
-	  fprintf(file, "%s[%s", name, prop->value);
-	first = 0;
-      }
-      else
-	fprintf(file, "\n%s", prop->value);
-    }
-  }
-
-  if (!first) {
-    fprintf(file, "]");
-    return 1;
-  }
-  else
-    return 0;
+  
+  sgf_print_property(file, node, nam, 1);
 }
 
 
-/*
- * Write result of the game to the game tree.
- */
-
-void
-sgfWriteResult(SGFNode *node, float score, int overwrite)
-{
-  char text[8];
-  char winner;
-  float s;
-  int dummy;
-
-  /* If not writing to the SGF file, skip everything and return now. */
-  if (!node)
-    return;
-
-  /* If not overwriting and there already is a result property, return. */
-  if (!overwrite)
-    if (sgfGetIntProperty(node, "RE", &dummy))
-      return;
-
-  if (score > 0.0) {
-    winner = 'W';
-    s = score;
-  }
-  else if (score < 0.0) {
-    winner = 'B';
-    s = -score;
-  }
-  else {
-    winner = '0';
-    s = 0;
-  }
-
-  if (winner == '0')
-    gg_snprintf(text, 8, "0");
-  else if (score < 1000.0 && score > -1000.0)
-    gg_snprintf(text, 8, "%c+%3.1f", winner, s);
-  else
-    gg_snprintf(text, 8, "%c+%c", winner, 'R');
-  sgfOverwriteProperty(node, "RE", text);
-}
-
-
-
 static void
 unparse_node(FILE *file, SGFNode *node)
 {
-  fputs("\n ;", file);
+  sgf_putc(';', file);
   sgfPrintCharProperty(file, node, "B ");
   sgfPrintCharProperty(file, node, "W ");
   sgfPrintCommentProperty(file, node, "N ");
   sgfPrintCommentProperty(file, node, "C ");
-  if (sgfPrintCommentProperty(file, node, "PB")
-      || sgfPrintCommentProperty(file, node, "BR"))
-    fputs("\n", file);
-  if (sgfPrintCommentProperty(file, node, "PW")
-      || sgfPrintCommentProperty(file, node, "WR"))
-    fputs("\n", file);
-  sgfPrintAllProperties(file, node);
+  sgfPrintRemainingProperties(file, node);
 }
 
 
 static void
-unparse_root(FILE *file, SGFNode *node, int root)
+unparse_root(FILE *file, SGFNode *node)
 {
-  fputs(";", file);
+  sgf_putc(';', file);
+  
+  if (sgfHasProperty(node, "GM"))
+    sgfPrintCharProperty(file, node, "GM");
+  else {
+    fputs("GM[1]", file);
+    sgf_column += 5;
+  }
+  
   sgfPrintCharProperty(file, node, "FF");
-  if (sgfPrintCharProperty(file, node, "GM"))
-    fputs("\n", file);
-  else if (root)
-    fputs("GM[1]\n", file);
+  sgf_putc('\n', file);
 
-  if (sgfPrintCharProperty(file, node, "SZ"))
-    fputs("\n", file);
-  if (sgfPrintCharProperty(file, node, "GN"))
-    fputs("\n", file);
-  if (sgfPrintCharProperty(file, node, "DT"))
-    fputs("\n", file);
-  if (sgfPrintCommentProperty(file, node, "PB")
-      || sgfPrintCommentProperty(file, node, "BR"))
-    fputs("\n", file);
-  if (sgfPrintCommentProperty(file, node, "PW")
-      || sgfPrintCommentProperty(file, node, "WR"))
-    fputs("\n", file);
-  sgfPrintCharProperty(file, node, "B ");
-  sgfPrintCharProperty(file, node, "W ");
+  sgfPrintCharProperty(file, node, "SZ");
+  sgf_putc('\n', file);
+  
+  sgfPrintCharProperty(file, node, "GN");
+  sgf_putc('\n', file);
+  
+  sgfPrintCharProperty(file, node, "DT");
+  sgf_putc('\n', file);
+  
+  sgfPrintCommentProperty(file, node, "PB");
+  sgfPrintCommentProperty(file, node, "BR");
+  sgf_putc('\n', file);
+  
+  sgfPrintCommentProperty(file, node, "PW");
+  sgfPrintCommentProperty(file, node, "WR");
+  sgf_putc('\n', file);
+  
   sgfPrintCommentProperty(file, node, "N ");
   sgfPrintCommentProperty(file, node, "C ");
-  sgfPrintAllProperties(file, node);
+  sgfPrintRemainingProperties(file, node);
+
+  sgf_putc('\n', file);
 }
 
 
+/*
+ * p->child is the next move.
+ * p->next  is the next variation
+ */
+
 static void
 unparse_game(FILE *file, SGFNode *node, int root)
 {
   if (!root)
-    fputc('\n', file);
-  fputc('(', file);
-  unparse_root(file, node, root);
+    sgf_putc('\n', file);
+  sgf_putc('(', file);
+  if (root)
+    unparse_root(file, node);
+  else
+    unparse_node(file, node);
 
   node = node->child;
   while (node != NULL && node->next == NULL) {
@@ -1331,33 +1435,39 @@ unparse_game(FILE *file, SGFNode *node, int root)
     unparse_game(file, node, 0);
     node = node->next;
   }
-  fputs(")", file);
+  sgf_putc(')', file);
+  if (root)
+    sgf_putc('\n', file);
 }
 
-void
-sgf_write_header(SGFNode *root, int overwrite, int seed, float komi, int level)
+/* Printed properties are marked by adding the 0x20 bit to the
+ * property name (changing an upper case letter to lower case). This
+ * function removes this mark so that we can print the property next
+ * time too. It recurses to all properties in the linked list.
+ */
+static void
+restore_property(SGFProperty *prop)
 {
-  time_t curtime = time(NULL);
-  struct tm *loctime = localtime(&curtime);
-  char str[128];
-  int dummy;
-
-  gg_snprintf(str, 128, "GNU Go %s Random Seed %d level %d", 
-	      VERSION, seed, level);
-  if (overwrite || !sgfGetIntProperty(root, "GN", &dummy))
-    sgfOverwriteProperty(root, "GN", str);
-  gg_snprintf(str, 128, "%4.4i-%2.2i-%2.2i",
-	      loctime->tm_year+1900, loctime->tm_mon+1, loctime->tm_mday);
-  if (overwrite || !sgfGetIntProperty(root, "DT", &dummy))
-    sgfOverwriteProperty(root, "DT", str);
-  if (overwrite || !sgfGetIntProperty(root, "AP", &dummy))
-    sgfOverwriteProperty(root, "AP", PACKAGE":"VERSION);
-  if (overwrite || !sgfGetIntProperty(root, "RU", &dummy))
-    sgfOverwriteProperty(root, "RU", "Japanese");
-  sgfOverwriteProperty(root, "FF", "4");
-  sgfOverwritePropertyFloat(root, "KM", komi);
+  if (prop) {
+    restore_property(prop->next);
+    prop->name &= ~0x20;
+  }
 }
 
+/* When called with the tree root, recurses to all properties in the
+ * tree and removes all print marks.
+ */
+static void
+restore_node(SGFNode *node)
+{
+  if (node) {
+    restore_property(node->props);
+    restore_node(node->child);
+    restore_node(node->next);
+  }
+}
+
+
 /*
  * Opens filename and writes the game stored in the sgf structure.
  */
@@ -1377,8 +1487,15 @@ writesgf(SGFNode *root, const char *filename)
     return 0;
   }
 
+  sgf_column = 0;
   unparse_game(outfile, root, 1);
   fclose(outfile);
+  
+  /* Remove "printed" marks so that the tree can be written multiple
+   * times.
+   */
+  restore_node(root);
+  
   return 1;
 }
 
@@ -1415,4 +1532,3 @@ main()
  * c-basic-offset: 2
  * End:
  */
-
