@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -34,36 +35,84 @@ sgftree_clear(SGFTree *tree)
 int
 sgftree_readfile(SGFTree *tree, const char *infilename)
 {
-  tree->root = readsgffile(infilename);
-  tree->lastnode = NULL;
+  SGFNode *savetree = tree->root;
 
-  return tree->root != NULL;
+  tree->root = readsgffile(infilename);
+  if (tree->root == NULL) {
+    tree->root = savetree;
+    return 0;
+  }
+  
+  sgfFreeNode(savetree);
+  tree->lastnode = NULL;
+  return 1;
 }
 
+
+/* Go back one node in the tree. If lastnode is NULL, go to the last
+ * node (the one in main variant which has no children).
+ */
+
+int
+sgftreeBack(SGFTree *tree)
+{
+  if (tree->lastnode) {
+    if (tree->lastnode->parent)
+      tree->lastnode = tree->lastnode->parent;
+    else
+      return 0;
+  }
+  else
+    while (sgftreeForward(tree));
+  
+  return 1;
+}
+
+
+/* Go forward one node in the tree. If lastnode is NULL, go to the
+ * tree root.
+ */
+
+int
+sgftreeForward(SGFTree *tree)
+{
+  if (tree->lastnode) {
+    if (tree->lastnode->child)
+      tree->lastnode = tree->lastnode->child;
+    else
+      return 0;
+  }
+  else
+    tree->lastnode = tree->root;
+  
+  return 1;
+}
+
+
 /* ================================================================ */
 /*                        High level functions                      */
 /* ================================================================ */
 
 /*
- * Returns the node to modify. When node==NULL then lastnode is used,
- * except if lastnode is NULL, then the current end of game is used.
+ * Returns the node to modify. Use lastnode if available, otherwise
+ * follow the main variation to the current end of the game.
  */
 
 SGFNode *
-sgftreeNodeCheck(SGFTree *tree, SGFNode *node)
+sgftreeNodeCheck(SGFTree *tree)
 {
-   assert(tree->root);
+  SGFNode *node = NULL;
+  assert(tree->root);
 
-   if (!node) {
-     if (tree->lastnode)
-       node = tree->lastnode;
-     else {
-       node = tree->root;
-       while (node->child)
-	 node = node->child;
-     }
-   }
-   return node;
+  if (tree->lastnode)
+    node = tree->lastnode;
+  else {
+    node = tree->root;
+    while (node->child)
+      node = node->child;
+  }
+
+  return node;
 }
 
 
@@ -72,13 +121,11 @@ sgftreeNodeCheck(SGFTree *tree, SGFNode *node)
  * Return the node where the stone was added.
  */
 
-SGFNode *
-sgftreeAddStone(SGFTree *tree, SGFNode *node, int color, int movex, int movey)
+void
+sgftreeAddStone(SGFTree *tree, int color, int movex, int movey)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfAddStone(node, color, movex, movey);
-
-  return node;
 }
 
 
@@ -86,14 +133,11 @@ sgftreeAddStone(SGFTree *tree, SGFNode *node, int color, int movex, int movey)
  * Add a move to the gametree.
  */
 
-SGFNode *
-sgftreeAddPlay(SGFTree *tree, SGFNode *node, int color, int movex, int movey)
+void
+sgftreeAddPlay(SGFTree *tree, int color, int movex, int movey)
 {
-  node = sgftreeNodeCheck(tree, node);
-  node = sgfAddPlay(node, color, movex, movey);
-  tree->lastnode = node;
-
-  return node;
+  SGFNode *node = sgftreeNodeCheck(tree);
+  tree->lastnode = sgfAddPlay(node, color, movex, movey);
 }
 
 
@@ -102,19 +146,15 @@ sgftreeAddPlay(SGFTree *tree, SGFNode *node, int color, int movex, int movey)
  * ones rather than before.
  */
 
-SGFNode *
-sgftreeAddPlayLast(SGFTree *tree, SGFNode *node, int color,
-		   int movex, int movey)
+void
+sgftreeAddPlayLast(SGFTree *tree, int color, int movex, int movey)
 {
-  node = sgftreeNodeCheck(tree, node);
-  node = sgfAddPlayLast(node, color, movex, movey);
-  tree->lastnode = node;
-
-  return node;
+  SGFNode *node = sgftreeNodeCheck(tree);
+  tree->lastnode = sgfAddPlayLast(node, color, movex, movey);
 }
 
 
-SGFNode *
+void
 sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi)
 {
   SGFNode *root = sgfNewNode();
@@ -122,8 +162,7 @@ sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi)
   sgfAddPropertyInt(root, "SZ", boardsize);
   sgfAddPropertyFloat(root, "KM", komi);
   tree->root = root;
-
-  return root;
+  tree->lastnode = root;
 }
 
 
@@ -131,15 +170,14 @@ sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi)
  * Add a comment to a gametree.
  */
 
-SGFNode *
-sgftreeAddComment(SGFTree *tree, SGFNode *node, const char *comment)
+void
+sgftreeAddComment(SGFTree *tree, const char *comment)
 {
+  SGFNode *node;
   assert(tree && tree->root);
 
-  node = sgftreeNodeCheck(tree, node);
+  node = sgftreeNodeCheck(tree);
   sgfAddComment(node, comment);
-
-  return node;
 }
 
 
@@ -147,15 +185,14 @@ sgftreeAddComment(SGFTree *tree, SGFNode *node, const char *comment)
  * Place text on the board at position (i, j).
  */
 
-SGFNode *
-sgftreeBoardText(SGFTree *tree, SGFNode *node, int i, int j, const char *text)
+void
+sgftreeBoardText(SGFTree *tree, int i, int j, const char *text)
 {
+  SGFNode *node;
   assert(tree->root);
 
-  node = sgftreeNodeCheck(tree, node);
+  node = sgftreeNodeCheck(tree);
   sgfBoardText(node, i, j, text);
-
-  return node;
 }
 
 
@@ -163,15 +200,14 @@ sgftreeBoardText(SGFTree *tree, SGFNode *node, int i, int j, const char *text)
  * Place a character on the board at position (i, j).
  */
 
-SGFNode *
-sgftreeBoardChar(SGFTree *tree, SGFNode *node, int i, int j, char c)
+void
+sgftreeBoardChar(SGFTree *tree, int i, int j, char c)
 {
+  SGFNode *node;
   assert(tree->root);
 
-  node = sgftreeNodeCheck(tree, node);
+  node = sgftreeNodeCheck(tree);
   sgfBoardChar(node, i, j, c);
-
-  return node;
 }
 
 
@@ -179,13 +215,11 @@ sgftreeBoardChar(SGFTree *tree, SGFNode *node, int i, int j, char c)
  * Place a number on the board at position (i, j).
  */
 
-SGFNode * 
-sgftreeBoardNumber(SGFTree *tree, SGFNode *node, int i, int j, int number)
+void
+sgftreeBoardNumber(SGFTree *tree, int i, int j, int number)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfBoardNumber(node, i, j, number);
-
-  return node;
 }
 
 
@@ -193,13 +227,11 @@ sgftreeBoardNumber(SGFTree *tree, SGFNode *node, int i, int j, int number)
  * Place a circle mark on the board at position (i, j).
  */
 
-SGFNode * 
-sgftreeTriangle(SGFTree *tree, SGFNode *node, int i, int j)
+void
+sgftreeTriangle(SGFTree *tree, int i, int j)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfTriangle(node, i, j);
-
-  return node;
 }
 
 
@@ -207,13 +239,11 @@ sgftreeTriangle(SGFTree *tree, SGFNode *node, int i, int j)
  * Place a circle mark on the board at position (i, j).
  */
 
-SGFNode * 
-sgftreeCircle(SGFTree *tree, SGFNode *node, int i, int j)
+void
+sgftreeCircle(SGFTree *tree, int i, int j)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfCircle(node, i, j);
-
-  return node;
 }
 
 
@@ -221,13 +251,11 @@ sgftreeCircle(SGFTree *tree, SGFNode *node, int i, int j)
  * Place a square mark on the board at position (i, j).
  */
 
-SGFNode * 
-sgftreeSquare(SGFTree *tree, SGFNode *node, int i, int j)
+void
+sgftreeSquare(SGFTree *tree, int i, int j)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfSquare(node, i, j);
-
-  return node;
 }
 
 
@@ -235,38 +263,35 @@ sgftreeSquare(SGFTree *tree, SGFNode *node, int i, int j)
  * Place a (square) mark on the board at position (i, j).
  */
 
-SGFNode *
-sgftreeMark(SGFTree *tree, SGFNode *node, int i, int j)
+void
+sgftreeMark(SGFTree *tree, int i, int j)
 {
-  node = sgftreeNodeCheck(tree, node);
+  SGFNode *node = sgftreeNodeCheck(tree);
   sgfMark(node, i, j);
-
-  return node;
 }
 
 
 /*
- * Start a new variant. Returns a pointer to the new node.
+ * Start a new variant.
  */
 
-SGFNode *
-sgftreeStartVariant(SGFTree *tree, SGFNode *node)
+void
+sgftreeStartVariant(SGFTree *tree)
 {
-  node = sgftreeNodeCheck(tree, node);
-  return sgfStartVariant(node);
+  SGFNode *node = sgftreeNodeCheck(tree);
+  tree->lastnode = sgfStartVariant(node);
 }
 
 
 /*
- * Start a new variant as first child. Returns a pointer to the new node.
+ * Start a new variant as first child.
  */
 
-SGFNode *
-sgftreeStartVariantFirst(SGFTree *tree, SGFNode *node)
+void
+sgftreeStartVariantFirst(SGFTree *tree)
 {
-  node = sgftreeNodeCheck(tree, node);
-  return sgfStartVariantFirst(node);
-
+  SGFNode *node = sgftreeNodeCheck(tree);
+  tree->lastnode = sgfStartVariantFirst(node);
 }
 
 
