@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -52,13 +53,13 @@ void *xalloc(unsigned int);
 
 typedef struct SGFProperty_t {
   struct SGFProperty_t *next;
-  short name;           /* FIXME: should be char with enumed flags */
-  char  *value;         /* FIXME: different types */
+  short name;
+  char *value;
 } SGFProperty;
 
     
 typedef struct SGFNode_t {
-  SGFProperty      *props;
+  SGFProperty *props;
   struct SGFNode_t *parent;
   struct SGFNode_t *child;
   struct SGFNode_t *next;
@@ -80,7 +81,7 @@ void sgfAddPropertyFloat(SGFNode *node, const char *name, float val);
 void sgfOverwriteProperty(SGFNode *node, const char *name, const char *text);
 void sgfOverwritePropertyFloat(SGFNode *node, const char *name, float val);
 void sgfOverwritePropertyInt(SGFNode *node, const char *name, int val);
-void * xrealloc(void *pt, unsigned int size);
+void *xrealloc(void *pt, unsigned int size);
 SGFProperty *sgfMkProperty(const char *name, const  char *value,
 			   SGFNode *node, SGFProperty *last);
 void sgfFreeProperty(SGFProperty *prop);
@@ -88,10 +89,13 @@ void sgfFreeProperty(SGFProperty *prop);
 SGFNode *sgfAddStone(SGFNode *node, int color, int movex, int movey);
 SGFNode *sgfAddPlay(SGFNode *node, int who, int movex, int movey);
 SGFNode *sgfAddPlayLast(SGFNode *node, int who, int movex, int movey);
-int sgfPrintCharProperty(FILE *file, SGFNode *node, const char *name);
-int sgfPrintCommentProperty(FILE *file, SGFNode *node, const char *name);
-void sgfWriteResult(SGFNode *node, float score, int overwrite);
 
+void sgfWriteResult(SGFNode *node, float score, int overwrite);
+void sgf_write_header(SGFNode *root, int overwrite, int seed, float komi,
+		      int level, int rules);
+
+SGFNode *sgfLabel(SGFNode *node, const char *label, int i, int j);
+SGFNode *sgfLabelInt(SGFNode *node, int num, int i, int j);
 SGFNode *sgfCircle(SGFNode *node, int i, int j);
 SGFNode *sgfSquare(SGFNode *node, int i, int j);
 SGFNode *sgfTriangle(SGFNode *node, int i, int j);
@@ -112,8 +116,6 @@ SGFNode *readsgffile(const char *filename);
 SGFNode *readsgffilefuseki(const char *filename, int moves_per_game);
 
 /* Write SGF tree to a file. */
-void sgf_write_header(SGFNode *root, int overwrite, int seed, float komi,
-		      int level);
 int writesgf(SGFNode *root, const char *filename);
 
 
@@ -129,31 +131,29 @@ typedef struct SGFTree_t {
 
 
 void sgftree_clear(SGFTree *tree);
-int  sgftree_readfile(SGFTree *tree, const char *infilename);
+int sgftree_readfile(SGFTree *tree, const char *infilename);
 
-SGFNode *sgftreeAddPlay(SGFTree *tree, SGFNode *node, int color,
-			int movex, int movey);
-SGFNode *sgftreeAddPlayLast(SGFTree *tree, SGFNode *node, int color,
-			    int movex, int movey);
-SGFNode *sgftreeAddStone(SGFTree *tree, SGFNode *node,
-			 int color, int movex, int movey);
-void     sgftreeWriteResult(SGFTree *tree, float score, int overwrite);
-SGFNode *sgftreeNodeCheck(SGFTree *tree, SGFNode *node);
+int sgftreeBack(SGFTree *tree);
+int sgftreeForward(SGFTree *tree);
+
+void sgftreeAddPlay(SGFTree *tree, int color, int movex, int movey);
+void sgftreeAddPlayLast(SGFTree *tree, int color, int movex, int movey);
+void sgftreeAddStone(SGFTree *tree, int color, int movex, int movey);
+void sgftreeWriteResult(SGFTree *tree, float score, int overwrite);
+SGFNode *sgftreeNodeCheck(SGFTree *tree);
 	 
-SGFNode *sgftreeCircle  (SGFTree *tree, SGFNode *node, int i, int j);
-SGFNode *sgftreeSquare  (SGFTree *tree, SGFNode *node, int i, int j);
-SGFNode *sgftreeTriangle(SGFTree *tree, SGFNode *node, int i, int j);
-SGFNode *sgftreeMark    (SGFTree *tree, SGFNode *node, int i, int j);
-SGFNode *sgftreeAddComment(SGFTree *tree, SGFNode *node, const char *comment);
-SGFNode *sgftreeBoardText(SGFTree *tree, SGFNode *node, int i, int j,
-			  const char *text);
-SGFNode *sgftreeBoardChar(SGFTree *tree, SGFNode *node, int i, int j, char c);
-SGFNode *sgftreeBoardNumber(SGFTree *tree, SGFNode *node,
-			    int i, int j, int number);
-SGFNode *sgftreeStartVariant(SGFTree *tree, SGFNode *node);
-SGFNode *sgftreeStartVariantFirst(SGFTree *tree, SGFNode *node);
-SGFNode *sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi);
-void     sgftreeSetLastNode(SGFTree *tree, SGFNode *lastnode);
+void sgftreeCircle(SGFTree *tree, int i, int j);
+void sgftreeSquare(SGFTree *tree, int i, int j);
+void sgftreeTriangle(SGFTree *tree, int i, int j);
+void sgftreeMark(SGFTree *tree, int i, int j);
+void sgftreeAddComment(SGFTree *tree, const char *comment);
+void sgftreeBoardText(SGFTree *tree, int i, int j, const char *text);
+void sgftreeBoardChar(SGFTree *tree, int i, int j, char c);
+void sgftreeBoardNumber(SGFTree *tree, int i, int j, int number);
+void sgftreeStartVariant(SGFTree *tree);
+void sgftreeStartVariantFirst(SGFTree *tree);
+void sgftreeCreateHeaderNode(SGFTree *tree, int boardsize, float komi);
+void sgftreeSetLastNode(SGFTree *tree, SGFNode *lastnode);
 
 
 /* ---------------------------------------------------------------- */
