@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -24,6 +25,7 @@
 #include <assert.h>
 
 #include "gg_utils.h"
+#include "random.h"
 
 #ifdef HAVE_GLIB_H
 #include <glib.h>
@@ -245,8 +247,8 @@ gg_snprintf(char *dest, unsigned long len, const char *fmt, ...)
  * if available, otherwise substituting a workaround for portability.
  */
 
-int
-gg_gettimeofday2(struct timeval *tv, void *p)
+static int
+gg_gettimeofday2(struct timeval *tv)
 {
 #ifdef HAVE_GETTIMEOFDAY
   return gettimeofday(tv, NULL);
@@ -263,7 +265,7 @@ double
 gg_gettimeofday(void)
 {
   struct timeval tv;
-  gg_gettimeofday2(&tv, NULL);
+  gg_gettimeofday2(&tv);
   return tv.tv_sec + 1.e-6*tv.tv_usec;
 }
 
@@ -396,6 +398,40 @@ gg_sort(void *base, size_t nel, size_t width,
 }
 
 
+/* Linearly interpolate f(x) from the data given in interpolation_data. */
+float
+gg_interpolate(struct interpolation_data *f, float x)
+{
+  int i;
+  float ratio;
+  float diff;
+  if (x < f->range_lowerbound)
+    return f->values[0];
+  else if (x > f->range_upperbound)
+    return f->values[f->sections];
+  else {
+    ratio = ((float) f->sections) * (x - f->range_lowerbound)
+              /(f->range_upperbound - f->range_lowerbound);
+    i = (int) ratio;
+    diff = ratio - ((float) i);
+    if (0)
+      fprintf(stderr, "Floating point Ratio: %f, integer: %d, diff %f",
+	      ratio, i, diff);
+    return ((1 - diff) * f->values[i] + diff * f->values[i+1]);
+  }
+}
+
+
+/* This is the simplest function that returns appr. a when a is small,
+ * and approximately b when a is large.
+ */
+float
+soft_cap(float a, float b)
+{
+  return ((a * b) / ( a + b));
+}
+
+
 /* Reorientation of point (i, j) into (*ri, *rj) */
 void
 rotate(int i, int j, int *ri, int *rj, int bs, int rot)
@@ -472,6 +508,53 @@ inv_rotate(int i, int j, int *ri, int *rj, int bs, int rot)
 }
 
 
+/* Intermediate layer to random.c. gg_srand() should only be called via the
+ * functions below.
+ */
+ 
+/* Private variable remembering the random seed. */
+static unsigned int random_seed;
+
+unsigned int
+get_random_seed()
+{
+  return random_seed;
+}
+
+void
+set_random_seed(unsigned int seed)
+{
+  random_seed = seed;
+  gg_srand(seed);
+}
+
+/* Update the random seed with the current value in the random sequence.
+ * This should be called at the start of each new game.
+ */
+void
+update_random_seed(void)
+{
+  random_seed = gg_rand();
+  /* Since random seed 0 has a special interpretation when given as
+   * command line argument with the -r option, we make sure to avoid
+   * it.
+   */
+  if (random_seed == 0)
+    random_seed = 1;
+  gg_srand(random_seed);
+}
+
+
+/* Restart the pseudo-random sequence with the initialization given
+ * by the random seed. Should be called at each move.
+ */
+void
+reuse_random_seed()
+{
+  gg_srand(random_seed);
+}
+
+
 
 /*
  * Local Variables:
