@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -58,7 +59,6 @@ void gg_vsnprintf(char *dest, unsigned long len, const char *fmt,
 		  va_list args);
 void gg_snprintf(char *dest, unsigned long len, const char *fmt, ...);
 
-int gg_gettimeofday2(struct timeval *tv, void *p);
 double gg_gettimeofday(void);
 double gg_cputime(void);
 
@@ -67,13 +67,30 @@ int gg_normalize_float2int(float x, float a);
 void gg_sort(void *base, size_t nel, size_t width,
 	     int (*compar)(const void *, const void *));
 
+#define MAX_INTERPOLATION_STEPS 20
+struct interpolation_data
+{
+  int sections;
+  float range_lowerbound;
+  float range_upperbound;
+  float values[MAX_INTERPOLATION_STEPS + 1];
+};
+
+float gg_interpolate(struct interpolation_data *f, float x);
+float soft_cap(float a, float b);
 
 const char *gg_version(void);
 
 /* prototypes for basic reorientation functions */
 
-void  rotate(int i, int j, int *ri, int *rj, int bs, int rot);
-void  inv_rotate(int i, int j, int *ri, int *rj, int bs, int rot);
+void rotate(int i, int j, int *ri, int *rj, int bs, int rot);
+void inv_rotate(int i, int j, int *ri, int *rj, int bs, int rot);
+
+void update_random_seed(void);
+void set_random_seed(unsigned int seed);
+unsigned int get_random_seed(void);
+void reuse_random_seed(void);
+
 
 #endif /* _GG_UTILS_H_ */
 
